#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
It generates the list of banned candidate fusion genes. This list is hard coded inhere.



Author: Daniel Nicorici, Daniel.Nicorici@gmail.com

Copyright (c) 2009-2017 Daniel Nicorici

This file is part of FusionCatcher.

FusionCatcher is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FusionCatcher is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with FusionCatcher (see file 'COPYING.txt').  If not, see
<http://www.gnu.org/licenses/>.

By default, FusionCatcher is running BLAT aligner
<http://users.soe.ucsc.edu/~kent/src/> but it offers also the option to disable
all its scripts which make use of BLAT aligner if you choose explicitly to do so.
BLAT's license does not allow to be used for commercial activities. If BLAT
license does not allow to be used in your case then you may still use
FusionCatcher by forcing not use the BLAT aligner by specifying the option
'--skip-blat'. Fore more information regarding BLAT please see its license.

Please, note that FusionCatcher does not require BLAT in order to find
candidate fusion genes!

This file is not running/executing/using BLAT.

"""
import sys
import os
import optparse
import symbols

if __name__ == '__main__':

    #command line parsing

    usage = "%prog [options]"
    description = """It generates the list of banned candidate fusion genes."""
    version = "%prog 0.10 beta"

    parser = optparse.OptionParser(usage=usage,description=description,version=version)

    parser.add_option("--organism",
                      action = "store",
                      type = "string",
                      dest = "organism",
                      default = "homo_sapiens",
                      help="""The name of the organism for which the list of banned candidate fusion genes is generated, e.g. homo_sapiens, mus_musculus, etc. Default is '%default'.""")

    parser.add_option("--output",
                      action="store",
                      type="string",
                      dest="output_directory",
                      default = '.',
                      help="""The output directory where the list of banned candidate fusion genes is generated. Default is '%default'.""")


    (options,args) = parser.parse_args()

    # validate options
    if not (options.output_directory
            ):
        parser.print_help()
        sys.exit(1)


    #
    #
    #

    print "Generating the list of banned fusion genes..."
    fusions = dict()

    fusions['rattus_norvegicus'] = [
        ['BIN2','CELA1'],
        ['ALB','FETUB'],
        ['C8G','LCN12'],
        ['FETUB','TF'],
        ['ALB','TF'],
        ['APOC1','APOC4'],
        ['GGH','TTPA'],
        ['GM7694','NOS1AP'],
        ['ALB','C3'],
        ['APOC2','APOC4'],
        ['DDT','GSTT3'],
        ['CES1H','CES2I'],
        ['PHYH','TF'],
        ['AHSG','ALB'],
        ['ALB','APOE'],
        ['ALB','SERPINF1'],
        ['NDUFB9','RNF139'],
        ['PROC','TF'],
        ['ALB','CTSH'],
        ['ALB','FGA'],
        ['ALB','SERPINA1'],
        ['ALB','SERPINA3K'],
        ['CRYGA','D630023F18RIK'],
        ['RBP4','SERPINA1'],
        ['ACOX2','TF'],
        ['ALB','APLP2'],
        ['ALB','HP'],
        ['ALB','PHYH'],
        ['ALB','TTR'],
        ['ALB','VTN'],
        ['ALDOB','MRPL50'],
        ['APOA5','TF'],
        ['ATP5L','UBE4A'],
        ['CRP','TF'],
        ['FABP7','SMPDL3A'],
        ['GOLT1A','KISS1'],
        ['GSTA2','TF'],
        ['IGKC','RGD1559916'],
        ['SERPINF2','WDR81'],
        ['TF','TTR'],
        ['ADNP','DPM1'],
        ['AHCY','ALB'],
        ['AHSG','HP'],
        ['AHSG','RBP4'],
        ['ALB','ALDH2'],
        ['ALB','AMACR'],
        ['ALB','AMBP'],
        ['ALB','APOA1'],
        ['ALB','C2'],
        ['ALB','CES1C'],
        ['ALB','CLU'],
        ['ALB','CPS1'],
        ['ALB','CRP'],
        ['ALB','CYP2E1'],
        ['ALB','FGB'],
        ['ALB','GLUL'],
        ['ALB','HMGCS2'],
        ['ALB','HPX'],
        ['ALB','MAT1A'],
        ['ALB','SERPINF2'],
        ['ALB','SLCO1B3'],
        ['AMACR','C1QTNF3'],
        ['AMBP','TF'],
        ['APOB','FETUB'],
        ['APOE','SERPINF1'],
        ['APOH','CRP'],
        ['APOH','HP'],
        ['ATP5B','SLC34A1'],
        ['C2','PHYH'],
        ['CAR3','TF'],
        ['CEACAM18-PS1','ZFP819'],
        ['CYP1A2','SERPINA1'],
        ['FGB','SERPINA1'],
        ['GC','RBP4'],
        ['GPD1','SERPINA1'],
        ['IGFBP5','ITM2B'],
        ['KDELC1','TEX30'],
        ['METTL21B','TSFM'],
        ['MIOX','SLC34A1'],
        ['SERPINA1','TTR'],
        ['ALB','ENSRNOG00000005599'],
        ['ALB','ENSRNOG00000009273'],
        ['ALB','ENSRNOG00000050117'],
        ['ARHGAP5','ENSRNOG00000047751'],
        ['CORO7','ENSRNOG00000046980'],
        ['ENSRNOG00000005599','PROC'],
        ['ENSRNOG00000011640','MECOM'],
        ['ENSRNOG00000015605','PTPRK'],
        ['ENSRNOG00000042512','PSMD10'],
        ['ENSRNOG00000047366','RGD1311595'],
        ['ENSRNOG00000048877','PLCB1'],
        ['ENSRNOG00000049177','PSD3'],
        ['ENSRNOG00000050066','IGKC'],
        ['ENSRNOG00000050188','PITPNC1'],
        ['GM10324','GM1976'],
        ['ISY1','RAB43'],
        ['SYS1','DBNDD2'],
        ['GM9900','ZFP672'],
        ['BRDT','EPHX4'],
        ['A930018M24RIK','KLHL33'],
        ['GM13308','GM12394'],
        ['2610524H06RIK','1500011B03RIK'],
        ['GRIA1','KIF5A'],
        ['APP','CPLX2'],
        ['ARF3','SPOCK2'],
        ['GTF2H5','TULP4'],
        ['ATP2B3','CAMK2A'],
        ['GM20687','THTPA'],
        ['PYGO1','GM20510'],
        ['A930011G23RIK','RASGEF1B'],
        ['PIGZ','NCBP2'],
        ['YOD1','AA986860'],
        ['PCSK2','CAMK2A'],
        ['SAMD5','SASH1'],
        ['PARP6','PKM'],
        ['PCP4L1','SDHC'],
        ['PVALB','IFT27'],
        ['C630016N16RIK','SBSN'],
        ['SIN3B','F2RL3'],
        ['TTBK1','KIF5A'],
        ['PHACTR1','CAMK2A'],
        ['NOLC1','GM5124'],
        ['TMEM219','TAOK2'],
        ['6330407J23RIK','9330159F19RIK'],
        ['MBP','KIF5A'],
        ['ZSCAN22','RPS5'],
        ['CPLX2','APP'],
        ['CAMK2A','ARF3'],
        ['VSNL1','CAMK2A'],
        ['SPARCL1','CPLX2'],
        ['GM13832','MSI1'],
        ['AV099323','CDS2'],
        ['PTPRN2','KIF5A'],
        ['A230083G16RIK','MLF2'],
        ['TOP3B','PPM1F'],
        ['METTL10','FAM53B'],
        ['RNF139','NDUFB9'],
        ['SPOP','STX1B'],
        ['PANX2','TRABD'],
        ['GM5124','NOLC1'],
        ['OTUD5','PIM2'],
        ['YWHAG','GNG7'],
        ['ARF3','DZANK1'],
        ['GALNTL6','AC116875.1'],
        ['CAMK2A','SYT1'],
        ['E130111B04RIK','MPP3'],
        ['GEMIN7','PPP1R37'],
        ['PCED1A','CPXM1'],
        ['ARF3','STX1B'],
        ['CAMK2A','YWHAG'],
        ['NAT15','CLUAP1'],
        ['IQCJ','SCHIP1'],
        ['PCDHGA3','CAMK2A'],
        ['SYNA','CLIP2'],
        ['ACSS2','GM14256'],
        ['CNRIP1','PPP3R1'],
        ['FTL','TIMP1'],
        ['MFGE8','HAPLN3'],
        ['CLCF1','POLD4'],
        ['CTBS','GNG5'],
        ['MFSD7','ATP5I']
    ]

    fusions['mus_musculus'] = [
        ['A930011G23RIK','RASGEF1B'],
        ['GM10324','GM1976'],
        ['ISY1','RAB43'],
        ['SYS1','DBNDD2'],
        ['GM9900','ZFP672'],
        ['BRDT','EPHX4'],
        ['A930018M24RIK','KLHL33'],
        ['GM13308','GM12394'],
        ['2610524H06RIK','1500011B03RIK'],
        ['GRIA1','KIF5A'],
        ['APP','CPLX2'],
        ['ARF3','SPOCK2'],
        ['GTF2H5','TULP4'],
        ['ATP2B3','CAMK2A'],
        ['GM20687','THTPA'],
        ['PYGO1','GM20510'],
        ['A930011G23RIK','RASGEF1B'],
        ['PIGZ','NCBP2'],
        ['YOD1','AA986860'],
        ['PCSK2','CAMK2A'],
        ['SAMD5','SASH1'],
        ['PARP6','PKM'],
        ['PCP4L1','SDHC'],
        ['PVALB','IFT27'],
        ['C630016N16RIK','SBSN'],
        ['SIN3B','F2RL3'],
        ['TTBK1','KIF5A'],
        ['PHACTR1','CAMK2A'],
        ['NOLC1','GM5124'],
        ['TMEM219','TAOK2'],
        ['6330407J23RIK','9330159F19RIK'],
        ['MBP','KIF5A'],
        ['ZSCAN22','RPS5'],
        ['CPLX2','APP'],
        ['CAMK2A','ARF3'],
        ['VSNL1','CAMK2A'],
        ['SPARCL1','CPLX2'],
        ['GM13832','MSI1'],
        ['AV099323','CDS2'],
        ['PTPRN2','KIF5A'],
        ['A230083G16RIK','MLF2'],
        ['TOP3B','PPM1F'],
        ['METTL10','FAM53B'],
        ['RNF139','NDUFB9'],
        ['SPOP','STX1B'],
        ['PANX2','TRABD'],
        ['GM5124','NOLC1'],
        ['OTUD5','PIM2'],
        ['YWHAG','GNG7'],
        ['ARF3','DZANK1'],
        ['GALNTL6','AC116875.1'],
        ['CAMK2A','SYT1'],
        ['E130111B04RIK','MPP3'],
        ['GEMIN7','PPP1R37'],
        ['PCED1A','CPXM1'],
        ['ARF3','STX1B'],
        ['CAMK2A','YWHAG'],
        ['NAT15','CLUAP1'],
        ['IQCJ','SCHIP1'],
        ['PCDHGA3','CAMK2A'],
        ['SYNA','CLIP2'],
        ['ACSS2','GM14256'],
        ['CNRIP1','PPP3R1']
    ]

    fusions['canis_familiaris'] = [
        ['ENSCAFG00000011681','ENSCAFG00000040733'],
        ['ENSCAFG00000028546','ENSCAFG00000000193'],
        ['ENSCAFG00000028546','ENSCAFG00000000193'],
        ['ENSCAFG00000028546','ENSCAFG00000000193'],
        ['ENSCAFG00000038498','ENSCAFG00000004924'],
        ['ENSCAFG00000035698','ENSCAFG00000007015'],
        ['ENSCAFG00000040551','ENSCAFG00000007135']

    ]


    fusions['homo_sapiens'] = [
        ['7SK','CLK4'],
        ['7SK','COG7'],
        ['A2M','ALB'],
        ['A2M','CRP'],
        ['A2M','EPAS1'],
        ['A2M','FGG'],
        ['DUS4L','BCAP29'],
        ['A2M','HP'],
        ['A2M','MB'],
        ['A2M','SFTPA1'],
        ['A2M','SFTPC'],
        ['A2M','SIK1'],
        ['A2ML1','KRT4'],
        ['A2ML1','MYH11'],
        ['A2ML1','NDE1'],
        ['A2ML1','PHC1'],
        ['A2ML1','RIMKLB'],
        ['A3GALT2','PHC2'],
        ['AACSP1','GBP6'],
        ['AADACL3','C1ORF158'],
        ['AAK1','CTC-513N18.7'],
        ['AAK1','ZNF626'],
        ['AAMP','COL1A1'],
        ['AARS','SMR3B'],
        ['AB015752.3','TMEM64'],
        ['ABAT','ALB'],
        ['ABCA13','LYZ'],
        ['ABCA17P','CCNF'],
        ['ABCA3','SFTPC'],
        ['ABCA8','LYZ'],
        ['ABHD15-AS1','INTS7'],
        ['ABHD17C','CTGF'],
        ['ABHD2','CCNY'],
        ['ABHD2','LYZ'],
        ['ABHD2','SMR3B'],
        ['ABI1','LYZ'],
        ['ABI2','CYP20A1'],
        ['ABI2','SMR3B'],
        ['ABI2','TG'],
        ['ABL1','ZG16B'],
        ['ABL2','SFTPB'],
        ['ABLIM1','HSF2'],
        ['ABLIM1','SFTPB'],
        ['ABR','LYZ'],
        ['ABTB2','MMADHC'],
        ['AC000124.1','ARF5'],
        ['AC002116.7','WDR62'],
        ['AC002398.11','AD000671.6'],
        ['AC002398.12','ACTG1'],
        ['AC002398.12','DES'],
        ['AC002398.12','MYL2'],
        ['AC002398.12','RP11-814P5.1'],
        ['AC002398.12','SPARCL1'],
        ['AC002480.2','STEAP1B'],
        ['AC002519.6','PPP2CB'],
        ['AC002550.5','IQCK'],
        ['AC004237.1','BAIAP2-AS1'],
        ['AC004381.7','THUMPD1'],
        ['AC004510.3','ZNF861P'],
        ['AC004895.4','USP42'],
        ['AC004967.7','OR7E7P'],
        ['AC004980.8','PRKRIP1'],
        ['AC004980.9','PMS2'],
        ['AC005013.5','CTB-113D17.1'],
        ['AC005077.5','SRRM3'],
        ['AC005077.7','GTF2IP7'],
        ['AC005154.7','GGCT'],
        ['AC005237.4','TG'],
        ['AC005253.4','CRLF1'],
        ['AC005336.4','IPMKP1'],
        ['AC005487.2','DNAJB9'],
        ['AC005534.6','RPL26'],
        ['AC005562.1','SMURF2'],
        ['AC005578.3','USP34'],
        ['AC005703.3','PMP22'],
        ['AC005740.3','NDUFS1'],
        ['AC005775.2','ODF3L2'],
        ['AC005786.7','FZR1'],
        ['AC005944.2','PLEC'],
        ['AC005944.2','SFTPC'],
        ['AC006042.7','AC009473.1'],
        ['AC006050.2','KRT42P'],
        ['AC006116.15','AC006116.21'],
        ['AC006262.10','AC006262.5'],
        ['AC006277.2','ZNF77'],
        ['AC006538.1','SLC39A3'],
        ['AC006547.14','KB-1269D1.8'],
        ['AC007038.7','RPE'],
        ['AC007192.4','LYZ'],
        ['AC007228.11','CFLAR'],
        ['AC007228.9','TOB2'],
        ['AC007255.8','ZNRF2P2'],
        ['AC007365.1','AFTPH'],
        ['AC007563.5','MYH11'],
        ['AC007566.10','GATAD1'],
        ['AC008268.2','GPAT2'],
        ['AC008278.2','DDX1'],
        ['AC008279.1','RPL5'],
        ['AC008280.5','ACYP2'],
        ['AC008427.2','MFF'],
        ['AC008440.5','IGF2'],
        ['AC008440.5','MTCH2'],
        ['AC008440.5','NLRP1'],
        ['AC008440.5','SPARCL1'],
        ['AC009014.3','TRPC7'],
        ['AC009133.14','SFTPC'],
        ['AC009133.22','LYZ'],
        ['AC009228.1','FAM228A'],
        ['AC009237.11','RP11-440D17.5'],
        ['AC009362.2','RPL27'],
        ['AC009413.2','RPS12'],
        ['AC009951.1','ZEB2'],
        ['AC009961.3','BAZ2B'],
        ['AC010095.6','COX7B'],
        ['AC010136.2','RUFY4'],
        ['AC010149.4','CLASP2'],
        ['AC010149.4','MORC1'],
        ['AC010149.4','RP11-79E3.2'],
        ['AC010504.2','IGF2'],
        ['AC010518.3','LILRA2'],
        ['AC010524.2','TEAD2'],
        ['AC010642.1','ZSCAN22'],
        ['AC010677.5','RPL23'],
        ['AC010731.4','ADAM23'],
        ['AC010878.3','STRAP'],
        ['AC011513.3','LYZ'],
        ['AC011513.3','PIGR'],
        ['AC011523.2','KLK3'],
        ['AC011558.5','CTSB'],
        ['AC012309.5','LINC00665'],
        ['AC012322.1','EBPL'],
        ['AC012442.6','CHCHD5'],
        ['AC012485.2','PER2'],
        ['AC013463.2','AC019181.2'],
        ['AC015849.2','RP11-1143G9.4'],
        ['AC015849.2','TAF15'],
        ['AC015971.2','B2M'],
        ['AC015987.1','IGF2'],
        ['AC016582.2','WDR87'],
        ['AC016586.1','ZBTB7A'],
        ['AC016717.1','NYAP2'],
        ['AC016722.2','SOCS5'],
        ['AC016831.7','RP11-138A9.2'],
        ['AC016907.3','ALK'],
        ['AC018731.3','NEB'],
        ['AC018737.3','TSN'],
        ['AC018890.6','TG'],
        ['AC019172.2','DCDC2C'],
        ['AC019349.5','CRNN'],
        ['AC019349.5','FLNA'],
        ['AC019349.5','GSN'],
        ['AC019349.5','KRT15'],
        ['AC019349.5','KRT4'],
        ['AC019349.5','NDE1'],
        ['AC019349.5','S100A9'],
        ['AC019349.5','SPINK5'],
        ['AC020956.3','LYPD4'],
        ['AC022182.2','NASP'],
        ['AC024560.3','AC069213.1'],
        ['AC024592.9','RP1-122P22.4'],
        ['AC025335.1','FLG2'],
        ['AC025335.1','PLXDC2'],
        ['AC026167.1','AC027119.1'],
        ['AC026202.3','TEX101'],
        ['AC053503.6','CRNN'],
        ['AC053503.6','DLGAP4-AS1'],
        ['AC053503.6','KRT13'],
        ['AC053503.6','MYH6'],
        ['AC053503.6','MYL2'],
        ['AC053503.6','NPPA'],
        ['AC053503.6','PPP1R12B'],
        ['AC053503.6','SYNPO'],
        ['AC053503.6','TNNC1'],
        ['AC053503.6','TNNI3'],
        ['AC053503.6','TNNT2'],
        ['AC053503.6','TPM1'],
        ['AC055811.1','MPRIP'],
        ['AC068580.7','AC139143.2'],
        ['AC068831.8','RCCD1'],
        ['AC069363.1','CCL4'],
        ['AC073063.10','SARNP'],
        ['AC073133.1','LINC01006'],
        ['AC073343.13','IGF2'],
        ['AC073343.13','ZNF316'],
        ['AC073610.5','IGH@'],
        ['AC073842.19','LAMTOR4'],
        ['AC074008.1','ALMS1'],
        ['AC074117.10','GPD1L'],
        ['AC078842.3','DGKI'],
        ['AC078842.3','PTN'],
        ['AC078899.3','ZNF66'],
        ['AC079354.5','DAZAP2P1'],
        ['AC079753.5','IL37'],
        ['AC079767.4','PPP1R14BP2'],
        ['AC083900.1','PLEKHM3'],
        ['AC084219.2','ZNF224'],
        ['AC087380.14','UBQLNL'],
        ['AC087650.1','RPL29'],
        ['AC091493.2','DAZL'],
        ['AC092431.1','ANXA4'],
        ['AC092687.4','RP11-245G13.2'],
        ['AC092798.2','RPL32'],
        ['AC092933.3','RPS29'],
        ['AC093106.5','NDUFB9'],
        ['AC093698.4','CRYGA'],
        ['AC093698.4','CRYGD'],
        ['AC096579.13','ANPEP'],
        ['AC096579.13','CALD1'],
        ['AC096579.13','CD53'],
        ['AC096579.13','CTGF'],
        ['AC096579.13','DES'],
        ['AC096579.13','DPCR1'],
        ['AC096579.13','EIF2B5'],
        ['AC096579.13','HBB'],
        ['AC096579.13','IGKV1D-39'],
        ['AC096579.13','IGKV2D-29'],
        ['AC096579.13','IGKV2D-30'],
        ['AC096579.13','IGKV3D-11'],
        ['AC096579.13','IGKV3D-20'],
        ['AC096579.13','IGKV4-1'],
        ['AC096579.13','JCHAIN'],
        ['AC096579.13','MUC6'],
        ['AC096579.13','NDE1'],
        ['AC096579.13','PGC'],
        ['AC096579.13','PIGR'],
        ['AC096579.13','PRH1'],
        ['AC096579.13','PRH1-PRR4'],
        ['AC096579.13','PRM2'],
        ['AC096579.13','PRR4'],
        ['AC096579.13','SMR3B'],
        ['AC096579.13','TFF1'],
        ['AC096579.13','TRBC2'],
        ['AC096579.7','CD74'],
        ['AC096579.7','HLA@'],
        ['AC096579.7','IGH@'],
        ['AC096579.7','IGK@'],
        ['AC096579.7','IGL@'],
        ['AC097468.7','LINC01494'],
        ['AC099344.3','SLC52A3'],
        ['AC099850.1','PRR11'],
        ['AC100802.3','EMP2'],
        ['AC104297.1','PPA1'],
        ['AC104532.2','MKLN1'],
        ['AC104699.1','LAMP2'],
        ['AC104820.2','ITGA4'],
        ['AC105053.3','ATOH8'],
        ['AC105342.1','ANKRD11'],
        ['AC111186.1','SLC16A5'],
        ['AC112721.1','COL6A3'],
        ['AC112721.2','COL6A3'],
        ['AC113607.1','TMEM18'],
        ['AC116366.6','IRF1'],
        ['AC132217.4','SPARC'],
        ['AC138430.4','FAM129A'],
        ['AC141586.5','KCTD5'],
        ['AC195454.1','CHRNA9'],
        ['AC226119.4','SFTPB'],
        ['AC226119.5','SFTPB'],
        ['ACACA','EIF3G'],
        ['ACAD8','GLB1L3'],
        ['ACADL','KANSL1L'],
        ['ACADVL','LTBP2'],
        ['ACE2','TMEM27'],
        ['ACIN1','MTFR1L'],
        ['ACIN1','MUC7'],
        ['ACIN1','RALGAPA1'],
        ['ACIN1','RP1-317E23.6'],
        ['ACKR2','ALB'],
        ['ACKR2','CALR'],
        ['ACO2','MYH6'],
        ['ACPP','ACTB'],
        ['ACPP','ACTG2'],
        ['ACPP','B2M'],
        ['ACPP','CALD1'],
        ['ACPP','EEF1A1'],
        ['ACPP','ITGB1'],
        ['ACPP','MSMB'],
        ['ACPP','MYH11'],
        ['ACPP','MYLK'],
        ['ACPP','NCAPD3'],
        ['ACPP','SGMS1'],
        ['ACR','SHANK3'],
        ['ACRBP','RMI2'],
        ['ACSF2','ALB'],
        ['ACSL1','ALB'],
        ['ACSL1','FTX'],
        ['ACSL1','HBB'],
        ['ACSL1','RP11-351N6.1'],
        ['ACSL1','RP11-701P16.5'],
        ['ACSL4','SULT2A1'],
        ['ACSS1','APMAP'],
        ['ACTA1','ACTA2-AS1'],
        ['ACTA1','HSPB6'],
        ['ACTA1','IGFBP5'],
        ['ACTA1','MYL2'],
        ['ACTA1','PTRF'],
        ['ACTA1','RP11-814P5.1'],
        ['ACTA1','STAMBPL1'],
        ['ACTA1','TNNI3'],
        ['ACTA2','IGFBP5'],
        ['ACTA2','NPPA-AS1'],
        ['ACTA2-AS1','ACTB'],
        ['ACTA2-AS1','ACTC1'],
        ['ACTA2-AS1','ACTG1'],
        ['ACTA2-AS1','ACTG2'],
        ['ACTA2-AS1','COL1A1'],
        ['ACTA2-AS1','NPPA'],
        ['ACTA2-AS1','RP11-814P5.1'],
        ['ACTB','ACTG1'],
        ['ACTB','ACTN1'],
        ['ACTB','AQP3'],
        ['ACTB','ATP8B4'],
        ['ACTB','AZU1'],
        ['ACTB','CAPN12'],
        ['ACTB','CFL1'],
        ['ACTB','CH17-12M21.1'],
        ['ACTB','COL1A1'],
        ['ACTB','CPA1'],
        ['ACTB','CTSB'],
        ['ACTB','DYNC1H1'],
        ['ACTB','ENO1'],
        ['ACTB','FBN1'],
        ['ACTB','FLNA'],
        ['ACTB','FN1'],
        ['ACTB','GAPDH'],
        ['ACTB','GPX3'],
        ['ACTB','HDGF'],
        ['ACTB','HERC2'],
        ['ACTB','HLA@'],
        ['ACTB','HSPA5'],
        ['ACTB','HSPA8'],
        ['ACTB','IGF2'],
        ['ACTB','IQGAP1'],
        ['ACTB','ITGA3'],
        ['ACTB','ITGA5'],
        ['ACTB','ITGB2'],
        ['ACTB','KLK3'],
        ['ACTB','KRT13'],
        ['ACTB','LAPTM5'],
        ['ACTB','LGALS1'],
        ['ACTB','LLFOS-48D6.2'],
        ['ACTB','LOXL2'],
        ['ACTB','LRP1'],
        ['ACTB','LYZ'],
        ['ACTB','MARCH6'],
        ['ACTB','MBOAT7'],
        ['ACTB','MDK'],
        ['ACTB','MPO'],
        ['ACTB','MUC7'],
        ['ACTB','NAMPT'],
        ['ACTB','NDE1'],
        ['ACTB','OAZ1'],
        ['ACTB','PIGR'],
        ['ACTB','PKM'],
        ['ACTB','PLEC'],
        ['ACTB','POTEM'],
        ['ACTB','PTBP1'],
        ['ACTB','PTMA'],
        ['ACTB','RAB11FIP1'],
        ['ACTB','RP5-972B16.2'],
        ['ACTB','RPS4X'],
        ['ACTB','SCUBE3'],
        ['ACTB','SENP3-EIF4A1'],
        ['ACTB','SERPINH1'],
        ['ACTB','SFTPA1'],
        ['ACTB','SFTPA2'],
        ['ACTB','SFTPC'],
        ['ACTB','SLC16A3'],
        ['ACTB','SMR3B'],
        ['ACTB','SNRPD3'],
        ['ACTB','SPARC'],
        ['ACTB','STAMBPL1'],
        ['ACTB','STATH'],
        ['ACTB','SULF1'],
        ['ACTB','TBCA'],
        ['ACTB','TG'],
        ['ACTB','TGFBI'],
        ['ACTB','THBS1'],
        ['ACTB','TIMM44'],
        ['ACTB','TIMP1'],
        ['ACTB','TNC'],
        ['ACTB','TOP1'],
        ['ACTB','TPM2'],
        ['ACTB','TRAM1'],
        ['ACTB','VIM'],
        ['ACTB','YWHAG'],
        ['ACTBP8','RP11-459O1.2'],
        ['ACTC1','CRYAB'],
        ['ACTC1','HSPB6'],
        ['ACTC1','NPPA-AS1'],
        ['ACTC1','POLDIP2'],
        ['ACTG1','ALB'],
        ['ACTG1','AMY2A'],
        ['ACTG1','CFL1'],
        ['ACTG1','COL1A1'],
        ['ACTG1','COL1A2'],
        ['ACTG1','COL3A1'],
        ['ACTG1','EEF2'],
        ['ACTG1','KRT13'],
        ['ACTG1','LYZ'],
        ['ACTG1','MYL6'],
        ['ACTG1','PCOLCE'],
        ['ACTG1','SLC7A5'],
        ['ACTG1','SPARC'],
        ['ACTG1','STAMBPL1'],
        ['ACTG1','TMEM259'],
        ['ACTG1','TMSB10'],
        ['ACTG1','TPM2'],
        ['ACTG1','TRB@'],
        ['ACTG1','XIST'],
        ['ACTG2','AHNAK'],
        ['ACTG2','CALR'],
        ['ACTG2','COL1A2'],
        ['ACTG2','GREB1L'],
        ['ACTG2','MYLK'],
        ['ACTG2','MYLK-AS1'],
        ['ACTG2','NDE1'],
        ['ACTG2','RP11-814P5.1'],
        ['ACTG2','STAMBPL1'],
        ['ACTN1','FN1'],
        ['ACTN1','HBB'],
        ['ACTN1','KRT4'],
        ['ACTN1','LASP1'],
        ['ACTN1','MYH9'],
        ['ACTN1','NOC2L'],
        ['ACTN2','CSRP3'],
        ['ACTN2','TNNT2'],
        ['ACTN2','TPM1'],
        ['ACTN4','FASN'],
        ['ACTN4','LYZ'],
        ['ACTN4','PLIN4'],
        ['ACTN4','PRSS1'],
        ['ACTN4','PTRF'],
        ['ACTN4','STATH'],
        ['ACTN4','TG'],
        ['ACTN4','TRB@'],
        ['ACTR10','CRHR1-IT1_'],
        ['ACTR6','MPO'],
        ['ACTR8','SELK'],
        ['ACVR1B','STATH'],
        ['ACYP2','GBP6'],
        ['ADAD1','IL21-AS1'],
        ['ADAM10','MRI1'],
        ['ADAM10','PTMS'],
        ['ADAM12','IGF2'],
        ['ADAM32','ADAM9'],
        ['ADAM5','RP11-122L4.2'],
        ['ADAMTS16','ICE1'],
        ['ADAMTS17','SPATA41'],
        ['ADAMTS4','PLIN4'],
        ['ADAMTSL3','LYZ'],
        ['ADAMTSL3','SH3GL3'],
        ['ADAMTSL4-AS1','IGF2'],
        ['ADAP2','RNF135'],
        ['ADAR','SFTPB'],
        ['ADCK3','PSEN2'],
        ['ADCK4','NUMBL'],
        ['ADCY1','ZSCAN5A'],
        ['ADCY10','FAM177A1'],
        ['ADCY10P1','BGN'],
        ['ADCY10P1','NFYA'],
        ['ADCY3','COL1A1'],
        ['ADCY5','NPPA'],
        ['ADD1','ERGIC1'],
        ['ADD1','LYZ'],
        ['ADD1','SH3BP2'],
        ['ADD1','TG'],
        ['ADD3','ENAM'],
        ['ADD3','LPP'],
        ['ADD3','TG'],
        ['ADGRA3','TECRL'],
        ['ADGRE4P','RP11-1137G4.4'],
        ['ADGRE5','AK2'],
        ['ADGRG1','EEF1A1'],
        ['ADGRG5','HMGB3P32'],
        ['ADGRG7','TFG'],
        ['ADGRL1','SUZ12'],
        ['ADGRL2','FAM134C'],
        ['ADGRL2','ZG16B'],
        ['ADH1B','C1S'],
        ['ADH1B','HP'],
        ['ADH1B','RP11-573D15.8'],
        ['ADH1B','TXNL4B'],
        ['ADH4','ALB'],
        ['ADH4','C3'],
        ['ADIG','ARHGAP40'],
        ['ADIPOQ','ANKRD31'],
        ['ADIPOQ','CD36'],
        ['ADIPOQ','CLU'],
        ['ADIPOQ','CPM'],
        ['ADIPOQ','DNAH14'],
        ['ADIPOQ','IGK@'],
        ['ADIPOQ','TLDC2'],
        ['ADIPOQ','VKORC1L1'],
        ['ADIPOQ-AS1','CWF19L1'],
        ['ADIPOQ-AS1','PLIN4'],
        ['ADIPOQ-AS1','PTRF'],
        ['ADIPOR1','CYB5R1'],
        ['ADM','TMEM214'],
        ['ADNP','DPM1'],
        ['ADPRHL1','DCUN1D2'],
        ['ADPRHL1','RP11-102K13.5'],
        ['ADRBK1','LYZ'],
        ['ADSL','SGSM3'],
        ['AEBP2','PLEKHA5'],
        ['AES','COL5A1'],
        ['AES','EPN1'],
        ['AES','FLNC'],
        ['AES','MUC7'],
        ['AES','PRRX2'],
        ['AES','SFTPC'],
        ['AES','SMR3B'],
        ['AF001548.5','KRT13'],
        ['AF064860.7','IGSF5'],
        ['AF127936.9','HBB'],
        ['AF213884.2','RP11-499E18.1'],
        ['AFAP1L2','TG'],
        ['AFF1','TG'],
        ['AFF4','F11R'],
        ['AFF4','RP11-544M22.13'],
        ['AFG3L2P1','LYZ'],
        ['AFMID','LRG1'],
        ['AFMID','LYZ'],
        ['AGAP11','BMS1'],
        ['AGAP3','SFTPC'],
        ['AGAP6','TIMM23B'],
        ['AGBL2','MTCH2'],
        ['AGBL5','HBB'],
        ['AGER','CD74'],
        ['AGER','IFITM2'],
        ['AGER','SFTPA2'],
        ['AGER','SFTPC'],
        ['AGFG2','RP11-44M6.1'],
        ['AGL','HIAT1'],
        ['AGMAT','FBXW12'],
        ['AGPAT2','LRP1'],
        ['AGPAT3','TG'],
        ['AGR2','LYZ'],
        ['AGRN','PRNP'],
        ['AGT','ALB'],
        ['AGT','C1ORF198'],
        ['AGT','FGA'],
        ['AGT','FGG'],
        ['AGXT','SERPINA1'],
        ['AHCYL2','PRPF4'],
        ['AHNAK','CSRP1'],
        ['AHNAK','FN1'],
        ['AHNAK','HLA@'],
        ['AHNAK','IGF2'],
        ['AHNAK','KRT10'],
        ['AHNAK','KRT14'],
        ['AHNAK','KRT2'],
        ['AHNAK','LYZ'],
        ['AHNAK','MYH11'],
        ['AHNAK','NEGR1'],
        ['AHNAK','RP11-1143G9.4'],
        ['AHNAK','SLC34A2'],
        ['AHNAK','SMR3B'],
        ['AHSA2','C2ORF74'],
        ['AHSP','RP11-452L6.7'],
        ['AIF1L','NUP214'],
        ['AIFM3','LZTR1'],
        ['AIG1','RP1-95L4.4'],
        ['AJUBA','SERPINE1'],
        ['AK3','RP11-265B8.5'],
        ['AK4P4','ERMP1'],
        ['AKAP10','SFTPB'],
        ['AKAP13','CLPS'],
        ['AKAP13','CPB1'],
        ['AKAP13','LYZ'],
        ['AKAP13','STATH'],
        ['AKIRIN1','LYZ'],
        ['AKNA','EMP2'],
        ['AKR1B1','CYP17A1'],
        ['AKR1B1','GML'],
        ['AKR1B1','STAG3'],
        ['AKR1B10','IGK@'],
        ['AKR1B10','RP11-1143G9.4'],
        ['AKR1C2','AKR1C8P'],
        ['AKR1C5P','AKR1C8P'],
        ['AKT1','COL1A1'],
        ['AKT1','EXT1'],
        ['AKT2','C19ORF47'],
        ['AKT3','TRB@'],
        ['AL121578.2','SYTL5'],
        ['AL121656.5','LTBP1'],
        ['AL137145.1','PFKFB3'],
        ['AL928768.3','IGHA1'],
        ['ALB','ALDH2'],
        ['ALB','ALDOB'],
        ['ALB','AMBP'],
        ['ALB','APOA1-AS'],
        ['ALB','APOB'],
        ['ALB','APOC3'],
        ['ALB','APOH'],
        ['ALB','BAAT'],
        ['ALB','C3'],
        ['ALB','C4BPA'],
        ['ALB','C9'],
        ['ALB','CANX'],
        ['ALB','CCDC152'],
        ['ALB','CCNY'],
        ['ALB','CES1'],
        ['ALB','CFH'],
        ['ALB','CLU'],
        ['ALB','CP'],
        ['ALB','CPS1'],
        ['ALB','CRP'],
        ['ALB','CTSB'],
        ['ALB','CTSD'],
        ['ALB','CYP2A6'],
        ['ALB','CYP2C9'],
        ['ALB','CYP2E1'],
        ['ALB','DMGDH'],
        ['ALB','EEF1A1'],
        ['ALB','EIF2B5'],
        ['ALB','F2'],
        ['ALB','F9'],
        ['ALB','FFAR4'],
        ['ALB','FGA'],
        ['ALB','FGB'],
        ['ALB','FGG'],
        ['ALB','FGL1'],
        ['ALB','FMO5'],
        ['ALB','FN1'],
        ['ALB','GATM'],
        ['ALB','GP2'],
        ['ALB','GSTA2'],
        ['ALB','HAMP'],
        ['ALB','HP'],
        ['ALB','HPS3'],
        ['ALB','HPX'],
        ['ALB','IGHG1'],
        ['ALB','IGK@'],
        ['ALB','IGKC'],
        ['ALB','INHBE'],
        ['ALB','ITIH2'],
        ['ALB','ITM2B'],
        ['ALB','ITPR2'],
        ['ALB','KRBOX1'],
        ['ALB','LBP'],
        ['ALB','LIPA'],
        ['ALB','MAOB'],
        ['ALB','MAT1A'],
        ['ALB','MGST1'],
        ['ALB','MT2A'],
        ['ALB','MYO1B'],
        ['ALB','NDUFS6'],
        ['ALB','NNMT'],
        ['ALB','NUCB1'],
        ['ALB','OPN3'],
        ['ALB','ORM1'],
        ['ALB','PCBP1-AS1'],
        ['ALB','PCSK6'],
        ['ALB','PDIA4'],
        ['ALB','PIPOX'],
        ['ALB','PLG'],
        ['ALB','PRAP1'],
        ['ALB','PROC'],
        ['ALB','PTGR1'],
        ['ALB','RP11-136C24.3'],
        ['ALB','RP11-141M3.6'],
        ['ALB','RP11-295K3.1'],
        ['ALB','RP11-35N6.6'],
        ['ALB','RP11-573D15.8'],
        ['ALB','RP11-64D24.2'],
        ['ALB','RP11-696N14.1'],
        ['ALB','RP11-903H12.3'],
        ['ALB','RP11-96H19.1'],
        ['ALB','RRBP1'],
        ['ALB','SAA1'],
        ['ALB','SAA2'],
        ['ALB','SARM1'],
        ['ALB','SCD'],
        ['ALB','SCP2'],
        ['ALB','SERPINA1'],
        ['ALB','SERPINA3'],
        ['ALB','SERPIND1'],
        ['ALB','SERPINE1'],
        ['ALB','SERPING1'],
        ['ALB','SNAP91'],
        ['ALB','ST6GAL1'],
        ['ALB','STAU1'],
        ['ALB','STOM'],
        ['ALB','TAT'],
        ['ALB','TAT-AS1'],
        ['ALB','TF'],
        ['ALB','TMBIM6'],
        ['ALB','TXNL4B'],
        ['ALB','UGT1A8'],
        ['ALB','UGT2B11'],
        ['ALDH1L1','SMR3B'],
        ['ALDH1L2','RP11-61E11.2'],
        ['ALDH2','LYZ'],
        ['ALDH2','SFTPB'],
        ['ALDH3A2','SLC47A1'],
        ['ALDH3B2','TMEM99'],
        ['ALDH5A1','LYZ'],
        ['ALDH6A1','LRG1'],
        ['ALDH6A1','SFTPB'],
        ['ALDOA','IGF2'],
        ['ALDOA','MUC5AC'],
        ['ALDOA','PIGR'],
        ['ALDOA','SMR3B'],
        ['ALDOA','TPM2'],
        ['ALDOA','VIM'],
        ['ALDOB','ANPEP'],
        ['ALDOB','CRP'],
        ['ALDOB','HRG'],
        ['ALDOB','PIGR'],
        ['ALDOB','TXNL4B'],
        ['ALG11','YES1'],
        ['ALG9','OSMR'],
        ['ALMS1','ALMS1-IT1'],
        ['ALOX12','RNASEK'],
        ['ALOX12','RNASEK-C17ORF49'],
        ['ALS2CR12','TRAK2'],
        ['ALX1','LRRIQ1'],
        ['AMACR','C1QTNF3'],
        ['AMACR','SLC45A2'],
        ['AMBP','FGA'],
        ['AMBP','SERPINA1'],
        ['AMBRA1','CHRM4'],
        ['AMBRA1','GFAP'],
        ['AMOTL1','IGF2'],
        ['AMY2A','CLPS'],
        ['AMY2A','PRSS2'],
        ['AMY2A','TRB@'],
        ['AMY2B','CLPS'],
        ['AMY2B','PNLIP'],
        ['AMY2B','PRSS2'],
        ['AMY2B','TRB@'],
        ['AMZ1','APOLD1'],
        ['AMZ1','IGF2'],
        ['ANAPC15','LAMTOR1'],
        ['ANAPC16','TG'],
        ['ANAPC16','UGDH'],
        ['ANGPTL3','RP11-696N14.1'],
        ['ANGPTL4','PTRF'],
        ['ANK1','AZU1'],
        ['ANK1','LYZ'],
        ['ANK1','S100A9'],
        ['ANK3','TG'],
        ['ANKIB1','CYP51A1-AS1'],
        ['ANKLE2','GOLGA3'],
        ['ANKMY2','ZNF708'],
        ['ANKRD1','LDB3'],
        ['ANKRD11','LYZ'],
        ['ANKRD11','STOM'],
        ['ANKRD11','TRB@'],
        ['ANKRD12','PTPRM'],
        ['ANKRD13A','CRISPLD2'],
        ['ANKRD13D','LINC00337'],
        ['ANKRD17','SMR3B'],
        ['ANKRD28','TRB@'],
        ['ANKRD34A','LIX1L'],
        ['ANKRD37','RP11-714G18.1'],
        ['ANO10','TG'],
        ['ANO2','VWF'],
        ['ANO3','REEP5'],
        ['ANP32B','FN1'],
        ['ANP32B','HBB'],
        ['ANP32B','HMGB2'],
        ['ANP32B','NCL'],
        ['ANP32B','NPM1'],
        ['ANP32E','NCL'],
        ['ANPEP','CELA2A'],
        ['ANPEP','COL6A2'],
        ['ANPEP','FN1'],
        ['ANPEP','HBB'],
        ['ANPEP','IGH@'],
        ['ANPEP','IGK@'],
        ['ANPEP','IGKC'],
        ['ANPEP','LYZ'],
        ['ANPEP','PIGR'],
        ['ANPEP','REG3A'],
        ['ANPEP','TRB@'],
        ['ANTXR1','IGK@'],
        ['ANXA1','CRNN'],
        ['ANXA1','HBB'],
        ['ANXA1','HTN3'],
        ['ANXA1','KRT4'],
        ['ANXA11','PTRF'],
        ['ANXA11','TRB@'],
        ['ANXA2','IGF2'],
        ['ANXA2','KRT10'],
        ['ANXA2','PNLIP'],
        ['ANXA4','PRSS1'],
        ['ANXA5','PCSK7'],
        ['ANXA6','PLIN1'],
        ['ANXA6','TNIP1'],
        ['AOAH','CXCL8'],
        ['AOC3','FTL'],
        ['AOX1','PRSS1'],
        ['AOX1','SPTLC2'],
        ['AOX1','TRB@'],
        ['AP000295.9','ZNF595'],
        ['AP000304.12','HTN1'],
        ['AP000322.53','KCNE1'],
        ['AP000347.2','KB-1572G7.3'],
        ['AP000350.4','SFTPA1'],
        ['AP000438.2','CYP4F3'],
        ['AP000688.14','PIGR'],
        ['AP000695.6','CHAF1B'],
        ['AP000860.2','SLC25A25-AS1'],
        ['AP001350.4','CNTF'],
        ['AP001350.4','ZFP91-CNTF'],
        ['AP006621.8','PRH1'],
        ['AP006621.8','PRH1-PRR4'],
        ['AP006621.8','PRR4'],
        ['AP2M1','FN1'],
        ['AP2M1','VIM'],
        ['AP3B2','SRSF9'],
        ['AP3D1','FLNA'],
        ['AP3D1','KRT4'],
        ['AP3D1','SMR3B'],
        ['AP3S1','COMMD10'],
        ['AP4B1','PTPN22'],
        ['AP4B1-AS1','RP5-1073O3.2'],
        ['AP5B1','LYZ'],
        ['APBB2','TG'],
        ['APBB3','NDUFA2'],
        ['APLP2','CANX'],
        ['APLP2','EEF1A1'],
        ['APLP2','HLA@'],
        ['APLP2','LAMB2'],
        ['APLP2','PRSS1'],
        ['APLP2','SFTPA2'],
        ['APLP2','SFTPC'],
        ['APOA1','CLU'],
        ['APOA1','FGB'],
        ['APOA1','IGHG1'],
        ['APOA1','PTMS'],
        ['APOA1','RP11-573D15.8'],
        ['APOA1','SARM1'],
        ['APOA1','SERPINA1'],
        ['APOA1','SIK3'],
        ['APOA1-AS','ATF5'],
        ['APOA1-AS','C3'],
        ['APOA1-AS','FGA'],
        ['APOA1-AS','HRG'],
        ['APOA1-AS','MIDN'],
        ['APOA1-AS','SERPINA1'],
        ['APOA4','PIGR'],
        ['APOB','C3'],
        ['APOB','CRP'],
        ['APOB','IGKC'],
        ['APOB','PIGR'],
        ['APOB','SERPINA1'],
        ['APOB','SERPINA3'],
        ['APOBEC3C','PTRF'],
        ['APOC1','SERPINA1'],
        ['APOC1P1','APOC4-APOC2'],
        ['APOC3','C3'],
        ['APOC3','SERPINA1'],
        ['APOD','MYH11'],
        ['APOE','FFAR4'],
        ['APOE','FGG'],
        ['APOH','C3'],
        ['APOH','FGB'],
        ['APOH','SERPINA1'],
        ['APOL1','DA750114'],
        ['APOL1','SCYL3'],
        ['APOL1','TNFRSF1B'],
        ['APOLD1','DPF3'],
        ['APOLD1','PARVA'],
        ['APOLD1','RP11-108K3.1'],
        ['APOLD1','TG'],
        ['APOPT1','RP11-894P9.2'],
        ['APP','GFAP'],
        ['APP','HBB'],
        ['APP','HLA@'],
        ['APP','IGF2'],
        ['APP','KRT10'],
        ['APP','LYZ'],
        ['APP','MYH7'],
        ['APP','PIGR'],
        ['APP','PNLIP'],
        ['APP','PRSS1'],
        ['APP','PRSS2'],
        ['APP','SFTPC'],
        ['APP','SMR3B'],
        ['APP','STATH'],
        ['APP','TRB@'],
        ['AQP1','NPPA-AS1'],
        ['AQP2','GPX3'],
        ['AQP3','MYH11'],
        ['AQP3','SMR3B'],
        ['AQP4','CHST9'],
        ['AQP4','RP11-862L9.3'],
        ['AQP4-AS1','MBP'],
        ['AQP8','PRSS1'],
        ['AQP8','PRSS2'],
        ['AQP8','TRB@'],
        ['ARCN1','FN1'],
        ['ARCN1','LRRC28'],
        ['ARCN1','NPPA-AS1'],
        ['ARCN1','PTRF'],
        ['ARF1','LYZ'],
        ['ARF3','FKBP11'],
        ['ARF3','WNT10B'],
        ['ARF4','DENND6A'],
        ['ARF6','SFTPC'],
        ['ARFGAP1','PNLIP'],
        ['ARFGAP3','PACSIN2'],
        ['ARG1','FGG'],
        ['ARHGAP1','FLG2'],
        ['ARHGAP1','PTRF'],
        ['ARHGAP11A','SCG5'],
        ['ARHGAP18','PNLIP'],
        ['ARHGAP23','COL5A1'],
        ['ARHGAP23','MARCKS'],
        ['ARHGAP27','PLEKHM1'],
        ['ARHGAP35','CTSS'],
        ['ARHGAP35','TG'],
        ['ARHGAP4','OPN1LW'],
        ['ARHGDIA','FLNA'],
        ['ARHGDIB','ERP27'],
        ['ARHGDIB','MYH9'],
        ['ARHGEF1','NPPA-AS1'],
        ['ARHGEF17','P2RY6'],
        ['ARHGEF2','SSR2'],
        ['ARHGEF3','FGD5-AS1'],
        ['ARID1A','CTSB'],
        ['ARID1B','CD24'],
        ['ARID1B','PIGR'],
        ['ARID1B','SMR3B'],
        ['ARID4B','LYZ'],
        ['ARIH2','LYZ'],
        ['ARL10','KRT13'],
        ['ARL15','KLK3'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARL6','EPHA6'],
        ['ARL6IP1','SMG1'],
        ['ARL8A','PTPN7'],
        ['ARL9','SRP72'],
        ['ARMCX4','TG'],
        ['ARMS2','PLEKHA1'],
        ['ARPC1A','PAPOLA'],
        ['ARPC2','EDC3'],
        ['ARPC3','ARPC3P1'],
        ['ARPC4','TTLL3'],
        ['ARPC4-TTLL3','CNTN4'],
        ['ARPC4-TTLL3','CTD-2267D19.3'],
        ['ARPC4-TTLL3','SFTPB'],
        ['ARPC5L','RP11-944L7.4'],
        ['ARPIN','RAI14'],
        ['ARPP19','RPL37A'],
        ['ARRB2','LYZ'],
        ['ARRDC1','OSTF1'],
        ['ARRDC3','MPO'],
        ['ARSD','PIK3C2B'],
        ['ARSD','SSH2'],
        ['ARSD','ZNF668'],
        ['ARSE','TAF13'],
        ['ARSG','CARNS1'],
        ['ART3','RP11-630D6.5'],
        ['ART4','SMCO3'],
        ['AS3MT','CNNM2'],
        ['ASB15','LMOD2'],
        ['ASCC1','SPOCK2'],
        ['ASF1B','PRKACA'],
        ['ASH1L','GBP6'],
        ['ASH1L','STATH'],
        ['ASH1L','TG'],
        ['ASIP','EIF5'],
        ['ASLP1','KB-1572G7.3'],
        ['ASMTL','SLC25A6'],
        ['ASNS','RP11-420N3.2'],
        ['ASNS','RP11-420N3.3'],
        ['ASPG','TDRD9'],
        ['ASRGL1','SCGB1A1'],
        ['ASS1','ASS1P11'],
        ['ASS1','ASS1P12'],
        ['ASS1','HMCN2'],
        ['ASXL1','KIF3B'],
        ['ATF3','HNRNPH1'],
        ['ATF3','TG'],
        ['ATF5','CLU'],
        ['ATF5','SERPINA1'],
        ['ATF5','TG'],
        ['ATF5','TXNL4B'],
        ['ATF6','DUSP12'],
        ['ATF7','SFTPC'],
        ['ATF7IP','IGF2'],
        ['ATG12','CDO1'],
        ['ATG14','LYZ'],
        ['ATG16L1','INPP5D'],
        ['ATG16L1','SAG'],
        ['ATG16L2','DSC3'],
        ['ATHL1','TRB@'],
        ['ATL3','TPM4'],
        ['ATM','PRH1'],
        ['ATM','PRH1-PRR4'],
        ['ATM','PRR4'],
        ['ATM','RHOBTB3'],
        ['ATN1','SFTPA1'],
        ['ATOH8','TXNDC5'],
        ['ATP11B','MYH11'],
        ['ATP12A','TUBGCP2'],
        ['ATP1A1','CTSB'],
        ['ATP1A1','GP2'],
        ['ATP1A1','IGH@'],
        ['ATP1A1','LYZ'],
        ['ATP1A1','PIGR'],
        ['ATP1A1','SUN1'],
        ['ATP1A1-AS1','ATP1A3'],
        ['ATP1A1-AS1','IGH@'],
        ['ATP1A1-AS1','MUC7'],
        ['ATP1A1-AS1','TG'],
        ['ATP1A2','GFAP'],
        ['ATP1A2','NRXN3'],
        ['ATP1B3P1','RP11-416L21.2'],
        ['ATP2A1','LRP1'],
        ['ATP2A2','LYZ'],
        ['ATP2A2','MMP14'],
        ['ATP2A2','MYH6'],
        ['ATP2A2','MYH9'],
        ['ATP2A3','GANAB'],
        ['ATP2A3','GP2'],
        ['ATP2A3','LYZ'],
        ['ATP2A3','PNLIP'],
        ['ATP2B4','NPPA-AS1'],
        ['ATP5A1','IGF2'],
        ['ATP5A1','NPPA-AS1'],
        ['ATP5B','FN1'],
        ['ATP5B','KRT13'],
        ['ATP5D','FAM96B'],
        ['ATP5I','MFSD7'],
        ['ATP5L','SLC25A37'],
        ['ATP6AP1','GNAS'],
        ['ATP6AP1','PTMS'],
        ['ATP6V0A2','TCTN2'],
        ['ATP6V0A4','SVOPL'],
        ['ATP6V0C','DDX5'],
        ['ATP6V0E1','GSN'],
        ['ATP6V1B1','VAX2'],
        ['ATP6V1C1','ECE1'],
        ['ATP6V1C1','KB-1507C5.4'],
        ['ATP6V1F','FLNC'],
        ['ATP6V1G2-DDX39B','DDX39A'],
        ['ATP6V1G2-DDX39B','LYZ'],
        ['ATP6V1G2-DDX39B','XXBAC-BPG299F13.16'],
        ['ATP6V1G3','RP11-553K8.2'],
        ['ATP6V1H','STARD4-AS1'],
        ['ATP7A','LYZ'],
        ['ATP7A','RP5-1000K24.2'],
        ['ATP7A','ZC3H12B'],
        ['ATP7B','CTAGE3P'],
        ['ATP8B1','AZGP1'],
        ['ATPIF1','HBB'],
        ['ATXN2','PEX26'],
        ['AUH','RNF4'],
        ['AURKA','RAB3GAP2'],
        ['AURKC','ZNF264'],
        ['AUTS2','LYZ'],
        ['AUTS2','SMR3B'],
        ['AVEN','CCDC69'],
        ['AXL','SFTPB'],
        ['AZGP1','CA6'],
        ['AZGP1','CD36'],
        ['AZGP1','GJC3'],
        ['AZGP1','LYZ'],
        ['AZGP1','MYH11'],
        ['AZGP1','PRB1'],
        ['AZGP1','PRB2'],
        ['AZGP1','PRH1'],
        ['AZGP1','PRH1-PRR4'],
        ['AZGP1','PRR4'],
        ['AZGP1','RP11-573D15.8'],
        ['AZGP1','SMR3B'],
        ['AZGP1','TGM4'],
        ['AZIN1','TG'],
        ['AZIN1','TRB@'],
        ['AZIN1','YWHAZ'],
        ['AZU1','CLTC'],
        ['AZU1','DDX39B'],
        ['AZU1','DNM2'],
        ['AZU1','EEF1A1'],
        ['AZU1','IGHG1'],
        ['AZU1','ILF3'],
        ['AZU1','LCP1'],
        ['AZU1','MYH9'],
        ['AZU1','NARF'],
        ['AZU1','NEMP1'],
        ['AZU1','NR3C1'],
        ['AZU1','OOEP-AS1'],
        ['AZU1','PLEK'],
        ['AZU1','PTP4A1'],
        ['AZU1','RBM3'],
        ['AZU1','RP11-1143G9.4'],
        ['AZU1','S100A9'],
        ['AZU1','SRGN'],
        ['AZU1','TUBGCP3'],
        ['AZU1','ZMYM1'],
        ['B2M','CXCR4'],
        ['B2M','EEF1A1'],
        ['B2M','FGA'],
        ['B2M','GNAS'],
        ['B2M','HBA2'],
        ['B2M','KLF13'],
        ['B2M','LINC01578'],
        ['B2M','PTP4A1'],
        ['B2M','REG3A'],
        ['B2M','RP11-437B10.1'],
        ['B2M','SYN3'],
        ['B3GNT6','RP11-191A15.2'],
        ['B3GNTL1','TRAF6'],
        ['B4GALNT2','EIF3I'],
        ['BAAT','TRMT112P4'],
        ['BACE1','GP2'],
        ['BACE2','IGF2'],
        ['BACH1','BACH1-IT1'],
        ['BACH2','CPA4'],
        ['BAG2','ZNF451'],
        ['BAG6','C6ORF47'],
        ['BAG6','FN1'],
        ['BAG6','SFTPC'],
        ['BAG6','SMR3B'],
        ['BAG6','SOD2'],
        ['BAG6','TG'],
        ['BAGE2','GBP6'],
        ['BAHD1','IVD'],
        ['BAIAP2','TFB2M'],
        ['BAIAP2L2','SLC16A8'],
        ['BANK1','LYZ'],
        ['BANP','RRN3'],
        ['BBS2','NUDT21'],
        ['BBS5','KLHL41'],
        ['BCAM','LYZ'],
        ['BCAM','SFTPA2'],
        ['BCAP29','DUS4L'],
        ['BCAS1','LYZ'],
        ['BCAS2','DENND2C'],
        ['BCKDHA','TMEM91'],
        ['BCKDHB','DENND1A'],
        ['BCL2L12','PRMT1'],
        ['BCL2L13','LYZ'],
        ['BCL2L15','PTPN22'],
        ['BCL2L2-PABPN1','KRT1'],
        ['BCL6','SERPINA1'],
        ['BCL9L','LYZ'],
        ['BCL9L','SMR3B'],
        ['BCLAF1','LYZ'],
        ['BCO1','TRIM56'],
        ['BDH1','CTSB'],
        ['BDH1','IGF2'],
        ['BDKRB1','BDKRB2'],
        ['BEAN1-AS1','PIGR'],
        ['BEST1','EEF2'],
        ['BEST1','LYZ'],
        ['BEST1','TG'],
        ['BFAR','SYN3'],
        ['BFSP2','KRT13'],
        ['BFSP2','KRT14'],
        ['BFSP2','RP11-91K8.4'],
        ['BGN','CKMT2-AS1'],
        ['BGN','GADL1'],
        ['BGN','IGHG1'],
        ['BGN','IGHM'],
        ['BGN','NPPA-AS1'],
        ['BGN','PRKCH'],
        ['BGN','RP11-452D21.1'],
        ['BHLHB9','LINC00630'],
        ['BHLHB9','RP4-769N13.6'],
        ['BHLHE40','TRB@'],
        ['BICD1','DES'],
        ['BICD2','PTMS'],
        ['BIRC6','YIPF4'],
        ['BLOC1S3','PTRF'],
        ['BLOC1S5-TXNDC5','HTN3'],
        ['BLOC1S5-TXNDC5','SMR3B'],
        ['BLOC1S6','LYZ'],
        ['BLVRA','VMA21'],
        ['BMP10','NPPA-AS1'],
        ['BMPR2','SFTPB'],
        ['BMPR2','TG'],
        ['BMS1','MBL1P'],
        ['BMS1P10','LL22NC03-80A10.6'],
        ['BMS1P20','LL22NC03-2H8.4'],
        ['BMS1P4','PARG'],
        ['BNIP3L','HBB'],
        ['BNIP3L','TG'],
        ['BNIPL','FLG2'],
        ['BOLA2B','SMG1'],
        ['BP-21264C1.2','TEKT4P2'],
        ['BPGM','CALD1'],
        ['BPGM','HBB'],
        ['BPI','S100A9'],
        ['BPIFA2','HTN1'],
        ['BPIFA2','LYZ'],
        ['BPIFA2','MOCOS'],
        ['BPIFA2','PIGR'],
        ['BPIFA2','SMR3B'],
        ['BPIFB1','LYZ'],
        ['BPIFB2','SMR3B'],
        ['BPNT1','PDXDC2P'],
        ['BRD7','SLC22A10'],
        ['BRD9P2','TPPP'],
        ['BRDT','EPHX4'],
        ['BRF1','TRIM22'],
        ['BRI3BP','MYO1D'],
        ['BRS3','HTATSF1'],
        ['BSCL2','LYZ'],
        ['BSG','LYZ'],
        ['BSG','NR1H2'],
        ['BSG','PTBP1'],
        ['BSG','RNF126'],
        ['BST1','FAM200B'],
        ['BTBD1','SPP1'],
        ['BTBD2','CAPNS1'],
        ['BTBD6','LOXL1'],
        ['BTBD8','KIAA1107'],
        ['BTBD9','RP11-1143G9.4'],
        ['BTG1','HBB'],
        ['BTG2','CEL'],
        ['BTG2','GP2'],
        ['BTG2','IGF2'],
        ['BTG2','SRGN'],
        ['BTG2','TG'],
        ['BTG4','POU2AF1'],
        ['BTN3A2','SCAMP1-AS1'],
        ['BVES','NUBPL'],
        ['BX470102.3','S100A6'],
        ['BZW1','CBL'],
        ['C10ORF10','PNLIP'],
        ['C10ORF10','SFTPA2'],
        ['C10ORF10','SFTPC'],
        ['C10ORF25','CEP164P1'],
        ['C10ORF32-ASMT','CNNM2'],
        ['C10ORF32-ASMT','LYZ'],
        ['C10ORF67','LINC01552'],
        ['C12ORF60','TG'],
        ['C12ORF65','NIPAL1'],
        ['C12ORF75','PTOV1'],
        ['C12ORF79','RP11-756G20.1'],
        ['C13ORF45','LMO7'],
        ['C14ORF166','TG'],
        ['C15ORF26','IL16'],
        ['C15ORF38-AP3S2','RAI14'],
        ['C15ORF57','CBX3'],
        ['C16ORF45','CBX3'],
        ['C16ORF46','GCSH'],
        ['C16ORF52','VWA3A'],
        ['C16ORF62','PNLIP'],
        ['C16ORF72','HOOK2'],
        ['C16ORF72','LYZ'],
        ['C16ORF89','SFTPB'],
        ['C17ORF100','CTC-281F24.3'],
        ['C17ORF104','DBF4B'],
        ['C17ORF108','NOS2'],
        ['C17ORF74','TMEM102'],
        ['C17ORF99','SYNGR2'],
        ['C19MC','RPLP1'],
        ['C19MC','TIMM50'],
        ['C19ORF10','FLNA'],
        ['C19ORF18','ZNF606'],
        ['C1D','HZGJ'],
        ['C1D','WDR92'],
        ['C1ORF105','DNM3'],
        ['C1ORF159','RP11-172C16.5'],
        ['C1ORF167','TG'],
        ['C1ORF194','UQCR10'],
        ['C1ORF220','RP5-1098D14.1'],
        ['C1ORF27','RP11-295K2.3'],
        ['C1ORF43','PNLIP'],
        ['C1ORF61','RP11-98G7.1'],
        ['C1ORF74','IRF6'],
        ['C1QC','GML'],
        ['C1QC','SFTPA1'],
        ['C1QTNF3-AMACR','RP11-1023L17.1'],
        ['C1QTNF3-AMACR','SLC45A2'],
        ['C1QTNF9B','MIPEP'],
        ['C1R','SERPING1'],
        ['C1RL-AS1','CLSTN3'],
        ['C1S','HLA-E'],
        ['C1S','IGFBP5'],
        ['C1S','KRT10'],
        ['C20ORF187','RP11-103J8.2'],
        ['C20ORF203','COMMD7'],
        ['C20ORF62','LINC01430'],
        ['C2CD5','ST8SIA1'],
        ['C2ORF48','RRM2'],
        ['C2ORF50','PECAM1'],
        ['C3','CLU'],
        ['C3','CREB3L3'],
        ['C3','CRP'],
        ['C3','DHCR24'],
        ['C3','EMP2'],
        ['C3','FGA'],
        ['C3','FGB'],
        ['C3','FGG'],
        ['C3','GML'],
        ['C3','HLA@'],
        ['C3','HP'],
        ['C3','HRG'],
        ['C3','ITIH1'],
        ['C3','RBP4'],
        ['C3','RP11-573D15.8'],
        ['C3','SERPINA1'],
        ['C3','SERPING1'],
        ['C3','TMEM176B'],
        ['C3','TNFSF14'],
        ['C3','TXNL4B'],
        ['C3','ZFP36L1'],
        ['C3ORF22','TXNRD3'],
        ['C4BPB','PFKFB2'],
        ['C4ORF3','CD36'],
        ['C4ORF3','FABP2'],
        ['C4ORF48','EIF5A'],
        ['C5','SERPINA1'],
        ['C5AR1','INAFM1'],
        ['C5AR1','SEPT9'],
        ['C5ORF24','TG'],
        ['C5ORF4','CCDC106'],
        ['C5ORF58','LCP2'],
        ['C5ORF66','KRT13'],
        ['C6ORF141','KIAA1671'],
        ['C6ORF165','RP4-669H2.1'],
        ['C6ORF165','RP4-678D15.1'],
        ['C6ORF183','CCDC162P'],
        ['C6ORF183','ZNF528-AS1'],
        ['C6ORF201','RP3-400B16.1'],
        ['C6ORF48','HSPA1A'],
        ['C6ORF58','LYZ'],
        ['C6ORF58','PGC'],
        ['C6ORF62','LYZ'],
        ['C6ORF89','LYZ'],
        ['C7','CHGB'],
        ['C7','COL1A2'],
        ['C7','IGF2'],
        ['C7','IGH@'],
        ['C7','MYH9'],
        ['C7','PNLIP'],
        ['C7','TBC1D2B'],
        ['C7','TIMP2'],
        ['C7ORF49','WDR91'],
        ['C7ORF50','COX19'],
        ['C7ORF50','PNLIP'],
        ['C7ORF60','HRAT17'],
        ['C8ORF89','RPL7'],
        ['C9ORF3','IGF2'],
        ['C9ORF3','MIR205HG'],
        ['C9ORF3','PIGR'],
        ['C9ORF43','RGS3'],
        ['C9ORF78','LYZ'],
        ['CA12','FLG-AS1'],
        ['CA5B','CA5BP1'],
        ['CA6','LYZ'],
        ['CA6','MUC7'],
        ['CA6','PRB2'],
        ['CA6','PRH1'],
        ['CA6','PRR27'],
        ['CA6','SMR3B'],
        ['CA6','STATH'],
        ['CA6','ZG16B'],
        ['CACNA1H','TG'],
        ['CADM1','ST3GAL2'],
        ['CADM3-AS1','IGF2'],
        ['CADM4','PLAUR'],
        ['CADPS2','RNF148'],
        ['CALCB','INSC'],
        ['CALCOCO1','KRT13'],
        ['CALD1','CTSB'],
        ['CALD1','IGF2'],
        ['CALD1','KRT13'],
        ['CALD1','KRT4'],
        ['CALD1','MYH11'],
        ['CALD1','PNLIP'],
        ['CALD1','SFTPC'],
        ['CALM1','CXCR4'],
        ['CALM1','YWHAE'],
        ['CALM2','PNLIP'],
        ['CALM3','CAMK2N1'],
        ['CALM3','NRGN'],
        ['CALM3','TG'],
        ['CALM3','VAMP2'],
        ['CALML4','CLN6'],
        ['CALML5','SMR3B'],
        ['CALN1','CD24'],
        ['CALR','COL1A1'],
        ['CALR','CTC-326K19.6'],
        ['CALR','EEF2'],
        ['CALR','FGA'],
        ['CALR','KRBOX1'],
        ['CALR','KRT13'],
        ['CALR','MDN1'],
        ['CALR','PLIN1'],
        ['CALR','RP11-136C24.3'],
        ['CALR','RP11-141M3.6'],
        ['CALR','SERPINA1'],
        ['CALR','SFTPC'],
        ['CALR','SMR3B'],
        ['CALR','UBXN11'],
        ['CAMK1D','IL1RAPL1'],
        ['CAMK2N1','MBP'],
        ['CAMK2N1','VSNL1'],
        ['CAMKK1','P2RX1'],
        ['CAMLG','SEC24A'],
        ['CAMTA1','CAMTA1-IT1'],
        ['CAMTA2','FBXW8'],
        ['CAND1','PTMS'],
        ['CANT1','FSCN1'],
        ['CANX','HMGB3'],
        ['CANX','MAML1'],
        ['CANX','MUC7'],
        ['CAP1','RP11-550F7.1'],
        ['CAP1','SERPINA1'],
        ['CAP1','SMR3B'],
        ['CAPN1','TG'],
        ['CAPN2','IGKC'],
        ['CAPN2','SFTPC'],
        ['CAPN2','TG'],
        ['CAPN3','GANC'],
        ['CAPN8','IGH@'],
        ['CAPN8','LYZ'],
        ['CAPN8','RP11-1143G9.4'],
        ['CAPNS1','PTMS'],
        ['CAPNS1','VEGFB'],
        ['CAPRIN1','KIF5B'],
        ['CAPZB','EEF2'],
        ['CAPZB','IGF2'],
        ['CAPZB','SFTPB'],
        ['CARD17','CASP1'],
        ['CARD8','TG'],
        ['CARHSP1','PTRF'],
        ['CARM1P1','FZD4'],
        ['CASC1','RP11-713N11.4'],
        ['CASC15','PRSS23'],
        ['CASC4','PTRF'],
        ['CASC4','TPM4'],
        ['CASP14','TMEM99'],
        ['CASQ2','MYH7'],
        ['CAST','CTD-2260A17.1'],
        ['CAST','HBB'],
        ['CAV1','CLU'],
        ['CAV1','COL6A1'],
        ['CAV2','TG'],
        ['CBR4','DES'],
        ['CBX3','FLG-AS1'],
        ['CBX4','IGF2'],
        ['CBX5','PTRF'],
        ['CBX5','SFTPB'],
        ['CBX5','TG'],
        ['CBX7','RP4-742C19.8'],
        ['CC2D2B','RP11-248J23.6'],
        ['CCAR2','PNLIP'],
        ['CCDC11','MBD1'],
        ['CCDC113','RP11-459F6.1'],
        ['CCDC142','MOGS'],
        ['CCDC144A','USP32'],
        ['CCDC144NL-AS1','TBC1D27'],
        ['CCDC146','PMS2'],
        ['CCDC15','SLC37A2'],
        ['CCDC152','EEF1A1'],
        ['CCDC152','IGF2'],
        ['CCDC152','SFTPC'],
        ['CCDC152','TF'],
        ['CCDC152','TG'],
        ['CCDC163P','TESK2'],
        ['CCDC169','SOHLH2'],
        ['CCDC17','GPBP1L1'],
        ['CCDC176','LIN52'],
        ['CCDC183','TMEM141'],
        ['CCDC25','PPIE'],
        ['CCDC47','STRADA'],
        ['CCDC80','LINC01279'],
        ['CCDC80','MYH9'],
        ['CCDC83','SLC25A1P1'],
        ['CCDC85B','EVA1B'],
        ['CCDC88C','HBB'],
        ['CCER2','CTC-360G5.8'],
        ['CCIN','GLIPR2'],
        ['CCL28','LYZ'],
        ['CCL28','ORAI2'],
        ['CCL28','SMR3B'],
        ['CCL5','LYZ'],
        ['CCND1','COL1A1'],
        ['CCND1','CXCL12'],
        ['CCND1','CYR61'],
        ['CCND1','TG'],
        ['CCNG2','RP11-255H23.4'],
        ['CCNI','CD74'],
        ['CCNI','FLNA'],
        ['CCNI','SFTPC'],
        ['CCNO','DHX29'],
        ['CCNT2-AS1','TMEM163'],
        ['CCNY','CREM'],
        ['CCP110','TMC5'],
        ['CCPG1','PIGBOS1'],
        ['CCPG1','PNLIP'],
        ['CCPG1','RAB27A'],
        ['CCR10','EZH1'],
        ['CCSER1','RP11-1143G9.4'],
        ['CCT4','FAM161A'],
        ['CCT6P1','VKORC1L1'],
        ['CD164','RP11-960L18.1'],
        ['CD22','MAG'],
        ['CD22','U62631.5'],
        ['CD24','CEBPD'],
        ['CD24','CNTN4'],
        ['CD24','COL6A2'],
        ['CD24','CTSB'],
        ['CD24','ERCC1'],
        ['CD24','FLNA'],
        ['CD24','GLTP'],
        ['CD24','IYD'],
        ['CD24','KRT13'],
        ['CD24','KRT4'],
        ['CD24','MACROD2'],
        ['CD24','PCYOX1'],
        ['CD24','PRKCA'],
        ['CD24','PVRL2'],
        ['CD24','RP11-58E21.4'],
        ['CD24','RP11-73B8.1'],
        ['CD24','SAP30L-AS1'],
        ['CD24','ZNF773'],
        ['CD248','COL1A1'],
        ['CD248','FN1'],
        ['CD248','SPARC'],
        ['CD248','VIM'],
        ['CD27-AS1','VAMP1'],
        ['CD34','NPPA'],
        ['CD36','EHD2'],
        ['CD36','IGF2'],
        ['CD44','FLG-AS1'],
        ['CD44','HTN1'],
        ['CD44','PRB1'],
        ['CD44','PRH1'],
        ['CD44','PRH1-PRR4'],
        ['CD44','PRR4'],
        ['CD44','SMR3B'],
        ['CD44','TG'],
        ['CD46','KRT4'],
        ['CD46','SMR3B'],
        ['CD53','RP11-96K19.2'],
        ['CD55','SMR3B'],
        ['CD59','HTN3'],
        ['CD59','MUC7'],
        ['CD59','TIMP3'],
        ['CD63','HBB'],
        ['CD63','HTN3'],
        ['CD63','PRSS1'],
        ['CD68','MPDU1'],
        ['CD68','SCUBE3'],
        ['CD68','SFTPC'],
        ['CD74','FLNA'],
        ['CD74','FOS'],
        ['CD74','GNAS'],
        ['CD74','HLA@'],
        ['CD74','IGH@'],
        ['CD74','IGK@'],
        ['CD74','LAMP1'],
        ['CD74','LYZ'],
        ['CD74','PIGR'],
        ['CD74','PSCA'],
        ['CD74','RP11-1143G9.4'],
        ['CD74','SFTPA1'],
        ['CD74','SFTPA2'],
        ['CD74','SFTPB'],
        ['CD74','SMR3B'],
        ['CD74','SOD2'],
        ['CD74','STATH'],
        ['CD74','SYNRG'],
        ['CD74','TG'],
        ['CD74','TNS1'],
        ['CD79A','RPS19'],
        ['CD81','KIAA2013'],
        ['CD81','PRSS2'],
        ['CD81','TSSC4'],
        ['CD81','YKT6'],
        ['CD82','COLEC12'],
        ['CD9','NDE1'],
        ['CD93','IGF2'],
        ['CD93','SYN3'],
        ['CD99','KANK2'],
        ['CD99','MYH9'],
        ['CDA','PINK1'],
        ['CDADC1','GTF2IP23'],
        ['CDADC1','LYZ'],
        ['CDADC1','RP11-203I16.7'],
        ['CDC16','UPF3A'],
        ['CDC40','LYZ'],
        ['CDC40','WDR78'],
        ['CDC42BPA','FGD5-AS1'],
        ['CDC42BPG','EHD1'],
        ['CDC42EP1','TMEM200A'],
        ['CDC42EP2','POLA2'],
        ['CDCA7L','SFTPB'],
        ['CDH2','FN1'],
        ['CDH23','CXCL16'],
        ['CDH24','GPC1'],
        ['CDH5','TG'],
        ['CDHR1','FLG-AS1'],
        ['CDIPT','RTN4'],
        ['CDK11B','IGHG1'],
        ['CDK11B','SLC35E2'],
        ['CDK12','HBB'],
        ['CDK12','MGEA5'],
        ['CDK14','CLDN12'],
        ['CDK16','TXNDC5'],
        ['CDK2','RAB5B'],
        ['CDK20','RP11-350E12.5'],
        ['CDK2AP2','TG'],
        ['CDKL4','MAP4K3'],
        ['CDKN1A','MYH9'],
        ['CDKN1C','FN1'],
        ['CDKN2B-AS1','UBA52'],
        ['CDR1-AS','LINC00632'],
        ['CDR2','RRN3'],
        ['CDRT4','FAM18B2'],
        ['CDT1','HBB'],
        ['CDV3','CDV3P1'],
        ['CEACAM1','PRH1'],
        ['CEACAM1','PRH1-PRR4'],
        ['CEACAM20','CEACAM22P'],
        ['CEACAM20','ZNF180'],
        ['CEACAM5','ENAM'],
        ['CEACAM5','IGH@'],
        ['CEACAM5','IGK@'],
        ['CEACAM5','IGKC'],
        ['CEACAM5','PIGR'],
        ['CEBPB','ITGB2'],
        ['CECR2','TERF2IP'],
        ['CEL','CELA2A'],
        ['CEL','CELA2B'],
        ['CEL','CELP'],
        ['CEL','CLPS'],
        ['CEL','CPA1'],
        ['CEL','CPB1'],
        ['CEL','CTRB1'],
        ['CEL','CTRB2'],
        ['CEL','CTRC'],
        ['CEL','CUZD1'],
        ['CEL','GP2'],
        ['CEL','HDLBP'],
        ['CEL','MT1A'],
        ['CEL','PLA2G1B'],
        ['CEL','PNLIP'],
        ['CEL','PNLIPRP1'],
        ['CEL','PRSS1'],
        ['CEL','PRSS2'],
        ['CEL','RP11-680B3.2'],
        ['CEL','SPINK1'],
        ['CEL','SYCN'],
        ['CEL','TMEM59'],
        ['CEL','TRB@'],
        ['CELA2A','CFTR'],
        ['CELA2A','CLPS'],
        ['CELA2A','CPA1'],
        ['CELA2A','CPA2'],
        ['CELA2A','CPB1'],
        ['CELA2A','CTC-479C5.12'],
        ['CELA2A','GP2'],
        ['CELA2A','PNLIP'],
        ['CELA2A','PRSS1'],
        ['CELA2A','PRSS2'],
        ['CELA2A','PRSS3'],
        ['CELA2A','RP11-331F4.4'],
        ['CELA2A','RP11-680B3.2'],
        ['CELA2A','RRBP1'],
        ['CELA2A','SYCN'],
        ['CELA2A','TRB@'],
        ['CELA2A','UBE2R2-AS1'],
        ['CELA2B','CLPS'],
        ['CELA2B','CPA1'],
        ['CELA2B','CPA2'],
        ['CELA2B','CPB1'],
        ['CELA2B','GP2'],
        ['CELA2B','PNLIP'],
        ['CELA2B','PRSS1'],
        ['CELA2B','PRSS2'],
        ['CELA2B','RP11-331F4.4'],
        ['CELA2B','RP11-680B3.2'],
        ['CELA2B','SYCN'],
        ['CELA2B','TRB@'],
        ['CELA3A','CLPS'],
        ['CELA3A','CPA1'],
        ['CELA3A','CPA2'],
        ['CELA3A','CTC-479C5.12'],
        ['CELA3A','GP2'],
        ['CELA3A','KLK1'],
        ['CELA3A','LINC00339'],
        ['CELA3A','PNLIP'],
        ['CELA3A','PRSS1'],
        ['CELA3A','PRSS2'],
        ['CELA3A','PRSS3'],
        ['CELA3A','TRB@'],
        ['CELA3A','UBE2R2-AS1'],
        ['CELA3B','CLPS'],
        ['CELA3B','GP2'],
        ['CELA3B','PNLIP'],
        ['CELA3B','PRSS1'],
        ['CELA3B','PRSS2'],
        ['CELA3B','PRSS3'],
        ['CELA3B','TRB@'],
        ['CELP','CPA1'],
        ['CELP','CPB1'],
        ['CELP','PNLIP'],
        ['CENPB','VEGFB'],
        ['CENPM','TNFRSF13C'],
        ['CENPT','RPL14'],
        ['CEP152','RP11-227D13.5'],
        ['CEP63','EPHB1'],
        ['CEP76','GP2'],
        ['CEP78','PSAT1'],
        ['CEP97','PDCL3'],
        ['CERK','LCN10'],
        ['CERK','SLC25A51'],
        ['CERS6','SYTL2'],
        ['CES1','CES1P1'],
        ['CES2','H6PD'],
        ['CES2','PIGR'],
        ['CFAP20','CSNK2A2'],
        ['CFAP54','NEDD1'],
        ['CFB','TXNL4B'],
        ['CFH','CFHR1'],
        ['CFI','LRG1'],
        ['CFI','SULT2A1'],
        ['CFL1','COL1A1'],
        ['CFL1','HBB'],
        ['CFL1','LYZ'],
        ['CFL1','MUC7'],
        ['CFL1','PIGR'],
        ['CFL1','TG'],
        ['CFL1','TPM2'],
        ['CFLAR','PKP2'],
        ['CFLAR','PTH2R'],
        ['CFLAR','RP11-13J8.1'],
        ['CFTR','HTN3'],
        ['CGNL1','PRSS1'],
        ['CGNL1','PRSS2'],
        ['CH17-140K24.2','TAF7'],
        ['CH17-212P11.4','IGH@'],
        ['CH17-212P11.5','IGH@'],
        ['CH17-212P11.5','IGHG1'],
        ['CH17-224D4.1','IGH@'],
        ['CH17-224D4.2','IGH@'],
        ['CHCHD10','VPREB3'],
        ['CHCHD2P6','DDI2'],
        ['CHCHD3','SRSF2'],
        ['CHD2','COL1A1'],
        ['CHD4','NOP2'],
        ['CHD8','SMR3B'],
        ['CHGA','CHGB'],
        ['CHGA','NR4A1'],
        ['CHGA','PRSS2'],
        ['CHGB','CYP11B1'],
        ['CHGB','HNRNPA2B1'],
        ['CHGB','MEG3'],
        ['CHMP3','LYZ'],
        ['CHP1','LRG1'],
        ['CHP1','SULT2A1'],
        ['CHPF','COL6A2'],
        ['CHPF','OBSL1'],
        ['CHPT1','LYZ'],
        ['CHST11','IGF2'],
        ['CHST5','RP11-77K12.5'],
        ['CIDEC','EMC3'],
        ['CIITA','PIK3C2A'],
        ['CIRBP','MUC7'],
        ['CIRBP','PNLIP'],
        ['CIRBP','SMR3B'],
        ['CIRH1A','PHACTR4'],
        ['CITED2','ITGA5'],
        ['CITED2','PTMS'],
        ['CITED2','RGMB'],
        ['CITF22-1A6.3','ZBED4'],
        ['CIZ1','PNLIP'],
        ['CKAP4','HS3ST3B1'],
        ['CKAP4','MT2A'],
        ['CKAP4','TWIST1'],
        ['CKM','CTSB'],
        ['CKM','DAB2'],
        ['CKM','MYH6'],
        ['CKM','NPPA'],
        ['CKM','NPPA-AS1'],
        ['CKM','NUCB1'],
        ['CKM','SYNPO2'],
        ['CKMT2','ZCCHC9'],
        ['CLASP2','HBB'],
        ['CLCA1','PIGR'],
        ['CLCA2','RAPH1'],
        ['CLCF1','POLD4'],
        ['CLCN3','PRSS1'],
        ['CLCN3','TG'],
        ['CLCN3P1','TTC39B'],
        ['CLCN7','TG'],
        ['CLDN11','SERPINE1'],
        ['CLDN18','ICAM1'],
        ['CLDN18','LYZ'],
        ['CLDN18','PGC'],
        ['CLDN18','RP11-1143G9.4'],
        ['CLDN4','SMR3B'],
        ['CLEC11A','S100A9'],
        ['CLEC2B','ZNF708'],
        ['CLEC3B','NPPA-AS1'],
        ['CLEC4F','FIGLA'],
        ['CLEC6A','PIGR'],
        ['CLEC6A','UTP23'],
        ['CLEC7A','ERC1'],
        ['CLIC1','HBB'],
        ['CLIC5','RP1-8B1.4'],
        ['CLIP1','RP11-512M8.5'],
        ['CLIP1','VPS33A'],
        ['CLK1','PPIL3'],
        ['CLK3','FOXN3'],
        ['CLK3','H2AFZP5'],
        ['CLK3','HEATR4'],
        ['CLK3','MPO'],
        ['CLK3','RP11-33N16.3'],
        ['CLMN','TRB@'],
        ['CLMP','FOXO4'],
        ['CLMP','HSPA8'],
        ['CLNK','ZNF518B'],
        ['CLPS','CPA1'],
        ['CLPS','CPA2'],
        ['CLPS','CPB1'],
        ['CLPS','CTRC'],
        ['CLPS','CUZD1'],
        ['CLPS','GP2'],
        ['CLPS','PLA2G1B'],
        ['CLPS','PNLIP'],
        ['CLPS','PNLIPRP2'],
        ['CLPS','PRSS1'],
        ['CLPS','PRSS2'],
        ['CLPS','PRSS3'],
        ['CLPS','RP11-331F4.4'],
        ['CLPS','TRB@'],
        ['CLPS','UBE2R2-AS1'],
        ['CLSTN1','CTNNBIP1'],
        ['CLSTN1','EIF3B'],
        ['CLSTN1','IGFBP5'],
        ['CLSTN1','KRT10'],
        ['CLSTN1','MARCKS'],
        ['CLSTN1','PRKCDBP'],
        ['CLTC','PPP1R12B'],
        ['CLTC','PTRF'],
        ['CLTC','SFTPC'],
        ['CLTCL1','HIRA'],
        ['CLU','CRYAB'],
        ['CLU','CYP4A11'],
        ['CLU','EEF1A1'],
        ['CLU','EIF2B5'],
        ['CLU','GFAP'],
        ['CLU','GRINA'],
        ['CLU','NRGN'],
        ['CLU','PLIN1'],
        ['CLU','POLR2A'],
        ['CLU','PRSS2'],
        ['CLU','PTGDS'],
        ['CLU','RMI2'],
        ['CLU','RP11-573D15.8'],
        ['CLU','SERPINA1'],
        ['CLU','SPARCL1'],
        ['CLU','TRB@'],
        ['CLU','TRBC2'],
        ['CLU','TRBV25-1'],
        ['CLU','TUBB4A'],
        ['CLUHP3','ZNF720'],
        ['CLVS1','KRT78'],
        ['CLYBL','PIGR'],
        ['CMAHP','FAM65B'],
        ['CMAHP','RP11-367G6.3'],
        ['CMAHP','RP3-522P13.3'],
        ['CMIP','PNLIP'],
        ['CMSS1','TBC1D23'],
        ['CMTM4','RP11-403P17.2'],
        ['CMYA5','TRDN'],
        ['CNDP2','LYZ'],
        ['CNDP2','TG'],
        ['CNN2','ECE1'],
        ['CNN2','FBLIM1'],
        ['CNN2','GALNT6'],
        ['CNN2','KIAA1191'],
        ['CNN2','LEPREL4'],
        ['CNN2','PPP1R12C'],
        ['CNN2','PXDC1'],
        ['CNN2','RBM3'],
        ['CNN2','RFC2'],
        ['CNN2','SFTPB'],
        ['CNN2','SLC25A37'],
        ['CNN3','TG'],
        ['CNOT1','IGF2'],
        ['CNOT1','PNLIP'],
        ['CNOT3','SMR3B'],
        ['CNPY3','RP3-475N16.1'],
        ['CNRIP1','HZGJ'],
        ['CNRIP1','PPP3R1'],
        ['CNRIP1','WDR92'],
        ['CNTN4','TTLL3'],
        ['CNTNAP2','RN7SL72P'],
        ['COBL','MUC7'],
        ['COCH','GP2'],
        ['COG5','CPD'],
        ['COG5','TRB@'],
        ['COL12A1','COL1A1'],
        ['COL14A1','IGF2'],
        ['COL14A1','IGFBP5'],
        ['COL15A1','IGF2'],
        ['COL15A1','TG'],
        ['COL16A1','COL1A1'],
        ['COL16A1','EMILIN1'],
        ['COL17A1','KRT1'],
        ['COL18A1','IGF2'],
        ['COL18A1','SPARC'],
        ['COL18A1','TRB@'],
        ['COL1A1','COL6A1'],
        ['COL1A1','COL6A2'],
        ['COL1A1','COL6A3'],
        ['COL1A1','CPSF6'],
        ['COL1A1','CTSB'],
        ['COL1A1','DLGAP4'],
        ['COL1A1','EFEMP1'],
        ['COL1A1','EMILIN1'],
        ['COL1A1','FAM129B'],
        ['COL1A1','FTL'],
        ['COL1A1','GAPDH'],
        ['COL1A1','GPC1'],
        ['COL1A1','IGK@'],
        ['COL1A1','KLHL22'],
        ['COL1A1','KMT2D'],
        ['COL1A1','KRT18'],
        ['COL1A1','LAMB2'],
        ['COL1A1','LEPRE1'],
        ['COL1A1','LGALS1'],
        ['COL1A1','MAP4K4'],
        ['COL1A1','MDH2'],
        ['COL1A1','MMP14'],
        ['COL1A1','MYH11'],
        ['COL1A1','MYL9'],
        ['COL1A1','MYO1C'],
        ['COL1A1','ORAOV1'],
        ['COL1A1','P4HB'],
        ['COL1A1','PCSK7'],
        ['COL1A1','PIGR'],
        ['COL1A1','PKM'],
        ['COL1A1','PLEC'],
        ['COL1A1','PLOD1'],
        ['COL1A1','PRRC2B'],
        ['COL1A1','PTMS'],
        ['COL1A1','RAB7A'],
        ['COL1A1','SFTPB'],
        ['COL1A1','SH3BGRL3'],
        ['COL1A1','SHC1'],
        ['COL1A1','SMTN'],
        ['COL1A1','SPARC'],
        ['COL1A1','STIM1'],
        ['COL1A1','SYMPK'],
        ['COL1A1','SYN3'],
        ['COL1A1','TCOF1'],
        ['COL1A1','TGFBI'],
        ['COL1A1','THOP1'],
        ['COL1A1','TLN1'],
        ['COL1A1','TRAK1'],
        ['COL1A1','TRIP12'],
        ['COL1A1','TSPO'],
        ['COL1A1','UNC45A'],
        ['COL1A1','VCL'],
        ['COL1A1','VIM-AS1'],
        ['COL1A1','XXBAC-BPG252P9.9'],
        ['COL1A2','COL6A3'],
        ['COL1A2','NOTCH3'],
        ['COL1A2','PLEKHM2'],
        ['COL1A2','RPL4'],
        ['COL1A2','SMR3B'],
        ['COL1A2','SPTBN1'],
        ['COL1A2','TPM4'],
        ['COL1A2','VIM-AS1'],
        ['COL23A1','TG'],
        ['COL3A1','COL6A2'],
        ['COL3A1','GNB2L1'],
        ['COL3A1','KRT4'],
        ['COL3A1','SFTPA2'],
        ['COL3A1','TNNT3'],
        ['COL3A1','UBC'],
        ['COL4A1','IGF2'],
        ['COL4A1','MEG3'],
        ['COL4A1','SYN3'],
        ['COL4A2','CTGF'],
        ['COL4A2','H19'],
        ['COL4A2','IGF2'],
        ['COL4A2','PRH1'],
        ['COL4A2','PRH1-PRR4'],
        ['COL4A2','PRR4'],
        ['COL4A2','SPARC'],
        ['COL5A1','EEF2'],
        ['COL5A1','EMILIN1'],
        ['COL5A1','IGF2'],
        ['COL5A1','ITGB5'],
        ['COL5A1','LRP1'],
        ['COL5A1','MXRA8'],
        ['COL5A1','PTPRS'],
        ['COL5A1','TMEM99'],
        ['COL5A1','VIM-AS1'],
        ['COL5A3','COL6A2'],
        ['COL6A1','FN1'],
        ['COL6A1','HSPB6'],
        ['COL6A1','LMNA'],
        ['COL6A1','MAZ'],
        ['COL6A1','MRC2'],
        ['COL6A1','NID2'],
        ['COL6A1','PLXNB2'],
        ['COL6A1','SEC16A'],
        ['COL6A1','SEPT9'],
        ['COL6A1','SERPINE2'],
        ['COL6A1','SPARC'],
        ['COL6A1','VIM'],
        ['COL6A2','CTSB'],
        ['COL6A2','FBN2'],
        ['COL6A2','FLNC'],
        ['COL6A2','GNAI2'],
        ['COL6A2','HNRNPK'],
        ['COL6A2','IGF2'],
        ['COL6A2','MUC7'],
        ['COL6A2','MXRA8'],
        ['COL6A2','PRKACA'],
        ['COL6A2','SFTPB'],
        ['COL6A2','SH3BP2'],
        ['COL6A2','TAF10'],
        ['COL6A2','TLN1'],
        ['COL6A2','VIM'],
        ['COL6A2','WNT5B'],
        ['COL6A3','LAMB2'],
        ['COL6A3','PLEC'],
        ['COL6A3','PTK7'],
        ['COL6A3','RTN4'],
        ['COL6A3','THY1'],
        ['COL6A3','VIM'],
        ['COL7A1','UCN2'],
        ['COL8A1','RP11-201E8.1'],
        ['COLCA1','POU2AF1'],
        ['COLEC12','IGF2'],
        ['COLGALT1','FAM129C'],
        ['COLGALT1','PTRF'],
        ['COLQ','HACL1'],
        ['COMMD3-BMI1','RP11-573G6.9'],
        ['COMMD6','LYZ'],
        ['COPA','RP11-1143G9.4'],
        ['COPA','RP11-574F21.3'],
        ['COPB1','MUC7'],
        ['COPS5','TCF24'],
        ['COPS7A','FN1'],
        ['COPZ1','RP11-753H16.5'],
        ['COPZ2','RP5-890E16.2'],
        ['COQ10B','LYZ'],
        ['COQ9','POLR2C'],
        ['COTL1','HNRNPA0'],
        ['COTL1','SCUBE3'],
        ['COTL1','SFTPB'],
        ['COX10-AS1','GHR'],
        ['COX18','DSG3'],
        ['COX20','TET1'],
        ['COX4I1','RP11-568J23.6'],
        ['COX4I1','TG'],
        ['COX6C','SMR3B'],
        ['COX7A2','TMEM30A'],
        ['COX7B','TCEANC2'],
        ['COX7C','KDM5B'],
        ['COX8A','NAA40'],
        ['CPA1','CTRB1'],
        ['CPA1','CTRB2'],
        ['CPA1','CTRC'],
        ['CPA1','CTRL'],
        ['CPA1','DDIT4'],
        ['CPA1','EPAS1'],
        ['CPA1','ERN1'],
        ['CPA1','EZR'],
        ['CPA1','FAM162A'],
        ['CPA1','FURIN'],
        ['CPA1','GLTSCR2'],
        ['CPA1','GP2'],
        ['CPA1','GRB10'],
        ['CPA1','HUWE1'],
        ['CPA1','LAMB2'],
        ['CPA1','PHLDA1'],
        ['CPA1','PLEC'],
        ['CPA1','PNLIP'],
        ['CPA1','PNLIPRP2'],
        ['CPA1','PRSS1'],
        ['CPA1','PRSS2'],
        ['CPA1','PRSS3'],
        ['CPA1','PTPRN2'],
        ['CPA1','RBM3'],
        ['CPA1','RBPJL'],
        ['CPA1','REG1A'],
        ['CPA1','RP11-331F4.4'],
        ['CPA1','RP11-680B3.2'],
        ['CPA1','SPTBN1'],
        ['CPA1','TIMP3'],
        ['CPA1','TRB@'],
        ['CPA1','UBE2R2-AS1'],
        ['CPA2','GP2'],
        ['CPA2','PNLIP'],
        ['CPA2','PRSS1'],
        ['CPA2','PRSS2'],
        ['CPA2','TRB@'],
        ['CPA2','UBE2R2-AS1'],
        ['CPB1','CTRB1'],
        ['CPB1','CYB5A'],
        ['CPB1','GP2'],
        ['CPB1','MYH9'],
        ['CPB1','PNLIP'],
        ['CPB1','PNLIPRP2'],
        ['CPB1','PRSS1'],
        ['CPB1','PRSS2'],
        ['CPB1','PRSS3'],
        ['CPB1','RNASE1'],
        ['CPB1','TRB@'],
        ['CPB1','UBA52'],
        ['CPB1','UBE2R2-AS1'],
        ['CPEB2-AS1','LINC00504'],
        ['CPED1','GBP6'],
        ['CPED1','WNT16'],
        ['CPM','RP11-702H23.4'],
        ['CPM','TMED9'],
        ['CPNE1','GP2'],
        ['CPQ','PNLIP'],
        ['CPSF3L','RP5-890O3.9'],
        ['CPT1B','NPPA-AS1'],
        ['CR2','HLA-E'],
        ['CRBN','TG'],
        ['CRCP','LYZ'],
        ['CREB3L1','PRSS1'],
        ['CREB3L1','PSPC1'],
        ['CREB3L2','IGF2'],
        ['CREB3L2','TPO'],
        ['CREBBP','TRB@'],
        ['CREBZF','LYZ'],
        ['CRIM1','GNAS'],
        ['CRIM1','MARCKS'],
        ['CRIM1','TG'],
        ['CRIPAK','UVSSA'],
        ['CRISP3','HTN1'],
        ['CRISP3','SMR3B'],
        ['CRISPLD2','GPNMB'],
        ['CRK','FAHD1'],
        ['CRK','YWHAE'],
        ['CRLF1','HBB'],
        ['CRLS1','MCM8'],
        ['CRNN','FLNA'],
        ['CRNN','HSPB1'],
        ['CRNN','KRT13'],
        ['CRNN','KRT4'],
        ['CRNN','KRT6A'],
        ['CRNN','MYH11'],
        ['CRNN','PPL'],
        ['CRNN','S100A9'],
        ['CRNN','TAGLN'],
        ['CRNN','TGM3'],
        ['CRP','CYP2E1'],
        ['CRP','FGA'],
        ['CRP','FGB'],
        ['CRP','FGG'],
        ['CRP','HP'],
        ['CRP','ORM1'],
        ['CRP','RDH11'],
        ['CRP','SERPINA1'],
        ['CRP','SERPINA3'],
        ['CRP','TXNL4B'],
        ['CRTAP','MCAM'],
        ['CRTAP','MYO1C'],
        ['CRTAP','SMAD3'],
        ['CRTAP','TG'],
        ['CRTAP','TPM4'],
        ['CRTC1','FZD4'],
        ['CRYAB','DGAT2'],
        ['CRYAB','MBP'],
        ['CRYAB','MYL2'],
        ['CRYAB','PLIN4'],
        ['CRYAB','TNNT2'],
        ['CRYM-AS1','CTD-2547E10.3'],
        ['CS','LYZ'],
        ['CSAG1','CSAG2'],
        ['CSAG1','CSAG3'],
        ['CSAG2','CSAG3'],
        ['CSDE1','DES'],
        ['CSDE1','LYZ'],
        ['CSF2RB','IGF2'],
        ['CSF3R','LYZ'],
        ['CSF3R','MRPS15'],
        ['CSGALNACT1','GYG2'],
        ['CSH1','IGF2'],
        ['CSH1','STAT6'],
        ['CSNK1A1','LYZ'],
        ['CSNK1D','METRNL'],
        ['CSNK1D','TG'],
        ['CSNK1G2','LLFOS-48D6.2'],
        ['CSNK1G2','OAZ1'],
        ['CSPG4','SDC3'],
        ['CSRNP1','SFTPC'],
        ['CSRP1','MSLN'],
        ['CSRP1','NRGN'],
        ['CSRP1','SMR3B'],
        ['CST1','MUC7'],
        ['CST1','PRH1'],
        ['CST1','PRH1-PRR4'],
        ['CST1','PRH2'],
        ['CST1','PRR4'],
        ['CST1','SMR3B'],
        ['CST2','MUC7'],
        ['CST2','PRH1'],
        ['CST2','PRH1-PRR4'],
        ['CST2','PRH2'],
        ['CST2','PRR4'],
        ['CST2','SMR3B'],
        ['CST3','FN1'],
        ['CST3','PRB3'],
        ['CST3','RP11-218C14.5'],
        ['CST3','SMR3B'],
        ['CST3','TNFAIP2'],
        ['CST4','MUC7'],
        ['CST4','PRH1'],
        ['CST4','PRH1-PRR4'],
        ['CST4','PRH2'],
        ['CST4','PRR4'],
        ['CST4','SMR3B'],
        ['CST4','STATH'],
        ['CST5','MUC7'],
        ['CST5','PRH2'],
        ['CST5','SMR3B'],
        ['CSTA','FGD5-AS1'],
        ['CSTB','KRT5'],
        ['CSTF3-AS1','PIGR'],
        ['CSTF3-AS1','RP1-89D4.1'],
        ['CSTL1','RP3-333B15.5'],
        ['CTA-109P11.4','STAM'],
        ['CTA-221G9.10','STMN3'],
        ['CTA-253N17.1','RPL22'],
        ['CTA-796E4.5','MYO18B'],
        ['CTAGE11P','CTAGE5'],
        ['CTAGE5','MIA2'],
        ['CTB-119C2.1','HNRNPA2B1'],
        ['CTB-11I22.1','UBLCP1'],
        ['CTB-11I22.2','UBLCP1'],
        ['CTB-120L21.1','ITK'],
        ['CTB-129P6.4','CTC-458I2.2'],
        ['CTB-131B5.5','IGIP'],
        ['CTB-14A14.2','TMTC4'],
        ['CTB-164N12.1','CTB-32H22.1'],
        ['CTB-175E5.7','IGF2'],
        ['CTB-186H2.3','SFTPC'],
        ['CTB-31O20.2','KLF16'],
        ['CTB-50L17.14','LYZ'],
        ['CTB-60B18.10','NTF4'],
        ['CTB-60B18.6','GINM1'],
        ['CTB-78H18.1','SMIM23'],
        ['CTBP2','WLS'],
        ['CTBS','GNG5'],
        ['CTC-235G5.2','SMS'],
        ['CTC-251I16.1','HRH2'],
        ['CTC-321K16.1','KRT10'],
        ['CTC-321K16.1','KRT14'],
        ['CTC-321K16.1','RP11-119J18.1'],
        ['CTC-338M12.2','TRIM7'],
        ['CTC-338M12.4','STX18-AS1'],
        ['CTC-340A15.2','TMED8'],
        ['CTC-347C20.1','CTD-2631K10.1'],
        ['CTC-360J11.4','ZNF264'],
        ['CTC-463A16.1','NDFIP1'],
        ['CTC-467M3.1','IGF2'],
        ['CTC-479C5.12','CTRL'],
        ['CTC-479C5.12','PRSS1'],
        ['CTC-479C5.12','PRSS2'],
        ['CTC-479C5.12','TRB@'],
        ['CTC-479C5.12','TRBC2'],
        ['CTC-479C5.12','TRBV25-1'],
        ['CTC-487M23.8','IGFBP4'],
        ['CTC-487M23.8','MUC15'],
        ['CTC-487M23.8','RP11-326C3.16'],
        ['CTC-510F12.4','TMEM205'],
        ['CTC-523E23.4','ZNF599'],
        ['CTC-554D6.1','RP11-326C3.16'],
        ['CTD-2020K17.1','RP13-890H12.2'],
        ['CTD-2031P19.5','TG'],
        ['CTD-2118P12.1','CTD-2134P3.2'],
        ['CTD-2140B24.6','ZNF10'],
        ['CTD-2154I11.2','SPATA9'],
        ['CTD-2184D3.5','DGKH'],
        ['CTD-2215E18.1','LINC01340'],
        ['CTD-2231E14.4','LINC00905'],
        ['CTD-2231E14.8','CYP4F3'],
        ['CTD-2231H16.1','RP11-811I15.1'],
        ['CTD-2267D19.3','TTLL3'],
        ['CTD-2270P14.5','TIGD4'],
        ['CTD-2291D10.2','ZNF728'],
        ['CTD-2306A12.1','SQRDL'],
        ['CTD-2308N23.2','S100A9'],
        ['CTD-2315A10.1','LINC01500'],
        ['CTD-2342J14.6','RAB3D'],
        ['CTD-2342J14.6','TMEM205'],
        ['CTD-2528L19.4','ZNF573'],
        ['CTD-2535I10.1','SMR3B'],
        ['CTD-2540M10.1','TNRC6A'],
        ['CTD-2542C24.8','ZNF85'],
        ['CTD-2545G14.7','RP1-4G17.5'],
        ['CTD-2545H1.2','SERINC3'],
        ['CTD-2568P8.1','PPM1A'],
        ['CTD-2583A14.10','SDCBP2-AS1'],
        ['CTD-2583A14.9','TEX41'],
        ['CTD-2616J11.11','NKG7'],
        ['CTD-3065J16.9','OPLAH'],
        ['CTD-3074O7.2','CTSF'],
        ['CTD-3187F8.14','CTD-3187F8.7'],
        ['CTD-3214H19.4','LYZ'],
        ['CTD-3222D19.2','FDPSP7'],
        ['CTD-3222D19.2','LYZ'],
        ['CTD-3233P19.7','ZNF738'],
        ['CTDSP1','SLC11A1'],
        ['CTDSP2','KLK3'],
        ['CTGF','IGHG1'],
        ['CTGF','IGHM'],
        ['CTGF','IGK@'],
        ['CTGF','IGKC'],
        ['CTGF','IGKV1-8'],
        ['CTGF','IGKV3-11'],
        ['CTIF','SLC25A3'],
        ['CTNNA1','KRT4'],
        ['CTNNA3','KRT8'],
        ['CTRB1','GP2'],
        ['CTRB1','PLA2G1B'],
        ['CTRB1','PNLIP'],
        ['CTRB1','PNLIPRP2'],
        ['CTRB1','PRSS1'],
        ['CTRB1','PRSS2'],
        ['CTRB1','PRSS3'],
        ['CTRB1','TRB@'],
        ['CTRB1','UBE2R2-AS1'],
        ['CTRB2','PNLIP'],
        ['CTRB2','PRSS1'],
        ['CTRB2','PRSS2'],
        ['CTRB2','PRSS3'],
        ['CTRB2','TRB@'],
        ['CTRB2','UBE2R2-AS1'],
        ['CTRC','GP2'],
        ['CTRC','PNLIP'],
        ['CTRC','PRSS1'],
        ['CTRC','PRSS2'],
        ['CTRC','PRSS3'],
        ['CTRC','TRB@'],
        ['CTRC','UBE2R2-AS1'],
        ['CTRL','PRSS1'],
        ['CTRL','PRSS2'],
        ['CTRL','TRB@'],
        ['CTSA','PLTP'],
        ['CTSA','TERF2IP'],
        ['CTSB','EEF1A1'],
        ['CTSB','EHD2'],
        ['CTSB','GPD2'],
        ['CTSB','GTF3C1'],
        ['CTSB','HNRNPH1'],
        ['CTSB','IYD'],
        ['CTSB','LOXL2'],
        ['CTSB','LTBP2'],
        ['CTSB','LYZ'],
        ['CTSB','MT1A'],
        ['CTSB','MYOZ1'],
        ['CTSB','NEAT1'],
        ['CTSB','P4HB'],
        ['CTSB','PPP1R36'],
        ['CTSB','PTRF'],
        ['CTSB','RP11-479O9.4'],
        ['CTSB','RP11-573D15.8'],
        ['CTSB','RP11-983P16.4'],
        ['CTSB','SCUBE3'],
        ['CTSB','TG'],
        ['CTSB','THBS1'],
        ['CTSB','TMED10'],
        ['CTSB','TPT1'],
        ['CTSB','VSTM1'],
        ['CTSB','WWP2'],
        ['CTSC','RAB38'],
        ['CTSD','IFITM10'],
        ['CTSD','IGF2'],
        ['CTSD','SFTPA1'],
        ['CTSD','SMR3B'],
        ['CTSE','DPCR1'],
        ['CTSE','LYZ'],
        ['CTSE','MUC6'],
        ['CTSG','HOOK2'],
        ['CTSG','IGH@'],
        ['CTSG','RP11-1143G9.4'],
        ['CTSG','S100A9'],
        ['CTSL','CTSL3P'],
        ['CTSS','GPRC5A'],
        ['CTSS','HLA@'],
        ['CTSS','HORMAD1'],
        ['CTSS','IGH@'],
        ['CTSS','KIRREL3'],
        ['CTSS','LYZ'],
        ['CTSS','P2RX7'],
        ['CTSS','TBXAS1'],
        ['CTSS','VPRBP'],
        ['CTSZ','PTOV1'],
        ['CUBN','RSU1'],
        ['CUL4B','EMP2'],
        ['CUZD1','GP2'],
        ['CUZD1','PNLIP'],
        ['CWC25','LYZ'],
        ['CWC25','PIP4K2B'],
        ['CWF19L1','PTRF'],
        ['CXCL12','PXDC1'],
        ['CXCL14','TMEM99'],
        ['CXCR4','PTMA'],
        ['CXCR4','PTP4A1'],
        ['CXCR4','UBB'],
        ['CXCR4','YPEL5'],
        ['CYB5R2','OVCH2'],
        ['CYB5R3','FTH1'],
        ['CYB5R3','HLA@'],
        ['CYB5R3','POLDIP3'],
        ['CYB5R3','TNNC2'],
        ['CYB5R4','RP4-676J13.2'],
        ['CYC1','FN1'],
        ['CYGB','ST6GALNAC2'],
        ['CYP11B1','DLK1'],
        ['CYP17A1-AS1','WBP1L'],
        ['CYP19A1','IGF2'],
        ['CYP27A1','TTLL4'],
        ['CYP2A6','LRP1'],
        ['CYP2B6','ECE1'],
        ['CYP2B6','LRG1'],
        ['CYP2B6','QPRT'],
        ['CYP2B6','RP11-696N14.1'],
        ['CYP2B6','SULT2A1'],
        ['CYP2B7P1','LRG1'],
        ['CYP2C8','RP11-310E22.5'],
        ['CYP2C8','SERPINA1'],
        ['CYP2D7','CYP2D8P'],
        ['CYP2E1','FGB'],
        ['CYP2E1','FGG'],
        ['CYP2E1','HP'],
        ['CYP2E1','SERPINA1'],
        ['CYP2U1','HADH'],
        ['CYP3A4','SEPP1'],
        ['CYP3A5','TATDN2P2'],
        ['CYP4F3','LA16C-83F12.6'],
        ['CYP4F3','PLCH1'],
        ['CYP4F3','QRFPR'],
        ['CYP4F3','RSPH14'],
        ['CYP4F3','ZNF536'],
        ['CYP4F32P','SNX18P14'],
        ['CYP8B1','HRG'],
        ['CYR61','DUSP7'],
        ['CYR61','IGHG1'],
        ['CYTH1','USP36'],
        ['DAB2','ENAM'],
        ['DAB2','MEG3'],
        ['DAB2IP','LYZ'],
        ['DAD1','KIAA0922'],
        ['DAGLB','PNLIP'],
        ['DAP','HBB'],
        ['DAZAP2','LYZ'],
        ['DBN1','MYH9'],
        ['DBN1','NUP62'],
        ['DBN1','TBC1D10B'],
        ['DBNDD2','SYS1'],
        ['DBT','RTCA-AS1'],
        ['DCAF7','LYZ'],
        ['DCAF8','RP11-536C5.7'],
        ['DCAF8','SFTPC'],
        ['DCN','IGF2'],
        ['DCTD','PNLIP'],
        ['DCTN1','FN1'],
        ['DCTN4','RP11-1143G9.4'],
        ['DCTN6','LYZ'],
        ['DDA1','MRPL34'],
        ['DDA1','RP11-857B24.1'],
        ['DDIT4','TRB@'],
        ['DDOST','LYZ'],
        ['DDOST','TTL'],
        ['DDR1-AS1','LINC00243'],
        ['DDR2','NPPA'],
        ['DDRGK1','PROSAPIP1'],
        ['DDX17','KHDRBS3'],
        ['DDX17','LYZ'],
        ['DDX17','MYH9'],
        ['DDX17','SFTPB'],
        ['DDX17','SFTPC'],
        ['DDX17','UBC'],
        ['DDX21','KIF1BP'],
        ['DDX24','HBB'],
        ['DDX39B','LYZ'],
        ['DDX39B','XXBAC-BPG299F13.16'],
        ['DDX3X','NPPA'],
        ['DDX3X','SFTPA1'],
        ['DDX3Y','HBB'],
        ['DDX5','EEF1A1'],
        ['DDX5','EEF1D'],
        ['DDX5','HBB'],
        ['DDX5','POLG2'],
        ['DDX5','PRSS1'],
        ['DDX5','PTP4A1'],
        ['DDX5','RP11-20I23.1'],
        ['DEDD','RP11-297K8.2'],
        ['DEF6','PPARD'],
        ['DEFA4','DEFA8P'],
        ['DEFA4','HBB'],
        ['DEFB122','VCL'],
        ['DEGS1','FN1'],
        ['DENND3','HBB'],
        ['DENND4A','LYZ'],
        ['DENND5A','PCBP1'],
        ['DENND6A-AS1','PDE12'],
        ['DEPDC1B','PSMD12'],
        ['DEPDC5','LYZ'],
        ['DERL2','DHX33'],
        ['DES','DLGAP4-AS1'],
        ['DES','DUSP3'],
        ['DES','EEF1A2'],
        ['DES','FKBP8'],
        ['DES','FLG-AS1'],
        ['DES','HHATL'],
        ['DES','HSPB6'],
        ['DES','IGH@'],
        ['DES','KLK2'],
        ['DES','KRT1'],
        ['DES','KRT13'],
        ['DES','KRT14'],
        ['DES','KRT16'],
        ['DES','KRT4'],
        ['DES','KRT5'],
        ['DES','KRT6A'],
        ['DES','LDHB'],
        ['DES','MB'],
        ['DES','NPPA'],
        ['DES','NPPA-AS1'],
        ['DES','NPPB'],
        ['DES','OGN'],
        ['DES','RP1-178F15.4'],
        ['DES','RP1-178F15.5'],
        ['DES','RP5-857K21.4'],
        ['DES','S100A13'],
        ['DES','S100A9'],
        ['DES','SRL'],
        ['DES','SYNPO'],
        ['DES','TIMP2'],
        ['DES','TIMP3'],
        ['DES','TNNI3'],
        ['DES','TNNT2'],
        ['DEXI','FAM102B'],
        ['DFNA5','OSBPL3'],
        ['DGAT2','PSAP'],
        ['DGAT2','RN7SL786P'],
        ['DGKD','HBB'],
        ['DGKG','IGF2'],
        ['DGKI','TG'],
        ['DGKZ','LYZ'],
        ['DHCR24','LYZ'],
        ['DHCR24','STAR'],
        ['DHRS1','RABGGTA'],
        ['DHRS12','LINC00282'],
        ['DHRSX','RP11-617F23.1'],
        ['DHRSX','SORBS2'],
        ['DHTKD1','LYZ'],
        ['DHX40','TBC1D3P1-DHX40P1'],
        ['DHX9','NPL'],
        ['DIAPH1','TG'],
        ['DICER1','HBB'],
        ['DIP2A','DIP2A-IT1'],
        ['DIRAS3','WLS'],
        ['DISP2','KNSTRN'],
        ['DIXDC1','DLAT'],
        ['DKK3','MYH6'],
        ['DKK3','MYL4'],
        ['DLG2','PIGR'],
        ['DLGAP4','ITGAM'],
        ['DLGAP4','LOXL2'],
        ['DLGAP4','TRB@'],
        ['DLGAP4-AS1','FLG-AS1'],
        ['DLGAP4-AS1','KRT13'],
        ['DLGAP4-AS1','KRT4'],
        ['DLGAP4-AS1','MYH11'],
        ['DLGAP4-AS1','MYL12A'],
        ['DLGAP4-AS1','NDE1'],
        ['DLGAP4-AS1','TPM2'],
        ['DLGAP4-AS1','WWTR1'],
        ['DLK1','MEG3'],
        ['DMBT1','LYZ'],
        ['DMBT1','PIGR'],
        ['DMBT1','PRB1'],
        ['DMBT1','PRH1'],
        ['DMBT1','PRH1-PRR4'],
        ['DMBT1','PRR4'],
        ['DMD','STATH'],
        ['DMKN','KRTDAP'],
        ['DMTN','LYZ'],
        ['DMTN','TG'],
        ['DMWD','RSPH6A'],
        ['DNA2','SLC25A16'],
        ['DNAAF1','LINC00470'],
        ['DNAAF5','LYZ'],
        ['DNAH1','GLYCTK'],
        ['DNAJC15','LINC00400'],
        ['DNAJC18','ECSCR'],
        ['DNAJC3','HTN3'],
        ['DNAJC5','TPD52L2'],
        ['DNAJC9','MRPS16'],
        ['DNASE2','KLF1'],
        ['DNASE2B','SAMD13'],
        ['DNER','GFAP'],
        ['DNER','SMR3B'],
        ['DNLZ','ITGB5'],
        ['DNM1','DNM1P33'],
        ['DNM1L','LYZ'],
        ['DNM1L','STX3'],
        ['DNM2','ZG16B'],
        ['DNMT1','HBB'],
        ['DOCK1','FUS'],
        ['DOCK9','IGF2'],
        ['DOCK9','SMR3B'],
        ['DOPEY2','LYZ'],
        ['DOT1L','LLFOS-48D6.2'],
        ['DOT1L','OAZ1'],
        ['DPCR1','IGK@'],
        ['DPCR1','IGKC'],
        ['DPCR1','IGKV1-8'],
        ['DPCR1','IGKV3-11'],
        ['DPCR1','MUC6'],
        ['DPH6-AS1','TAP2'],
        ['DPH6-AS1','XXBAC-BPG246D15.9'],
        ['DPM2','PIP5KL1'],
        ['DPP3','LYZ'],
        ['DPP4','IGF2'],
        ['DPP6','PABPC1'],
        ['DPY19L1','KCNK3'],
        ['DPY19L2','DPY19L2P2'],
        ['DPY19L2P2','NAPEPLD'],
        ['DPY19L4','INTS8'],
        ['DPY30','RPS9P1'],
        ['DRAIC','PCAT29'],
        ['DRAP1','GPR124'],
        ['DRAP1','PALLD'],
        ['DRG1','TG'],
        ['DRICH1','KB-208E9.1'],
        ['DROSHA','TG'],
        ['DSC1','KRT1'],
        ['DSC3','FLG2'],
        ['DSC3','RFX4'],
        ['DSC3','TMEM99'],
        ['DSCAML1','FXYD6-FXYD2'],
        ['DSCAS','KRT10'],
        ['DSCC1','KB-1471A8.1'],
        ['DSCR3','PTRF'],
        ['DSE','RP11-274B18.4'],
        ['DSE','TG'],
        ['DSG2','LYZ'],
        ['DSG2','PRH1'],
        ['DSG2','PRH1-PRR4'],
        ['DSG3','IL1RAPL1'],
        ['DSG3','NXN'],
        ['DSP','IGF2'],
        ['DSP','KRT13'],
        ['DSP','MUC7'],
        ['DSP','TMEM99'],
        ['DST','KRT10'],
        ['DSTYK','LYZ'],
        ['DTD2','GRB2'],
        ['DTX2','UPK3B'],
        ['DUOX1','SFTPB'],
        ['DUS1L','FASN'],
        ['DUS3L','FUT6'],
        ['DUSP1','HBB'],
        ['DUSP1','SFTPC'],
        ['DUSP11','TPRKB'],
        ['DUSP16','GPR19'],
        ['DUSP3','FGG'],
        ['DUT','ZNF534'],
        ['DUX4L26','DUX4L50'],
        ['DUX4L50','LINC01410'],
        ['DVL3','IRF2BPL'],
        ['DYDC1','MAT1A'],
        ['DYNC1H1','HBB'],
        ['DYNC1H1','SFTPA1'],
        ['DYNC1H1','SFTPA2'],
        ['DYNLL2','PTRF'],
        ['DYNLRB1','ITCH'],
        ['DYNLT1','TATDN2P2'],
        ['DYRK4','RAD51AP1'],
        ['DYX1C1-CCPG1','PIGBOS1'],
        ['DYX1C1-CCPG1','RAB27A'],
        ['E2F1','NECAB3'],
        ['E2F3P2','MRE11A'],
        ['E2F4','RPL14'],
        ['EBF4','RP4-686C3.7'],
        ['ECE1','KRT13'],
        ['ECE1','LYZ'],
        ['ECE1','SFTPC'],
        ['ECE1','TRB@'],
        ['ECH1','HNRNPL'],
        ['ECHDC2','GP2'],
        ['ECHDC2','PNLIP'],
        ['ECM1','KRT13'],
        ['EDARADD','ENO1'],
        ['EEF1A1','G0S2'],
        ['EEF1A1','GP2'],
        ['EEF1A1','HBB'],
        ['EEF1A1','HMGB2'],
        ['EEF1A1','HNRNPA2B1'],
        ['EEF1A1','HNRNPH1'],
        ['EEF1A1','IGH@'],
        ['EEF1A1','KLK3'],
        ['EEF1A1','KRT13'],
        ['EEF1A1','LYZ'],
        ['EEF1A1','MPO'],
        ['EEF1A1','MYADM'],
        ['EEF1A1','PIGR'],
        ['EEF1A1','PRH1-PRR4'],
        ['EEF1A1','PTP4A1'],
        ['EEF1A1','SERPINA3'],
        ['EEF1A1','SFTPB'],
        ['EEF1A1','SFTPC'],
        ['EEF1A1','SLC25A37'],
        ['EEF1A1','SRSF5'],
        ['EEF1A1','SRSF7'],
        ['EEF1A1','TGFBR3'],
        ['EEF1A1','TMBIM6'],
        ['EEF1A1','TRB@'],
        ['EEF1D','PARK2'],
        ['EEF1D','SFTPC'],
        ['EEF1D','ZC3H3'],
        ['EEF1DP3','FRY'],
        ['EEF2','EGR1'],
        ['EEF2','FASN'],
        ['EEF2','FN1'],
        ['EEF2','GP2'],
        ['EEF2','HBB'],
        ['EEF2','KRT14'],
        ['EEF2','LAMP3'],
        ['EEF2','LGALS1'],
        ['EEF2','LYZ'],
        ['EEF2','MGLL'],
        ['EEF2','MUC7'],
        ['EEF2','NDE1'],
        ['EEF2','PFN1'],
        ['EEF2','PIGR'],
        ['EEF2','PLIN1'],
        ['EEF2','PLIN4'],
        ['EEF2','PRB3'],
        ['EEF2','PRSS1'],
        ['EEF2','PRSS2'],
        ['EEF2','TPM2'],
        ['EEF2','TRB@'],
        ['EEF2','VIM-AS1'],
        ['EEF2','WDR1'],
        ['EFCAB11','SYNC'],
        ['EFCAB14','LYZ'],
        ['EFCAB14','SFTPC'],
        ['EFCAB3','RP11-212P7.3'],
        ['EFCAB5','NSRP1'],
        ['EGFR','IGF2'],
        ['EGFR','SSR3'],
        ['EGLN1','SFTPB'],
        ['EGR1','KRT13'],
        ['EGR1','MUC6'],
        ['EGR1','PLIN4'],
        ['EGR1','SERPINA1'],
        ['EGR1','SFTPA2'],
        ['EGR1','SFTPB'],
        ['EGR1','SFTPC'],
        ['EGR1','SPRR1B'],
        ['EGR2','TG'],
        ['EHBP1L1','FAM89B'],
        ['EHBP1L1','SSSCA1'],
        ['EHD1','SF1'],
        ['EHD2','FBLIM1'],
        ['EHD2','FGFR1'],
        ['EHD2','FKBP14'],
        ['EHD2','GLTP'],
        ['EHD2','KIAA1191'],
        ['EHD2','LEPREL4'],
        ['EHD2','MYADM'],
        ['EHD2','PLIN3'],
        ['EHD2','PLIN4'],
        ['EHD2','PTRF'],
        ['EHD2','RGS5'],
        ['EHD2','SFTPB'],
        ['EHD2','SLC2A6'],
        ['EHD2','TPM4'],
        ['EHD2','TRAM2'],
        ['EHD3','IGF2'],
        ['EHMT1','SFTPB'],
        ['EHMT2','SLC44A4'],
        ['EI24','UFM1'],
        ['EIF1','LYZ'],
        ['EIF1','NAMPT'],
        ['EIF1','RP11-1143G9.4'],
        ['EIF2B5','KRT1'],
        ['EIF2B5','LYZ'],
        ['EIF2B5','MUC7'],
        ['EIF2B5','MYH7'],
        ['EIF2B5','NPPA-AS1'],
        ['EIF2B5','PRH1'],
        ['EIF2B5','PRH1-PRR4'],
        ['EIF2B5','PRR4'],
        ['EIF2B5','SFTPB'],
        ['EIF2B5','SMR3B'],
        ['EIF2B5','ST3GAL1'],
        ['EIF2B5','TG'],
        ['EIF2D','SFTPC'],
        ['EIF2S1','TG'],
        ['EIF2S3','TPM4'],
        ['EIF3B','ITGAV'],
        ['EIF3E','PRB3'],
        ['EIF3E','PRSS1'],
        ['EIF3E','RAB8A'],
        ['EIF3E','SMR3B'],
        ['EIF3E','TG'],
        ['EIF3H','HSP90AA1'],
        ['EIF3L','IGF2'],
        ['EIF3L','RP11-666E17.1'],
        ['EIF4E','TMEM30B'],
        ['EIF4E3','FOXP1'],
        ['EIF4G1','FLNC'],
        ['EIF4G1','KRT13'],
        ['EIF4G1','SPARC'],
        ['EIF4G3','HP1BP3'],
        ['EIF5','HMGB2'],
        ['EIF5A','FBRS'],
        ['EIF5A','FTH1'],
        ['EIF5A','IRF2BPL'],
        ['EIF5A','PTMS'],
        ['EIF6','LYZ'],
        ['ELANE','HBB'],
        ['ELANE','KBTBD11'],
        ['ELANE','PTBP1'],
        ['ELANE','S100A9'],
        ['ELAVL1','HBB'],
        ['ELAVL1','MUC7'],
        ['ELAVL1','PRB1'],
        ['ELAVL1','SMR3B'],
        ['ELAVL1','TIMM44'],
        ['ELAVL1','TRB@'],
        ['ELF1','TG'],
        ['ELF3','RNPEP'],
        ['ELF5','LYZ'],
        ['ELK1','IGH@'],
        ['ELK2BP','IGH@'],
        ['ELK4','SLC45A3'],
        ['ELL2','PNLIP'],
        ['ELMO1','MPO'],
        ['ELMO1','PNLIP'],
        ['ELN','TNRC18'],
        ['ELOVL1','MED8'],
        ['ELP2','GLTPD2'],
        ['EMB','WWC1'],
        ['EMC3','PRRT3'],
        ['EMG1','PPP3CA'],
        ['EMILIN1','EWSR1'],
        ['EMILIN1','GNAS'],
        ['EMILIN1','MVP'],
        ['EMILIN1','TSC22D4'],
        ['EMILIN2','FLNA'],
        ['EMP1','EVPL'],
        ['EMP1','MYH11'],
        ['EMP2','F11R'],
        ['EMP2','FN1'],
        ['EMP2','HSPG2'],
        ['EMP2','MYH9'],
        ['EMP2','NCOR2'],
        ['EMP2','PTCD1'],
        ['EMP2','SFTPA1'],
        ['ENAM','IGFBP5'],
        ['ENAM','KLF6'],
        ['ENAM','LYZ'],
        ['ENAM','PIGR'],
        ['ENAM','ZG16B'],
        ['ENG','PTRF'],
        ['ENO1','LYZ'],
        ['ENO3','NPPA-AS1'],
        ['ENPP1','KCNE3'],
        ['ENPP5','RP1-8B1.4'],
        ['ENSA','MCL1'],
        ['ENTPD1','TG'],
        ['ENTPD1-AS1','TCTN3'],
        ['ENTPD4','LYZ'],
        ['ENTPD6','MUC7'],
        ['EOGT','TMF1'],
        ['EPAS1','IGF2'],
        ['EPAS1','MYH9'],
        ['EPAS1','SFTPA2'],
        ['EPAS1','TMEM247'],
        ['EPB41L1','RP4-550H1.4'],
        ['EPB41L2','IGFBP5'],
        ['EPCAM','TG'],
        ['EPCAM','TIAL1'],
        ['EPDR1','IGFBP5'],
        ['EPG5','LYZ'],
        ['EPG5','PSTPIP2'],
        ['EPHX1','FGG'],
        ['EPHX1','SERPINA1'],
        ['EPHX2','GULOP'],
        ['EPN1','ITGA3'],
        ['EPN1','MMP14'],
        ['EPOR','SYNC'],
        ['EPRS','HTN3'],
        ['EPRS','LYZ'],
        ['EPS15L1','IGFBP5'],
        ['EPS8L2','TRIP6'],
        ['EPSTI1','LYZ'],
        ['EQTN','MOB3B'],
        ['ERAP1','PNLIP'],
        ['ERBB2','GRB7'],
        ['ERBB2','KRT8'],
        ['ERCC1','MYH11'],
        ['ERCC1','S100A9'],
        ['ERCC2','SERPINE1'],
        ['ERGIC1','FGB'],
        ['ERGIC1','FGG'],
        ['ERGIC1','FLNA'],
        ['ERGIC1','SMR3B'],
        ['ERGIC2','RP11-996F15.2'],
        ['ERGIC3','ZG16B'],
        ['ERLIN2','RP11-863K10.2'],
        ['ERMN','GFAP'],
        ['ERN1','TEX2'],
        ['ERP27','GP2'],
        ['ERVK13-1','PDPK1'],
        ['ERVK3-1','ZSCAN22'],
        ['ERVW-1','PEX1'],
        ['ESD','HTR2A'],
        ['ESRP2','RP11-96D1.5'],
        ['ESRRAP2','TPTE2'],
        ['ESRRG','SMR3B'],
        ['ESYT1','PA2G4'],
        ['ESYT1','RP11-603J24.9'],
        ['ESYT2','LYZ'],
        ['ETF1','HSPA9'],
        ['ETF1','LYZ'],
        ['ETFB','NKG7'],
        ['ETHE1','XRCC1'],
        ['EVPL','TG'],
        ['EVPLL','TNFRSF13B'],
        ['EXD1','RP11-492E3.2'],
        ['EXO1','WDR64'],
        ['EXOC6','HHEX'],
        ['EXOC7','FN1'],
        ['EXOSC6','SMG1'],
        ['EXT2','PPIC'],
        ['EYS','LYZ'],
        ['EZH1','TG'],
        ['EZR','SFTPC'],
        ['EZR','SMR3B'],
        ['F11-AS1','SLC25A5'],
        ['F11R','FOXM1'],
        ['F11R','LRG1'],
        ['F11R','RBM22P2'],
        ['F11R','TSTD1'],
        ['F13A1','IGF2'],
        ['F3','PIN1'],
        ['FA2H','MLKL'],
        ['FABP1','FGA'],
        ['FABP3','NPPA'],
        ['FABP4','FAM213A'],
        ['FABP6','WSB1'],
        ['FADS3','SLC1A5'],
        ['FAH','ZFAND6'],
        ['FAM102A','IGH@'],
        ['FAM107A','FAM3D'],
        ['FAM107B','HBB'],
        ['FAM107B','LYZ'],
        ['FAM111A','PROX2'],
        ['FAM117A','SLC35B1'],
        ['FAM129A','GCSAML'],
        ['FAM129A','HMCN2'],
        ['FAM129A','MYL2'],
        ['FAM129A','PNLIP'],
        ['FAM129A','RP11-72L22.1'],
        ['FAM129A','TRB@'],
        ['FAM129B','FKBP8'],
        ['FAM129B','KRT13'],
        ['FAM129B','PRSS2'],
        ['FAM129C','MMP23B'],
        ['FAM134B','ZNF622'],
        ['FAM135A','SSR3'],
        ['FAM136A','FAM136BP'],
        ['FAM13A','PRH1'],
        ['FAM13A','PRH1-PRR4'],
        ['FAM13A','PRR4'],
        ['FAM13A-AS1','HERC3'],
        ['FAM149A','TLR3'],
        ['FAM157A','FAM157B'],
        ['FAM162B','ZUFSP'],
        ['FAM163B','LL09NC01-254D11.1'],
        ['FAM167A','TG'],
        ['FAM167A-AS1','IYD'],
        ['FAM167A-AS1','TG'],
        ['FAM170B','VSTM4'],
        ['FAM172A','KIAA0825'],
        ['FAM175A','HELQ'],
        ['FAM185A','FAM185BP'],
        ['FAM193B','LYZ'],
        ['FAM198A','KRBOX1'],
        ['FAM205BP','FAM205C'],
        ['FAM209B','RTFDC1'],
        ['FAM210B','LYZ'],
        ['FAM211A-AS1','LGALS1'],
        ['FAM212B','PSMB2'],
        ['FAM213A','LYZ'],
        ['FAM221B','HINT2'],
        ['FAM227A','RP3-508I15.9'],
        ['FAM228B','RP1-167A14.2'],
        ['FAM230B','SNX1'],
        ['FAM3A','SFTPB'],
        ['FAM3D-AS1','SMR3B'],
        ['FAM46A','LYZ'],
        ['FAM49B','MPO'],
        ['FAM49B','ZFHX3'],
        ['FAM53B','METTL10'],
        ['FAM53C','KDM3B'],
        ['FAM71D','MPP5'],
        ['FAM73A','NEXN'],
        ['FAM8A1','SUMO2P13'],
        ['FAR1','RP11-98J9.1'],
        ['FARP1','TG'],
        ['FARSA','SYCE2'],
        ['FASN','MGST1'],
        ['FASN','PLIN4'],
        ['FBLIM1','NONO'],
        ['FBLIM1','PDGFRA'],
        ['FBLIM1','SCUBE3'],
        ['FBLN1','FSCN1'],
        ['FBLN1','IGF2'],
        ['FBLN1','MEG3'],
        ['FBLN1','MYH11'],
        ['FBLN1','NDE1'],
        ['FBLN1','SERPINH1'],
        ['FBLN2','MBOAT7'],
        ['FBN1','SERPINE1'],
        ['FBRS','MUC7'],
        ['FBXL2','IGF2'],
        ['FBXL22','USP3'],
        ['FBXL5','LYZ'],
        ['FBXL5','METTL16'],
        ['FBXL5','RP11-799M12.2'],
        ['FBXO11','LYZ'],
        ['FBXO16','STOM'],
        ['FBXO21','NOS1'],
        ['FBXO22','UBE2Q2'],
        ['FBXW12','ZNF589'],
        ['FBXW9','TNPO2'],
        ['FCAR','KIAA1211L'],
        ['FCAR','LYZ'],
        ['FCAR','NRF1'],
        ['FCER1G','TOMM40L'],
        ['FCGBP','PAX8-AS1'],
        ['FCGBP','PIGR'],
        ['FCGBP','TPO'],
        ['FCN1','MPO'],
        ['FCN3','MAP3K6'],
        ['FDCSP','SMR3B'],
        ['FDFT1','NEIL2'],
        ['FDFT1','SUB1'],
        ['FDPSP7','SLC35E1'],
        ['FDX1','RP11-347E10.1'],
        ['FER','RP11-1143G9.4'],
        ['FER1L4','IGHG1'],
        ['FEZ2','IGF2'],
        ['FFAR4','RP11-573D15.8'],
        ['FFAR4','SERPINA1'],
        ['FGA','HP'],
        ['FGA','IGF2'],
        ['FGA','KTN1'],
        ['FGA','P4HB'],
        ['FGA','RP11-573D15.8'],
        ['FGA','SAA1'],
        ['FGA','SAA2'],
        ['FGA','SARM1'],
        ['FGA','SERPINA1'],
        ['FGA','SERPINA3'],
        ['FGA','TXNL4B'],
        ['FGB','GC'],
        ['FGB','HP'],
        ['FGB','ORM1'],
        ['FGB','PEBP1'],
        ['FGB','RP11-573D15.8'],
        ['FGB','SERPINA1'],
        ['FGD3','LYZ'],
        ['FGD3','SFTPC'],
        ['FGD5-AS1','RCL1'],
        ['FGF14','IGF2'],
        ['FGFR1','PPIC'],
        ['FGFR4','ZNF346'],
        ['FGFRL1','IGF2'],
        ['FGFRL1','PLEC'],
        ['FGG','GC'],
        ['FGG','HP'],
        ['FGG','HPX'],
        ['FGG','LRG1'],
        ['FGG','PEBP1'],
        ['FGG','RBP4'],
        ['FGG','RP11-573D15.8'],
        ['FGG','RP11-696N14.1'],
        ['FGG','SERPINA1'],
        ['FGG','SERPINA3'],
        ['FGG','TMBIM6'],
        ['FGG','TXNL4B'],
        ['FGL1','HP'],
        ['FGL1','SERPINA3'],
        ['FHL2','TTN'],
        ['FHL3','SF3A3'],
        ['FHOD3','NPPA'],
        ['FIK','ZNF674'],
        ['FKBP10','JUNB'],
        ['FKBP15','HTN3'],
        ['FKBP15','LYZ'],
        ['FKBP15','PRB3'],
        ['FKBP1A','SDCBP2'],
        ['FKBP5','SFTPB'],
        ['FKBP8','LYZ'],
        ['FKRP','IGF2'],
        ['FLG-AS1','KRT1'],
        ['FLG-AS1','KRT13'],
        ['FLG-AS1','KRT2'],
        ['FLG-AS1','KRT4'],
        ['FLG-AS1','KRT6A'],
        ['FLG-AS1','NUCKS1'],
        ['FLG-AS1','PROM2'],
        ['FLG-AS1','RNF213'],
        ['FLG-AS1','RP5-884C9.2'],
        ['FLG-AS1','S100A2'],
        ['FLG-AS1','SBSN'],
        ['FLG-AS1','TGM3'],
        ['FLG-AS1','VAMP2'],
        ['FLG2','GSDMA'],
        ['FLG2','KRT5'],
        ['FLG2','SGPL1'],
        ['FLG2','TSIX'],
        ['FLG2','TTC39C'],
        ['FLG2','TUFT1'],
        ['FLJ22447','PRKCH'],
        ['FLJ37505','LINC00507'],
        ['FLJ46284','TRIQK'],
        ['FLNA','FTH1'],
        ['FLNA','GNAS'],
        ['FLNA','HBB'],
        ['FLNA','HNRNPUL1'],
        ['FLNA','IGF2'],
        ['FLNA','IGH@'],
        ['FLNA','KRT4'],
        ['FLNA','LYZ'],
        ['FLNA','MIF'],
        ['FLNA','NCOR2'],
        ['FLNA','NME4'],
        ['FLNA','NPPA'],
        ['FLNA','PRSS2'],
        ['FLNA','PUF60'],
        ['FLNA','RP11-309L24.2'],
        ['FLNA','RUSC2'],
        ['FLNA','SFN'],
        ['FLNA','SFTPA2'],
        ['FLNA','SFTPC'],
        ['FLNA','TUBB'],
        ['FLNA','UBC'],
        ['FLNB','SFTPB'],
        ['FLNC','FSTL1'],
        ['FLNC','PLEC'],
        ['FLOT1','HTN3'],
        ['FLOT2','NPPA-AS1'],
        ['FLRT2','TRB@'],
        ['FLT1','IGF2'],
        ['FLT1','IGFBP4'],
        ['FLT1','RP11-27M24.1'],
        ['FMN2','GREM2'],
        ['FMNL2','LYZ'],
        ['FN1','HDGF'],
        ['FN1','HLA@'],
        ['FN1','IGH@'],
        ['FN1','IGHG1'],
        ['FN1','INPPL1'],
        ['FN1','KIAA0930'],
        ['FN1','LAMB1'],
        ['FN1','LEPRE1'],
        ['FN1','LGALS1'],
        ['FN1','LMO7'],
        ['FN1','MCAM'],
        ['FN1','MED15'],
        ['FN1','MEF2D'],
        ['FN1','MYADM'],
        ['FN1','NLGN2'],
        ['FN1','NOL6'],
        ['FN1','P4HB'],
        ['FN1','PCSK7'],
        ['FN1','PECAM1'],
        ['FN1','PPP1R18'],
        ['FN1','PRSS2'],
        ['FN1','PTPN23'],
        ['FN1','PTRF'],
        ['FN1','RCN3'],
        ['FN1','RUSC2'],
        ['FN1','SEPT9'],
        ['FN1','SERPINE1'],
        ['FN1','SERPINE2'],
        ['FN1','SETD3'],
        ['FN1','SFTPA2'],
        ['FN1','SH3BGRL3'],
        ['FN1','SIPA1L3'],
        ['FN1','SLC35E1'],
        ['FN1','SLC4A2'],
        ['FN1','SLC7A5'],
        ['FN1','SLIT2'],
        ['FN1','SMARCA4'],
        ['FN1','SND1'],
        ['FN1','SNX32'],
        ['FN1','STEAP3'],
        ['FN1','TAGLN'],
        ['FN1','THBS1'],
        ['FN1','TIMP1'],
        ['FN1','TUBB'],
        ['FN1','VAMP2'],
        ['FN1','XXBAC-BPG252P9.9'],
        ['FN1','ZBTB4'],
        ['FN1','ZNF282'],
        ['FN1','ZYX'],
        ['FNBP1','LYZ'],
        ['FNBP4','Y_RNA'],
        ['FNDC3A','IGF2'],
        ['FNDC3A','RP11-203I16.7'],
        ['FOS','KLF6'],
        ['FOS','KRT13'],
        ['FOS','MMP8'],
        ['FOS','PTRF'],
        ['FOS','SFTPA1'],
        ['FOSB','RGCC'],
        ['FOSL2','SMR3B'],
        ['FOXC1','PRH1'],
        ['FOXC1','PRH1-PRR4'],
        ['FOXC1','PRR4'],
        ['FOXC2','IGFBP7'],
        ['FOXC2','PLEC'],
        ['FOXJ1','RNF157'],
        ['FOXJ3','TRB@'],
        ['FOXK2','IGFBP3'],
        ['FOXO3','LYZ'],
        ['FOXP1','HBB'],
        ['FOXP1','PRSS1'],
        ['FOXP1','TG'],
        ['FPGS','LYZ'],
        ['FRG1HP','RP11-88I18.2'],
        ['FRG1JP','RP11-87H9.4'],
        ['FRMD6-AS2','RP11-255G12.3'],
        ['FRRS1L','RP11-446E24.4'],
        ['FRS2','ZNF529-AS1'],
        ['FRS3','RBBP4'],
        ['FSCN1','PFKL'],
        ['FSCN1','PHLDA2'],
        ['FSTL1','TG'],
        ['FTH1','GNB1'],
        ['FTH1','GP2'],
        ['FTH1','HBB'],
        ['FTH1','HNRNPA0'],
        ['FTH1','HNRNPL'],
        ['FTH1','LYZ'],
        ['FTH1','MTCH1'],
        ['FTH1','PDZD2'],
        ['FTH1','PRELID1'],
        ['FTL','HDAC5'],
        ['FTL','S100A9'],
        ['FTL','SFTPA2'],
        ['FTL','TIMP1'],
        ['FTL','TPO'],
        ['FTL','TPT1'],
        ['FTO','FTO-IT1'],
        ['FTO','RP11-1143G9.4'],
        ['FTX','KRT10'],
        ['FTX','PAQR3'],
        ['FURIN','SMR3B'],
        ['FURIN','TG'],
        ['FUS','LGMN'],
        ['FUS','NCL'],
        ['FUT2','SEC1P'],
        ['FUT8','RP11-762H8.4'],
        ['FXN','TJP2'],
        ['FXYD2','SMR3B'],
        ['FZD4','RP11-736K20.5'],
        ['G0S2','MPO'],
        ['GAA','SMR3B'],
        ['GAB1','SMARCA5'],
        ['GABARAP','NPPA-AS1'],
        ['GABARAPL1','NPPA-AS1'],
        ['GABARAPL2','NPPA-AS1'],
        ['GABPB1-AS1','GBP6'],
        ['GABPB2','MLLT11'],
        ['GABRR2','UBE2J1'],
        ['GAK','ZG16B'],
        ['GAL3ST1','PES1'],
        ['GALNT10','SAMD1'],
        ['GALNT2','LAMP1'],
        ['GAMT','PPP2R5B'],
        ['GANAB','LYZ'],
        ['GAPDH','GAPDHP1'],
        ['GAPDH','GAPDHP14'],
        ['GAPDH','GAPDHP15'],
        ['GAPDH','GAPDHP16'],
        ['GAPDH','GAPDHP17'],
        ['GAPDH','GAPDHP19'],
        ['GAPDH','GAPDHP2'],
        ['GAPDH','GAPDHP20'],
        ['GAPDH','GAPDHP21'],
        ['GAPDH','GAPDHP22'],
        ['GAPDH','GAPDHP23'],
        ['GAPDH','GAPDHP24'],
        ['GAPDH','GAPDHP25'],
        ['GAPDH','GAPDHP26'],
        ['GAPDH','GAPDHP27'],
        ['GAPDH','GAPDHP28'],
        ['GAPDH','GAPDHP29'],
        ['GAPDH','GAPDHP31'],
        ['GAPDH','GAPDHP32'],
        ['GAPDH','GAPDHP33'],
        ['GAPDH','GAPDHP34'],
        ['GAPDH','GAPDHP36'],
        ['GAPDH','GAPDHP37'],
        ['GAPDH','GAPDHP38'],
        ['GAPDH','GAPDHP39'],
        ['GAPDH','GAPDHP40'],
        ['GAPDH','GAPDHP41'],
        ['GAPDH','GAPDHP42'],
        ['GAPDH','GAPDHP44'],
        ['GAPDH','GAPDHP45'],
        ['GAPDH','GAPDHP46'],
        ['GAPDH','GAPDHP47'],
        ['GAPDH','GAPDHP48'],
        ['GAPDH','GAPDHP49'],
        ['GAPDH','GAPDHP50'],
        ['GAPDH','GAPDHP51'],
        ['GAPDH','GAPDHP52'],
        ['GAPDH','GAPDHP53'],
        ['GAPDH','GAPDHP54'],
        ['GAPDH','GAPDHP55'],
        ['GAPDH','GAPDHP56'],
        ['GAPDH','GAPDHP57'],
        ['GAPDH','GAPDHP58'],
        ['GAPDH','GAPDHP59'],
        ['GAPDH','GAPDHP60'],
        ['GAPDH','GAPDHP61'],
        ['GAPDH','GAPDHP62'],
        ['GAPDH','GAPDHP63'],
        ['GAPDH','GAPDHP64'],
        ['GAPDH','GAPDHP65'],
        ['GAPDH','GAPDHP66'],
        ['GAPDH','GAPDHP67'],
        ['GAPDH','GAPDHP68'],
        ['GAPDH','GAPDHP69'],
        ['GAPDH','GAPDHP70'],
        ['GAPDH','GAPDHP71'],
        ['GAPDH','GAPDHP72'],
        ['GAPDH','GAPDHP73'],
        ['GAPDH','GAPDHP74'],
        ['GAPDH','GAPDHP75'],
        ['GAPDH','GAPDHP76'],
        ['GAPDH','GAPDHS'],
        ['GAPDH','HBB'],
        ['GAPDH','KRT13'],
        ['GAPDH','MCAM'],
        ['GAPDH','PCSK7'],
        ['GAPDH','PGC'],
        ['GAPDH','PLEC'],
        ['GAPDH','RMDN3'],
        ['GAPDH','SNX22'],
        ['GAPDH','TIMP3'],
        ['GAPDHP38','GAPDHP70'],
        ['GAPLINC','TGIF1'],
        ['GAREM','IGH@'],
        ['GAS6','PITPNA'],
        ['GAS6','TG'],
        ['GAST','LIPF'],
        ['GAST','PGC'],
        ['GATA2-AS1','TMED10P2'],
        ['GATAD2A','LYZ'],
        ['GATM','GP2'],
        ['GATSL2','GTF2I'],
        ['GBGT1','RALGDS'],
        ['GBP6','LRP11'],
        ['GBP6','LRP1B'],
        ['GBP6','MYH11'],
        ['GBP6','NRG1'],
        ['GBP6','PTPN12'],
        ['GBP6','RYR2'],
        ['GBP6','SLTM'],
        ['GC','SERPINA1'],
        ['GCA','LYZ'],
        ['GCLC','YES1'],
        ['GCNT4','LINC01336'],
        ['GCSAM','SLC9C1'],
        ['GCSH','NDUFS1'],
        ['GDPGP1','RP11-697E2.6'],
        ['GDPGP1','TTLL13P'],
        ['GFAP','HK1'],
        ['GFAP','IL3RA'],
        ['GFAP','KRT1'],
        ['GFAP','KRT13'],
        ['GFAP','KRT14'],
        ['GFAP','KRT16'],
        ['GFAP','KRT4'],
        ['GFAP','KRT5'],
        ['GFAP','MBP'],
        ['GFAP','MEG3'],
        ['GFAP','PHYHIP'],
        ['GFAP','PLP1'],
        ['GFAP','RP11-862L9.3'],
        ['GFAP','SERINC1'],
        ['GFAP','SPARCL1'],
        ['GFM1','MFSD1'],
        ['GFPT2','SIRT3'],
        ['GGA2','ZFAND5'],
        ['GGACT','TMTC4'],
        ['GGNBP2','LYZ'],
        ['GGT6','IGH@'],
        ['GIMAP8','LINC00996'],
        ['GINM1','LYZ'],
        ['GIPC1','RAB32'],
        ['GIPC2','MYADM'],
        ['GJA9-MYCBP','RHBDL2'],
        ['GK5','LYZ'],
        ['GK5','XRN1'],
        ['GKAP1','KIF27'],
        ['GKN1','IGH@'],
        ['GKN1','IGHA1'],
        ['GKN1','IGK@'],
        ['GKN1','IGKC'],
        ['GKN1','LYZ'],
        ['GKN2','LYZ'],
        ['GKN2','PGC'],
        ['GKN2','TFF2'],
        ['GLIS2','IGF2'],
        ['GLRX','TG'],
        ['GLT8D1','TG'],
        ['GLTP','IDE'],
        ['GLTP','LYZ'],
        ['GLTP','MAP2K5'],
        ['GLTP','MRGPRX4'],
        ['GLTP','POLE2'],
        ['GLTP','TRAPPC9'],
        ['GLTSCR2','PRSS1'],
        ['GLTSCR2','TRB@'],
        ['GLUL','IGFBP5'],
        ['GLUL','S100A9'],
        ['GLUL','SFTPB'],
        ['GLYR1','TG'],
        ['GMCL1','SNRNP27'],
        ['GMPPB','LRG1'],
        ['GMPPB','TG'],
        ['GMPR','SPTLC2'],
        ['GNAI2','HBB'],
        ['GNAI2','SFTPB'],
        ['GNAO1','MBP'],
        ['GNAS','GSPT1'],
        ['GNAS','HNRNPA0'],
        ['GNAS','ID4'],
        ['GNAS','KRT13'],
        ['GNAS','MAPK1'],
        ['GNAS','MAZ'],
        ['GNAS','MMP2'],
        ['GNAS','MUC7'],
        ['GNAS','MYH11'],
        ['GNAS','PRB3'],
        ['GNAS','PRH1'],
        ['GNAS','PRH1-PRR4'],
        ['GNAS','PRH2'],
        ['GNAS','PRR4'],
        ['GNAS','RAB21'],
        ['GNAS','SLA'],
        ['GNAS','TMBIM6'],
        ['GNAS','TRB@'],
        ['GNB1','MYH11'],
        ['GNB2L1','LYZ'],
        ['GNE','LYZ'],
        ['GNE','PRH1'],
        ['GNG10','UGCG'],
        ['GNG12','SMR3B'],
        ['GNG12-AS1','PPP4R3A'],
        ['GNG7','RAB21'],
        ['GNMT','RP3-475N16.1'],
        ['GNS','PTRF'],
        ['GOLGA2P8','ZNF774'],
        ['GOLGA3','SFTPC'],
        ['GOLGA4','TG'],
        ['GOLGA4','TJP1'],
        ['GOLGA6L2','GOLGA6L7P'],
        ['GOLIM4','MYH9'],
        ['GOLT1A','KISS1'],
        ['GORASP2','PRH1'],
        ['GORASP2','PRH1-PRR4'],
        ['GORASP2','PRR4'],
        ['GOT2','GOT2P2'],
        ['GOT2','GOT2P3'],
        ['GOT2','TG'],
        ['GP2','GPT2'],
        ['GP2','HNRNPH1'],
        ['GP2','LMNA'],
        ['GP2','NPHP3-ACAD11'],
        ['GP2','NTN4'],
        ['GP2','NUPR1'],
        ['GP2','P4HB'],
        ['GP2','PLA2G1B'],
        ['GP2','PNLIP'],
        ['GP2','PNLIPRP1'],
        ['GP2','PNLIPRP2'],
        ['GP2','PRSS1'],
        ['GP2','PRSS2'],
        ['GP2','PRSS3'],
        ['GP2','RP11-680B3.2'],
        ['GP2','SEL1L'],
        ['GP2','SETD2'],
        ['GP2','SLC4A4'],
        ['GP2','TEX11'],
        ['GP2','TRB@'],
        ['GP2','UBE2R2-AS1'],
        ['GPC1','MZT2B'],
        ['GPC5','GPC5-IT1'],
        ['GPD1','GSN'],
        ['GPD1','MCAM'],
        ['GPI','LRG1'],
        ['GPM6B','PIGR'],
        ['GPR107','TG'],
        ['GPR141','NME8'],
        ['GPR65','LINC01146'],
        ['GPRC5A','SFTPB'],
        ['GPRIN1','SNCB'],
        ['GPX3','HNF4A'],
        ['GPX3','RP11-807H22.7'],
        ['GRAP2','TMA7'],
        ['GRB2','LYZ'],
        ['GRB2','RNF130'],
        ['GRB2','RP11-176H8.1'],
        ['GRB2','SCUBE3'],
        ['GRB2','SFTPB'],
        ['GREB1','MARK4'],
        ['GREB1','PCMT1'],
        ['GREB1','RP11-397A16.1'],
        ['GREM1','QKI'],
        ['GREM2','RP11-1143G9.4'],
        ['GRHL1','RP11-95D17.1'],
        ['GRID1-AS1','RP11-93H12.4'],
        ['GRIN2D','KDELR1'],
        ['GRIP1','RP11-123O10.3'],
        ['GRK6','PRR7'],
        ['GRN','LYZ'],
        ['GRWD1','PTRF'],
        ['GS1-124K5.11','GS1-124K5.12'],
        ['GS1-124K5.12','KCTD7'],
        ['GS1-124K5.12','RABGEF1'],
        ['GS1-124K5.2','RABGEF1'],
        ['GS1-124K5.2','SKP1'],
        ['GS1-259H13.11','LYZ'],
        ['GS1-259H13.2','ZNF655'],
        ['GS1-259H13.2','ZSCAN25'],
        ['GS1-304P7.3','RP11-295K2.3'],
        ['GSDMB','ORMDL3'],
        ['GSE1','TG'],
        ['GSN','ITGB1'],
        ['GSN','KRT13'],
        ['GSN','LYZ'],
        ['GSN','NPPA'],
        ['GSN','PLIN4'],
        ['GSN','PRSS1'],
        ['GSN','PRSS2'],
        ['GSN','PTRF'],
        ['GSN','SPTBN1'],
        ['GSN','TIMP3'],
        ['GSN','TRBC2'],
        ['GSN','TRBV25-1'],
        ['GSN','ZNF106'],
        ['GSTA1','GSTA7P'],
        ['GSTM3','NFIC'],
        ['GSTP1','PIGR'],
        ['GTF2F2','GTF2F2P1'],
        ['GTF2I','HBB'],
        ['GTF2I','HTN1'],
        ['GTF2I','HTN3'],
        ['GTF2I','PNLIP'],
        ['GTF2I','PRSS1'],
        ['GTF2I','STATH'],
        ['GTF2IP7','LINC00174'],
        ['GTPBP1','LAMC1'],
        ['GTPBP1','TG'],
        ['GUCA1B','KRT13'],
        ['GUCA1B','KRT14'],
        ['GUCA1B','KRT16'],
        ['GUCD1','LDLR'],
        ['GUCD1','LRG1'],
        ['GUCD1','RFC2'],
        ['GUCY2C','PLBD1'],
        ['GUSB','RP5-1132H15.3'],
        ['GUSBP4','LINC00680'],
        ['GYG2','XG'],
        ['GYPA','GYPB'],
        ['GYPA','GYPE'],
        ['GYPB','GYPE'],
        ['GYS1','MYL2'],
        ['H1F0','HBB'],
        ['H1F0','LYZ'],
        ['H1F0','S100A9'],
        ['H1FX','RABL6'],
        ['H1FX-AS1','HBB'],
        ['H1FX-AS1','S100A8'],
        ['H2AFJ','HBA1'],
        ['H2AFJ','HIST1H3J'],
        ['H2AFV','HMGCL'],
        ['H2AFV','RNF128'],
        ['H3F3A','LYZ'],
        ['H3F3B','HBB'],
        ['H3F3B','LYZ'],
        ['H3F3B','MPO'],
        ['H6PD','LRG1'],
        ['H6PD','SMR3B'],
        ['H6PD','SPSB1'],
        ['HACD3','IGF2'],
        ['HADH','SGMS2'],
        ['HADHA','HBB'],
        ['HAIR','KRT13'],
        ['HAIR','KRT14'],
        ['HAND2-AS1','NPPA-AS1'],
        ['HAO2','RP5-834N19.1'],
        ['HAPLN3','MFGE8'],
        ['HARS','RP11-259G18.3'],
        ['HARS2','ZMAT2'],
        ['HAS2-AS1','RP11-3G20.2'],
        ['HAUS4','PRMT5'],
        ['HBA2','HERPUD1'],
        ['HBA2','MPO'],
        ['HBA2','NFE2L1'],
        ['HBA2','SLC4A1'],
        ['HBB','HIPK3'],
        ['HBB','HLA-A'],
        ['HBB','HLA-C'],
        ['HBB','HLA-DRA'],
        ['HBB','HLA@'],
        ['HBB','HNRNPAB'],
        ['HBB','HOOK2'],
        ['HBB','ICE1'],
        ['HBB','IER2'],
        ['HBB','IGH@'],
        ['HBB','IGK@'],
        ['HBB','IGKC'],
        ['HBB','KDM3A'],
        ['HBB','KLF6'],
        ['HBB','LAPTM5'],
        ['HBB','LMNA'],
        ['HBB','LTF'],
        ['HBB','LYST'],
        ['HBB','MBNL3'],
        ['HBB','MCL1'],
        ['HBB','MPO'],
        ['HBB','MTMR14'],
        ['HBB','NCOA6'],
        ['HBB','NOP56'],
        ['HBB','NSD1'],
        ['HBB','NUDT4'],
        ['HBB','PDE4B'],
        ['HBB','PDIA3'],
        ['HBB','PINK1'],
        ['HBB','PKM'],
        ['HBB','PLBD1'],
        ['HBB','PNP'],
        ['HBB','POLDIP2'],
        ['HBB','PRR13'],
        ['HBB','PRTN3'],
        ['HBB','RP11-1334A24.6'],
        ['HBB','RP11-138I1.4'],
        ['HBB','RP11-296A16.1'],
        ['HBB','RP11-404F10.2'],
        ['HBB','RP11-571M6.7'],
        ['HBB','RP11-762H8.4'],
        ['HBB','RP11-968A15.8'],
        ['HBB','RP4-781K5.2'],
        ['HBB','RP5-877J2.1'],
        ['HBB','RP5-940J5.9'],
        ['HBB','RP5-972B16.2'],
        ['HBB','RPL3'],
        ['HBB','RPL37A'],
        ['HBB','RPL8'],
        ['HBB','RPLP1'],
        ['HBB','RPLP2'],
        ['HBB','RPS14'],
        ['HBB','RPS7'],
        ['HBB','RPS8'],
        ['HBB','RRP12'],
        ['HBB','RUNX3'],
        ['HBB','S100A9'],
        ['HBB','SESN3'],
        ['HBB','SFTPC'],
        ['HBB','SLC6A9'],
        ['HBB','SNCA'],
        ['HBB','SNX32'],
        ['HBB','SPPL2B'],
        ['HBB','SPTA1'],
        ['HBB','STAT6'],
        ['HBB','STK35'],
        ['HBB','SULF2'],
        ['HBB','SYNE1'],
        ['HBB','TET3'],
        ['HBB','TMSB4X'],
        ['HBB','TNFAIP3'],
        ['HBB','TPM3'],
        ['HBB','TRAK2'],
        ['HBB','TXNIP'],
        ['HBB','UBE2Z'],
        ['HBB','UNC13D'],
        ['HBB','VIM-AS1'],
        ['HBB','VPS13A'],
        ['HBB','VPS28'],
        ['HBB','XPO6'],
        ['HBB','ZBTB1'],
        ['HBB','ZFAND5'],
        ['HBB','ZWINT'],
        ['HBG2','IGF2'],
        ['HBG2','TRIM5'],
        ['HCG21','IGH@'],
        ['HCG21','IGHG1'],
        ['HCG21','SFTA2'],
        ['HCG23','XXBAC-BPG154L12.5'],
        ['HCG4','XXBAC-BPG248L24.12'],
        ['HCG4P5','HLA@'],
        ['HDAC1','RNF123'],
        ['HDAC2','LYZ'],
        ['HDAC2','NPPA-AS1'],
        ['HDGFRP2','TAOK2'],
        ['HDLBP','IGF2'],
        ['HDLBP','IGFBP5'],
        ['HDLBP','LYZ'],
        ['HDLBP','MYH11'],
        ['HDLBP','PGC'],
        ['HDLBP','PIGR'],
        ['HDLBP','PNLIP'],
        ['HEATR1','PRSS1'],
        ['HEATR1','PRSS2'],
        ['HEATR1','TRBC2'],
        ['HECA','SLC48A1'],
        ['HECTD1','SMR3B'],
        ['HEG1','MAZ'],
        ['HEIH','ZFP62'],
        ['HELLS','MASP2'],
        ['HELZ2','SFTPA1'],
        ['HELZ2','SFTPC'],
        ['HEMGN','RP11-535C21.3'],
        ['HEPHL1','PANX1'],
        ['HERC1','LYZ'],
        ['HERC1','USP3-AS1'],
        ['HERC2P10','RP11-540B6.3'],
        ['HERC2P3','RP11-603B24.1'],
        ['HERPUD1','SLC12A3'],
        ['HERPUD2','LYZ'],
        ['HGF','IGF2'],
        ['HHIPL2','TAF1A'],
        ['HIAT1','SLC35A3'],
        ['HIC2','PI4KA'],
        ['HIF1A','NEK9'],
        ['HIF1A','SF3B1'],
        ['HIF1A-AS2','MYO1F'],
        ['HIF1A-AS2','SFT2D2'],
        ['HIGD1C','METTL7A'],
        ['HILPDA','RCAN3'],
        ['HIP1','IGF2'],
        ['HIPK2','TG'],
        ['HIST1H2AA','SLC17A1'],
        ['HIST1H2APS1','HIST1H2BA'],
        ['HLA-A','HLA-B'],
        ['HLA-A','HLA-C'],
        ['HLA-A','HLA-DRA'],
        ['HLA-A','HLA-E'],
        ['HLA-B','HLA-C'],
        ['HLA-B','HLA-DRA'],
        ['HLA-B','HLA-E'],
        ['HLA-B','SFTPC'],
        ['HLA-C','HLA-DRA'],
        ['HLA-C','HLA-E'],
        ['HLA-C','LTF'],
        ['HLA-C','LYZ'],
        ['HLA-DRA','TGFBI'],
        ['HLA-DRB1','HLA-DRB6'],
        ['HLA-DRB5','HLA-DRB6'],
        ['HLA-E','MPO'],
        ['HLA-E','PLIN4'],
        ['HLA-E','ZNF638'],
        ['HLA@','HLA@'],
        ['HLA@','IGH@'],
        ['HLA@','IGK@'],
        ['HLA@','INMT'],
        ['HLA@','LTF'],
        ['HLA@','MACF1'],
        ['HLA@','MPO'],
        ['HLA@','PIGR'],
        ['HLA@','RAP1A'],
        ['HLA@','SFTPA2'],
        ['HLA@','SFTPC'],
        ['HLA@','SPTBN1'],
        ['HLA@','TGFBI'],
        ['HLA@','XXBAC-BPG254F23.6'],
        ['HLA@','ZNF638'],
        ['HLX','MARC1'],
        ['HM13','RP3-324O17.4'],
        ['HM13','SMR3B'],
        ['HMGB1P20','RP1-34L19.1'],
        ['HMGB2','NCL'],
        ['HMGB2','PTMA'],
        ['HMGN2','PTMA'],
        ['HMGN3','SFTPB'],
        ['HMGXB4','TOM1'],
        ['HMHA1','TFE3'],
        ['HMOX1','SFTPC'],
        ['HMP19','METTL7A'],
        ['HNF1A-AS1','LYZ'],
        ['HNRNPA1','KRT13'],
        ['HNRNPA1P27','XKRX'],
        ['HNRNPA2B1','PTP4A1'],
        ['HNRNPA3','LYZ'],
        ['HNRNPAB','TPBG'],
        ['HNRNPC','LYZ'],
        ['HNRNPD','IGFBP4'],
        ['HNRNPD','SMR3B'],
        ['HNRNPDL','RNF128'],
        ['HNRNPH1','KRT4'],
        ['HNRNPH1','LYZ'],
        ['HNRNPK','PRB3'],
        ['HNRNPL','SPP1'],
        ['HNRNPM','LYZ'],
        ['HNRNPM','MARCH2'],
        ['HNRNPU-AS1','TG'],
        ['HNRNPUL1','PRB3'],
        ['HNRNPUL2-BSCL2','LYZ'],
        ['HOOK2','HTN3'],
        ['HOOK2','KLK2'],
        ['HOOK2','VIM'],
        ['HOXB7','RP11-357H14.17'],
        ['HOXB9','RP11-357H14.17'],
        ['HOXC8','HOXC9'],
        ['HP','ORM1'],
        ['HP','SAA1'],
        ['HP','SERPINA1'],
        ['HP','SERPINA3'],
        ['HP','TF'],
        ['HP1BP3','LYZ'],
        ['HPCAL1','LYZ'],
        ['HPN','TRB@'],
        ['HPS1','LYZ'],
        ['HPS5','SAA2'],
        ['HPX','SERPINA1'],
        ['HRG','RBP4'],
        ['HRG','SERPINA1'],
        ['HRH4','IMPACT'],
        ['HS1BP3','HS1BP3-IT1'],
        ['HS2ST1','LINC01140'],
        ['HSF5','RNF43'],
        ['HSP90AA1','HSP90AA4P'],
        ['HSP90AA1','LYZ'],
        ['HSP90AB1','SLC29A1'],
        ['HSP90B1','HSP90B3P'],
        ['HSP90B1','SYN3'],
        ['HSPA2','ZBTB1'],
        ['HSPA5','TG'],
        ['HSPA8','MALAT1'],
        ['HSPB1','TIMP3'],
        ['HSPB6','PLEKHA4'],
        ['HSPB6','PLIN1'],
        ['HSPB6','PLIN4'],
        ['HSPB7','MYL7'],
        ['HSPB7','NPPA-AS1'],
        ['HSPB8','NPPA'],
        ['HSPE1','MOB4'],
        ['HSPE1P22','ZNF512B'],
        ['HSPG2','MYH6'],
        ['HSPG2','SFTPB'],
        ['HSPG2','TG'],
        ['HSPG2','TLN1'],
        ['HTN1','IGH@'],
        ['HTN1','IGK@'],
        ['HTN1','IGKC'],
        ['HTN1','IGKV1-8'],
        ['HTN1','PRB2'],
        ['HTN1','SMR3B'],
        ['HTN1','TTC3'],
        ['HTN3','IGH@'],
        ['HTN3','IGK@'],
        ['HTN3','IGKC'],
        ['HTN3','IGKV1-8'],
        ['HTN3','IGKV3-11'],
        ['HTN3','ITPR2'],
        ['HTN3','LYZ'],
        ['HTN3','MAOA'],
        ['HTN3','MCFD2'],
        ['HTN3','MGEA5'],
        ['HTN3','MSN'],
        ['HTN3','NBAS'],
        ['HTN3','PDIA3'],
        ['HTN3','PITPNA'],
        ['HTN3','PRB3'],
        ['HTN3','PRH1'],
        ['HTN3','PRH1-PRR4'],
        ['HTN3','PRR4'],
        ['HTN3','RNF150'],
        ['HTN3','SLC12A2'],
        ['HTN3','SMR3B'],
        ['HTN3','SSBP2'],
        ['HTN3','TCN1'],
        ['HTN3','TMEM123'],
        ['HTN3','TRAM1'],
        ['HTN3','TRIM2'],
        ['HTN3','USO1'],
        ['HTN3','VPS13B'],
        ['HTRA1','LYZ'],
        ['HTRA1','PLIN4'],
        ['HTT','TPO'],
        ['HUWE1','RPS2'],
        ['HUWE1','TG'],
        ['HVCN1','METTL7A'],
        ['HVCN1','PPP1CC'],
        ['HYDIN2','LYZ'],
        ['HYKK','IREB2'],
        ['HYKK','PSMA4'],
        ['HYPK','WSB1'],
        ['ICAM1','SFTPB'],
        ['ICAM1','TNXB'],
        ['ID4','PTMS'],
        ['ID4','TMEM158'],
        ['IDI2-AS1','WDR37'],
        ['IER2','OSCAR'],
        ['IFITM2','SFTPC'],
        ['IFITM3','SFTPA2'],
        ['IFNAR2','ZNF595'],
        ['IFNG-AS1','LINC01479'],
        ['IFT122','RP11-529F4.1'],
        ['IGF2','IL6ST'],
        ['IGF2','INS'],
        ['IGF2','ISM2'],
        ['IGF2','KDM5B'],
        ['IGF2','KIAA0895L'],
        ['IGF2','KLF7'],
        ['IGF2','LAMB1'],
        ['IGF2','LGMN'],
        ['IGF2','LHFP'],
        ['IGF2','MACF1'],
        ['IGF2','MAF'],
        ['IGF2','MBNL1'],
        ['IGF2','MED15'],
        ['IGF2','MGRN1'],
        ['IGF2','MPEG1'],
        ['IGF2','MPRIP'],
        ['IGF2','MSN'],
        ['IGF2','MYH10'],
        ['IGF2','MYH11'],
        ['IGF2','MYH9'],
        ['IGF2','NID1'],
        ['IGF2','NPPA-AS1'],
        ['IGF2','OLR1'],
        ['IGF2','OS9'],
        ['IGF2','OSBPL9'],
        ['IGF2','PAFAH1B1'],
        ['IGF2','PALM2-AKAP2'],
        ['IGF2','PAPPA'],
        ['IGF2','PCBP2'],
        ['IGF2','PECAM1'],
        ['IGF2','PODN'],
        ['IGF2','PRG2'],
        ['IGF2','PTPRF'],
        ['IGF2','RAB7A'],
        ['IGF2','RAP2C-AS1'],
        ['IGF2','RBM6'],
        ['IGF2','REEP5'],
        ['IGF2','RHOA'],
        ['IGF2','RP11-123M6.2'],
        ['IGF2','RP11-295K3.1'],
        ['IGF2','RP11-537H15.3'],
        ['IGF2','RP11-69L16.4'],
        ['IGF2','RP11-728F11.4'],
        ['IGF2','RP11-779O18.3'],
        ['IGF2','RP11-872D17.8'],
        ['IGF2','RP4-613B23.1'],
        ['IGF2','RP5-972B16.2'],
        ['IGF2','SCARA3'],
        ['IGF2','SCARB1'],
        ['IGF2','SEC31A'],
        ['IGF2','SEPP1'],
        ['IGF2','SEPT9'],
        ['IGF2','SERPINE2'],
        ['IGF2','SFTPA1'],
        ['IGF2','SLC16A3'],
        ['IGF2','SNX2'],
        ['IGF2','SNX24'],
        ['IGF2','SPINT2'],
        ['IGF2','SYN3'],
        ['IGF2','SYNPO'],
        ['IGF2','TIMP2'],
        ['IGF2','TIMP3'],
        ['IGF2','TP53I11'],
        ['IGF2','TREML2'],
        ['IGF2','TRIOBP'],
        ['IGF2','TXNL4B'],
        ['IGF2','VSNL1'],
        ['IGF2','VWF'],
        ['IGF2','ZFAT'],
        ['IGF2BP3','RPS6KA2'],
        ['IGF2R','IGFBP4'],
        ['IGFBP2','LAMC1'],
        ['IGFBP2','TRIM28'],
        ['IGFBP3','MYH9'],
        ['IGFBP4','KLF2'],
        ['IGFBP4','NELFB'],
        ['IGFBP4','PLEC'],
        ['IGFBP4','SCARF2'],
        ['IGFBP4','SERPINA1'],
        ['IGFBP4','SPIRE1'],
        ['IGFBP4','SPTBN1'],
        ['IGFBP4','TRIM28'],
        ['IGFBP4','TSR3'],
        ['IGFBP5','NCOR2'],
        ['IGFBP5','NDE1'],
        ['IGFBP5','NFIX'],
        ['IGFBP5','SMR3B'],
        ['IGFBP6','PLEC'],
        ['IGFBP7','PRSS2'],
        ['IGFBP7','TRB@'],
        ['IGFBP7-AS1','NPPA'],
        ['IGFBP7-AS1','TNNT2'],
        ['IGH@','IGH@'],
        ['IGH@','IGHA1'],
        ['IGH@','IGHA2'],
        ['IGH@','IGHG1'],
        ['IGH@','IGHG3'],
        ['IGH@','IGHV2-70'],
        ['IGH@','IGHV3-21'],
        ['IGH@','IGK@'],
        ['IGH@','JCHAIN'],
        ['IGH@','KRT13'],
        ['IGH@','KRT19'],
        ['IGH@','KRT4'],
        ['IGH@','LCN2'],
        ['IGH@','LCT'],
        ['IGH@','LINC00221'],
        ['IGH@','LYZ'],
        ['IGH@','MEP1B'],
        ['IGH@','MUC13'],
        ['IGH@','MUC5AC'],
        ['IGH@','MUC5B'],
        ['IGH@','MUC6'],
        ['IGH@','MUC7'],
        ['IGH@','MYL2'],
        ['IGH@','NCL'],
        ['IGH@','NEAT1'],
        ['IGH@','PGC'],
        ['IGH@','PIGR'],
        ['IGH@','PRB1'],
        ['IGH@','PRB2'],
        ['IGH@','PRB3'],
        ['IGH@','PRH1'],
        ['IGH@','PRH1-PRR4'],
        ['IGH@','PRSS2'],
        ['IGH@','PTPRF'],
        ['IGH@','REG4'],
        ['IGH@','RP11-1166P10.1'],
        ['IGH@','RP11-1166P10.6'],
        ['IGH@','RP11-19N8.2'],
        ['IGH@','RP11-294C11.4'],
        ['IGH@','RP11-302B13.5'],
        ['IGH@','RP11-731F5.2'],
        ['IGH@','RP11-812E19.9'],
        ['IGH@','RPS8'],
        ['IGH@','SFTPB'],
        ['IGH@','SIAE'],
        ['IGH@','SMR3B'],
        ['IGH@','SULT1C2'],
        ['IGH@','TFF2'],
        ['IGH@','TG'],
        ['IGH@','TRB@'],
        ['IGH@','ZG16B'],
        ['IGHA1','MPO'],
        ['IGHA1','MUC6'],
        ['IGHA1','MUC7'],
        ['IGHA1','PGC'],
        ['IGHA1','PIGR'],
        ['IGHA1','PRB1'],
        ['IGHA1','PRB2'],
        ['IGHA1','REG3A'],
        ['IGHA1','SFTPA1'],
        ['IGHA1','SFTPC'],
        ['IGHA1','SMR3B'],
        ['IGHA1','SRRM2'],
        ['IGHA2','MUC6'],
        ['IGHA2','MUC7'],
        ['IGHA2','SMR3B'],
        ['IGHG1','IGLC1'],
        ['IGHG1','LCT'],
        ['IGHG1','LINC01578'],
        ['IGHG1','LIPF'],
        ['IGHG1','MUC7'],
        ['IGHG1','OLFM4'],
        ['IGHG1','PRH1'],
        ['IGHG1','PRH1-PRR4'],
        ['IGHG1','RP11-1143G9.4'],
        ['IGHG1','RP11-731F5.1'],
        ['IGHG3','PRH1'],
        ['IGHG3','PRH1-PRR4'],
        ['IGHG3','SFTPB'],
        ['IGHM','JCHAIN'],
        ['IGHM','MUC5AC'],
        ['IGHM','MUC7'],
        ['IGHM','PRH1'],
        ['IGHM','PRH1-PRR4'],
        ['IGHM','PTPRF'],
        ['IGHM','RP11-1143G9.4'],
        ['IGHM','RP11-731F5.1'],
        ['IGHV3-49','RP11-731F5.2'],
        ['IGK@','IGL@'],
        ['IGK@','JCHAIN'],
        ['IGK@','LAPTM5'],
        ['IGK@','LL22NC03-2H8.5'],
        ['IGK@','LYZ'],
        ['IGK@','MUC5AC'],
        ['IGK@','MUC6'],
        ['IGK@','MUC7'],
        ['IGK@','NDE1'],
        ['IGK@','PGC'],
        ['IGK@','PIGR'],
        ['IGK@','PRH1'],
        ['IGK@','PRH1-PRR4'],
        ['IGK@','PRR4'],
        ['IGK@','RP11-1143G9.4'],
        ['IGK@','SFTPB'],
        ['IGK@','SMR3B'],
        ['IGK@','STAG3'],
        ['IGK@','SYNPO2'],
        ['IGK@','TFF1'],
        ['IGKC','ILF3'],
        ['IGKC','JCHAIN'],
        ['IGKC','LAPTM5'],
        ['IGKC','LYZ'],
        ['IGKC','MUC5AC'],
        ['IGKC','MUC6'],
        ['IGKC','MUC7'],
        ['IGKC','NDE1'],
        ['IGKC','PGC'],
        ['IGKC','PIGR'],
        ['IGKC','PRH1'],
        ['IGKC','PRH1-PRR4'],
        ['IGKC','PRR27'],
        ['IGKC','PRR4'],
        ['IGKC','REG3A'],
        ['IGKC','RP11-1143G9.4'],
        ['IGKC','SEPT9'],
        ['IGKC','SMR3B'],
        ['IGKC','TFF1'],
        ['IGKC','TMBIM6'],
        ['IGKV1-12','MUC5AC'],
        ['IGKV1-8','MUC5AC'],
        ['IGKV1-8','NDRG2'],
        ['IGKV1-8','PIGR'],
        ['IGKV1-8','PRH1'],
        ['IGKV1-8','PRH1-PRR4'],
        ['IGKV1-8','PRR4'],
        ['IGKV1-8','RP11-1143G9.4'],
        ['IGKV1-8','TFF1'],
        ['IGKV3-11','LYZ'],
        ['IGKV3-11','PIGR'],
        ['IGKV3-11','PRH1'],
        ['IGKV3-11','PRH1-PRR4'],
        ['IGKV3-11','PRR4'],
        ['IGKV3-11','RP11-1143G9.4'],
        ['IGKV3-11','TFF1'],
        ['IGL@','IGL@'],
        ['IGLVIV-64','LL22NC03-23C6.13'],
        ['IGLVIV-65','LL22NC03-23C6.13'],
        ['IGLVIV-66-1','LL22NC03-23C6.13'],
        ['IGSF5','PCP4'],
        ['IKZF1','LYZ'],
        ['IL11','PTRF'],
        ['IL17RA','LYZ'],
        ['IL18','LITAF'],
        ['IL1R1','PNLIP'],
        ['IL1RN','RP11-109E10.1'],
        ['IL36A','MRPL11'],
        ['IL6R','LYZ'],
        ['IL6ST','PRH1'],
        ['ILF3','LYZ'],
        ['IMMP2L','PIGR'],
        ['IMP3','PTRF'],
        ['IMPA1','SLC10A5'],
        ['IMPDH1','IMPDH1P4'],
        ['INA','KRT13'],
        ['INA','KRT14'],
        ['INA','KRT16'],
        ['INA','KRT4'],
        ['INA','KRT5'],
        ['INA','KRT6A'],
        ['INE1','UBA1'],
        ['INHBE','LRG1'],
        ['INPP5B','LYZ'],
        ['INPP5B','SFTPB'],
        ['INPP5D','LYZ'],
        ['INPP5K','MYO1C'],
        ['INPPL1','UBA1'],
        ['INS','TRB@'],
        ['INSL3','JAK3'],
        ['INTS1','TG'],
        ['INTS2','MED13'],
        ['INTS6-AS1','WDFY2'],
        ['INTS7','LYZ'],
        ['IQCE','RP11-1143G9.4'],
        ['IQGAP1','LYZ'],
        ['IQGAP1','NSMCE1'],
        ['IQSEC1','RP11-767C1.1'],
        ['IQSEC1','SFTPC'],
        ['IQUB','NDUFA5'],
        ['IRF1','LYZ'],
        ['IRS4','RP6-24A23.7'],
        ['ISPD','SOSTDC1'],
        ['IST1','LYZ'],
        ['IST1','TG'],
        ['ITGA2B','LYZ'],
        ['ITGA3','SFTPB'],
        ['ITGA5','MAZ'],
        ['ITGA5','MYH9'],
        ['ITGA5','PTOV1'],
        ['ITGA5','VPS37B'],
        ['ITGA5','ZFP36L2'],
        ['ITGA7','NPPA-AS1'],
        ['ITGAD','RPL37A'],
        ['ITGAL','LYZ'],
        ['ITGAM','LYZ'],
        ['ITGAX','SFTPB'],
        ['ITGB1','PLEC'],
        ['ITGB1','SMR3B'],
        ['ITGB1','TG'],
        ['ITGB2','SUMO3'],
        ['ITGB4','LYZ'],
        ['ITM2B','LYZ'],
        ['ITM2B','S100A9'],
        ['ITPKB','SFTPA2'],
        ['ITPKB','SMR3B'],
        ['ITPR1','TG'],
        ['ITPR2','STATH'],
        ['IVD','TG'],
        ['IWS1','NCL'],
        ['IWS1','SDF4'],
        ['IYD','LAMC1'],
        ['IYD','METTL7A'],
        ['IYD','RP11-950C14.7'],
        ['IYD','SLA'],
        ['IYD','TPO'],
        ['JAG2','NUDT14'],
        ['JAM3','RP11-713P17.3'],
        ['JCHAIN','LYZ'],
        ['JCHAIN','PGC'],
        ['JCHAIN','PIGR'],
        ['JMJD6','SRSF2'],
        ['JUND','KIAA1683'],
        ['JUND','NPAS1'],
        ['JUND','SERPINE1'],
        ['JUND','SFTPA1'],
        ['JUND','SFTPC'],
        ['JUP','KRT13'],
        ['JUP','KRT14'],
        ['JUP','KRT16'],
        ['JUP','KRT5'],
        ['JUP','LCE1C'],
        ['JUP','MYH11'],
        ['JUP','SFTPB'],
        ['JUP','TMEM99'],
        ['K14','KRT13'],
        ['K14','KRT14'],
        ['K14','KRT16'],
        ['KALRN','STOM'],
        ['KANSL2','LYZ'],
        ['KANSL3','SPN'],
        ['KARS','SMR3B'],
        ['KAT8','MYH9'],
        ['KAT8','SNX24'],
        ['KATNAL2','PAX8-AS1'],
        ['KATNAL2','RP11-65I12.1'],
        ['KB-1980E6.3','ODF1'],
        ['KCNA3','RP11-284N8.3'],
        ['KCNE3','ZNF346'],
        ['KCNH8','PRKDC'],
        ['KCNIP3','PROM2'],
        ['KCNIP4','RP4-758J18.10'],
        ['KCNIP4','UBE2V2'],
        ['KCNJ15','TG'],
        ['KCNMA1','KRT13'],
        ['KCNMA1','NDE1'],
        ['KCNMA1','SMR3B'],
        ['KCNMB3','LRRFIP1'],
        ['KCNN4','LYPD5'],
        ['KCNN4','MUC7'],
        ['KCNQ1','TG'],
        ['KCP','SENP1'],
        ['KCTD1','PCAT18'],
        ['KCTD14','LARS'],
        ['KCTD14','SMR3B'],
        ['KCTD20','LYZ'],
        ['KCTD7','LYZ'],
        ['KDELR2','TRB@'],
        ['KDM2A','LYZ'],
        ['KDM2B','STOM'],
        ['KDM4C','TMEM50B'],
        ['KDM4E','RP11-60C6.3'],
        ['KDM5A','PLD5'],
        ['KDM5A','SLC6A13'],
        ['KDM5C','KDM5C-IT1'],
        ['KDSR','STAG1'],
        ['KIAA0020','SSR3'],
        ['KIAA0319L','LYZ'],
        ['KIAA0355','TRB@'],
        ['KIAA0430','PAX8-AS1'],
        ['KIAA0586','TOMM20L'],
        ['KIAA0753','MED31'],
        ['KIAA0753','PITPNM3'],
        ['KIAA0754','MACF1'],
        ['KIAA1109','LYZ'],
        ['KIAA1191','PRKCSH'],
        ['KIAA1217','PRB3'],
        ['KIAA1217','TG'],
        ['KIAA1324','PAGE4'],
        ['KIAA1324','PRH1'],
        ['KIAA1324','PRH1-PRR4'],
        ['KIAA1324','PRR4'],
        ['KIAA1324','PRSS1'],
        ['KIAA1324','TRB@'],
        ['KIAA1462','SFTPB'],
        ['KIAA1522','SMR3B'],
        ['KIAA1671','TG'],
        ['KIAA1731','SCARNA9'],
        ['KIAA1841','PEX13'],
        ['KIDINS220','PTH'],
        ['KIDINS220','TG'],
        ['KIF1C','SFTPB'],
        ['KIF22','MCM3AP'],
        ['KIF26B','PIGR'],
        ['KIF5A','PCNT'],
        ['KIFAP3','TG'],
        ['KIFC3','TG'],
        ['KIT','TG'],
        ['KLC1','SMR3B'],
        ['KLF16','LLFOS-48D6.2'],
        ['KLF16','OAZ1'],
        ['KLF16','REXO1'],
        ['KLF2','LOXL1'],
        ['KLF2','MAZ'],
        ['KLF3','PNLIP'],
        ['KLF6','NAMPT'],
        ['KLF6','TG'],
        ['KLF8','RP13-188A5.1'],
        ['KLHL23','SSB'],
        ['KLHL3','RP11-381K20.2'],
        ['KLHL33','TEP1'],
        ['KLK1','SMR3B'],
        ['KLK1','TRB@'],
        ['KLK2','NDE1'],
        ['KLK2','STAMBPL1'],
        ['KLK3','LLFOS-48D6.2'],
        ['KLK3','MALAT1'],
        ['KLK3','MYLK'],
        ['KLK3','NR4A1'],
        ['KLK4','KLKP1'],
        ['KLRAP1','MAGOHB'],
        ['KMT2D','S100A8'],
        ['KNOP1P1','TRIM69'],
        ['KPNA1','PARP9'],
        ['KRBOX1','ZNF662'],
        ['KRBOX4','ZNF674'],
        ['KREMEN1','LYZ'],
        ['KRIT1','LRRD1'],
        ['KRR1','RP11-585P4.5'],
        ['KRT1','KRT10'],
        ['KRT1','KRT12'],
        ['KRT1','KRT13'],
        ['KRT1','KRT14'],
        ['KRT1','KRT15'],
        ['KRT1','KRT16'],
        ['KRT1','KRT17'],
        ['KRT1','KRT18'],
        ['KRT1','KRT19'],
        ['KRT1','KRT1B'],
        ['KRT1','KRT2'],
        ['KRT1','KRT24'],
        ['KRT1','KRT3'],
        ['KRT1','KRT4'],
        ['KRT1','KRT5'],
        ['KRT1','KRT6A'],
        ['KRT1','KRT6B'],
        ['KRT1','KRT6C'],
        ['KRT1','KRT7'],
        ['KRT1','KRT71'],
        ['KRT1','KRT72'],
        ['KRT1','KRT73'],
        ['KRT1','KRT73-AS1'],
        ['KRT1','KRT74'],
        ['KRT1','KRT75'],
        ['KRT1','KRT76'],
        ['KRT1','KRT77'],
        ['KRT1','KRT78'],
        ['KRT1','KRT79'],
        ['KRT1','KRT8'],
        ['KRT1','KRT80'],
        ['KRT1','KRT81'],
        ['KRT1','KRT82'],
        ['KRT1','KRT83'],
        ['KRT1','KRT84'],
        ['KRT1','KRT85'],
        ['KRT1','KRT86'],
        ['KRT1','KRT9'],
        ['KRT1','KRTHB1'],
        ['KRT1','KRTHB2'],
        ['KRT1','KRTHB6'],
        ['KRT1','LYPD3'],
        ['KRT1','MACF1'],
        ['KRT1','MYH9'],
        ['KRT1','NEFL'],
        ['KRT1','PCGF3'],
        ['KRT1','PERP'],
        ['KRT1','PITPNM3'],
        ['KRT1','POLR2A'],
        ['KRT1','PRPH'],
        ['KRT1','PTPRF'],
        ['KRT1','SBSN'],
        ['KRT1','SRGAP2'],
        ['KRT1','TM6SF1'],
        ['KRT1','TMEM99'],
        ['KRT1','TTBK2'],
        ['KRT1','VIM'],
        ['KRT1','ZFP36L1'],
        ['KRT10','KRT13'],
        ['KRT10','KRT14'],
        ['KRT10','KRT16'],
        ['KRT10','KRT4'],
        ['KRT10','KRT5'],
        ['KRT10','KRT6A'],
        ['KRT10','PSORS1C1'],
        ['KRT10','PVRL4'],
        ['KRT12','KRT13'],
        ['KRT12','KRT14'],
        ['KRT12','KRT16'],
        ['KRT12','KRT4'],
        ['KRT121P','KRT13'],
        ['KRT121P','KRT4'],
        ['KRT121P','KRT5'],
        ['KRT121P','KRT6A'],
        ['KRT128P','KRT73'],
        ['KRT13','KRT14'],
        ['KRT13','KRT15'],
        ['KRT13','KRT16'],
        ['KRT13','KRT17'],
        ['KRT13','KRT18'],
        ['KRT13','KRT19'],
        ['KRT13','KRT2'],
        ['KRT13','KRT20'],
        ['KRT13','KRT23'],
        ['KRT13','KRT24'],
        ['KRT13','KRT25'],
        ['KRT13','KRT26'],
        ['KRT13','KRT27'],
        ['KRT13','KRT28'],
        ['KRT13','KRT3'],
        ['KRT13','KRT31'],
        ['KRT13','KRT32'],
        ['KRT13','KRT33A'],
        ['KRT13','KRT33B'],
        ['KRT13','KRT34'],
        ['KRT13','KRT35'],
        ['KRT13','KRT36'],
        ['KRT13','KRT37'],
        ['KRT13','KRT38'],
        ['KRT13','KRT39'],
        ['KRT13','KRT4'],
        ['KRT13','KRT40'],
        ['KRT13','KRT5'],
        ['KRT13','KRT6B'],
        ['KRT13','KRT6C'],
        ['KRT13','KRT7'],
        ['KRT13','KRT73'],
        ['KRT13','KRT75'],
        ['KRT13','KRT76'],
        ['KRT13','KRT77'],
        ['KRT13','KRT78'],
        ['KRT13','KRT79'],
        ['KRT13','KRT8'],
        ['KRT13','KRT83'],
        ['KRT13','KRT85'],
        ['KRT13','KRT9'],
        ['KRT13','KRTHA7'],
        ['KRT13','KRTHB1'],
        ['KRT13','MCAM'],
        ['KRT13','MGLL'],
        ['KRT13','MUC4'],
        ['KRT13','MYH11'],
        ['KRT13','MYLK'],
        ['KRT13','NDE1'],
        ['KRT13','NEFL'],
        ['KRT13','NEFM'],
        ['KRT13','PALLD'],
        ['KRT13','PARD3'],
        ['KRT13','PARP9'],
        ['KRT13','PKP1'],
        ['KRT13','PPP1CB'],
        ['KRT13','PRMT2'],
        ['KRT13','PRPH'],
        ['KRT13','PTPRF'],
        ['KRT13','RHCG'],
        ['KRT13','RP11-290L1.3'],
        ['KRT13','RP11-779O18.3'],
        ['KRT13','RP11-968A15.8'],
        ['KRT13','S100A14'],
        ['KRT13','S100A9'],
        ['KRT13','SF3B2'],
        ['KRT13','SFN'],
        ['KRT13','SH3PXD2A'],
        ['KRT13','SNX32'],
        ['KRT13','SORBS1'],
        ['KRT13','SORT1'],
        ['KRT13','SPARCL1'],
        ['KRT13','SPRR1B'],
        ['KRT13','SREBF2'],
        ['KRT13','SRPK2'],
        ['KRT13','TACSTD2'],
        ['KRT13','TAGLN'],
        ['KRT13','TBCD'],
        ['KRT13','TGFB1'],
        ['KRT13','TIMP2'],
        ['KRT13','TMBIM1'],
        ['KRT13','TNS1'],
        ['KRT13','TPM2'],
        ['KRT13','TRIM29'],
        ['KRT13','TYPE'],
        ['KRT13','VIM'],
        ['KRT13','YWHAE'],
        ['KRT13','ZBTB4'],
        ['KRT14','KRT15'],
        ['KRT14','KRT16'],
        ['KRT14','KRT17'],
        ['KRT14','KRT18'],
        ['KRT14','KRT19'],
        ['KRT14','KRT2'],
        ['KRT14','KRT20'],
        ['KRT14','KRT23'],
        ['KRT14','KRT24'],
        ['KRT14','KRT25'],
        ['KRT14','KRT26'],
        ['KRT14','KRT27'],
        ['KRT14','KRT28'],
        ['KRT14','KRT31'],
        ['KRT14','KRT32'],
        ['KRT14','KRT33A'],
        ['KRT14','KRT33B'],
        ['KRT14','KRT34'],
        ['KRT14','KRT35'],
        ['KRT14','KRT36'],
        ['KRT14','KRT37'],
        ['KRT14','KRT38'],
        ['KRT14','KRT39'],
        ['KRT14','KRT4'],
        ['KRT14','KRT40'],
        ['KRT14','KRT5'],
        ['KRT14','KRT7'],
        ['KRT14','KRT77'],
        ['KRT14','KRT8'],
        ['KRT14','KRT9'],
        ['KRT14','KRTHA7'],
        ['KRT14','MYH9'],
        ['KRT14','NEAT1'],
        ['KRT14','NEFM'],
        ['KRT14','PRPH'],
        ['KRT14','TMEM99'],
        ['KRT14','TYPE'],
        ['KRT14','VIM'],
        ['KRT15','KRT16'],
        ['KRT15','KRT4'],
        ['KRT15','KRT5'],
        ['KRT15','TMEM99'],
        ['KRT16','KRT17'],
        ['KRT16','KRT18'],
        ['KRT16','KRT19'],
        ['KRT16','KRT2'],
        ['KRT16','KRT20'],
        ['KRT16','KRT23'],
        ['KRT16','KRT24'],
        ['KRT16','KRT25'],
        ['KRT16','KRT26'],
        ['KRT16','KRT27'],
        ['KRT16','KRT28'],
        ['KRT16','KRT3'],
        ['KRT16','KRT31'],
        ['KRT16','KRT32'],
        ['KRT16','KRT33A'],
        ['KRT16','KRT33B'],
        ['KRT16','KRT34'],
        ['KRT16','KRT35'],
        ['KRT16','KRT36'],
        ['KRT16','KRT37'],
        ['KRT16','KRT38'],
        ['KRT16','KRT39'],
        ['KRT16','KRT4'],
        ['KRT16','KRT40'],
        ['KRT16','KRT5'],
        ['KRT16','KRT6B'],
        ['KRT16','KRT6C'],
        ['KRT16','KRT7'],
        ['KRT16','KRT75'],
        ['KRT16','KRT76'],
        ['KRT16','KRT77'],
        ['KRT16','KRT8'],
        ['KRT16','KRT9'],
        ['KRT16','KRTHA7'],
        ['KRT16','NEFL'],
        ['KRT16','NEFM'],
        ['KRT16','PRPH'],
        ['KRT16','VIM'],
        ['KRT17','KRT4'],
        ['KRT17','KRT5'],
        ['KRT18','KRT4'],
        ['KRT18','KRT5'],
        ['KRT18P3','RP11-359G22.2'],
        ['KRT18P59','PKNOX2'],
        ['KRT19','KRT4'],
        ['KRT19','LYZ'],
        ['KRT19','MUC5AC'],
        ['KRT1B','KRT4'],
        ['KRT1B','KRT5'],
        ['KRT1B','KRT6A'],
        ['KRT2','KRT4'],
        ['KRT2','KRT5'],
        ['KRT2','KRT6A'],
        ['KRT2','SRSF5'],
        ['KRT2','TMEM99'],
        ['KRT20','PIGR'],
        ['KRT222','KRT5'],
        ['KRT23','KRT5'],
        ['KRT23','SMR3B'],
        ['KRT24','KRT5'],
        ['KRT25','KRT5'],
        ['KRT26','KRT5'],
        ['KRT27','KRT5'],
        ['KRT28','KRT5'],
        ['KRT3','KRT4'],
        ['KRT3','KRT5'],
        ['KRT3','KRT6A'],
        ['KRT35','KRT5'],
        ['KRT37','KRT4'],
        ['KRT37','KRT5'],
        ['KRT38','KRT5'],
        ['KRT4','KRT5'],
        ['KRT4','KRT6A'],
        ['KRT4','KRT6B'],
        ['KRT4','KRT6C'],
        ['KRT4','KRT7'],
        ['KRT4','KRT71'],
        ['KRT4','KRT72'],
        ['KRT4','KRT73'],
        ['KRT4','KRT74'],
        ['KRT4','KRT75'],
        ['KRT4','KRT76'],
        ['KRT4','KRT77'],
        ['KRT4','KRT78'],
        ['KRT4','KRT79'],
        ['KRT4','KRT8'],
        ['KRT4','KRT80'],
        ['KRT4','KRT81'],
        ['KRT4','KRT82'],
        ['KRT4','KRT83'],
        ['KRT4','KRT84'],
        ['KRT4','KRT85'],
        ['KRT4','KRT86'],
        ['KRT4','KRT9'],
        ['KRT4','KRTHA7'],
        ['KRT4','KRTHB1'],
        ['KRT4','KRTHB2'],
        ['KRT4','KRTHB6'],
        ['KRT4','LMNA'],
        ['KRT4','LMNB2'],
        ['KRT4','MCM7'],
        ['KRT4','MYH11'],
        ['KRT4','MYLK'],
        ['KRT4','MYO5B'],
        ['KRT4','NEFL'],
        ['KRT4','NEFM'],
        ['KRT4','NPEPPS'],
        ['KRT4','PERP'],
        ['KRT4','PPP1R12B'],
        ['KRT4','PRPH'],
        ['KRT4','RP11-753H16.3'],
        ['KRT4','RP11-753H16.5'],
        ['KRT4','RP11-779O18.3'],
        ['KRT4','RPS11'],
        ['KRT4','RPS25'],
        ['KRT4','S100A14'],
        ['KRT4','SORBS1'],
        ['KRT4','SPRR3'],
        ['KRT4','SVIL'],
        ['KRT4','SYN3'],
        ['KRT4','SYNPO2'],
        ['KRT4','TGM3'],
        ['KRT4','TNFAIP2'],
        ['KRT4','TRIM29'],
        ['KRT4','TTBK2'],
        ['KRT4','VIM'],
        ['KRT4','ZMIZ1'],
        ['KRT40','KRT5'],
        ['KRT42P','USP6'],
        ['KRT5','KRT6A'],
        ['KRT5','KRT6B'],
        ['KRT5','KRT6C'],
        ['KRT5','KRT7'],
        ['KRT5','KRT71'],
        ['KRT5','KRT72'],
        ['KRT5','KRT73'],
        ['KRT5','KRT74'],
        ['KRT5','KRT75'],
        ['KRT5','KRT76'],
        ['KRT5','KRT77'],
        ['KRT5','KRT78'],
        ['KRT5','KRT79'],
        ['KRT5','KRT8'],
        ['KRT5','KRT80'],
        ['KRT5','KRT81'],
        ['KRT5','KRT82'],
        ['KRT5','KRT83'],
        ['KRT5','KRT84'],
        ['KRT5','KRT85'],
        ['KRT5','KRT86'],
        ['KRT5','KRT9'],
        ['KRT5','KRTHA7'],
        ['KRT5','KRTHB1'],
        ['KRT5','KRTHB2'],
        ['KRT5','KRTHB6'],
        ['KRT5','LMNA'],
        ['KRT5','LMNB2'],
        ['KRT5','LYPD3'],
        ['KRT5','MYH9'],
        ['KRT5','NEFH'],
        ['KRT5','NEFL'],
        ['KRT5','NEFM'],
        ['KRT5','PRPH'],
        ['KRT5','TMEM99'],
        ['KRT5','TTBK2'],
        ['KRT5','VIM'],
        ['KRT6A','KRT6B'],
        ['KRT6A','KRT6C'],
        ['KRT6A','KRT7'],
        ['KRT6A','KRT71'],
        ['KRT6A','KRT72'],
        ['KRT6A','KRT73'],
        ['KRT6A','KRT74'],
        ['KRT6A','KRT75'],
        ['KRT6A','KRT76'],
        ['KRT6A','KRT77'],
        ['KRT6A','KRT78'],
        ['KRT6A','KRT79'],
        ['KRT6A','KRT8'],
        ['KRT6A','KRT80'],
        ['KRT6A','KRT81'],
        ['KRT6A','KRT82'],
        ['KRT6A','KRT83'],
        ['KRT6A','KRT84'],
        ['KRT6A','KRT85'],
        ['KRT6A','KRT86'],
        ['KRT6A','KRT9'],
        ['KRT6A','KRTHB1'],
        ['KRT6A','KRTHB2'],
        ['KRT6A','NEFL'],
        ['KRT6A','PRPH'],
        ['KRT6A','RHCG'],
        ['KRT6A','TTBK2'],
        ['KRT6A','VIM'],
        ['KRT7','SFTPB'],
        ['KRT7','TG'],
        ['KRT78','MYH11'],
        ['KRT8','LYZ'],
        ['KRT8','MUC7'],
        ['KRT8P26','RP11-770G2.2'],
        ['KRTAP5-8','NADSYN1'],
        ['KSR1','TRB@'],
        ['KTN1','TG'],
        ['KXD1','UBA52'],
        ['L2HGDH','SOS2'],
        ['LACC1','LINC00284'],
        ['LAD1','TNNI1'],
        ['LAIR1','LYZ'],
        ['LAMA1','TMPO'],
        ['LAMA5','PNLIP'],
        ['LAMB1','TG'],
        ['LAMB2','TG'],
        ['LAMC1','PPP1R14B'],
        ['LAMP1','MAMDC2'],
        ['LAMP2','LYZ'],
        ['LAMP3','SFTPA1'],
        ['LAMP3','SFTPB'],
        ['LAMP3','SLC22A5'],
        ['LAPTM4B','SFTPC'],
        ['LAPTM5','SFTPA2'],
        ['LARP1','MUC7'],
        ['LARP1','TRB@'],
        ['LARS','MUC7'],
        ['LARS','NDUFC2'],
        ['LARS','NDUFC2-KCTD14'],
        ['LARS','TG'],
        ['LASP1','TAGLN'],
        ['LBP','RPN2'],
        ['LBR','LYZ'],
        ['LCAT','PSMB10'],
        ['LCLAT1','MLIP'],
        ['LCP1','LYZ'],
        ['LCP1','SRGN'],
        ['LDB1','RP11-862L9.3'],
        ['LDHB','TG'],
        ['LDLR','LYZ'],
        ['LDLR','TAPBP'],
        ['LENG8','PAX8-AS1'],
        ['LEP','SPTBN1'],
        ['LEPRE1','TRNP1'],
        ['LEPREL1','NCOR2'],
        ['LEPREL4','SCUBE3'],
        ['LEPROTL1','LYZ'],
        ['LGALS1','PDXP'],
        ['LGALS8','SFTPB'],
        ['LHFPL3','METAZOA_SRP'],
        ['LHFPL3','RN7SL8P'],
        ['LILRA1','LILRP2'],
        ['LILRB2','LILRP2'],
        ['LIMCH1','SFTPB'],
        ['LIMK2','LYZ'],
        ['LIN54','SEC31A'],
        ['LIN54','THAP9-AS1'],
        ['LINC00202-1','LINC00264'],
        ['LINC00202-1','LINC00614'],
        ['LINC00309','PELI1'],
        ['LINC00452','LINC00453'],
        ['LINC00521','OTUB2'],
        ['LINC00598','RP11-172E9.2'],
        ['LINC00665','LINC01535'],
        ['LINC00670','MYOCD'],
        ['LINC00680','WDR70'],
        ['LINC00694','ZNF445'],
        ['LINC00707','RP11-554I8.2'],
        ['LINC00843','PARG'],
        ['LINC00879','RPS18P6'],
        ['LINC00882','RP11-446H18.5'],
        ['LINC00937','RP11-481A20.10'],
        ['LINC00950','TMEM8B'],
        ['LINC00963','RGMA'],
        ['LINC00963','SLC12A6'],
        ['LINC00964','ZNF572'],
        ['LINC00969','LYZ'],
        ['LINC00969','SDHAP3'],
        ['LINC00994','PSMD6'],
        ['LINC01057','RP11-86H7.7'],
        ['LINC01091','RP11-381N20.1'],
        ['LINC01091','RP11-381N20.2'],
        ['LINC01115','TMEM18'],
        ['LINC01119','SOCS5'],
        ['LINC01128','RP11-63E5.6'],
        ['LINC01198','TPT1-AS1'],
        ['LINC01227','RP11-109P11.1'],
        ['LINC01251','UBE2R2-AS1'],
        ['LINC01278','MUC7'],
        ['LINC01291','POLE4'],
        ['LINC01317','LYZ'],
        ['LINC01324','PEX5L'],
        ['LINC01340','RP11-455B3.1'],
        ['LINC01358','RP11-145M4.3'],
        ['LINC01370','RP11-101E14.3'],
        ['LINC01378','SCD5'],
        ['LINC01492','ST6GALNAC2'],
        ['LINC01496','NUDT11'],
        ['LINC01507','RP11-327L9.1'],
        ['LINC01517','RP11-478H13.1'],
        ['LINC01518','ZNF33B'],
        ['LINC01524','RP4-715N11.2'],
        ['LINC01524','ZNF281'],
        ['LINC01605','RP11-527N22.2'],
        ['LINCR-0001','MSRA'],
        ['LIPA','LYZ'],
        ['LIPF','LYZ'],
        ['LIPF','PIGR'],
        ['LIPG','RP11-110H1.8'],
        ['LIX1L','PTRF'],
        ['LIX1L','UGT8'],
        ['LL0XNC01-16G2.1','PNMA3'],
        ['LL22NC03-2H8.5','RP11-989E6.10'],
        ['LLFOS-48D6.2','SGTA'],
        ['LLGL1','MYO1C'],
        ['LLGL2','LYZ'],
        ['LMAN1L','TMOD3'],
        ['LMAN2','MXD3'],
        ['LMF1','SMR3B'],
        ['LMNA','PNLIP'],
        ['LMNA','SLC10A3'],
        ['LMNA','TLN1'],
        ['LMNA','WIZ'],
        ['LMO7','SFTPC'],
        ['LMO7','TG'],
        ['LMO7','UCHL3'],
        ['LMOD2','NPPA-AS1'],
        ['LNP1','TMEM45A'],
        ['LONP1','SCAF1'],
        ['LONRF2','RAPH1'],
        ['LOX','SH3GL1'],
        ['LOXL1','MAFF'],
        ['LOXL1','MARVELD1'],
        ['LOXL1','ZFP36L2'],
        ['LOXL2','PKN1'],
        ['LOXL2','PTRF'],
        ['LPCAT1','SLC6A3'],
        ['LPCAT3','LRG1'],
        ['LPGAT1','TG'],
        ['LPHN1','SUZ12P'],
        ['LPL','VIM'],
        ['LPL','ZFP36L1'],
        ['LPO','LYZ'],
        ['LPO','PRH1'],
        ['LPO','PRH1-PRR4'],
        ['LPO','PRH2'],
        ['LPO','PRR4'],
        ['LPO','RP11-1143G9.4'],
        ['LPO','SMR3B'],
        ['LPP','MYH11'],
        ['LPP','RNF128'],
        ['LPP','STXBP4'],
        ['LPP','SUGCT'],
        ['LRBA','TG'],
        ['LRCH3','PTRF'],
        ['LRG1','LYZ'],
        ['LRG1','SAA2'],
        ['LRG1','SS18'],
        ['LRG1','TAT'],
        ['LRIG1','PRSS1'],
        ['LRIG1','PRSS2'],
        ['LRIG1','TRB@'],
        ['LRP1','PECAM1'],
        ['LRP10','LYZ'],
        ['LRP10','REM2'],
        ['LRP11','TTYH3'],
        ['LRP2','TG'],
        ['LRP4-AS1','LYZ'],
        ['LRP5','NPPA'],
        ['LRP5','TG'],
        ['LRRC14B','PLEKHG4B'],
        ['LRRC37A15P','MANBA'],
        ['LRRC37B','SMURF2'],
        ['LRRC56','QRSL1'],
        ['LRRC61','RP4-584D14.7'],
        ['LRRC75A','METTL7A'],
        ['LRRC75A-AS1','LYZ'],
        ['LRRC75A-AS1','SYN3'],
        ['LRRCC1','RP11-146E23.2'],
        ['LRRFIP1','SMR3B'],
        ['LRRK2','MUC19'],
        ['LSG1','RP11-114H23.1'],
        ['LSP1','TNNT3'],
        ['LTA','NFKBIL1'],
        ['LTF','LYZ'],
        ['LTF','NFKBIZ'],
        ['LTF','RBM25'],
        ['LTF','SMR3B'],
        ['LTF','STATH'],
        ['LTV1','RP11-718O11.1'],
        ['LYG1','TXNDC9'],
        ['LYPLA1','TCEA1'],
        ['LYRM7','LYZ'],
        ['LYSMD1','SEMA6C'],
        ['LYSMD3','RP1-225E12.2'],
        ['LYSMD3','RP1-225E12.3'],
        ['LYST','LYZ'],
        ['LYZ','MAFF'],
        ['LYZ','MAGED1'],
        ['LYZ','MAP3K5'],
        ['LYZ','MCM4'],
        ['LYZ','MKNK2'],
        ['LYZ','MLLT6'],
        ['LYZ','MLXIP'],
        ['LYZ','MOGS'],
        ['LYZ','MON1B'],
        ['LYZ','MROH1'],
        ['LYZ','MYH11'],
        ['LYZ','MYH9'],
        ['LYZ','MYO18A'],
        ['LYZ','MYO1A'],
        ['LYZ','MYO1F'],
        ['LYZ','NARF'],
        ['LYZ','NCL'],
        ['LYZ','NCOA6'],
        ['LYZ','NEAT1'],
        ['LYZ','NEBL'],
        ['LYZ','NF1'],
        ['LYZ','NFKBIB'],
        ['LYZ','NFKBIZ'],
        ['LYZ','NMD3'],
        ['LYZ','NONO'],
        ['LYZ','NPC1'],
        ['LYZ','NRAS'],
        ['LYZ','NT5C2'],
        ['LYZ','NXN'],
        ['LYZ','OBSL1'],
        ['LYZ','OPA1'],
        ['LYZ','ORAI2'],
        ['LYZ','OSBPL2'],
        ['LYZ','OSBPL9'],
        ['LYZ','OTUD5'],
        ['LYZ','P4HB'],
        ['LYZ','PABPC1'],
        ['LYZ','PARP8'],
        ['LYZ','PCBP1-AS1'],
        ['LYZ','PCBP2'],
        ['LYZ','PCDHA1'],
        ['LYZ','PCDHA10'],
        ['LYZ','PCDHA11'],
        ['LYZ','PCDHA12'],
        ['LYZ','PCDHA13'],
        ['LYZ','PCDHA2'],
        ['LYZ','PCDHA3'],
        ['LYZ','PCDHA4'],
        ['LYZ','PCDHA5'],
        ['LYZ','PCDHA7'],
        ['LYZ','PCDHA8'],
        ['LYZ','PCDHA9'],
        ['LYZ','PCDHAC1'],
        ['LYZ','PCDHAC2'],
        ['LYZ','PCYOX1'],
        ['LYZ','PDLIM5'],
        ['LYZ','PET100'],
        ['LYZ','PGC'],
        ['LYZ','PHACTR4'],
        ['LYZ','PHF2'],
        ['LYZ','PIK3R2'],
        ['LYZ','PIP'],
        ['LYZ','PITPNA'],
        ['LYZ','PLAUR'],
        ['LYZ','PLEKHM2'],
        ['LYZ','PLEKHS1'],
        ['LYZ','PLIN3'],
        ['LYZ','PLPP3'],
        ['LYZ','PPIB'],
        ['LYZ','PPP1CB'],
        ['LYZ','PRB1'],
        ['LYZ','PRB2'],
        ['LYZ','PRELP'],
        ['LYZ','PRH1'],
        ['LYZ','PRH1-PRR4'],
        ['LYZ','PRKX'],
        ['LYZ','PRPF8'],
        ['LYZ','PRR27'],
        ['LYZ','PRR4'],
        ['LYZ','PRRC2B'],
        ['LYZ','PSAP'],
        ['LYZ','PSKH1'],
        ['LYZ','PSMB5'],
        ['LYZ','PSMC6'],
        ['LYZ','PSME4'],
        ['LYZ','PTAFR'],
        ['LYZ','PTPRA'],
        ['LYZ','PTPRC'],
        ['LYZ','PTPRN2'],
        ['LYZ','PTRF'],
        ['LYZ','R3HDM2'],
        ['LYZ','RAB11FIP1'],
        ['LYZ','RAB1A'],
        ['LYZ','RAB8B'],
        ['LYZ','RABGEF1'],
        ['LYZ','RANBP10'],
        ['LYZ','RBM3'],
        ['LYZ','RBM39'],
        ['LYZ','RBM5'],
        ['LYZ','RBSN'],
        ['LYZ','REV1'],
        ['LYZ','REXO2'],
        ['LYZ','RFWD3'],
        ['LYZ','RGCC'],
        ['LYZ','RIOK3'],
        ['LYZ','RNF115'],
        ['LYZ','RP11-307N16.6'],
        ['LYZ','RP11-468E2.1'],
        ['LYZ','RP11-473I1.9'],
        ['LYZ','RP11-613M10.9'],
        ['LYZ','RP11-618G20.1'],
        ['LYZ','RP3-331H24.5'],
        ['LYZ','RPL26'],
        ['LYZ','RPL30'],
        ['LYZ','RPL37A'],
        ['LYZ','RPL4'],
        ['LYZ','RPL7L1'],
        ['LYZ','RPS23'],
        ['LYZ','RSPH6A'],
        ['LYZ','RTF1'],
        ['LYZ','RUNDC1'],
        ['LYZ','SAMHD1'],
        ['LYZ','SAT1'],
        ['LYZ','SBF2'],
        ['LYZ','SCAF11'],
        ['LYZ','SCAMP1'],
        ['LYZ','SENP3-EIF4A1'],
        ['LYZ','SF3B1'],
        ['LYZ','SFPQ'],
        ['LYZ','SFTPB'],
        ['LYZ','SGSM3'],
        ['LYZ','SLC20A2'],
        ['LYZ','SLC25A37'],
        ['LYZ','SLC25A39'],
        ['LYZ','SLC2A3'],
        ['LYZ','SLC43A2'],
        ['LYZ','SLC4A1'],
        ['LYZ','SLC7A5'],
        ['LYZ','SMG1'],
        ['LYZ','SMIM14'],
        ['LYZ','SMR3B'],
        ['LYZ','SNRNP200'],
        ['LYZ','SNTB2'],
        ['LYZ','SP3'],
        ['LYZ','SPARC'],
        ['LYZ','SPATA13'],
        ['LYZ','SPC24'],
        ['LYZ','SPINT2'],
        ['LYZ','SPN'],
        ['LYZ','SPTBN1'],
        ['LYZ','SPTY2D1'],
        ['LYZ','SRGN'],
        ['LYZ','SRRM2'],
        ['LYZ','SRSF10'],
        ['LYZ','SRSF5'],
        ['LYZ','SSH1'],
        ['LYZ','STAT1'],
        ['LYZ','STAT3'],
        ['LYZ','STK32C'],
        ['LYZ','STK39'],
        ['LYZ','STT3B'],
        ['LYZ','STX16'],
        ['LYZ','STX16-NPEPL1'],
        ['LYZ','STX8'],
        ['LYZ','SUGCT'],
        ['LYZ','SUGP2'],
        ['LYZ','SYAP1'],
        ['LYZ','TAF15'],
        ['LYZ','TAOK3'],
        ['LYZ','TBRG1'],
        ['LYZ','TEF'],
        ['LYZ','TENM3'],
        ['LYZ','TFCP2L1'],
        ['LYZ','TFRC'],
        ['LYZ','TGFBRAP1'],
        ['LYZ','THEMIS2'],
        ['LYZ','TIA1'],
        ['LYZ','TIGD6'],
        ['LYZ','TIMP2'],
        ['LYZ','TMBIM6'],
        ['LYZ','TMED3'],
        ['LYZ','TMEM220-AS1'],
        ['LYZ','TMEM56'],
        ['LYZ','TMEM87A'],
        ['LYZ','TPD52L1'],
        ['LYZ','TPM4'],
        ['LYZ','TPT1'],
        ['LYZ','TRA2A'],
        ['LYZ','TRIM58'],
        ['LYZ','TRIP12'],
        ['LYZ','TSC2'],
        ['LYZ','TSG101'],
        ['LYZ','TSN'],
        ['LYZ','TXNL4A'],
        ['LYZ','TYK2'],
        ['LYZ','UBE2D3'],
        ['LYZ','UBE2H'],
        ['LYZ','UBXN4'],
        ['LYZ','UBXN7'],
        ['LYZ','UFD1L'],
        ['LYZ','UGDH-AS1'],
        ['LYZ','USP4'],
        ['LYZ','VPS16'],
        ['LYZ','VPS4B'],
        ['LYZ','VPS53'],
        ['LYZ','VTI1A'],
        ['LYZ','WDR1'],
        ['LYZ','WDR82'],
        ['LYZ','WIPF2'],
        ['LYZ','WSB1'],
        ['LYZ','XRCC6'],
        ['LYZ','YIPF4'],
        ['LYZ','YIPF5'],
        ['LYZ','YME1L1'],
        ['LYZ','ZBTB43'],
        ['LYZ','ZG16B'],
        ['LYZ','ZKSCAN1'],
        ['LYZ','ZMYND8'],
        ['LYZ','ZNF251'],
        ['LYZ','ZNF394'],
        ['LYZ','ZNF431'],
        ['LYZ','ZNF655'],
        ['MAATS1','NR1I2'],
        ['MACF1','PAX8'],
        ['MACF1','SFTPB'],
        ['MACF1','TG'],
        ['MAD2L1','MRI1'],
        ['MAF','RP11-679B19.2'],
        ['MAFG-AS1','RP11-498C9.12'],
        ['MALAT1','RPPH1'],
        ['MALAT1','TNN'],
        ['MALAT1','WDR74'],
        ['MALRD1','PLXDC2'],
        ['MALRD1','RP11-1143G9.4'],
        ['MAMDC2-AS1','SMC5-AS1'],
        ['MAML1','SQSTM1'],
        ['MAML1','TG'],
        ['MAN1A2','STATH'],
        ['MAOA','MUC7'],
        ['MAP1A','VIM'],
        ['MAP2K2','PIGR'],
        ['MAP2K5','SKOR1'],
        ['MAP3K9','PGD'],
        ['MAP4K4','TG'],
        ['MAP6D1','PARL'],
        ['MAP7D1','PRELID1'],
        ['MAPK1','TRB@'],
        ['MAPK7','RNF112'],
        ['MAPKAPK2','MPO'],
        ['MAPKAPK3','RP11-804H8.6'],
        ['MAPRE1','MAPRE1P1'],
        ['MARCH1','SNRPA1'],
        ['MARCH6','MUC7'],
        ['MARCH6','TG'],
        ['MARCKS','RCC2'],
        ['MAT1A','RP11-573D15.8'],
        ['MAT2A','TG'],
        ['MAT2B','STATH'],
        ['MATN2','TG'],
        ['MATN4','SLPI'],
        ['MAVS','PANK2'],
        ['MAVS','RP5-1009E24.9'],
        ['MAVS','ZNF444'],
        ['MAZ','PTMS'],
        ['MAZ','TMEM160'],
        ['MB','NPPA-AS1'],
        ['MB','RP11-814P5.1'],
        ['MB','TTN-AS1'],
        ['MBD3','TG'],
        ['MBP','RTN3'],
        ['MBP','SYT11'],
        ['MBP','TMSB4X'],
        ['MBP','TUBA1A'],
        ['MBP','TUBB4A'],
        ['MBP','VAMP2'],
        ['MCAM','MYL7'],
        ['MCAM','RPS24'],
        ['MCAM','RPS29'],
        ['MCFD2','TG'],
        ['MCL1','PNLIP'],
        ['MCL1','S100A9'],
        ['MCM3AP-AS1','SSR3'],
        ['MCM4','PGAM5'],
        ['MCPH1','PIGR'],
        ['MCUR1','SRSF10'],
        ['MDC1','SCARB2'],
        ['MDM1','SFTPB'],
        ['MECOM','RPL22'],
        ['MECP2','TG'],
        ['MED13L','PNLIP'],
        ['MED13L','SMR3B'],
        ['MED13L','TG'],
        ['MED15','ZNF890P'],
        ['MED21','RP11-582E3.4'],
        ['MED25','PDLIM1'],
        ['MED29','TG'],
        ['MED6','RP11-486O13.4'],
        ['MEG3','SCARB1'],
        ['MEG3','SLC16A3'],
        ['MEG3','SPARC'],
        ['MEG3','STAR'],
        ['MEG3','TFAP2D'],
        ['MEG3','TIMP3'],
        ['MEMO1','RPS9P1'],
        ['MEMO1P1','RPS9P1'],
        ['METAZOA_SRP','PAFAH1B1'],
        ['METTL22','RP11-475D10.4'],
        ['METTL23','MFSD11'],
        ['METTL7A','USP37'],
        ['METTL9','SRM'],
        ['MFAP5','PIGR'],
        ['MFGE8','NPPA-AS1'],
        ['MFSD1','RP11-170K4.2'],
        ['MGAT4A','PRSS2'],
        ['MGEA5','S100A9'],
        ['MGLL','PPP1R1A'],
        ['MICA','ZDHHC20P2'],
        ['MICAL3','SPAG9'],
        ['MICALL1','POLR2F'],
        ['MICALL1','RP5-1039K5.19'],
        ['MICU2','ZDHHC20'],
        ['MICU3','SERPINB1'],
        ['MINK1','SFTPB'],
        ['MINOS1','NEK10'],
        ['MINOS1-NBL1','NEK10'],
        ['MIR497HG','SFTPC'],
        ['MKLN1','VMAC'],
        ['MKNK2','TG'],
        ['MLF2','TG'],
        ['MLLT10P1','NCOA3'],
        ['MLLT4','PRSS1'],
        ['MLLT6','SMR3B'],
        ['MMD','TG'],
        ['MMD2','RP11-284F21.7'],
        ['MMP2','MRC2'],
        ['MMP2','SFTPB'],
        ['MNAT1','SMR3B'],
        ['MND1','TRIM2'],
        ['MOB2','MUC7'],
        ['MORC2-AS1','OSBP2'],
        ['MPDU1','RP11-186B7.4'],
        ['MPHOSPH9','RP11-282O18.3'],
        ['MPO','NUP210L'],
        ['MPO','PDZD2'],
        ['MPO','PIEZO1'],
        ['MPO','SNX32'],
        ['MPO','ZFP36'],
        ['MRE11A','RP11-867G2.8'],
        ['MRPL28','VIM'],
        ['MRPL30','RN7SL377P'],
        ['MRPL30P1','MS4A8'],
        ['MRPL42P3','STX11'],
        ['MRPL51','RBM3'],
        ['MRPS27','SULT2A1'],
        ['MRPS31P2','TPTE2'],
        ['MS4A10','RP11-158I13.2'],
        ['MS4A10','ZNF512'],
        ['MS4A12','THCAT158'],
        ['MSI1','PLA2G1B'],
        ['MSMB','NCOA4'],
        ['MSN','MUC7'],
        ['MSN','NPPA-AS1'],
        ['MSN','SFTPC'],
        ['MT1A','RSG1'],
        ['MT1A','TG'],
        ['MT1M','MT2P1'],
        ['MT1X','RSG1'],
        ['MTERF4','PASK'],
        ['MTG1','RP11-108K14.4'],
        ['MTG1','SCART1'],
        ['MTMR12','SFTPB'],
        ['MTMR6','RP11-271M24.2'],
        ['MTR','RYR2'],
        ['MTSS1L','RANBP3'],
        ['MTUS1','PNLIP'],
        ['MUC16','MUC5AC'],
        ['MUC21','MYH11'],
        ['MUC21','PPL'],
        ['MUC5AC','MUC6'],
        ['MUC5AC','PGC'],
        ['MUC5AC','PTBP1'],
        ['MUC5B','MUC7'],
        ['MUC5B','SMR3B'],
        ['MUC5B','STATH'],
        ['MUC6','PGC'],
        ['MUC7','NFIC'],
        ['MUC7','PABPC1'],
        ['MUC7','PDLIM5'],
        ['MUC7','PFN1'],
        ['MUC7','PHLDA1'],
        ['MUC7','PIGR'],
        ['MUC7','POLR2A'],
        ['MUC7','PRB1'],
        ['MUC7','PRB2'],
        ['MUC7','PRB3'],
        ['MUC7','PRH1'],
        ['MUC7','PRH1-PRR4'],
        ['MUC7','PRH2'],
        ['MUC7','PRR4'],
        ['MUC7','PSAP'],
        ['MUC7','RP11-432B6.3'],
        ['MUC7','RPL8'],
        ['MUC7','RPS11'],
        ['MUC7','SCARB2'],
        ['MUC7','SECISBP2L'],
        ['MUC7','SQSTM1'],
        ['MUC7','STATH'],
        ['MUC7','SYN3'],
        ['MUC7','TMED10'],
        ['MUC7','TNS1'],
        ['MUC7','TSHZ2'],
        ['MUC7','WNK2'],
        ['MUC7','ZG16B'],
        ['MUM1','MYH11'],
        ['MUM1','NDUFS7'],
        ['MXRA7','PTRF'],
        ['MXRA7','RBMS2'],
        ['MXRA8','SBF1'],
        ['MYADM','NEDD4L'],
        ['MYADM','PARN'],
        ['MYADM','SAMHD1'],
        ['MYADM','SEC11A'],
        ['MYADM','TNS1'],
        ['MYBPC1','SOD2'],
        ['MYBPC3','PPP1R12B'],
        ['MYEF2','RP11-605F22.1'],
        ['MYH10','SFTPC'],
        ['MYH11','MYH6'],
        ['MYH11','P4HB'],
        ['MYH11','PIGR'],
        ['MYH11','RHCG'],
        ['MYH11','RP11-2C24.9'],
        ['MYH11','SMR3B'],
        ['MYH11','SPINK5'],
        ['MYH11','SPTBN1'],
        ['MYH11','SRCAP'],
        ['MYH11','SYN3'],
        ['MYH11','SYNM'],
        ['MYH11','SYNPO2'],
        ['MYH11','TGM3'],
        ['MYH11','TPM1'],
        ['MYH11','TRB@'],
        ['MYH11','TRIM29'],
        ['MYH11','UBE2N'],
        ['MYH16','SCD5'],
        ['MYH3','SCO1'],
        ['MYH6','NEXN'],
        ['MYH6','NPPA'],
        ['MYH6','NPPA-AS1'],
        ['MYH6','RP5-857K21.4'],
        ['MYH6','RP5-857K21.6'],
        ['MYH6','SRRM2'],
        ['MYH7','NPPA-AS1'],
        ['MYH7','SLC4A3'],
        ['MYH7','TNNC1'],
        ['MYH7','TNS1'],
        ['MYH9','NDE1'],
        ['MYH9','PLIN4'],
        ['MYH9','PRH1'],
        ['MYH9','PRH1-PRR4'],
        ['MYH9','PRH2'],
        ['MYH9','PXDN'],
        ['MYH9','QSOX1'],
        ['MYH9','SFTPA2'],
        ['MYH9','SFTPC'],
        ['MYH9','SMR3B'],
        ['MYH9','TAGLN'],
        ['MYH9','TPM2'],
        ['MYHAS','RP11-1143G9.4'],
        ['MYL2','NCOA6'],
        ['MYL2','NDRG1'],
        ['MYL2','NPPA-AS1'],
        ['MYL2','PALM2-AKAP2'],
        ['MYL3','PRSS42'],
        ['MYL3','PRSS45'],
        ['MYL3','PRSS50'],
        ['MYL4','NPPA'],
        ['MYL6','MYL6P1'],
        ['MYL6','MYL6P2'],
        ['MYL6','MYL6P3'],
        ['MYL6','MYL6P4'],
        ['MYL6','MYL6P5'],
        ['MYL6','SPARC'],
        ['MYL6','TG'],
        ['MYL7','MYLK3'],
        ['MYL7','NPPA'],
        ['MYL7','PPP1R12B'],
        ['MYL7','RYR2'],
        ['MYLK','PRB3'],
        ['MYO18A','RP11-22N12.2'],
        ['MYO19','ZG16B'],
        ['MYO1C','TAPBP'],
        ['MYO5A','SFTPB'],
        ['MYO9B','PTRF'],
        ['MYOM1','RP13-270P17.1'],
        ['MYOZ1','SYNPO2L'],
        ['MYRFL','RAB3IP'],
        ['MYT1','OPRL1'],
        ['N6AMT1','RP11-178H8.3'],
        ['NAA20','RIN2'],
        ['NACA','NPPA-AS1'],
        ['NACA','PRIM1'],
        ['NACA','RAB3IL1'],
        ['NACC1','NFX1'],
        ['NANOGNBP2','ZNF605'],
        ['NAPEPLD','RP11-401L13.7'],
        ['NARS','RP11-706P11.2'],
        ['NAV3','RP1-34H18.1'],
        ['NBEAL2','S100A9'],
        ['NBEAP1','RP11-32B5.2'],
        ['NBEAP1','RP11-467L19.16'],
        ['NBPF20','NOTCH2NL'],
        ['NBR1','NBR2'],
        ['NCAM1','VSNL1'],
        ['NCAM1-AS1','RP11-839D17.3'],
        ['NCBP2','PIGZ'],
        ['NCBP2-AS1','SENP5'],
        ['NCDN','VSNL1'],
        ['NCKAP1','PNLIP'],
        ['NCKAP1L','PDE1B'],
        ['NCL','NEFM'],
        ['NCL','NPM1'],
        ['NCL','SFTPC'],
        ['NCLN','S1PR4'],
        ['NCOA1','RP11-1143G9.4'],
        ['NCOA6','NPPA'],
        ['NCOA6','PIGU'],
        ['NCOR2','OAF'],
        ['NCOR2','PLAUR'],
        ['NCOR2','SFTPA2'],
        ['NCOR2','SMR3B'],
        ['NCR3LG1','NUCB2'],
        ['NDE1','PGR'],
        ['NDE1','STAMBPL1'],
        ['NDRG1','ST3GAL1'],
        ['NDRG2','PRH1'],
        ['NDRG2','PRH1-PRR4'],
        ['NDRG2','SMR3B'],
        ['NDUFA6-AS1','SMDT1'],
        ['NDUFB8','SEC31B'],
        ['NDUFC2-KCTD14','SMR3B'],
        ['NDUFV1','NPPA-AS1'],
        ['NEAT1','SYN3'],
        ['NEAT1','TMEM99'],
        ['NEBL','SMR3B'],
        ['NECAB2','OSGIN1'],
        ['NEDD4L','TRB@'],
        ['NEDD8','TINF2'],
        ['NEDD8','UBA52'],
        ['NEDD8-MDP1','TINF2'],
        ['NEFH','RFPL1'],
        ['NEK10','SLC4A7'],
        ['NEK7','PIGR'],
        ['NEK9','RP11-618G20.1'],
        ['NEK9','TMED10'],
        ['NEMP1','PLEKHA2'],
        ['NEU3','SPCS2'],
        ['NF2','TG'],
        ['NFATC3','PLA2G15'],
        ['NFE2L1','NPPA-AS1'],
        ['NFE2L2','TG'],
        ['NFIB','PRH1-PRR4'],
        ['NFIC','PNLIP'],
        ['NFIX','PTMS'],
        ['NFIX','SMR3B'],
        ['NFIX','VPS26B'],
        ['NFYC','RP11-276H19.4'],
        ['NIN','SAV1'],
        ['NIPBL','TRB@'],
        ['NKAPP1','RHOXF1'],
        ['NKAPP1','RP4-755D9.1'],
        ['NKIRAS1','SMR3B'],
        ['NKRF','SEPT6'],
        ['NKTR','TMEM99'],
        ['NKX2-1','SFTA3'],
        ['NME4','PCSK7'],
        ['NMNAT1','RBP7'],
        ['NMNAT2','VSNL1'],
        ['NMT1','PABPC1'],
        ['NNT-AS1','TPT1P2'],
        ['NOL11','PITPNC1'],
        ['NOL6','SUGT1'],
        ['NOL6','SUGT1P'],
        ['NONO','PTRF'],
        ['NONO','SFTPB'],
        ['NOP10','RP11-290L1.5'],
        ['NOP9','TF'],
        ['NOS1','PKP1'],
        ['NOTCH3','PTMS'],
        ['NOX4','RP11-643G5.6'],
        ['NOXRED1','VIPAS39'],
        ['NPC2','SFTPB'],
        ['NPC2','SYNDIG1L'],
        ['NPEPL1','STX16'],
        ['NPHS1','PRODH2'],
        ['NPIPB5','SMG1'],
        ['NPIPP1','PKD1'],
        ['NPM','NPM1P43'],
        ['NPM1','NPM1P51'],
        ['NPPA','POLD2'],
        ['NPPA','PPP1R12B'],
        ['NPPA','PRRC2B'],
        ['NPPA','RP1-178F15.5'],
        ['NPPA','RP11-64D24.2'],
        ['NPPA','RP11-77K12.8'],
        ['NPPA','RP11-814P5.1'],
        ['NPPA','S100A13'],
        ['NPPA','SF3B1'],
        ['NPPA','SPOCK1'],
        ['NPPA','STAMBPL1'],
        ['NPPA','SYNPO'],
        ['NPPA','TECRL'],
        ['NPPA','TGM2'],
        ['NPPA','TIMP3'],
        ['NPPA','TNNT2'],
        ['NPPA','TNS1'],
        ['NPPA-AS1','NPPB'],
        ['NPPA-AS1','NUCB2'],
        ['NPPA-AS1','PBX1'],
        ['NPPA-AS1','PLEC'],
        ['NPPA-AS1','PSAP'],
        ['NPPA-AS1','RAB11B'],
        ['NPPA-AS1','RCAN2'],
        ['NPPA-AS1','RILPL1'],
        ['NPPA-AS1','S100A1'],
        ['NPPA-AS1','SGCD'],
        ['NPPA-AS1','SPARC'],
        ['NPPA-AS1','SQSTM1'],
        ['NPPA-AS1','SRRM2'],
        ['NPPA-AS1','SYNPO'],
        ['NPPA-AS1','TGOLN2'],
        ['NPPA-AS1','TNNC1'],
        ['NPPA-AS1','TNNI3'],
        ['NPPA-AS1','TNNT2'],
        ['NPPA-AS1','TNS1'],
        ['NPPA-AS1','TP53INP2'],
        ['NPPA-AS1','TPM1'],
        ['NPPA-AS1','TTN-AS1'],
        ['NPPA-AS1','TXNIP'],
        ['NPPA-AS1','USB1'],
        ['NPPA-AS1','WDPCP'],
        ['NPTXR','RAD52'],
        ['NQO1','THRIL'],
        ['NR4A1','STAT3'],
        ['NRBP2','PUF60'],
        ['NRGN','TMSB10'],
        ['NRIP3','TMEM9B'],
        ['NRSN2-AS1','TRB@'],
        ['NSD1','SFTPB'],
        ['NSF','PNLIP'],
        ['NSFL1C','SIRPB2'],
        ['NSUN7','RP11-632F7.3'],
        ['NT5C2','PRICKLE1'],
        ['NT5DC2','PTRF'],
        ['NTN4','TRB@'],
        ['NTN4','VASN'],
        ['NTRK2','TG'],
        ['NUCB1','SFTPC'],
        ['NUCB2','SMR3B'],
        ['NUDT10','PTMS'],
        ['NUDT15','SUCLA2-AS1'],
        ['NUDT17','POLR3C'],
        ['NUP160','PNLIP'],
        ['NUP188','PHYHD1'],
        ['NUP35','RP11-438L19.1'],
        ['NUP50','RP11-973N13.3'],
        ['NUP88','TG'],
        ['NUPR1','RP11-666O2.2'],
        ['NXN','PTMS'],
        ['NXN','TPM4'],
        ['OAZ1','PNLIP'],
        ['OAZ1','PTMA'],
        ['OAZ1','SGTA'],
        ['OAZ1','VIM-AS1'],
        ['OAZ2','RBPMS2'],
        ['OBFC1','SH3PXD2A'],
        ['OCA2','RP11-30G8.1'],
        ['ODF3B','SCO2'],
        ['OGDH','PAX8-AS1'],
        ['OLFM5P','UBQLNL'],
        ['OOEP-AS1','PTMA'],
        ['OR2J3','XXBAC-BPG258E24.10'],
        ['OR2T8','TRIM58'],
        ['OR52L1','OR56A4'],
        ['OR52M2P','TRIM21'],
        ['OR6K3','OR6K4P'],
        ['OR6K5P','OR6N2'],
        ['OR6M3P','TMEM225'],
        ['OR7D1P','OR7E24'],
        ['OR8T1P','ZNF641'],
        ['ORAI2','PRKRIP1'],
        ['ORAOV1','PTMS'],
        ['ORC1','ZCCHC11'],
        ['ORM1','SERPINA1'],
        ['OSMR','RICTOR'],
        ['OSMR','RP11-108O10.8'],
        ['OSTF1','SSR3'],
        ['OVGP1','RP11-552M11.4'],
        ['OXR1','PNLIP'],
        ['OXTR','SSUH2'],
        ['P4HB','SMR3B'],
        ['PA2G4','ZC3H10'],
        ['PADI4','TFRC'],
        ['PAFAH1B2','YIPF5'],
        ['PAICS','SRP72'],
        ['PAK3','TRB@'],
        ['PAK4','SMR3B'],
        ['PAM','PNLIP'],
        ['PANK1','SLC16A12'],
        ['PANK3','PIGR'],
        ['PAPD7','RP11-332J15.3'],
        ['PAPOLA','RP11-433J8.1'],
        ['PAPPA','TIMP3'],
        ['PAQR6','SMG5'],
        ['PAQR7','STMN1'],
        ['PARG','TIMM23B'],
        ['PARK2','UBA52'],
        ['PARL','RP1-95L4.4'],
        ['PARM1','PRH1'],
        ['PARM1','PRH1-PRR4'],
        ['PARP10','SFTPC'],
        ['PATL2','SPG11'],
        ['PATZ1','PIK3IP1'],
        ['PAX5','RHOBTB3'],
        ['PAX8-AS1','TG'],
        ['PAX8-AS1','TPO'],
        ['PBX1','TG'],
        ['PBXIP1','PMVK'],
        ['PCAT1','RP11-89K10.1'],
        ['PCBP1-AS1','S100A9'],
        ['PCBP1-AS1','SRGN'],
        ['PCBP1-AS1','TG'],
        ['PCBP2','SMR3B'],
        ['PCBP2','TG'],
        ['PCBP3','PTTG1IP'],
        ['PCCB','PDS5B'],
        ['PCDHGA12','PCDHGB8P'],
        ['PCDHGB8P','PCDHGC3'],
        ['PCDHGB8P','PCDHGC4'],
        ['PCDHGB8P','PCDHGC5'],
        ['PCK1','RP11-573D15.8'],
        ['PCNP','PCNPP3'],
        ['PCOLCE','PLEC'],
        ['PCSK7','PKM'],
        ['PCSK7','RRBP1'],
        ['PCSK7','TAF15'],
        ['PCSK7','TGFBI'],
        ['PCYOX1','SOD2'],
        ['PDCD7','UBAP1L'],
        ['PDE4B','SMR3B'],
        ['PDE8B','TG'],
        ['PDE8B','ZBED3-AS1'],
        ['PDHA1','STATH'],
        ['PDIA2','TRB@'],
        ['PDIA3','PIGR'],
        ['PDIA3','PNLIP'],
        ['PDIA3','STATH'],
        ['PDIA6','PRSS2'],
        ['PDIA6','RP11-791G15.2'],
        ['PDIA6','SMR3B'],
        ['PDIA6','STATH'],
        ['PDK4','TRB@'],
        ['PDS5A','TG'],
        ['PDXDC1','PNLIP'],
        ['PDXK','SFTPB'],
        ['PDZD2','PVRL4'],
        ['PDZK1IP1','TAL1'],
        ['PEA15','SMR3B'],
        ['PEBP1','SERPINA1'],
        ['PECAM1','SFTPB'],
        ['PECAM1','ZG16B'],
        ['PER1','PRH1'],
        ['PER1','PRH1-PRR4'],
        ['PER1','PRR4'],
        ['PEX11A','PLIN1'],
        ['PFKFB3','RP11-563J2.2'],
        ['PFKFB4','SHISA5'],
        ['PFKL','TG'],
        ['PFN1','RANBP3'],
        ['PGAM1P5','PRSS2'],
        ['PGAM1P5','TRB@'],
        ['PGAM1P5','TRBC2'],
        ['PGAM1P5','TRBV25-1'],
        ['PGC','PIGR'],
        ['PGC','POLR2A'],
        ['PGC','RP11-1143G9.4'],
        ['PGC','RP11-571M6.7'],
        ['PHACTR4','RCC1'],
        ['PHACTR4','SNTB2'],
        ['PHB','ZNF607'],
        ['PHF14','SMR3B'],
        ['PHF2','SFTPC'],
        ['PHGDH','SCD'],
        ['PHLDA1','PRH1'],
        ['PHLDA1','PRH1-PRR4'],
        ['PIAS1','SMR3B'],
        ['PICALM','PRB3'],
        ['PIEZO1','SFTPA2'],
        ['PIGL','UBB'],
        ['PIGO','TG'],
        ['PIGR','PIP'],
        ['PIGR','PLA2G2A'],
        ['PIGR','POC5'],
        ['PIGR','PRB1'],
        ['PIGR','PRB2'],
        ['PIGR','PRH1'],
        ['PIGR','PRH1-PRR4'],
        ['PIGR','PRR4'],
        ['PIGR','RBM20'],
        ['PIGR','RP11-319G6.1'],
        ['PIGR','RP11-403P17.5'],
        ['PIGR','SETD4'],
        ['PIGR','SFTPB'],
        ['PIGR','SLC44A4'],
        ['PIGR','SLC5A1'],
        ['PIGR','SPTBN1'],
        ['PIGR','STAT1'],
        ['PIGR','SUSD1'],
        ['PIGR','SYNPO2'],
        ['PIGR','TMBIM6'],
        ['PIGR','ZG16B'],
        ['PIGR','ZKSCAN1'],
        ['PIGR','ZNF232'],
        ['PIK3C2A','PNLIP'],
        ['PIM2','SLC35A2'],
        ['PIM3','SCO2'],
        ['PIN4','PNLIP'],
        ['PINK1-AS','TTL'],
        ['PITPNA','SMR3B'],
        ['PITPNA','TG'],
        ['PITPNC1','SLC25A3'],
        ['PIWIL1','RP11-117L5.1'],
        ['PJA2','SFTPB'],
        ['PK','RP11-814P5.1'],
        ['PKHD1L1','TG'],
        ['PKM','RHOC'],
        ['PKM','RP11-1143G9.4'],
        ['PKM','SERPINE1'],
        ['PKM','THBS1'],
        ['PKN1','UBE2M'],
        ['PKP1','TMEM99'],
        ['PLA2G1B','PRSS1'],
        ['PLA2G1B','PRSS2'],
        ['PLA2G1B','TRB@'],
        ['PLCD3','PTRF'],
        ['PLEC','PLOD3'],
        ['PLEC','TFPI2'],
        ['PLEC','TNC'],
        ['PLEC','VIM-AS1'],
        ['PLEKHA1','RAB17'],
        ['PLEKHA5','SRSF11'],
        ['PLEKHB1','VAMP2'],
        ['PLEKHB2','SETBP1'],
        ['PLEKHG2','ZFP36'],
        ['PLEKHH1','TG'],
        ['PLEKHO1','VPS45'],
        ['PLIN3','SCUBE3'],
        ['PLIN4','PSAP'],
        ['PLIN4','SERPING1'],
        ['PLIN4','SPARC'],
        ['PLIN4','SPTBN1'],
        ['PLIN4','SYN3'],
        ['PLIN4','TBX1'],
        ['PLIN5','SAA2'],
        ['PLP1','RP11-862L9.3'],
        ['PLXNA1','SFTPC'],
        ['PLXNA2','PRSS1'],
        ['PLXNA2','PRSS2'],
        ['PLXNB2','SFTPB'],
        ['PM20D2','WDFY3'],
        ['PMCH','PMCHL1'],
        ['PMCH','PMCHL2'],
        ['PMCHL1','PMCHL2'],
        ['PMEPA1','PNLIP'],
        ['PMS2','PMS2CL'],
        ['PMS2CL','RBAK'],
        ['PMS2CL','RBAK-RBAKDN'],
        ['PNLIP','PRSS1'],
        ['PNLIP','PRSS2'],
        ['PNLIP','PRSS3'],
        ['PNLIP','PRSS3P1'],
        ['PNLIP','PTPRN2'],
        ['PNLIP','RP11-1021N1.1'],
        ['PNLIP','RP11-64D24.2'],
        ['PNLIP','RPL30'],
        ['PNLIP','RPS20'],
        ['PNLIP','SERF2'],
        ['PNLIP','SERPINI2'],
        ['PNLIP','SGK1'],
        ['PNLIP','SLC37A4'],
        ['PNLIP','SLC38A10'],
        ['PNLIP','SLC4A4'],
        ['PNLIP','SND1'],
        ['PNLIP','SNTG2'],
        ['PNLIP','SOD2'],
        ['PNLIP','SPATS2L'],
        ['PNLIP','SPINT2'],
        ['PNLIP','SPTBN1'],
        ['PNLIP','SRRM2'],
        ['PNLIP','ST6GALNAC6'],
        ['PNLIP','SYCN'],
        ['PNLIP','SYN3'],
        ['PNLIP','TCIRG1'],
        ['PNLIP','TM9SF3'],
        ['PNLIP','TMBIM6'],
        ['PNLIP','TMED3'],
        ['PNLIP','TMEM179B'],
        ['PNLIP','TRAM1'],
        ['PNLIP','TRB@'],
        ['PNLIP','TRHDE'],
        ['PNLIP','TRIM28'],
        ['PNLIP','TRIM44'],
        ['PNLIP','TSC22D3'],
        ['PNLIP','UBE2R2-AS1'],
        ['PNLIP','USP7'],
        ['PNLIP','WLS'],
        ['PNLIP','ZNF664'],
        ['PNLIPRP2','PRSS1'],
        ['PNLIPRP2','PRSS2'],
        ['PNLIPRP2','TRB@'],
        ['PNPLA6','TG'],
        ['PNPLA8','THAP5P1'],
        ['PODXL','TG'],
        ['POLR2A','PRM2'],
        ['POLR2A','TMEM99'],
        ['POMGNT1','TG'],
        ['POU2F3','TMEM99'],
        ['POU6F1','SMAGP'],
        ['PPAP2A','RNF138'],
        ['PPARA','STATH'],
        ['PPDPF','RP4-697K14.3'],
        ['PPDPF','SDC3'],
        ['PPIF','RP11-342M3.5'],
        ['PPIL2','TG'],
        ['PPP1CB','TG'],
        ['PPP1R12B','SFTPB'],
        ['PPP1R12B','STT3A'],
        ['PPP1R12B','USHBP1'],
        ['PPP1R18','SFTPA2'],
        ['PPP1R1B','STARD3'],
        ['PPP1R21','STON1'],
        ['PPP1R8','STX12'],
        ['PPP2CB','TG'],
        ['PPP2CB','TRIM28'],
        ['PPP2R1A','PRB3'],
        ['PPP3CB-AS1','TG'],
        ['PPP3CC','SORBS3'],
        ['PPP6R1','TG'],
        ['PPTC7','VPS29'],
        ['PRB1','PRH1'],
        ['PRB1','PRH1-PRR4'],
        ['PRB1','PRH2'],
        ['PRB1','PRR4'],
        ['PRB1','SFRP1'],
        ['PRB1','SMR3B'],
        ['PRB1','STATH'],
        ['PRB1','ZG16B'],
        ['PRB2','PRH1'],
        ['PRB2','PRH1-PRR4'],
        ['PRB2','PRH2'],
        ['PRB2','PRR4'],
        ['PRB2','RP11-1143G9.4'],
        ['PRB2','SMR3B'],
        ['PRB2','STATH'],
        ['PRB3','PRH1'],
        ['PRB3','PRH1-PRR4'],
        ['PRB3','PRH2'],
        ['PRB3','PRR4'],
        ['PRB3','RP11-154D6.1'],
        ['PRB3','SCARB2'],
        ['PRB3','SLPI'],
        ['PRB3','SMR3B'],
        ['PRB3','STATH'],
        ['PRB3','SYVN1'],
        ['PRB4','PRH1'],
        ['PRB4','PRH1-PRR4'],
        ['PRB4','PRH2'],
        ['PRB4','PRR4'],
        ['PRB4','SMR3B'],
        ['PRB4','STATH'],
        ['PRCP','RAB30'],
        ['PRH1','PRKCSH'],
        ['PRH1','RP11-1143G9.4'],
        ['PRH1','RP11-285F7.2'],
        ['PRH1','RPS19'],
        ['PRH1','SMR3B'],
        ['PRH1','STATH'],
        ['PRH1','TMBIM6'],
        ['PRH1','TNS1'],
        ['PRH1','UBC'],
        ['PRH1','ZG16B'],
        ['PRH1-PRR4','PRKCSH'],
        ['PRH1-PRR4','RP11-1143G9.4'],
        ['PRH1-PRR4','RP11-285F7.2'],
        ['PRH1-PRR4','RPS19'],
        ['PRH1-PRR4','SMR3B'],
        ['PRH1-PRR4','STATH'],
        ['PRH1-PRR4','TMBIM6'],
        ['PRH1-PRR4','TNS1'],
        ['PRH1-PRR4','UBC'],
        ['PRH1-PRR4','ZG16B'],
        ['PRH2','SH3BP4'],
        ['PRH2','SMR3B'],
        ['PRH2','STATH'],
        ['PRIM2','SCARB2'],
        ['PRKAA1','TTC33'],
        ['PRKAB1','TMEM233'],
        ['PRKACA','RP11-120I21.2'],
        ['PRKACB','SAMD13'],
        ['PRKACB','TXN2'],
        ['PRKAR2A','RP11-1143G9.4'],
        ['PRKCA','TPM4'],
        ['PRKCD','TIMM50'],
        ['PRKCH','TG'],
        ['PRKCSH','PRR4'],
        ['PRKX','TG'],
        ['PRPF8','S100A9'],
        ['PRPSAP1','QRICH2'],
        ['PRR11','ZNF197'],
        ['PRR18','SFT2D1'],
        ['PRR27','SMR3B'],
        ['PRR4','RP11-1143G9.4'],
        ['PRR4','RP11-285F7.2'],
        ['PRR4','RPS19'],
        ['PRR4','SH3BP4'],
        ['PRR4','SMR3B'],
        ['PRR4','STATH'],
        ['PRR4','TACC2'],
        ['PRR4','TAS2R15'],
        ['PRR4','TMBIM6'],
        ['PRR4','UBC'],
        ['PRR4','ZG16B'],
        ['PRRC2A','PTMS'],
        ['PRRX2','YKT6'],
        ['PRSS1','RNASE1'],
        ['PRSS1','RP11-290D2.6'],
        ['PRSS1','RP11-331F4.4'],
        ['PRSS1','RP11-680B3.2'],
        ['PRSS1','RPL3'],
        ['PRSS1','RPS9'],
        ['PRSS1','SLC41A1'],
        ['PRSS1','SNRNP200'],
        ['PRSS1','SYCN'],
        ['PRSS1','TIMP2'],
        ['PRSS1','TPT1'],
        ['PRSS1','UBE2R2-AS1'],
        ['PRSS2','RNASE1'],
        ['PRSS2','RP11-290D2.6'],
        ['PRSS2','RP11-331F4.4'],
        ['PRSS2','RP11-680B3.2'],
        ['PRSS2','RRBP1'],
        ['PRSS2','RUVBL1'],
        ['PRSS2','SERPINE1'],
        ['PRSS2','SH3BP4'],
        ['PRSS2','SLC41A1'],
        ['PRSS2','SNRNP200'],
        ['PRSS2','SYCN'],
        ['PRSS2','TNS3'],
        ['PRSS2','TPT1'],
        ['PRSS2','UBE2R2-AS1'],
        ['PRSS3','TRB@'],
        ['PRSS3','UBE2R2'],
        ['PRTN3','RP11-1143G9.4'],
        ['PRX','SFTPC'],
        ['PSAP','SFTPC'],
        ['PSAP','SPARC'],
        ['PSMB7','SMR3B'],
        ['PSMD11','ZNF207'],
        ['PSMD12P','SHQ1'],
        ['PSME4','TG'],
        ['PSMG3','PTRF'],
        ['PTEN','RPL11'],
        ['PTGDS','VAMP2'],
        ['PTHLH','TGM1'],
        ['PTK6','ZNF611'],
        ['PTK7','ZG16B'],
        ['PTMA','S100A9'],
        ['PTMA','SMR3B'],
        ['PTMA','TG'],
        ['PTMA','TMC5'],
        ['PTMS','R3HDM4'],
        ['PTMS','RPS2'],
        ['PTMS','RRP36'],
        ['PTMS','SEPT9'],
        ['PTMS','SULF2'],
        ['PTMS','TMED2'],
        ['PTMS','TMEM99'],
        ['PTMS','TRIM28'],
        ['PTMS','UBE2M'],
        ['PTMS','VIM'],
        ['PTOV1','TG'],
        ['PTPN22','RSBN1'],
        ['PTPN3','YBX1P6'],
        ['PTPRM','TG'],
        ['PTPRN2','TRB@'],
        ['PTPRS','STK24'],
        ['PTRF','SCARB2'],
        ['PTRF','SCUBE3'],
        ['PTRF','SLC35E2B'],
        ['PTRF','SLX4'],
        ['PTRF','SNX15'],
        ['PTRF','SOD2'],
        ['PTRF','SPG7'],
        ['PTRF','SYN3'],
        ['PTRF','TAPBP'],
        ['PTRF','TNFRSF10B'],
        ['PTRF','TTL'],
        ['PTRF','UAP1L1'],
        ['PTRF','ZSCAN16-AS1'],
        ['PTRH1','RP11-56D16.8'],
        ['PUM1','SDC3'],
        ['PVR','SERPINE1'],
        ['PVRIG','STAG3L5P'],
        ['PVRL1','TMEM99'],
        ['PVRL2','SERINC5'],
        ['PVRL2','TOMM40'],
        ['PWP1','RP11-864J10.4'],
        ['PWRN1','PWRN3'],
        ['PXDC1','RP1-223B1.1'],
        ['PXK','RPP14'],
        ['PXT1','STK38'],
        ['PYGB','TRB@'],
        ['PYY','TMEM101'],
        ['R3HDM1','UBXN4'],
        ['R3HDM2','STAC3'],
        ['RAB1A','SFTPB'],
        ['RAB1A','STAT1'],
        ['RAB1B','RAB1C'],
        ['RAB35','TG'],
        ['RAB40C','TECPR1'],
        ['RAB43P1','RABGAP1'],
        ['RAB4A','RP4-803J11.2'],
        ['RAB9A','TCEANC'],
        ['RAC1','TG'],
        ['RAD23A','UBA52'],
        ['RAD51B','SMR3B'],
        ['RASSF2','RP4-756H11.5'],
        ['RBBP4','TRA@'],
        ['RBM14','RBM4'],
        ['RBM3','RP11-1148L6.5'],
        ['RBM3','RP11-518L10.5'],
        ['RBM3','RP11-60L3.6'],
        ['RBM5','RBM6'],
        ['RBM8A','SEH1L'],
        ['RBP4','RP11-573D15.8'],
        ['RBP4','SERPINA1'],
        ['RBP4','TXNL4B'],
        ['RBP7','VSIG1'],
        ['RBPJL','TRB@'],
        ['RBPMS','TUBBP1'],
        ['RBPMS-AS1','TNNT2'],
        ['RCAN3','RP11-155G14.6'],
        ['RCAN3','RP11-212P7.3'],
        ['RDH10-AS1','RP11-434I12.4'],
        ['RDH16','RP11-74M13.5'],
        ['REG1A','TRB@'],
        ['REG1A','UBE2R2-AS1'],
        ['RFC2','TPM1'],
        ['RFFL','TG'],
        ['RFPL3S','RTCB'],
        ['RGCC','SRGN'],
        ['RGP1','RP11-112J3.16'],
        ['RGS6','RP6-114E22.1'],
        ['RHBDF1','RHBDF1P1'],
        ['RHBDL2','RP5-864K19.6'],
        ['RHBDL2','RP5-864K19.7'],
        ['RHBG','TSACC'],
        ['RHCG','SPRR3'],
        ['RIIAD1','RP11-98D18.15'],
        ['RIN2','RP5-999L4.2'],
        ['RIOK3','SRGN'],
        ['RIPK1','SERPINB9'],
        ['RLF','RP11-115D7.3'],
        ['RMND1','ZBTB2'],
        ['RN7SL151P','RN7SL2'],
        ['RN7SL2','RN7SL381P'],
        ['RN7SL711P','RP11-359P5.1'],
        ['RN7SL837P','RP5-858B6.3'],
        ['RNASE1','RP11-680B3.2'],
        ['RNASE1','TNS1'],
        ['RNASE1','TRB@'],
        ['RNF111','RP11-707P17.1'],
        ['RNF130','RTN4RL1'],
        ['RNF168','UBXN7'],
        ['RNF20','TMEM246-AS1'],
        ['RNF212B','TMEM184A'],
        ['RNF213','SFTPC'],
        ['RNF216P1','XKR8'],
        ['RNF40','ZG16B'],
        ['RNF5','SFTPA1'],
        ['RNF5','SFTPC'],
        ['RNMTL1','RP11-1143G9.4'],
        ['ROCK2','SFTPB'],
        ['RP1-122P22.2','RP5-999L4.2'],
        ['RP1-137D17.2','WDR27'],
        ['RP1-167F1.2','RP11-686D16.1'],
        ['RP1-178F15.5','S100A16'],
        ['RP1-178F15.5','TNNT2'],
        ['RP1-199J3.6','TSTD3'],
        ['RP1-1J6.2','ZHX3'],
        ['RP1-20N18.4','SPRR3'],
        ['RP1-266L20.2','TCTE3'],
        ['RP1-317E23.6','TG'],
        ['RP1-7G5.6','TCHP'],
        ['RP11-1008C21.1','TMCO5A'],
        ['RP11-1035H13.3','SMG1'],
        ['RP11-108K14.8','SCART1'],
        ['RP11-1099M24.8','RPL21P44'],
        ['RP11-109P14.2','VCPKMT'],
        ['RP11-1143G9.4','RP11-799N11.1'],
        ['RP11-1143G9.4','RPL8'],
        ['RP11-1143G9.4','S100A8'],
        ['RP11-1143G9.4','SESN1'],
        ['RP11-1143G9.4','SLC25A39'],
        ['RP11-1143G9.4','SLC2A3'],
        ['RP11-1143G9.4','SLC4A1'],
        ['RP11-1143G9.4','SMARCA4'],
        ['RP11-1143G9.4','SPTBN1'],
        ['RP11-1143G9.4','SPTY2D1-AS1'],
        ['RP11-1143G9.4','SRGN'],
        ['RP11-1143G9.4','TACC2'],
        ['RP11-1143G9.4','TFF1'],
        ['RP11-1143G9.4','TPT1'],
        ['RP11-1143G9.4','TSPAN33'],
        ['RP11-134O21.1','RP11-378A12.1'],
        ['RP11-135A1.3','RP11-407B7.3'],
        ['RP11-138I1.4','UBC'],
        ['RP11-143A12.3','RP11-315A16.3'],
        ['RP11-145E5.5','UBA52'],
        ['RP11-146B14.1','TMEM50B'],
        ['RP11-151A6.4','TMTC4'],
        ['RP11-159D12.2','SRSF1'],
        ['RP11-15B24.5','SPATA31D3'],
        ['RP11-164H13.1','TCL1B'],
        ['RP11-166O4.6','STAG3L4'],
        ['RP11-173M1.3','WSB1'],
        ['RP11-227G15.10','ZNF207'],
        ['RP11-248J23.7','TG'],
        ['RP11-252M21.6','TEX26-AS1'],
        ['RP11-257I8.2','RP11-508M8.1'],
        ['RP11-274B21.1','TLK2'],
        ['RP11-278L15.2','TM4SF1'],
        ['RP11-27M24.1','ZNF483'],
        ['RP11-285M22.1','RP11-846F4.11'],
        ['RP11-290D2.6','TRB@'],
        ['RP11-290D2.6','TRBC2'],
        ['RP11-290D2.6','TRBV25-1'],
        ['RP11-290L1.3','TG'],
        ['RP11-302F12.5','SEL1L3'],
        ['RP11-314N13.10','TG'],
        ['RP11-31K23.1','SRSF3'],
        ['RP11-326C3.16','SRP19'],
        ['RP11-331F4.4','TRB@'],
        ['RP11-331F4.4','TRBC2'],
        ['RP11-331F4.4','TRBV25-1'],
        ['RP11-332H14.1','TGFBRAP1'],
        ['RP11-33N16.3','SYNC'],
        ['RP11-340A13.1','RP11-506N2.1'],
        ['RP11-343H19.2','RSG1'],
        ['RP11-356O9.1','TTC6'],
        ['RP11-357K6.2','SMR3B'],
        ['RP11-359E19.1','RP11-470M17.2'],
        ['RP11-359E19.2','RP11-470M17.2'],
        ['RP11-360D2.1','RP3-436N22.3'],
        ['RP11-364B6.2','THAP3'],
        ['RP11-365F18.1','RPL37A'],
        ['RP11-368M16.3','RP11-548K12.10'],
        ['RP11-369E15.2','RP11-421P23.1'],
        ['RP11-369E15.4','RP11-421P23.1'],
        ['RP11-377D9.3','RP11-392P7.6'],
        ['RP11-383C6.2','TRUB1'],
        ['RP11-392O17.1','RP11-95P13.1'],
        ['RP11-3G20.2','RP11-785H20.1'],
        ['RP11-415A20.1','TMEM236'],
        ['RP11-421L21.2','SLC30A7'],
        ['RP11-424M21.1','RP11-73G16.2'],
        ['RP11-434D2.11','TNFRSF13B'],
        ['RP11-435P24.2','SERF2'],
        ['RP11-448G15.3','SLC2A9'],
        ['RP11-448P19.1','SPATA19'],
        ['RP11-469J4.3','SKIL'],
        ['RP11-479O9.4','RP11-72L22.1'],
        ['RP11-489G11.3','SCRG1'],
        ['RP11-501O2.3','RP11-639B1.1'],
        ['RP11-503L23.1','RP11-73G16.3'],
        ['RP11-510J16.5','SDR42E1'],
        ['RP11-514O12.4','TG'],
        ['RP11-521C20.2','SEMA4D'],
        ['RP11-536C5.7','RP11-574F21.3'],
        ['RP11-545P7.4','WSB1'],
        ['RP11-548P2.2','ST3GAL5-AS1'],
        ['RP11-560G2.1','SET'],
        ['RP11-562A8.4','USP50'],
        ['RP11-571E6.3','SERPINB1'],
        ['RP11-573D15.8','SERPINC1'],
        ['RP11-573D15.8','TF'],
        ['RP11-573D15.8','TFP1'],
        ['RP11-573D15.8','TMEM176B'],
        ['RP11-574K11.27','RP11-96C23.12'],
        ['RP11-58E21.5','VCPKMT'],
        ['RP11-597D13.7','TMEM144'],
        ['RP11-603J24.9','ZC3H10'],
        ['RP11-610P16.1','RP11-731D1.4'],
        ['RP11-613D13.5','SLC9A9'],
        ['RP11-613M10.9','SLC25A52'],
        ['RP11-613M10.9','SMR3B'],
        ['RP11-618G20.1','SF3B1'],
        ['RP11-62H20.1','RTN1'],
        ['RP11-63E5.1','ZNF596'],
        ['RP11-640N20.9','WDR45B'],
        ['RP11-661A12.12','TSTA3'],
        ['RP11-661C8.2','TRIP13'],
        ['RP11-666A8.8','ST6GALNAC2'],
        ['RP11-666E17.1','RP5-1014D13.2'],
        ['RP11-680B3.2','TRB@'],
        ['RP11-680B3.2','TRBC2'],
        ['RP11-680B3.2','TRBV25-1'],
        ['RP11-696N14.1','SERPINA1'],
        ['RP11-697E2.12','ZNF774'],
        ['RP11-69E11.4','TG'],
        ['RP11-6N17.4','SP2-AS1'],
        ['RP11-702H23.4','SPNS3'],
        ['RP11-702H23.4','TG'],
        ['RP11-702H23.6','SPNS3'],
        ['RP11-737G21.1','TRIM2'],
        ['RP11-73M18.2','RP11-894P9.2'],
        ['RP11-742B18.1','SPARCL1'],
        ['RP11-745L13.2','RPL7AP28'],
        ['RP11-746M1.1','TBC1D27'],
        ['RP11-747D18.1','SI'],
        ['RP11-77K12.8','TG'],
        ['RP11-797H7.1','RPL34-AS1'],
        ['RP11-79E3.3','SP100'],
        ['RP11-79N23.1','RP13-476E20.1'],
        ['RP11-814P5.1','TNNT2'],
        ['RP11-81A1.6','SLAIN2'],
        ['RP11-829H16.3','STRN3'],
        ['RP11-88E10.4','TUBGCP3'],
        ['RP11-903H12.3','TF'],
        ['RP11-91K8.4','TMEM108'],
        ['RP11-923I11.6','SCN8A'],
        ['RP11-950C14.7','TG'],
        ['RP11-973N13.4','ZBTB25'],
        ['RP3-492J12.2','ZNF831'],
        ['RP4-595K12.2','RP4-682C21.5'],
        ['RP4-630C24.3','SPAM1'],
        ['RP4-669H2.1','SLC35A1'],
        ['RP4-671G15.2','ST7L'],
        ['RP4-673D20.4','SIRPB3P'],
        ['RP4-678D15.1','SLC35A1'],
        ['RP5-1000K24.2','ZC3H12B'],
        ['RP5-1024N4.2','SLC5A9'],
        ['RP5-1024N4.5','SPATA6'],
        ['RP5-1065J22.8','TAF13'],
        ['RP5-1098D14.1','TEX35'],
        ['RP5-857K21.4','TPM1'],
        ['RP5-928E24.2','ZNF711'],
        ['RP5-972B16.2','TG'],
        ['RPL11','TCEB3'],
        ['RPL14','S100A16'],
        ['RPL24','ZBTB11'],
        ['RPL3','TRB@'],
        ['RPL30','TG'],
        ['RPL37','TNFSF10'],
        ['RPL37A','TRB@'],
        ['RPL37A','VEGFA'],
        ['RPL38','TTYH2'],
        ['RPL7L1','RPS2P55'],
        ['RPL8','SFTPC'],
        ['RPS12','S100A9'],
        ['RPS19','SNX6'],
        ['RPS2','STK39'],
        ['RPS2','YWHAQ'],
        ['RPS20','RSF1'],
        ['RPS27A','UBA52'],
        ['RPS29','SLC34A2'],
        ['RPS29','SYN3'],
        ['RPS4X','RPS4XP1'],
        ['RPS4X','RPS4XP19'],
        ['RPS4X','RPS4XP2'],
        ['RPS4X','RPS4XP5'],
        ['RPS6KA2','TG'],
        ['RPS9','TRB@'],
        ['RRBP1','TG'],
        ['RSPH4A','RWDD1'],
        ['RUSC2','SMR3B'],
        ['RUVBL1','SMR3B'],
        ['RUVBL1','TRB@'],
        ['RUVBL1','TRBC2'],
        ['RUVBL1','TRBV25-1'],
        ['RUVBL2','SMR3B'],
        ['RWDD3','TMEM56'],
        ['RYBP','SFTPC'],
        ['RYR2','TG'],
        ['S100A10','TNS3'],
        ['S100A13','S100A16'],
        ['S100A13','TNNT2'],
        ['S100A16','S100A2'],
        ['S100A9','SLC25A37'],
        ['S100A9','SLC4A1'],
        ['S100A9','TKT'],
        ['SAA1','SERPINA1'],
        ['SAA1','TIMP3'],
        ['SAA2','SERPINA1'],
        ['SAA2','TMBIM6'],
        ['SAA2-SAA4','SERPINA1'],
        ['SAA4','SERPINA1'],
        ['SAG','VPS13C'],
        ['SAMD8','VDAC2'],
        ['SAR1A','SAR1AP4'],
        ['SAR1A','TG'],
        ['SARS','SMR3B'],
        ['SBF2','TG'],
        ['SCAMP4','SFTPB'],
        ['SCAND2P','ZSCAN2'],
        ['SCD5','SDC3'],
        ['SCD5','SEPT4-AS1'],
        ['SCD5','TPM4'],
        ['SCG5','TRB@'],
        ['SCGB1B2P','SCGB2B2'],
        ['SCNN1A','TNFRSF1A'],
        ['SCP2','TUBB'],
        ['SCUBE3','TG'],
        ['SCUBE3','ZNF76'],
        ['SDCBP2-AS1','ZNF587B'],
        ['SEC11A','SSR3'],
        ['SEC24C','SPAG9'],
        ['SEC61A1','TRB@'],
        ['SEC62','SMR3B'],
        ['SEL1L','SFTPB'],
        ['SELPLG','SFTPA1'],
        ['SEMA3B','U73166.2'],
        ['SEPN1','SFTPB'],
        ['SEPN1','TRB@'],
        ['SEPT11','SERPINE1'],
        ['SEPT2','TG'],
        ['SEPT9','SFTPC'],
        ['SEPT9','SMR3B'],
        ['SERINC1','SUMF2'],
        ['SERINC3','TG'],
        ['SERP1','SMR3B'],
        ['SERP1','TG'],
        ['SERPINA1','SERPINC1'],
        ['SERPINA1','TAT'],
        ['SERPINA1','TGM2'],
        ['SERPINA1','TMBIM6'],
        ['SERPINA1','TXNL4B'],
        ['SERPINA1','UGT2B4'],
        ['SERPINB3','SERPINB4'],
        ['SERPINE1','SHROOM3'],
        ['SERPINE1','TRBV25-1'],
        ['SERPINE1','TXNDC5'],
        ['SERPINF2','WDR81'],
        ['SERPING1','TXNL4B'],
        ['SET','SETP11'],
        ['SET','SETP3'],
        ['SF1','TRB@'],
        ['SFN','SLFN5'],
        ['SFPQ','SMR3B'],
        ['SFPQ','SRGN'],
        ['SFRP1','SMAD6'],
        ['SFTPA1','SFTPB'],
        ['SFTPA1','SFTPC'],
        ['SFTPA1','SPTBN1'],
        ['SFTPA1','SQSTM1'],
        ['SFTPA1','SUSD2'],
        ['SFTPA1','TAGLN'],
        ['SFTPA1','TMBIM6'],
        ['SFTPA2','SFTPC'],
        ['SFTPA2','SOCS3'],
        ['SFTPA2','STAT3'],
        ['SFTPA2','SYN3'],
        ['SFTPA2','USP36'],
        ['SFTPA2','ZFP36'],
        ['SFTPA2','ZMIZ1'],
        ['SFTPB','SFTPC'],
        ['SFTPB','SPTBN1'],
        ['SFTPB','SQSTM1'],
        ['SFTPB','SUMF2'],
        ['SFTPB','SYN3'],
        ['SFTPB','SYNE1'],
        ['SFTPB','TBCEL'],
        ['SFTPB','TFPI'],
        ['SFTPB','THAP5'],
        ['SFTPB','TIMM10B'],
        ['SFTPB','TMEM119'],
        ['SFTPB','TMEM50B'],
        ['SFTPB','TNFRSF10B'],
        ['SFTPB','TTLL3'],
        ['SFTPB','TYROBP'],
        ['SFTPB','WARS'],
        ['SFTPB','WFS1'],
        ['SFTPB','XXYLT1'],
        ['SFTPB','ZFP36'],
        ['SFTPB','ZFP36L1'],
        ['SFTPB','ZFP36L2'],
        ['SFTPC','SH3D19'],
        ['SFTPC','SLC34A2'],
        ['SFTPC','SLCO2A1'],
        ['SFTPC','SPARC'],
        ['SFTPC','SPOCK2'],
        ['SFTPC','SUPT5H'],
        ['SFTPC','TACSTD2'],
        ['SFTPC','TSIX'],
        ['SFTPC','VIM'],
        ['SFTPC','ZFP36'],
        ['SFTPC','ZNF609'],
        ['SFXN3','TPM4'],
        ['SH3BGR','WRB'],
        ['SH3BGRL2','SMR3B'],
        ['SH3BP4','TRB@'],
        ['SH3BP4','TRBC2'],
        ['SH3BP4','TRBV25-1'],
        ['SHISA7','UBE2S'],
        ['SHISA9','U91319.1'],
        ['SHQ1','TG'],
        ['SIDT2','TAGLN'],
        ['SIGLEC16','WDR35'],
        ['SIPA1L3','TG'],
        ['SIPA1L3','TRB@'],
        ['SIRPA','SMR3B'],
        ['SLAIN2','SLC10A4'],
        ['SLAIN2','ST20'],
        ['SLAIN2','ST20-MTHFS'],
        ['SLC16A3','TG'],
        ['SLC25A29','TPO'],
        ['SLC25A29','YY1'],
        ['SLC25A3','SRGN'],
        ['SLC25A3','SUGCT'],
        ['SLC25A3','TET1'],
        ['SLC26A4','TG'],
        ['SLC26A7','TG'],
        ['SLC29A3','UNC5B'],
        ['SLC2A2','TNIK'],
        ['SLC30A9','TG'],
        ['SLC35F6','TPM4'],
        ['SLC3A2','TG'],
        ['SLC41A1','TRB@'],
        ['SLC41A3','TG'],
        ['SLC43A3','TG'],
        ['SLC6A16','TEAD2'],
        ['SLC7A5','SMG1'],
        ['SLC9A8','ZNFX1-AS1'],
        ['SLPI','SMR3B'],
        ['SLTM','TG'],
        ['SMAD2','TG'],
        ['SMR3B','SPSB3'],
        ['SMR3B','SPTBN1'],
        ['SMR3B','SREBF2'],
        ['SMR3B','SRRM2'],
        ['SMR3B','STATH'],
        ['SMR3B','SUPT5H'],
        ['SMR3B','SURF4'],
        ['SMR3B','SYN3'],
        ['SMR3B','TLN2'],
        ['SMR3B','TMED10'],
        ['SMR3B','TMEM120B'],
        ['SMR3B','TNS1'],
        ['SMR3B','TPD52L1'],
        ['SMR3B','TRB@'],
        ['SMR3B','TRIM29'],
        ['SMR3B','TRPC4AP'],
        ['SMR3B','TXNDC5'],
        ['SMR3B','WNK2'],
        ['SMR3B','WWC1'],
        ['SMR3B','YBX3'],
        ['SMR3B','ZCCHC14'],
        ['SMR3B','ZFP36L1'],
        ['SMR3B','ZG16B'],
        ['SMR3B','ZNF827'],
        ['SMTN','VARS2'],
        ['SNAP25','VSNL1'],
        ['SNAP25-AS1','SYN2'],
        ['SNHG14','SNURF'],
        ['SNRPD2','TG'],
        ['SNX2','SNX24'],
        ['SOD2','TTN'],
        ['SOGA1','TG'],
        ['SORBS2','TG'],
        ['SPARC','TG'],
        ['SPARC','TNS1'],
        ['SPARC','TPM2'],
        ['SPARC','VWF'],
        ['SPATS2','TMEM99'],
        ['SPINK1','TRB@'],
        ['SPPL3','TG'],
        ['SPSB3','TAGLN'],
        ['SQLE','ZNF572'],
        ['SREBF2','TG'],
        ['SRGN','TKT'],
        ['SRGN','USP15'],
        ['SRGN','XIRP2'],
        ['SRI','STEAP4'],
        ['SRRM2','TIMP3'],
        ['SRRM2','TMEM99'],
        ['SRSF10','ZFP69'],
        ['SSR2','UBQLN4'],
        ['SSR3','WIPI2'],
        ['SSR3','ZNF684'],
        ['ST3GAL1','TPO'],
        ['ST6GAL1','TG'],
        ['STAG3','STAG3L5P'],
        ['STAG3','STAG3L5P-PVRIG2P-PILRB'],
        ['STARD7','TPT1'],
        ['STATH','UBE2Z'],
        ['STATH','UBXN4'],
        ['STC1','TG'],
        ['STIP1','TAPBP'],
        ['STMN4','TRIM35'],
        ['SULT2A1','TAT'],
        ['SUMF2','TG'],
        ['SUMO1','SUMO1P4'],
        ['SUMO3','UBE2G2'],
        ['SVEP1','TIMP3'],
        ['SVEP1','TXNDC8'],
        ['SYBU','TRB@'],
        ['SYCN','TRB@'],
        ['SYN1','TUBB4A'],
        ['SYN3','TG'],
        ['SYN3','TNS1'],
        ['SYNC','TRA@'],
        ['SYNE1','TG'],
        ['SYT8','TNNI2'],
        ['SYTL4','TSPAN6'],
        ['TAB2','TG'],
        ['TAPBP','TTC39B'],
        ['TAS2R5','TAS2R6'],
        ['TAT-AS1','TF'],
        ['TBC1D23','TMEM30C'],
        ['TBC1D9','TNRC18P1'],
        ['TBCEL','TECTA'],
        ['TCF25','TRB@'],
        ['TECPR1','TRB@'],
        ['TEX41','ZNF417'],
        ['TF','TXNL4B'],
        ['TFF1','TFF2'],
        ['TG','TM2D3'],
        ['TG','TMED10'],
        ['TG','UBA1'],
        ['TG','UBE2H'],
        ['TG','UQCC2'],
        ['TG','USP40'],
        ['TG','USP54'],
        ['TG','UTRN'],
        ['TG','VAC14'],
        ['TG','VAV3'],
        ['TG','VMP1'],
        ['TG','WAC'],
        ['TG','WDR86'],
        ['TG','WHSC1'],
        ['TG','XPO1'],
        ['TG','XRCC5'],
        ['TG','YWHAB'],
        ['TG','ZFAND3'],
        ['TG','ZNF704'],
        ['THY1','VIM-AS1'],
        ['TIMP1','VIM'],
        ['TIMP3','TPM2'],
        ['TLE1','TLE1P1'],
        ['TLK2','ZNF880'],
        ['TLN1','TRB@'],
        ['TMC6','TRIM69'],
        ['TMEM131','YIPF5'],
        ['TMEM189-UBE2V1','UBE2V1P6'],
        ['TMPRSS5','ZW10'],
        ['TMSB10','TPM2'],
        ['TNFSF10','TVP23B'],
        ['TNS3','TRB@'],
        ['TNS3','TRBC2'],
        ['TNS3','TRBV25-1'],
        ['TPO','VWF'],
        ['TPPP3','ZDHHC1'],
        ['TPST2','TRB@'],
        ['TPT1','TRB@'],
        ['TRAC','TRAJ5'],
        ['TRAC','TRAV10'],
        ['TRAC','TRAV12-1'],
        ['TRAC','TRAV12-2'],
        ['TRAC','TRAV12-3'],
        ['TRAC','TRAV14DV4'],
        ['TRAC','TRAV17'],
        ['TRAC','TRAV19'],
        ['TRAC','TRAV21'],
        ['TRAC','TRAV22'],
        ['TRAC','TRAV25'],
        ['TRAC','TRAV26-1'],
        ['TRAC','TRAV35'],
        ['TRAC','TRAV9-2'],
        ['TRAJ39','TRAV12-2'],
        ['TRAJ5','TRAV19'],
        ['TRAV18','TRAV19'],
        ['TRB@','TRBC2'],
        ['TRB@','TREH'],
        ['TRB@','TUBB'],
        ['TRB@','UBE2R2-AS1'],
        ['TRB@','ULK1'],
        ['TRBC2','TRBV10-2'],
        ['TRBC2','TRBV19'],
        ['TRBC2','TRBV2'],
        ['TRBC2','TRBV20-1'],
        ['TRBC2','TRBV28'],
        ['TRBC2','TRBV3-1'],
        ['TRBC2','TRBV30'],
        ['TRBC2','TRBV4-1'],
        ['TRBC2','TRBV4-2'],
        ['TRBC2','TRBV5-1'],
        ['TRBC2','TRBV6-5'],
        ['TRBC2','TRBV7-3'],
        ['TRBC2','TRBV9'],
        ['TREM1','TREML3P'],
        ['TTC38','UQCRB'],
        ['TUBA1C','TUBA3C'],
        ['TUBA1C','TUBAP2'],
        ['TUBA4A','TUBAP2'],
        ['TXNL4B','XXBAC-BPG116M5.17'],
        ['TYW1','TYW1B'],
        ['UBA52','UBB'],
        ['UBA52','UBBP4'],
        ['UBA52','UBC'],
        ['UBA52','UBL4A'],
        ['UBA52','ZFAND4'],
        ['UBE2I','UNKL'],
        ['UGT1A1','UGT1A2P'],
        ['UPP2','UPP2-IT1'],
        ['VN1R37P','VN1R40P'],
        ['VWF','ZG16B'],
        ['WDR41','WWTR1'],
        ['ZC3H18','ZFPM1'],
        ['ZFP30','ZNF607'],
        ['ZFP41','ZNF674'],
        ['ZFP90','ZNF674'],
        ['ZFS-4','ZNF674'],
        ['ZKSCAN5','ZNF674'],
        ['ZNF137P','ZNF701'],
        ['ZNF141','ZNF876P'],
        ['ZNF224','ZNF225'],
        ['ZNF227','ZNF233'],
        ['ZNF232','ZNF674'],
        ['ZNF28','ZNF860'],
        ['ZNF32','ZNF674'],
        ['ZNF331','ZNF765'],
        ['ZNF33A','ZNF33B'],
        ['ZNF345','ZNF790-AS1'],
        ['ZNF350','ZNF674'],
        ['ZNF37A','ZNF37BP'],
        ['ZNF415','ZNF674'],
        ['ZNF517','ZNF7'],
        ['ZNF573','ZNF607'],
        ['ZNF586','ZNF776'],
        ['ZNF621','ZNF674'],
        ['ZNF674','ZNF675'],
        ['ZNF674','ZNF702P'],
        ['ZNF674','ZNF78L1'],
        ['ZNF674','ZNF81'],
        ['ZNF674','ZNF93'],
        ['ZNF674','ZSCAN30'],
        ['ZNF708','ZNF738'],
        ['ZNF761','ZNF765'],
        ['ABHD11','SNAP23'],
        ['ABLIM1','BTK'],
        ['AC004066.3','PPA2'],
        ['AC005037.3','NDUFB3'],
        ['AC006116.14','AC006116.22'],
        ['AC015849.16','CCL5'],
        ['AC074141.3','ZNF577'],
        ['AC104532.2','SURF1'],
        ['ACAA1','FBP1'],
        ['ACADSB','ARHGAP32'],
        ['ACOX1','DNAL1'],
        ['ACSL1','COL4A2'],
        ['ACSL1','IPO11'],
        ['ACTB','TOMM20'],
        ['ACTN4','BCAP31'],
        ['ADCY10','MPC2'],
        ['AF131217.1','MSMB'],
        ['AIG1','GSK3B'],
        ['AIG1','PHF20L1'],
        ['AIM1','C6ORF203'],
        ['AIMP1','ZNF605'],
        ['AKAP12','RBM39'],
        ['AKR1A1','NBR2'],
        ['ALDOA','NUDC'],
        ['ANO6','BROX'],
        ['ANOS2P','TMSB4Y'],
        ['ANXA3','ZNF350'],
        ['AP003068.18','AP003068.23'],
        ['AP1B1','QRICH1'],
        ['AP3D1','RNF10'],
        ['APCDD1L-AS1','VAPB'],
        ['APPL1','COPA'],
        ['ARFGAP2','CIB1'],
        ['ARHGEF28','MTX2'],
        ['ARL2-SNX15','WSB1'],
        ['ARNT','CTSK'],
        ['ARPC1A','MMD'],
        ['ARPC1B','PDE3A'],
        ['ARPC2','LCORL'],
        ['ASAH1','B2M'],
        ['ASAH1','PCM1'],
        ['ASB12','MTMR8'],
        ['ASCC3','PDE10A'],
        ['ATG13','NRAS'],
        ['ATG13','SMIM14'],
        ['ATG7','RAF1'],
        ['ATL3','RAD17'],
        ['ATM','LINC01091'],
        ['ATP10B','UPK1A'],
        ['ATP13A5-AS1','HRASLS'],
        ['ATP2C1','MBNL1'],
        ['ATP5B','MIR762HG'],
        ['ATP9A','NFATC2'],
        ['ATRAID','GTF3C2'],
        ['B2M','CD36'],
        ['B2M','CD99'],
        ['B2M','ERV3-1'],
        ['B2M','KIF2A'],
        ['B2M','MYCBP'],
        ['B2M','SQRDL'],
        ['B2M','TLN1'],
        ['B2M','TMEM91'],
        ['B2M','TRAK2'],
        ['B2M','WIPF1'],
        ['B3GNT2','COMMD1'],
        ['BAZ2B','MARCH7'],
        ['BCAS1','MID1IP1'],
        ['BCAS1','PRKAA1'],
        ['BCL10','NIPAL1'],
        ['BHLHE41','CTR9'],
        ['BHMT','ZNF414'],
        ['BLOC1S6','GNG4'],
        ['BLOC1S6','SMAD4'],
        ['BPNT1','TSPAN31'],
        ['BZW1','NORAD'],
        ['C11ORF63','GOLGA4'],
        ['C11ORF80','SF1'],
        ['C12ORF45','LYRM2'],
        ['C15ORF52','PGPEP1'],
        ['C16ORF72','GTPBP10'],
        ['C21ORF62-AS1','HOMER1'],
        ['C6ORF25','CCL5'],
        ['C6ORF25','F2R'],
        ['C6ORF25','LIMS1'],
        ['C6ORF25','PLEKHB2'],
        ['C6ORF25','SNAP29'],
        ['C6ORF25','STAT3'],
        ['C6ORF25','TRIM58'],
        ['C7ORF55','TXNDC17'],
        ['C7ORF55-LUC7L2','TXNDC17'],
        ['C7ORF73','NSUN5'],
        ['CA2','MEIS1'],
        ['CA5B','NT5C'],
        ['CALM3','GMPR2'],
        ['CAMK2D','GNAS'],
        ['CAMTA1','CRYL1'],
        ['CASP6','PLA2G12A'],
        ['CASP6','TRA2A'],
        ['CASP8','FAM210B'],
        ['CASP8','LONP2'],
        ['CAT','OGFOD1'],
        ['CCAR1','SP140L'],
        ['CCDC113','CEP70'],
        ['CCDC34','LGR4'],
        ['CCDC69','TPMT'],
        ['CCL5','TMED4'],
        ['CCNL1','SMG1'],
        ['CCSER1','FUBP1'],
        ['CCT6A','CCT6P1'],
        ['CD2AP','GTF3C2'],
        ['CDC37','DDX42'],
        ['CDC42EP3','LINC00211'],
        ['CENPC','NR2F2-AS1'],
        ['CENPU','KIAA0232'],
        ['CERS2','FTH1'],
        ['CFD','ICA1'],
        ['CFL1','LDHA'],
        ['CHCHD2','TCEA3'],
        ['CHD1L','GAMT'],
        ['CHD2','TRIM52'],
        ['CHFR','YKT6'],
        ['CHP1','FBXO27'],
        ['CHP1','NAA40'],
        ['CHRAC1','PAXIP1-AS1'],
        ['CHURC1','GLMP'],
        ['CIAO1','ZNF264'],
        ['CLCC1','ZNF33A'],
        ['CLCN3','EGF'],
        ['CLCN3','RGS18'],
        ['CLHC1','LSMEM1'],
        ['CMBL','CNN2'],
        ['CMBL','LIG3'],
        ['CNPY2','PARP3'],
        ['CNST','MOB1A'],
        ['COL4A3BP','NSA2'],
        ['COMMD2','ICK'],
        ['COMMD7','FTH1'],
        ['COQ10B','R3HDM2'],
        ['COX10-AS1','SH2D4A'],
        ['COX6B1','MDK'],
        ['COX7B','XPR1'],
        ['CRELD2','DPF2'],
        ['CSNK1E','FBXO21'],
        ['CSTB','UBE2L3'],
        ['CTSA','TJP2'],
        ['CTSD','ZYX'],
        ['CTSL','ZC3H14'],
        ['CUL4B','MBD3'],
        ['CYB561D1','VPS53'],
        ['CYCS','ZNF561'],
        ['DAP3','UBAP2L'],
        ['DAPP1','SNAP23'],
        ['DBT','SFXN2'],
        ['DCAF11','PSME1'],
        ['DCAF13','RGS10'],
        ['DCLRE1A','IKZF5'],
        ['DDI2','SUMF2'],
        ['DEGS2','PALLD'],
        ['DHRS2','ZBTB40'],
        ['DMKN','EDEM2'],
        ['DNM1L','GIT2'],
        ['DRAM1','NR3C1'],
        ['DYRK2','EIF1'],
        ['E2F3','VPS53'],
        ['EIF1','HNRNPA0'],
        ['EIF2AK1','TMSB4X'],
        ['EIF2AK4','ZNF121'],
        ['EIF3I','HP1BP3'],
        ['EIF3M','FDFT1'],
        ['ELF1','HMGB1'],
        ['ENC1','UPK1A'],
        ['ERCC6L2','TMEM245'],
        ['ERI3','NOL10'],
        ['ERICH1','ZNF624'],
        ['ERP27','PSMB4'],
        ['ERV3-1','SDPR'],
        ['ESRP1','WRNIP1'],
        ['ESRP2','FMO9P'],
        ['ESRRA','KRT18'],
        ['ESRRG','NEDD9'],
        ['EVI5','PTCD3'],
        ['EXOC1','UPK1A'],
        ['EXOC7','RABEP1'],
        ['EZR','NBEAL2'],
        ['F2R','LPCAT3'],
        ['FAM117B','YWHAB'],
        ['FAM120AOS','LYST'],
        ['FAM210B','MRPL49'],
        ['FAM213A','LIN7C'],
        ['FAM63B','GTPBP3'],
        ['FAM89B','KLC1'],
        ['FASTKD2','ZNF638'],
        ['FBP1','UPF1'],
        ['FBXL20','PTGIR'],
        ['FGFR1OP2','MBNL1'],
        ['FIS1','HSPE1'],
        ['FKBP3','TOP2B'],
        ['FRMD4B','MALAT1'],
        ['FRS2','FTL'],
        ['FTH1','POLR2I'],
        ['FTH1','TSC22D1'],
        ['FTSJ3','SAMD11'],
        ['G3BP1','TNFAIP8'],
        ['GAN','TMEM50B'],
        ['GAPDH','TUBB4B'],
        ['GCNT3','PGPEP1'],
        ['GFOD2','VTI1B'],
        ['GGACT','ZC3H8'],
        ['GGPS1','MAPKAPK5'],
        ['GLIS3','RFX3'],
        ['GLRX3','SLC27A2'],
        ['GLTP','PCBD2'],
        ['GLTP','SAR1B'],
        ['GLTP','SUMF2'],
        ['GMPR','HCG18'],
        ['GNPTG','ZNF502'],
        ['GOLM1','SOCS6'],
        ['GSR','ZNF91'],
        ['GSTK1','UBA52'],
        ['GSTO2','PRRC2C'],
        ['HBP1','KMT2E'],
        ['HECA','XXBAC-BPG32J3.22'],
        ['HERPUD2','MSMO1'],
        ['HERPUD2','SEPT7-AS1'],
        ['HIST1H2AG','HIST1H4I'],
        ['HIST1H2AG','TSC22D1'],
        ['HIST1H2BF','HIST1H4E'],
        ['HIST1H2BH','TRIM13'],
        ['HMG20A','SRSF8'],
        ['HMG20B','TAX1BP3'],
        ['HNRNPA2B1','SKAP2'],
        ['HNRNPF','ZCCHC24'],
        ['HNRNPK','KIF2A'],
        ['HPGD','HSPA8'],
        ['HPS1','SMG1'],
        ['HS1BP3','PDF'],
        ['IFNAR2','NFX1'],
        ['IFT80','TMEM87A'],
        ['IL1R1','MRPS27'],
        ['INO80B-WBP1','MYL12B'],
        ['ISG20L2','SNAP23'],
        ['ITM2B','LINC00534'],
        ['ITM2B','MAPRE2'],
        ['JMJD7','TRAK1'],
        ['JUN','SLAIN2'],
        ['KCTD11','MIR4458HG'],
        ['KDM5B','PPP1R12B'],
        ['KHK','PDZK1IP1'],
        ['KIAA0196','VGLL4'],
        ['KIAA0368','N4BP2L2'],
        ['KIAA0391','NSD1'],
        ['KIAA0513','ORAI2'],
        ['KIAA1217','PPFIA1'],
        ['KIF13A','RPL8'],
        ['KIF2A','NDUFS4'],
        ['KLK4','KLKP1'],
        ['KRT19','MFN2'],
        ['KRT19','TOP1'],
        ['KRT7','P2RX2'],
        ['LEPROT','THOC2'],
        ['LINC00472','RALGPS2'],
        ['LINC00969','PYGL'],
        ['LINC00989','LINC01088'],
        ['LLPH','LYZ'],
        ['LNX2','MTHFR'],
        ['LRRFIP1','PCBD2'],
        ['LYPLAL1','SMYD3'],
        ['MAFF','STAP2'],
        ['MALAT1','SOD2'],
        ['MALAT1','VPS36'],
        ['MAP3K7CL','PPBP'],
        ['MARC2','ZFP91-CNTF'],
        ['MAX','SFI1'],
        ['MBD5','ORC4'],
        ['MCCC2','SMIM14'],
        ['MDH1','TMBIM6'],
        ['MDM2','VPS4B'],
        ['MFSD11','RND3'],
        ['MICU1','UPK1A'],
        ['MID2','PTK2'],
        ['MIF4GD','NFKBIA'],
        ['MKL2','PARN'],
        ['MKRN2OS','NONO'],
        ['MLPH','VPS28'],
        ['MNS1','TEX9'],
        ['MPI','SPINK1'],
        ['MPP7','RAB18'],
        ['MRPL14','RAB40C'],
        ['MRPL9','PI4KA'],
        ['MRPS27','SOCS2-AS1'],
        ['MRPS28','TPD52'],
        ['MTFMT','XXBAC-BPG299F13.17'],
        ['MTURN','ZNRF2'],
        ['NAP1L1','SEMA4D'],
        ['NCBP2','NUDT19'],
        ['NCCRP1','TMF1'],
        ['NDUFB10','USO1'],
        ['NFKBIZ','TGM4'],
        ['NIF3L1','ORC4'],
        ['NME2','SLK'],
        ['NQO1','ZNF486'],
        ['NRIP1','USP25'],
        ['NSD1','PWWP2A'],
        ['NSMCE2','PLCL1'],
        ['NT5C2','SNAP23'],
        ['NUDCD1','SNX16'],
        ['OCEL1','TMEM97'],
        ['OIP5-AS1','RSU1'],
        ['OPA3','ZNF451'],
        ['PACRG','TMEM242'],
        ['PAFAH1B2','USP54'],
        ['PAK2','SMIM14'],
        ['PANK3','TRIM68'],
        ['PAQR7','RPS23'],
        ['PCLO','UCP2'],
        ['PCMTD1','PXDNL'],
        ['PDXK','ZNF593'],
        ['PEA15','S100A11'],
        ['PFDN6','RNF19B'],
        ['PLCB3','TXLNA'],
        ['PLPP1','RBM47'],
        ['PMPCB','UBE2L6'],
        ['POLR3H','PRPSAP2'],
        ['PPARG','TSEN2'],
        ['PPARGC1A','RSF1'],
        ['PRICKLE2','RPRD1A'],
        ['PRKAR1B','RASA3'],
        ['PSPC1','ZMYM2'],
        ['PTN','TPK1'],
        ['QPRT','TMED4'],
        ['RAB2A','SGMS1'],
        ['RAB31','RASA3'],
        ['RAB31','VAPA'],
        ['RAI14','RTN4'],
        ['RASSF8','SSPN'],
        ['RBFA','ZMYND11'],
        ['RBM3','TBC1D25'],
        ['RBM5','RHOA'],
        ['RBM6','RHOA'],
        ['REPS2','SETDB1'],
        ['REXO2','RPUSD4'],
        ['RFFL','RUBCN'],
        ['RNF185','UQCR10'],
        ['RNF220','TRIM28'],
        ['RNF24','SLC35E1'],
        ['RNGTT','UTRN'],
        ['RNH1','VAMP2'],
        ['RNPS1','SRSF5'],
        ['RSPH1','VPS13A'],
        ['S100P','YTHDF2'],
        ['SAMD8','SELT'],
        ['SENP3-EIF4A1','UBL5'],
        ['SEPT2','SKIL'],
        ['SLC2A6','SMC6'],
        ['SLC35E3','THNSL1'],
        ['SNAP23','TRIM58'],
        ['SPP1','UBE2D1'],
        ['SSR2','TRIM25'],
        ['ST3GAL1','TCEA1'],
        ['TBC1D9','TM4SF1'],
        ['THSD4','TPD52'],
        ['TMED4','ZNFX1'],
        ['TMEM198B','YWHAB'],
        ['TMEM41B','ZNF217'],
        ['TOMM70','ZNF490'],
        ['TPI1','TRIM27'],
        ['TPM4','UBXN2B'],
        ['UQCRC2','ZSWIM7'],
        ['VPS8','ZNF699'],
        ['ZNF267','ZNF720'],
        ['A2ML1','CRNN'],
        ['ABCA8','PIGR'],
        ['AC019349.5','AL513122.2'],
        ['AC027682.1','SFTPC'],
        ['AC096579.15','IGKV4-1'],
        ['ACTR2','LYZ'],
        ['AES','LYZ'],
        ['AHNAK','TRB@'],
        ['AL109923.1','DYNLRB1'],
        ['AL513122.2','KRT13'],
        ['ALB','CYP51A1'],
        ['ALB','ZNF511-PRAP1'],
        ['ALDOB','REG3A'],
        ['ARL6IP1','LYZ'],
        ['ATP5A1','LYZ'],
        ['BAIAP2','LYZ'],
        ['C7','HNRNPU'],
        ['CLSTN1','PRSS1'],
        ['CLSTN1','TRB@'],
        ['COL18A1','PRSS1'],
        ['COX6C','LYZ'],
        ['CPA1','SLC3A1'],
        ['CPB1','MAN1A2'],
        ['CPB1','PPIB'],
        ['CTSG','ITGB2'],
        ['CUL3','SMR3B'],
        ['DET1','MRPL46'],
        ['DHX40','RNFT1'],
        ['DNM2','SMR3B'],
        ['DOCK1','TG'],
        ['EEF1A1','PRH1'],
        ['EEF1D','PNLIP'],
        ['ENAH','MUC7'],
        ['ERRFI1','PNLIP'],
        ['FCGBP','HNRNPU'],
        ['H6PD','LYZ'],
        ['HIF1A','LYZ'],
        ['HTN3','IL6ST'],
        ['IGF2','IGFBP5'],
        ['IGF2','ITGB1'],
        ['IGF2','PLPP3'],
        ['IGF2','STAU1'],
        ['KIAA1191','LYZ'],
        ['LYZ','NDRG2'],
        ['LYZ','PLEC'],
        ['LYZ','PPP2CA'],
        ['LYZ','PRTN3'],
        ['LYZ','SLC12A6'],
        ['LYZ','SOCS7'],
        ['PDLIM1','TG'],
        ['PKD1','PKD1P6-NPIPP1'],
        ['PLCB4','SMR3B'],
        ['PLEC','SMR3B'],
        ['PNLIP','SLC25A26'],
        ['PNPLA7','TG'],
        ['PPIB','PRH1-PRR4'],
        ['RPL3','TG'],
        ['SERPINA1','SERPING1'],
        ['SERPINE1','TRB@'],
        ['SVIL','ZG16B'],
        ['ACTB','EIF4A1'],
        ['AL513122.1','PTRF'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BRS3','HTATSF1'],
        ['COQ8B','NUMBL'],
        ['HSPE1','MOB4'],
        ['SIDT2','TAGLN'],
        ['AC004980.8','PMS2'],
        ['AC004980.8','UPK3B'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['CAP1','LHX4'],
        ['CATSPERB','POLR3G'],
        ['COQ8B','NUMBL'],
        ['GSR','GTF2E2'],
        ['IGK@','SLC31A1'],
        ['MAN1B1-AS1','XIAP'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['PSPC1','ZMYM5'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['ACOT8','TNNC2'],
        ['C2ORF49','RN7SL619P'],
        ['C5ORF46','SPINK1'],
        ['CADM4','ZNF428'],
        ['CD81','TSSC4'],
        ['CDR2L','ICT1'],
        ['CECR7','IL17RA'],
        ['CHCHD10','VPREB3'],
        ['CHD4','NOP2'],
        ['CLCF1','POLD4'],
        ['CLMP','HSPA8'],
        ['COA6','ACTB'],
        ['COL7A1','UCN2'],
        ['COL8A1','CMSS1'],
        ['CSF2RA','PLIN3'],
        ['CTBS','GNG5'],
        ['CTSC','RAB38'],
        ['CYR61','HLA-B'],
        ['DHRS1','RABGGTA'],
        ['DPM2','PIP5KL1'],
        ['EFEMP2','CFL1'],
        ['EIF3K','ACTN4'],
        ['ENO1','EDARADD'],
        ['FAM96B','RRAD'],
        ['FKBP1A','SDCBP2'],
        ['GABARAP','PHF23'],
        ['HOXD11','NFE2L2'],
        ['HSF4','NOL3'],
        ['HSPA4','B2M'],
        ['HSPE1','MOB4'],
        ['INTS3','SLC27A3'],
        ['ITCH','DYNLRB1'],
        ['KIFC1','PHF1'],
        ['KSR1','LGALS9'],
        ['LCAT','PSMB10'],
        ['LMAN2','MXD3'],
        ['MAD1L1','MROH3P'],
        ['MED22','SURF6'],
        ['MED8','ELOVL1'],
        ['MFSD7','ATP5I'],
        ['MTAP','CDKN2B-AS1'],
        ['MTATP6P1','MTRNR2L8'],
        ['MYL9','TGIF2'],
        ['NADKD1','SDK2'],
        ['NECAP1','CLEC4A'],
        ['NFATC2IP','SPNS1'],
        ['NXN','GLOD4'],
        ['OBSL1','CHPF'],
        ['PBXIP1','PMVK'],
        ['PELI3','DPP3'],
        ['PLEKHO2','ANKDD1A'],
        ['POLA2','CDC42EP2'],
        ['PRAF2','TFE3'],
        ['PRKAA1','TTC33'],
        ['PSPH','BNIPL'],
        ['RAB11FIP1','PRKCSH'],
        ['RASSF4','ZNF22'],
        ['RNASE10','PTGR1'],
        ['RPL11','TCEB3'],
        ['RPL37A','IGFBP2'],
        ['RPL38','TTYH2'],
        ['RPLP1','RBM15B'],
        ['SAR1A','TYSND1'],
        ['SCRN2','MRPL10'],
        ['SEPT7P2','PSPH'],
        ['SLC29A1','HSP90AB1'],
        ['SLIRP','LINC00545'],
        ['SMG5','PAQR6'],
        ['SRGAP2B','SRGAP2C'],
        ['STYXL1','TMEM120A'],
        ['SURF4','SURF1'],
        ['TIMM23','LINC00843'],
        ['TMEM190','RPL28'],
        ['TMX2','CTNND1'],
        ['TOMM40','APOE'],
        ['TOPORS','DDX58'],
        ['TP53RK','SLC13A3'],
        ['U1','KLF7P1'],
        ['UBE2D2','CXXC5'],
        ['UBE2J2','FAM132A'],
        ['USP32','APPBP2'],
        ['VAMP1','CD27-AS1'],
        ['VPS45','PLEKHO1'],
        ['WARS2','TBX15'],
        ['WRB','SH3BGR'],
        ['ZBTB1','PPP1R36'],
        ['ZNF264','AURKC'],
        ['ZNF674-AS1','CHST7'],
        ['ZNF765','TPM3P9'],
        ['ZYX','EPHA1-AS1'],
        ['AC019349.5','AL513122.1'],
        ['AC024560.3','SDHA'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['AL513122.1','KRT13'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['C19MC','RPLP1'],
        ['C2ORF48','RRM2'],
        ['C4ORF3','LYZ'],
        ['CANX','LYZ'],
        ['COQ8B','NUMBL'],
        ['CTSC','RAB38'],
        ['CYP2A6','CYP2A7'],
        ['DHX9','NPL'],
        ['FAM13A-AS1','HERC3'],
        ['GOLT1A','KISS1'],
        ['HBB','IQSEC1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['KRT78','TNNT1'],
        ['LRRFIP1','LYZ'],
        ['LYZ','STAT6'],
        ['NACA','PRIM1'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['PPP1R12B','TPM1'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['AC019349.5','AL513122.1'],
        ['AC024560.3','SDHA'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['AL513122.1','KRT13'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['C19MC','RPLP1'],
        ['C2ORF48','RRM2'],
        ['C4ORF3','LYZ'],
        ['CANX','LYZ'],
        ['COQ8B','NUMBL'],
        ['CTSC','RAB38'],
        ['CYP2A6','CYP2A7'],
        ['DHX9','NPL'],
        ['FAM13A-AS1','HERC3'],
        ['GOLT1A','KISS1'],
        ['HBB','IQSEC1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['KRT78','TNNT1'],
        ['LRRFIP1','LYZ'],
        ['LYZ','STAT6'],
        ['NACA','PRIM1'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['PPP1R12B','TPM1'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['ANXA1','COL6A1'],
        ['ATP6V0A1','YBX3'],
        ['CD44','CTSS'],
        ['CTSB','PNPLA6'],
        ['DES','FLNA'],
        ['EEF1A1','PLD3'],
        ['FBLN5','MYH11'],
        ['IGFBP5','VCL'],
        ['IGKC','TMED5'],
        ['KLF6','METAP1'],
        ['RNF216','XKR8'],
        ['SARS','TLR8-AS1'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['CDC42EP2','POLA2'],
        ['COQ8B','NUMBL'],
        ['GSR','GTF2E2'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['PSPC1','ZMYM5'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['A2ML1','ANXA2'],
        ['AC096664.1','RPL27'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['AP3S1','COMMD10'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['C19MC','RPLP1'],
        ['C2ORF48','RRM2'],
        ['CAST','KRT4'],
        ['CD36','MB'],
        ['CDC42EP2','POLA2'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CTSC','RAB38'],
        ['CYBRD1','MYH11'],
        ['DES','TCF4'],
        ['DES','ZBTB38'],
        ['DHX9','NPL'],
        ['EIF4E3','FOXP1'],
        ['FAM13A-AS1','HERC3'],
        ['GOLT1A','KISS1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['KRT4','SUN2'],
        ['KRT4','TBC1D14'],
        ['LYZ','RAB18'],
        ['MYH11','NUMA1'],
        ['MYH11','PPL'],
        ['MYH6','TTN'],
        ['NACA','PRIM1'],
        ['NPEPL1','STX16'],
        ['NPPA','P4HB'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['A2ML1','ANXA2'],
        ['AADAC','SUCNR1'],
        ['AC005336.4','IPMK'],
        ['AC008753.3','RPLP1'],
        ['AC096664.1','RPL27'],
        ['AC108488.4','ADI1'],
        ['ACPP','ERGIC3'],
        ['ALB','APLP2'],
        ['ASPH','TG'],
        ['CAST','KRT4'],
        ['CD36','MB'],
        ['CTTN','RACK1'],
        ['CYBRD1','MYH11'],
        ['CYP4F12','CYP4F24P'],
        ['DES','TCF4'],
        ['DES','ZBTB38'],
        ['DNAJB1','TG'],
        ['EEF1A1','SCMH1'],
        ['GKN1','NSD1'],
        ['KRT4','SUN2'],
        ['KRT4','TBC1D14'],
        ['LYZ','RAB18'],
        ['MYH11','NUMA1'],
        ['MYH11','PPL'],
        ['MYH6','TTN'],
        ['NOTCH3','TG'],
        ['NPPA','P4HB'],
        ['PCDHB15','PCDHB19P'],
        ['PDXDC1','RAC1'],
        ['SYTL4','TG'],
        ['TACC2','TG'],
        ['ZNF702P','ZNF816'],
        ['ZNF737','ZNF826P'],
        ['A2ML1','ANXA2'],
        ['AADAC','SUCNR1'],
        ['AC005336.4','IPMK'],
        ['AC008753.3','RPLP1'],
        ['AC096664.1','RPL27'],
        ['AC108488.4','ADI1'],
        ['ACCS','EXT2'],
        ['ACPP','ERGIC3'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['ALB','APLP2'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['ASPH','TG'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BCKDHA','TMEM91'],
        ['C19MC','RPLP1'],
        ['C1ORF194','UQCR10'],
        ['C2ORF48','RRM2'],
        ['CAST','KRT4'],
        ['CD36','MB'],
        ['CDC42EP2','POLA2'],
        ['CDK2','RAB5B'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CTSC','RAB38'],
        ['CTTN','RACK1'],
        ['CYBRD1','MYH11'],
        ['CYP4F12','CYP4F24P'],
        ['DES','TCF4'],
        ['DES','ZBTB38'],
        ['DHX9','NPL'],
        ['DNAJB1','TG'],
        ['EEF1A1','SCMH1'],
        ['EEF1A1','TG'],
        ['EIF4E3','FOXP1'],
        ['ELK4','SLC45A3'],
        ['FAM13A-AS1','HERC3'],
        ['GKN1','NSD1'],
        ['GOLT1A','KISS1'],
        ['HGSNAT','TG'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['KLK4','KLKP1'],
        ['KRT4','SUN2'],
        ['KRT4','TBC1D14'],
        ['LYZ','RAB18'],
        ['LYZ','SCYL1'],
        ['MSMB','NCOA4'],
        ['MUC7','RRBP1'],
        ['MYH11','NUMA1'],
        ['MYH11','PPL'],
        ['MYH6','TTN'],
        ['NACA','PRIM1'],
        ['NCOA6','PIGU'],
        ['NIT2','TBC1D23'],
        ['NOTCH3','TG'],
        ['NPEPL1','STX16'],
        ['NPPA','P4HB'],
        ['NUP188','PHYHD1'],
        ['PBXIP1','PMVK'],
        ['PCDHB15','PCDHB19P'],
        ['PDXDC1','RAC1'],
        ['PECAM1','TG'],
        ['PGM5','TGM4'],
        ['PIM3','SCO2'],
        ['PSPC1','ZMYM5'],
        ['PTPRJ','TG'],
        ['RASSF3','ZG16B'],
        ['RIPK1','SERPINB9'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['SLC25A23','TG'],
        ['SLC6A16','TEAD2'],
        ['SPIN1','TG'],
        ['SYTL4','TG'],
        ['TACC2','TG'],
        ['TBCEL','TECTA'],
        ['TGM4','WDR6'],
        ['ZNF702P','ZNF816'],
        ['ZNF737','ZNF826P'],
        ['A2ML1','ANXA2'],
        ['AADAC','SUCNR1'],
        ['AC005336.4','IPMK'],
        ['AC008753.3','RPLP1'],
        ['AC096664.1','RPL27'],
        ['AC108488.4','ADI1'],
        ['ACCS','EXT2'],
        ['ACPP','ERGIC3'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['AKAP13','CPA1'],
        ['ALB','APLP2'],
        ['ALDH7A1','CLPS'],
        ['ANXA1','STATH'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['ASPH','TG'],
        ['ATF6','PNLIP'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BCKDHA','TMEM91'],
        ['BNIP3L','HTN1'],
        ['BNIP3L','STATH'],
        ['C19MC','RPLP1'],
        ['C1ORF194','UQCR10'],
        ['C2ORF48','RRM2'],
        ['CAPN1','SMR3B'],
        ['CAST','KRT4'],
        ['CD36','MB'],
        ['CD44','PRH2'],
        ['CD63','PNLIP'],
        ['CD63','SMR3B'],
        ['CD63','STATH'],
        ['CDC42EP2','POLA2'],
        ['CDK2','RAB5B'],
        ['CEL','CELP'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CPA1','GTF2I'],
        ['CPA1','LARP4B'],
        ['CPA1','MOB3A'],
        ['CPA1','PDIA3'],
        ['CPA1','RGL3'],
        ['CPA1','TSC2'],
        ['CTSC','RAB38'],
        ['CTTN','RACK1'],
        ['CYBRD1','MYH11'],
        ['CYP4F12','CYP4F24P'],
        ['DES','LGMN'],
        ['DES','TCF4'],
        ['DES','ZBTB38'],
        ['DHX9','NPL'],
        ['DNAJB1','TG'],
        ['DNMT1','PRSS1'],
        ['EEF1A1','PRR4'],
        ['EEF1A1','SCMH1'],
        ['EEF1A1','TG'],
        ['EIF4E3','FOXP1'],
        ['ELK4','SLC45A3'],
        ['FAM13A-AS1','HERC3'],
        ['GANC','HTN3'],
        ['GAREM1','TRB@'],
        ['GKN1','NSD1'],
        ['GOLT1A','KISS1'],
        ['GTF2I','SMR3B'],
        ['HEBP2','SMR3B'],
        ['HGSNAT','TG'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['HTN1','WWC1'],
        ['HTN3','TOPORS-AS1'],
        ['HTN3','TSC2'],
        ['IFT20','PNLIP'],
        ['KLK4','KLKP1'],
        ['KRT4','SUN2'],
        ['KRT4','TBC1D14'],
        ['LYZ','RAB18'],
        ['LYZ','SCYL1'],
        ['MAOB','PRH2'],
        ['MEG3','SNRNP200'],
        ['METTL21B','TSFM'],
        ['MLPH','SMR3B'],
        ['MSMB','NCOA4'],
        ['MTA3','PNLIP'],
        ['MUC7','RRBP1'],
        ['MYH11','NUMA1'],
        ['MYH11','PPL'],
        ['MYH6','TTN'],
        ['NACA','PRIM1'],
        ['NCOA3','PNLIP'],
        ['NCOA6','PIGU'],
        ['NIT2','TBC1D23'],
        ['NOTCH3','TG'],
        ['NPEPL1','STX16'],
        ['NPPA','P4HB'],
        ['NSD1','PNLIP'],
        ['NSD1','SMR3B'],
        ['NUP188','PHYHD1'],
        ['OS9','TRB@'],
        ['PBXIP1','PMVK'],
        ['PCDHB15','PCDHB19P'],
        ['PDIA3','PRSS1'],
        ['PDIA3','SMR3B'],
        ['PDXDC1','RAC1'],
        ['PECAM1','TG'],
        ['PGM5','TGM4'],
        ['PIM3','SCO2'],
        ['PNLIP','POM121'],
        ['PNLIP','PPARA'],
        ['PNLIP','RALY'],
        ['PNLIP','RNF10'],
        ['PNLIP','SCAMP5'],
        ['PNLIP','SDHA'],
        ['PNLIP','SLIT2'],
        ['PNLIP','VPS28'],
        ['PRB1','PSAP'],
        ['PRB1','PTCH2'],
        ['PRSS1','TSC2'],
        ['PSPC1','ZMYM5'],
        ['PTPRJ','TG'],
        ['RASSF3','ZG16B'],
        ['RIPK1','SERPINB9'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['SAMD5','SASH1'],
        ['SCNN1A','TNFRSF1A'],
        ['SDHA','SMR3B'],
        ['SERINC2','SMR3B'],
        ['SH3GLB1','WSB1'],
        ['SIDT2','TAGLN'],
        ['SLC25A23','TG'],
        ['SLC6A16','TEAD2'],
        ['SMR3B','TSC2'],
        ['SMR3B','WFDC2'],
        ['SPIN1','TG'],
        ['SYTL4','TG'],
        ['TACC2','TG'],
        ['TBCEL','TECTA'],
        ['TGM4','WDR6'],
        ['ZNF702P','ZNF816'],
        ['ZNF737','ZNF826P'],
        ['AC010680.1','MYH6'],
        ['ACCS','EXT2'],
        ['ACSL1','HTN1'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['ANP32E','HMGB1'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17B','KANSL1'],
        ['ARL4A','NSUN7'],
        ['ART4','SMCO3'],
        ['ATP2A3','HBB'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BCKDHA','TMEM91'],
        ['C19MC','RPLP1'],
        ['C1ORF194','UQCR10'],
        ['C2ORF48','RRM2'],
        ['CDC42EP2','POLA2'],
        ['CDK2','RAB5B'],
        ['CEL','CELP'],
        ['COL14A1','TG'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CSDE1','STATH'],
        ['CTSC','RAB38'],
        ['CXCR4','HMGB1'],
        ['DDX5','HMGB1'],
        ['DHX9','NPL'],
        ['EEF1A1','TG'],
        ['EIF4E3','FOXP1'],
        ['ELK4','SLC45A3'],
        ['FAM13A-AS1','HERC3'],
        ['GOLT1A','KISS1'],
        ['GSTP1','HBB'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['HTN3','LGR4'],
        ['KLK4','KLKP1'],
        ['LINC00707','LINP1'],
        ['METTL21B','TSFM'],
        ['MSMB','NCOA4'],
        ['MUC7','RUNX1'],
        ['NACA','PRIM1'],
        ['NCOA6','PIGU'],
        ['NDST1','ZG16B'],
        ['NPEPL1','STX16'],
        ['NUP188','PHYHD1'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['PSPC1','ZMYM5'],
        ['RIPK1','SERPINB9'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['SAMD5','SASH1'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['SLC6A16','TEAD2'],
        ['TG','VWF'],
        ['ZNF208','ZNF676'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['CDC42EP2','POLA2'],
        ['COQ8B','NUMBL'],
        ['GSR','GTF2E2'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['PSPC1','ZMYM5'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BRS3','HTATSF1'],
        ['COQ8B','NUMBL'],
        ['HSPE1','MOB4'],
        ['SIDT2','TAGLN'],
        ['C5ORF34','EEF1A1'],
        ['CAPZB','EEF2'],
        ['EEF1A1','C5ORF34'],
        ['IGH@','GRK2'],
        ['IGH@','LDHA'],
        ['IGH@','WDFY4'],
        ['NCL','PTMA'],
        ['RPL38','TTYH2'],
        ['RRM2','C2ORF48'],
        ['TTC7B','MVD'],
        ['KANSL1','ARL17A'],
        ['ADH1B','PTRF'],
        ['ARHGAP27P1-BPTFP1-KPNA2P3','BPTF'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BPTF','KPNA2'],
        ['BRS3','HTATSF1'],
        ['COQ8B','NUMBL'],
        ['HSPE1','MOB4'],
        ['IGLV5-52','LL22NC03-80A10.6'],
        ['SIDT2','TAGLN'],
        ['ATP10B','LINC02159'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['C2ORF27A','DUXAP8'],
        ['CDC42EP2','POLA2'],
        ['COQ8B','NUMBL'],
        ['GSR','GTF2E2'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['PSPC1','ZMYM5'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['ADH1B','ALB'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['ATG16L1','INPP5D'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['C19MC','RPLP1'],
        ['C2ORF48','RRM2'],
        ['CDC42EP2','POLA2'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CTSC','RAB38'],
        ['DHX9','NPL'],
        ['EIF4E3','FOXP1'],
        ['FAM13A-AS1','HERC3'],
        ['GFAP','SPARC'],
        ['GOLT1A','KISS1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['LINC00174','RABGEF1'],
        ['LINC00174','SKP1'],
        ['LINC02009','LTF'],
        ['LINC02019','MAPKAPK3'],
        ['LINC02023','SI'],
        ['NACA','PRIM1'],
        ['NPEPL1','STX16'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['AC006262.10','IGFL2-AS1'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ADGRG7','TFG'],
        ['ADH1B','ALB'],
        ['AKT2','C19ORF47'],
        ['AP000344.4','LINC01659'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ART4','SMCO3'],
        ['ATG16L1','INPP5D'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BCKDHA','TMEM91'],
        ['C15ORF57','CBX3'],
        ['C19MC','RPLP1'],
        ['C1ORF194','UQCR10'],
        ['C2ORF48','RRM2'],
        ['CCT3','PRB3'],
        ['CDC42EP2','POLA2'],
        ['CDK2','RAB5B'],
        ['CEL','CELP'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CRISP3','HTN3'],
        ['CTSC','RAB38'],
        ['DHX9','NPL'],
        ['EEF1A1','TG'],
        ['EIF4E3','FOXP1'],
        ['ELK4','SLC45A3'],
        ['ERBB2','GRB7'],
        ['FAM107A','SMR3B'],
        ['FAM13A-AS1','HERC3'],
        ['FGFR4','ZNF346'],
        ['FLJ22447','PRKCH'],
        ['GFAP','SPARC'],
        ['GOLT1A','KISS1'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['IGH@','MXI1'],
        ['ITGA4','LINC01934'],
        ['KLK4','KLKP1'],
        ['LINC00174','RABGEF1'],
        ['LINC00174','SKP1'],
        ['LINC01764','ZNF861P'],
        ['LINC01823','TSN'],
        ['LINC01859','ZNF728'],
        ['LINC01979','TBC1D16'],
        ['LINC02009','LTF'],
        ['LINC02019','MAPKAPK3'],
        ['LINC02023','SI'],
        ['LIX1L-AS1','POLR3GL'],
        ['METTL21B','TSFM'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['NCL','TRB@'],
        ['NCOA6','PIGU'],
        ['NPEPL1','STX16'],
        ['NPIPB5','SMG1'],
        ['NUP188','PHYHD1'],
        ['PBXIP1','PMVK'],
        ['PIM3','SCO2'],
        ['PNLIP','SCARB2'],
        ['PSPC1','ZMYM5'],
        ['RCAN1','TG'],
        ['RIPK1','SERPINB9'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['SAMD5','SASH1'],
        ['SCNN1A','TNFRSF1A'],
        ['SIDT2','TAGLN'],
        ['SLC6A16','TEAD2'],
        ['AC004967.7','OR7E38P'],
        ['AC096574.4','TARDBP'],
        ['AC131056.5','CH17-431G21.1'],
        ['AC145124.2','FAM85B'],
        ['ACTB','HEBP2'],
        ['ACTB','TPO'],
        ['ACTG1','PLPP3'],
        ['ADIPOQ-AS1','GNPTG'],
        ['AGPAT3','SOD2'],
        ['ALK','LYZ'],
        ['ANAPC1','RMND5A'],
        ['ANAPC16','CNOT10'],
        ['ANAPC16','CRLF2'],
        ['ANP32B','CTSS'],
        ['ANXA1','TG'],
        ['APOC2','ISOC2'],
        ['APOL1','LINC01138'],
        ['ARHGEF26-AS1','TRA@'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['AZGP1','GJC3'],
        ['B2M','CDK6'],
        ['BCAP29','DUS4L'],
        ['BCL9','SH3PXD2A'],
        ['BP-21264C1.2','ZNF717'],
        ['BRS3','HTATSF1'],
        ['C12ORF45','PRKCSH'],
        ['C19MC','RPLP1'],
        ['C5','LRG1'],
        ['C7','RPS3'],
        ['C7','TRA@'],
        ['CANX','VEGFA'],
        ['CATSPER2','PPIP5K1'],
        ['CBFA2T3','CTSS'],
        ['CCDC144NL','NBEA'],
        ['CCDC6','MRLN'],
        ['CD24','SMIM14'],
        ['CD63','EEF1A1'],
        ['CD93','SYNPO'],
        ['CDH8','SYNPO'],
        ['CFLAR','METTL7A'],
        ['CLIP1','VHL'],
        ['COPA','FGFR2'],
        ['COQ8B','NUMBL'],
        ['CRLF2','IDS'],
        ['CRLF2','METTL7A'],
        ['CTSB','DNAJC7'],
        ['CTSB','EML4'],
        ['CTSB','FGFR2'],
        ['CTSB','PAX5'],
        ['CWF19L1','RFWD3'],
        ['CXCL5','GLYR1'],
        ['DTX2P1-UPK3BP1-PMS2P11','PMS2'],
        ['EEF1A1','EIF4B'],
        ['EGFR','IGSF6'],
        ['EMC10','MCAM'],
        ['FAM157B','FBXO25'],
        ['FAM98C','MRPS21'],
        ['FBXL18','PTGIS'],
        ['FGB','SERPINA3'],
        ['FGD5-AS1','FRRS1L'],
        ['GON4L','TG'],
        ['H6PD','PMPCA'],
        ['HCG4B','XXBAC-BPG248L24.12'],
        ['HNRNPU','HSP90AA1'],
        ['HP','SDHA'],
        ['HSP90AB1','SLC29A1'],
        ['HSP90B1','PTMS'],
        ['HSPA1B','TPT1'],
        ['HSPE1','MOB4'],
        ['HSPG2','PCBP2'],
        ['HSPG2','SCD'],
        ['IGFBP5','IGH@'],
        ['IGHM','STAT6'],
        ['IGK@','SOD2'],
        ['IL31RA','PRKCSH'],
        ['IL3RA','PTRF'],
        ['ITGB1','MYH11'],
        ['LIMS1','LYZ'],
        ['LINC00266-1','LINC01881'],
        ['LINC02193','RPL23AP82'],
        ['MECOM','RPL22'],
        ['MYBPC1','RYR1'],
        ['MYH11','YWHAZ'],
        ['NAIP','OCLN'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['PAFAH1B1','TG'],
        ['PDIA3','SPP1'],
        ['PDIA3','TG'],
        ['PECAM1','PTRF'],
        ['PIK3R1','TPM1'],
        ['PIPOX','SOD2'],
        ['PTRF','TMOD3'],
        ['RBFOX2','RPL41'],
        ['RPL13A','TG'],
        ['SCD','SORBS1'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SOD2','TRB@'],
        ['TG','UBE2Z'],
        ['WASH2P','WASH5P'],
        ['AC008746.10','LYZ'],
        ['AC008993.2','XXYAC-YRM2039.3'],
        ['AC019349.5','KRT6A'],
        ['AC026150.8','GOLGA8M'],
        ['AC027612.6','IGK@'],
        ['ACCS','EXT2'],
        ['ACOT11','MROH7-TTC4'],
        ['ACSL1','IGH@'],
        ['ACSS1','APMAP'],
        ['ACTA1','CRYAB'],
        ['ACTA1','SDHA'],
        ['ACTB','ALB'],
        ['ACTB','C16ORF72'],
        ['ACTB','DDX5'],
        ['ACTB','GLUL'],
        ['ACTB','RPL15'],
        ['ACTB','SF3B1'],
        ['ACTB','SPINK5'],
        ['ACTN1','IGH@'],
        ['ADGRG7','TFG'],
        ['ADH1A','TF'],
        ['ADSL','TNRC6B'],
        ['AGAP5','BMS1P4'],
        ['AHR','LYZ'],
        ['AHSG','HP'],
        ['ALB','CHD4'],
        ['ALB','CYP1A2'],
        ['ALB','CYP4A11'],
        ['ALK','CDK12'],
        ['ANAPC1','RMND5A'],
        ['ANKRD20A21P','BP-2189O9.2'],
        ['ANKRD54','LYZ'],
        ['ANP32B','PTMA'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['APP','KRT13'],
        ['AQP7','CH17-469D17.1'],
        ['ARHGAP1','NPPA'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARRDC3','HIST1H2BD'],
        ['ART4','SMCO3'],
        ['ATG16L1','INPP5D'],
        ['ATG16L2','IGK@'],
        ['ATP1B2','LINC00649'],
        ['ATP5G2','TFRC'],
        ['ATP5G3','NPPA-AS1'],
        ['AZGP1','GJC3'],
        ['B2M','CALM1'],
        ['B2M','HMGB1'],
        ['B2M','TMSB4X'],
        ['BANK1','IGH@'],
        ['BANK1','IGK@'],
        ['BANK1','OSGEP'],
        ['BCAP29','DUS4L'],
        ['BCL2','LYZ'],
        ['BCL9','TSPAN14'],
        ['BCLAF1','EEF1A1'],
        ['BGN','FAM102B'],
        ['BORCS7-ASMT','CCND3'],
        ['BPI','NUMA1'],
        ['BPTF','LRRC37A2'],
        ['C15ORF57','CBX3'],
        ['C19MC','RPLP1'],
        ['C1S','ORM2'],
        ['C2ORF48','RRM2'],
        ['C5AR1','LYZ'],
        ['C9ORF78','HBB'],
        ['CA1','MGEA5'],
        ['CATSPER2','PPIP5K1'],
        ['CCDC152','HP'],
        ['CCND3','FCAR'],
        ['CCND3','GRB2'],
        ['CCNG2','LRCH4'],
        ['CCNG2','S100A9'],
        ['CD164','WWOX'],
        ['CD24','IL3RA'],
        ['CD24','RACK1'],
        ['CD84','SLC26A2'],
        ['CDC42','S100A9'],
        ['CDC42EP2','POLA2'],
        ['CDH17','IGHA1'],
        ['CEACAM1','MUC2'],
        ['CES4A','TRA@'],
        ['CH17-164J11.1','IGK@'],
        ['CHGB','IGH@'],
        ['CHRDL1','CHST11'],
        ['CHST11','HIF1A'],
        ['CKMT2-AS1','LYZ'],
        ['CLU','GAPDH'],
        ['CMBL','WWOX'],
        ['CNN2','S100A9'],
        ['CNOT10','HBB'],
        ['COL1A1','PTGDS'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CPLX2','GLUL'],
        ['CSF2RA','LYZ'],
        ['CTSC','RAB38'],
        ['CUX1','HBB'],
        ['CWC25','LSINCT5'],
        ['CWC25','OSBPL2'],
        ['CXCL16','CYB561A3'],
        ['CXCR2','WSB1'],
        ['CXCR4','EEF1A1'],
        ['CXCR4','HNRNPA2B1'],
        ['CYP1A2','MICAL2'],
        ['CYP4V2','NSD2'],
        ['DA750114','TRA@'],
        ['DCTN5','GTDC1'],
        ['DDOST','TAS2R4'],
        ['DDX5','PTMA'],
        ['DENND3','HBA2'],
        ['DHODH','MYSM1'],
        ['DHX9','GLUL'],
        ['DHX9','NPL'],
        ['DKK3','MYH7'],
        ['DMBT1','IGH@'],
        ['DNAJB6','MPO'],
        ['DNAJC25-GNG10','UGCG'],
        ['DNMT1','LYZ'],
        ['DPYSL3','KRT13'],
        ['EBLN3P','ZCCHC7'],
        ['EEF1A1','HBA2'],
        ['EEF1A1','HMGB1'],
        ['EEF1A1','NFKBIZ'],
        ['EEF1A1','SLC4A1'],
        ['EEF1A1','SYNCRIP'],
        ['EEF1A1','UBC'],
        ['EIF4B','KCTD20'],
        ['EIF4E','PIK3C2A'],
        ['EIF4E3','FOXP1'],
        ['ELANE','HBA2'],
        ['EMX2OS','TRA@'],
        ['EPOR','SLC16A12'],
        ['ERBB2','GRB7'],
        ['ERVW-1','LINC01239'],
        ['ETV6','PIGR'],
        ['EWSR1','LYZ'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM20B','TFDP2'],
        ['FBXL18','TTLL3'],
        ['FBXL5','HBB'],
        ['FCGBP','IGH@'],
        ['FCGBP','IGHA2'],
        ['FGD5-AS1','LINC00969'],
        ['FGFR2','TMPO'],
        ['FOSB','PPM1N'],
        ['FOXP1','GPRC5A'],
        ['FRG1BP','FRG1HP'],
        ['FRG1HP','FRG1JP'],
        ['FUS','PTMA'],
        ['GEMIN6','TSPAN14'],
        ['GINM1','RN7SL470P'],
        ['GLTP','MALL'],
        ['GLUL','PTMA'],
        ['GOLT1A','KISS1'],
        ['GRK2','IGH@'],
        ['GSE1','SLC7A5'],
        ['GUSBP11','LYZ'],
        ['GYPA','HBB'],
        ['HBB','HGS'],
        ['HBB','MICAL2'],
        ['HBB','MYH10'],
        ['HBB','MYO9B'],
        ['HBB','NSUN3'],
        ['HBB','PAFAH1B1'],
        ['HBB','PHLDA1'],
        ['HBB','PLXND1'],
        ['HBB','PTMA'],
        ['HBB','RBMS1'],
        ['HBB','RPS27'],
        ['HBB','RPS3'],
        ['HBB','SDHA'],
        ['HBB','SFI1'],
        ['HBB','SMARCA2'],
        ['HBB','SP100'],
        ['HBB','TACC3'],
        ['HBB','TSC2'],
        ['HBB','ZNF726'],
        ['HHLA3','TESPA1'],
        ['HMGB2','TMSB4X'],
        ['HMGN2','SEPT5'],
        ['HNRNPA1','KRT4'],
        ['HNRNPH1','PTP4A1'],
        ['HP','PLG'],
        ['HRG','ORM1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPA5','MPO'],
        ['HSPA6','HSPA7'],
        ['HSPE1','MOB4'],
        ['IGH@','MUC2'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','PTMA'],
        ['IGH@','ZFP36L1'],
        ['IGK@','SAMHD1'],
        ['IL3RA','YIPF5'],
        ['KIAA2012','PEX26'],
        ['KRT13','MUC21'],
        ['KRT13','PFKL'],
        ['KRT13','RGS12'],
        ['KRT6A','MYH11'],
        ['KYNU','ZNF528-AS1'],
        ['LINC00491','LINC01807'],
        ['LINC01347','NUPR2'],
        ['LINC01641','RPS18'],
        ['LLFOS-48D6.2','PTMA'],
        ['LMX1B','PAPOLG'],
        ['LRIG2','ZNF106'],
        ['LRRC37A3','NSF'],
        ['LYZ','MAU2'],
        ['LYZ','NHSL2'],
        ['LYZ','NIPAL2'],
        ['LYZ','PAXBP1-AS1'],
        ['LYZ','PRAM1'],
        ['LYZ','SPATS2L'],
        ['LYZ','UBXN2A'],
        ['LYZ','WWOX'],
        ['LYZ','XIST'],
        ['MATR3','PAIP2'],
        ['MC2R','SH3PXD2A'],
        ['MED9','SLC37A2'],
        ['MPO','RPS21'],
        ['MRPS10','PPM1B'],
        ['MS4A10','SP140L'],
        ['MYH11','PABPC1'],
        ['MYH6','PLIN4'],
        ['MYH6','ZNF106'],
        ['MYH7','PSMC4'],
        ['MYH9','RPL13A'],
        ['MYL3','PRSS44'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NPPA','RAPGEF1'],
        ['OAZ2','SIK2'],
        ['PBXIP1','PMVK'],
        ['PCDHB13','PCDHB8'],
        ['PIGR','TRA@'],
        ['PIM3','SCO2'],
        ['PLIN1','WWOX'],
        ['PMS2','POLR2J3'],
        ['PPP1R12B','ZNF623'],
        ['RAB21','RNF24'],
        ['RAB2B','TOP3A'],
        ['RASSF8','SSPN'],
        ['RBM25','S100A9'],
        ['RFWD3','RSL1D1'],
        ['RHOA','TKT'],
        ['RHOH','SLC31A1'],
        ['RPL14','SRP14'],
        ['RPL37A','SRGN'],
        ['RPL38','TTYH2'],
        ['RPS14','S100A9'],
        ['S100A9','YWHAZ'],
        ['SCNN1A','TNFRSF1A'],
        ['SELENOP','TPT1'],
        ['SERF1A','SMN1'],
        ['SFN','TNS1'],
        ['SIDT2','TAGLN'],
        ['SLC8A1','TPM1'],
        ['STAT3','TRA@'],
        ['TBCEL','TECTA'],
        ['TRA@','TRIM69'],
        ['TUBB7P','TUBB8'],
        ['AC008746.10','LYZ'],
        ['AC008993.2','XXYAC-YRM2039.3'],
        ['AC019349.5','KRT6A'],
        ['AC026150.8','GOLGA8M'],
        ['AC027612.6','IGK@'],
        ['AC097374.2','BP-2171C21.2'],
        ['ACOT11','MROH7-TTC4'],
        ['ACSL1','IGH@'],
        ['ACTA1','CRYAB'],
        ['ACTA1','SDHA'],
        ['ACTB','ALB'],
        ['ACTB','C16ORF72'],
        ['ACTB','DDX5'],
        ['ACTB','EEF1A1'],
        ['ACTB','GLUL'],
        ['ACTB','PRH1-PRR4'],
        ['ACTB','RPL15'],
        ['ACTB','SF3B1'],
        ['ACTB','SPINK5'],
        ['ACTB','SPTAN1'],
        ['ACTN1','IGH@'],
        ['ADH1A','TF'],
        ['ADSL','TNRC6B'],
        ['AGPS','USP34'],
        ['AHR','LYZ'],
        ['AHSA2','TIAL1'],
        ['AHSG','HP'],
        ['AKT2','TRB@'],
        ['ALB','CHD4'],
        ['ALB','CYP1A2'],
        ['ALB','CYP4A11'],
        ['ALK','CDK12'],
        ['AMY2A','CELA2A'],
        ['AMY2A','CELA3A'],
        ['AMY2A','CELA3B'],
        ['AMY2A','CTRB1'],
        ['AMY2A','PNLIP'],
        ['AMY2A','PRSS1'],
        ['AMY2B','PRSS1'],
        ['ANAPC7','KDSR'],
        ['ANKRD20A21P','BP-2189O9.2'],
        ['ANKRD54','LYZ'],
        ['ANP32B','PTMA'],
        ['ANXA1','SMR3B'],
        ['APBB2','SPAG9'],
        ['APP','KRT13'],
        ['AQP7','CH17-469D17.1'],
        ['ARHGAP1','NPPA'],
        ['ARRDC3','HIST1H2BD'],
        ['ATG16L2','IGK@'],
        ['ATP1B2','LINC00649'],
        ['ATP5G2','TFRC'],
        ['ATP5G3','NPPA-AS1'],
        ['B2M','CALM1'],
        ['B2M','HMGB1'],
        ['B2M','TMSB4X'],
        ['BANK1','IGK@'],
        ['BANK1','OSGEP'],
        ['BCL2','LYZ'],
        ['BCL9','TSPAN14'],
        ['BCLAF1','EEF1A1'],
        ['BGN','FAM102B'],
        ['BORCS7-ASMT','CCND3'],
        ['BP-21264C1.2','MAFIP'],
        ['BPI','NUMA1'],
        ['C1S','ORM2'],
        ['C5AR1','LYZ'],
        ['C7','RPS4X'],
        ['C9ORF78','HBB'],
        ['CA1','MGEA5'],
        ['CALR','H19'],
        ['CAPZB','SMR3B'],
        ['CASP9','PNLIP'],
        ['CCDC152','HP'],
        ['CCND3','F11R'],
        ['CCND3','FCAR'],
        ['CCND3','GRB2'],
        ['CCNG2','LRCH4'],
        ['CCNG2','S100A9'],
        ['CD164','WWOX'],
        ['CD24','IL3RA'],
        ['CD24','MUC7'],
        ['CD24','RACK1'],
        ['CD63','CPB1'],
        ['CD84','SLC26A2'],
        ['CDC42','S100A9'],
        ['CDH17','IGHA1'],
        ['CEACAM1','MUC2'],
        ['CEL','CELA3A'],
        ['CEL','CELA3B'],
        ['CEL','CPA2'],
        ['CEL','PRSS3'],
        ['CEL','SLC7A2'],
        ['CELA3A','FLNA'],
        ['CELA3B','TPT1'],
        ['CELP','TRB@'],
        ['CES4A','TRA@'],
        ['CGNL1','SFTPA1'],
        ['CH17-164J11.1','IGK@'],
        ['CH17-264L24.1','IGK@'],
        ['CHGB','IGH@'],
        ['CHI3L2','SMR3B'],
        ['CHRDL1','CHST11'],
        ['CHST11','HIF1A'],
        ['CKMT2-AS1','LYZ'],
        ['CLEC2D','IGH@'],
        ['CLPS','CTRB1'],
        ['CLU','GAPDH'],
        ['CMBL','WWOX'],
        ['CNN2','S100A9'],
        ['CNNM2','LYZ'],
        ['CNOT10','HBB'],
        ['CNOT4','SPTLC2'],
        ['COASY','HSD17B1'],
        ['COL1A1','PTGDS'],
        ['CORO1A','EEF1A1'],
        ['COX7C','ZG16B'],
        ['CPB1','CTRB2'],
        ['CPB1','DDX24'],
        ['CPB1','GOLGA8B'],
        ['CPB1','REG1A'],
        ['CPB1','STAT6'],
        ['CPLX2','GLUL'],
        ['CSF2RA','LYZ'],
        ['CSH1','H19'],
        ['CTBP1','SMR3B'],
        ['CTRB1','SYCN'],
        ['CTRB2','GNAS'],
        ['CUX1','HBB'],
        ['CWC25','LSINCT5'],
        ['CWC25','OSBPL2'],
        ['CXCL16','CYB561A3'],
        ['CXCR2','WSB1'],
        ['CXCR4','EEF1A1'],
        ['CXCR4','HNRNPA2B1'],
        ['CYP1A2','MICAL2'],
        ['CYP4V2','NSD2'],
        ['DA750114','TRA@'],
        ['DCTN5','GTDC1'],
        ['DDOST','TAS2R4'],
        ['DDX5','PTMA'],
        ['DENND3','HBA2'],
        ['DGCR6','XXBAC-B444P24.13'],
        ['DHODH','MYSM1'],
        ['DHX9','GLUL'],
        ['DIS3L2','SMR3B'],
        ['DKK3','MYH7'],
        ['DMBT1','IGH@'],
        ['DNAJB6','MPO'],
        ['DNAJC25-GNG10','UGCG'],
        ['DNMT1','LYZ'],
        ['DPYSL3','KRT13'],
        ['DUXAP8','LINC01297'],
        ['EBLN3P','ZCCHC7'],
        ['ECI2','PRSS1'],
        ['EEF1A1','HBA2'],
        ['EEF1A1','HMGB1'],
        ['EEF1A1','ITGB1'],
        ['EEF1A1','NFKBIZ'],
        ['EEF1A1','RPS4X'],
        ['EEF1A1','SLC4A1'],
        ['EEF1A1','SYNCRIP'],
        ['EEF1A1','UBC'],
        ['EEF1A1','XPR1'],
        ['EGFR','ZG16B'],
        ['EIF3E','TRB@'],
        ['EIF4B','KCTD20'],
        ['EIF4E','PIK3C2A'],
        ['ELANE','HBA2'],
        ['EMX2OS','TRA@'],
        ['ENAH','LYZ'],
        ['EPOR','SLC16A12'],
        ['ERVW-1','LINC01239'],
        ['ETV6','PIGR'],
        ['EWSR1','LYZ'],
        ['FAM20B','TFDP2'],
        ['FBXL18','TTLL3'],
        ['FBXL5','HBB'],
        ['FCGBP','IGH@'],
        ['FCGBP','IGHA2'],
        ['FGD5-AS1','LINC00969'],
        ['FGFR2','PNLIP'],
        ['FGFR2','TMPO'],
        ['FLOT1','SMR3B'],
        ['FOSB','PPM1N'],
        ['FOXP1','GPRC5A'],
        ['FOXP1','HECA'],
        ['FRG1BP','FRG1HP'],
        ['FRG1HP','FRG1JP'],
        ['FUS','PTMA'],
        ['GABPB1-AS1','NSD2'],
        ['GALNT10','SFTPA1'],
        ['GAPDH','H19'],
        ['GAPDH','MUC7'],
        ['GEMIN6','TSPAN14'],
        ['GINM1','RN7SL470P'],
        ['GLTP','MALL'],
        ['GLUL','PTMA'],
        ['GNAS','IGF2'],
        ['GP2','GSN'],
        ['GP2','MAN1B1-AS1'],
        ['GP2','MEG3'],
        ['GSE1','SLC7A5'],
        ['GSN','TRB@'],
        ['GSTM4','MUC7'],
        ['GUSBP11','LYZ'],
        ['GYPA','HBB'],
        ['H19','NRP1'],
        ['H19','PFN1'],
        ['H19','PLAC4'],
        ['H19','PLPPR2'],
        ['H19','TBC1D14'],
        ['H19','TPT1'],
        ['HBB','HGS'],
        ['HBB','MICAL2'],
        ['HBB','MYH10'],
        ['HBB','MYO9B'],
        ['HBB','NSUN3'],
        ['HBB','PAFAH1B1'],
        ['HBB','PHLDA1'],
        ['HBB','PLXND1'],
        ['HBB','PTMA'],
        ['HBB','RBMS1'],
        ['HBB','RPS27'],
        ['HBB','RPS3'],
        ['HBB','SDHA'],
        ['HBB','SFI1'],
        ['HBB','SMARCA2'],
        ['HBB','SP100'],
        ['HBB','TACC3'],
        ['HBB','TSC2'],
        ['HBB','ZNF726'],
        ['HHLA3','TESPA1'],
        ['HKR1','ZNF790-AS1'],
        ['HMGB1','PNLIP'],
        ['HMGB2','TMSB4X'],
        ['HMGN2','SEPT5'],
        ['HNRNPA1','KRT4'],
        ['HNRNPA1L2','MRPS31P4'],
        ['HNRNPA3','SFTPC'],
        ['HNRNPH1','PTP4A1'],
        ['HP','PLG'],
        ['HRG','ORM1'],
        ['HSPA5','MPO'],
        ['HSPA6','HSPA7'],
        ['HTN1','PDIA6'],
        ['IGF2','NAP1L1'],
        ['IGFBP5','RPL30'],
        ['IGH@','LINC00926'],
        ['IGH@','MUC2'],
        ['IGH@','NPM1'],
        ['IGH@','PTMA'],
        ['IGH@','RAP1GAP'],
        ['IGH@','SFTPC'],
        ['IGH@','SLC25A6'],
        ['IGH@','SRRM2'],
        ['IGHA1','LTF'],
        ['IGK@','SAMHD1'],
        ['IGK@','STATH'],
        ['IGK@','TRA@'],
        ['IGLL5','SMR3B'],
        ['IL3RA','YIPF5'],
        ['IL6ST','LYZ'],
        ['KIAA2012','PEX26'],
        ['KLK3','PLPP3'],
        ['KRT13','MUC21'],
        ['KRT13','PFKL'],
        ['KRT13','RGS12'],
        ['KRT6A','MYH11'],
        ['KYNU','ZNF528-AS1'],
        ['LENG8','PRSS1'],
        ['LINC00491','LINC01807'],
        ['LINC01251','TRB@'],
        ['LINC01347','NUPR2'],
        ['LINC01641','RPS18'],
        ['LLFOS-48D6.2','PTMA'],
        ['LMX1B','PAPOLG'],
        ['LRIG2','ZNF106'],
        ['LTBR','SPEF2'],
        ['LYZ','MAU2'],
        ['LYZ','NHSL2'],
        ['LYZ','NIPAL2'],
        ['LYZ','PAXBP1-AS1'],
        ['LYZ','PRAM1'],
        ['LYZ','SAMD4B'],
        ['LYZ','SETX'],
        ['LYZ','SPATS2L'],
        ['LYZ','TATDN2'],
        ['LYZ','TMEM184C'],
        ['LYZ','TNS3'],
        ['LYZ','TRB@'],
        ['LYZ','UBXN2A'],
        ['LYZ','WWOX'],
        ['LYZ','XIST'],
        ['MATR3','PAIP2'],
        ['MC2R','SH3PXD2A'],
        ['MDM4','TRA@'],
        ['MED9','SLC37A2'],
        ['MEG3','MYH9'],
        ['MPO','RPS21'],
        ['MRPS10','PPM1B'],
        ['MS4A10','SP140L'],
        ['MS4A7','TCTN1'],
        ['MT1E','MT1L'],
        ['MUC7','NDRG2'],
        ['MUC7','SLC35E2B'],
        ['MYH11','PABPC1'],
        ['MYH6','PLIN4'],
        ['MYH6','ZNF106'],
        ['MYH7','PSMC4'],
        ['MYH9','RPL13A'],
        ['MYL3','PRSS44'],
        ['NORAD','TRB@'],
        ['NPPA','RAPGEF1'],
        ['NUDT4','PNLIP'],
        ['NUMA1','SMR3B'],
        ['OAZ2','SIK2'],
        ['PABPC1','RPS19'],
        ['PAFAH1B2','SUGP2'],
        ['PCDHB13','PCDHB8'],
        ['PDXDC1','PRSS1'],
        ['PDXDC1','SMR3B'],
        ['PIGN','PNLIP'],
        ['PIGR','TRA@'],
        ['PLIN1','WWOX'],
        ['PNLIP','SMIM19'],
        ['PNLIP','TMSB4X'],
        ['PPP1R12B','ZNF623'],
        ['PRPF8','PRSS1'],
        ['PRRC2A','SMR3B'],
        ['PRSS1','ZNF706'],
        ['PSD4','SMR3B'],
        ['RAB21','RNF24'],
        ['RAB2B','TOP3A'],
        ['RBM25','S100A9'],
        ['RFWD3','RSL1D1'],
        ['RHOA','TKT'],
        ['RHOH','SLC31A1'],
        ['RPL37A','SRGN'],
        ['RPS14','S100A9'],
        ['S100A9','YWHAZ'],
        ['SELENOP','TPT1'],
        ['SFI1','SMR3B'],
        ['SFN','TNS1'],
        ['SFTPA2','SLK'],
        ['SLC8A1','TPM1'],
        ['SMR3B','STAT6'],
        ['SMR3B','TGOLN2'],
        ['SMR3B','TSIX'],
        ['SMR3B','USP34'],
        ['SMR3B','VIM'],
        ['SQSTM1','STEAP2'],
        ['STAT3','TRA@'],
        ['TRA@','TRIM69'],
        ['TRB@','ZFP36L1'],
        ['TUBB7P','TUBB8'],
        ['ABR','PRH1-PRR4'],
        ['AC003973.4','LINC01859'],
        ['AC005077.7','LINC00174'],
        ['AC006004.1','FLVCR1'],
        ['AC007099.1','RGS6'],
        ['AC008132.13','CALM1'],
        ['AC008132.13','USP18'],
        ['AC008746.10','LYZ'],
        ['AC008993.2','XXYAC-YRM2039.3'],
        ['AC015849.16','LRRC37A9P'],
        ['AC019349.5','KRT6A'],
        ['AC025335.1','DNAH14'],
        ['AC026150.8','GOLGA8M'],
        ['AC027612.6','IGK@'],
        ['AC097374.2','BP-2171C21.2'],
        ['ACCS','EXT2'],
        ['ACIN1','WTIP'],
        ['ACOT11','MROH7-TTC4'],
        ['ACSL1','HTN3'],
        ['ACSL1','IGH@'],
        ['ACSS1','APMAP'],
        ['ACTA1','CRYAB'],
        ['ACTA1','SDHA'],
        ['ACTB','ALB'],
        ['ACTB','C16ORF72'],
        ['ACTB','DDX5'],
        ['ACTB','EEF1A1'],
        ['ACTB','GLUL'],
        ['ACTB','PRB2'],
        ['ACTB','PRH1-PRR4'],
        ['ACTB','RPL15'],
        ['ACTB','SF3B1'],
        ['ACTB','SPINK5'],
        ['ACTB','SPTAN1'],
        ['ACTN1','IGH@'],
        ['ADGRG1','FCGBP'],
        ['ADGRG7','TFG'],
        ['ADH1A','TF'],
        ['ADSL','TNRC6B'],
        ['AFMID','SLC48A1'],
        ['AGAP5','BMS1P4'],
        ['AGPS','USP34'],
        ['AHNAK','LIPF'],
        ['AHR','LYZ'],
        ['AHSA2','TIAL1'],
        ['AHSG','HP'],
        ['AKT2','C19ORF47'],
        ['AKT2','TRB@'],
        ['AL050303.11','LINC01087'],
        ['AL050303.11','LINC01297'],
        ['ALB','CHD4'],
        ['ALB','CYP1A2'],
        ['ALB','CYP4A11'],
        ['ALDH18A1','SMR3B'],
        ['ALDH1L1','HTN3'],
        ['ALG1L5P','TSSC2'],
        ['ALK','CDK12'],
        ['AMY2A','CELA2A'],
        ['AMY2A','CELA3A'],
        ['AMY2A','CELA3B'],
        ['AMY2A','CTRB1'],
        ['AMY2A','PNLIP'],
        ['AMY2A','PRSS1'],
        ['AMY2B','PRH1-PRR4'],
        ['AMY2B','PRH2'],
        ['AMY2B','PRSS1'],
        ['ANAPC1','RMND5A'],
        ['ANAPC7','KDSR'],
        ['ANKRD18CP','ZNF322'],
        ['ANKRD20A21P','BP-2189O9.2'],
        ['ANKRD54','LYZ'],
        ['ANKRD62','VWF'],
        ['ANP32B','IGH@'],
        ['ANP32B','PTMA'],
        ['ANXA1','SMR3B'],
        ['ANXA2','PRB2'],
        ['AP000351.4','GSTTP1'],
        ['AP005530.1','C2ORF27B'],
        ['AP3S1','COMMD10'],
        ['APBB2','SPAG9'],
        ['APBB3','SRA1'],
        ['APLP2','HSP90B1'],
        ['APP','KRT13'],
        ['AQP7','CH17-469D17.1'],
        ['ARAP1','PRB3'],
        ['AREL1','TG'],
        ['ARHGAP1','NPPA'],
        ['ARHGAP44','MAGT1'],
        ['ARID1A','TG'],
        ['ARIH2','TG'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARL6IP1','RPS15A'],
        ['ARRDC3','HIST1H2BD'],
        ['ART4','SMCO3'],
        ['ASCC1','TG'],
        ['ASL','GUSBP11'],
        ['ASPH','PRB3'],
        ['ATG16L1','INPP5D'],
        ['ATG16L2','IGK@'],
        ['ATL2','TG'],
        ['ATP13A1','TG'],
        ['ATP1B2','LINC00649'],
        ['ATP5G2','TFRC'],
        ['ATP5G3','NPPA-AS1'],
        ['AUTS2','PRB3'],
        ['AZGP1','GJC3'],
        ['AZGP1','PRB4'],
        ['B2M','CALM1'],
        ['B2M','HMGB1'],
        ['B2M','TMSB4X'],
        ['B3GAT3','FCGBP'],
        ['BANK1','CTSB'],
        ['BANK1','IGH@'],
        ['BANK1','IGK@'],
        ['BANK1','OSGEP'],
        ['BCAP29','DUS4L'],
        ['BCAP29','SLC26A4'],
        ['BCKDHA','TMEM91'],
        ['BCL2','LYZ'],
        ['BCL9','TSPAN14'],
        ['BCLAF1','EEF1A1'],
        ['BGN','FAM102B'],
        ['BMS1P4','TIMM23B'],
        ['BORCS7-ASMT','CCND3'],
        ['BP-21264C1.2','MAFIP'],
        ['BPI','NUMA1'],
        ['BPIFA2','MUC7'],
        ['BPTF','LRRC37A2'],
        ['C12ORF49','TG'],
        ['C15ORF57','CBX3'],
        ['C19MC','RPLP1'],
        ['C1ORF194','UQCR10'],
        ['C1S','ORM2'],
        ['C2ORF48','RRM2'],
        ['C5AR1','LYZ'],
        ['C7','RPS4X'],
        ['C9ORF78','HBB'],
        ['CA1','MGEA5'],
        ['CA6','PRH1-PRR4'],
        ['CALR','H19'],
        ['CAPZB','SMR3B'],
        ['CARF','IQUB'],
        ['CASP9','PNLIP'],
        ['CATSPER2','PPIP5K1'],
        ['CCDC152','HP'],
        ['CCL28','PRH1-PRR4'],
        ['CCND3','F11R'],
        ['CCND3','FCAR'],
        ['CCND3','GRB2'],
        ['CCNG2','LRCH4'],
        ['CCNG2','S100A9'],
        ['CD164','WWOX'],
        ['CD24','IGH@'],
        ['CD24','IL3RA'],
        ['CD24','MUC7'],
        ['CD24','RACK1'],
        ['CD2AP','DBT'],
        ['CD63','CPB1'],
        ['CD74','PRH1-PRR4'],
        ['CD74','PRH2'],
        ['CD84','SLC26A2'],
        ['CD9','CST4'],
        ['CDC42','S100A9'],
        ['CDC42EP2','POLA2'],
        ['CDH12','LINC02197'],
        ['CDH17','IGHA1'],
        ['CDK2','RAB5B'],
        ['CDK6','HTN3'],
        ['CEACAM1','MUC2'],
        ['CEL','CELA3A'],
        ['CEL','CELA3B'],
        ['CEL','CELP'],
        ['CEL','CPA2'],
        ['CEL','PRSS3'],
        ['CEL','SLC7A2'],
        ['CELA3A','FLNA'],
        ['CELA3B','TPT1'],
        ['CELP','TRB@'],
        ['CES4A','TRA@'],
        ['CGNL1','SFTPA1'],
        ['CH17-164J11.1','IGK@'],
        ['CH17-264L24.1','IGK@'],
        ['CH17-53B9.2','NCOR1'],
        ['CHCHD2','PHKG1'],
        ['CHGB','IGH@'],
        ['CHI3L2','SMR3B'],
        ['CHRDL1','CHST11'],
        ['CHRM3','PRB3'],
        ['CHST11','HIF1A'],
        ['CKMT2-AS1','LYZ'],
        ['CLCC1','KLHDC4'],
        ['CLCN3','LIPG'],
        ['CLEC2D','IGH@'],
        ['CLPS','CTRB1'],
        ['CLU','GAPDH'],
        ['CMBL','WWOX'],
        ['CNN2','S100A9'],
        ['CNNM2','LYZ'],
        ['CNOT10','HBB'],
        ['CNOT4','SPTLC2'],
        ['COASY','HSD17B1'],
        ['COL1A1','PTGDS'],
        ['COL6A2','KRT14'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CORO1A','EEF1A1'],
        ['COX7C','ZG16B'],
        ['CPB1','CTRB2'],
        ['CPB1','DDX24'],
        ['CPB1','GOLGA8B'],
        ['CPB1','REG1A'],
        ['CPB1','STAT6'],
        ['CPD','HTN3'],
        ['CPLX2','GLUL'],
        ['CPT1A','TG'],
        ['CSF2RA','LYZ'],
        ['CSH1','H19'],
        ['CST1','LPO'],
        ['CST2','IGK@'],
        ['CST2','STATH'],
        ['CTBP1','SMR3B'],
        ['CTRB1','SYCN'],
        ['CTRB2','GNAS'],
        ['CTSC','RAB38'],
        ['CTSD','PRB1'],
        ['CTSS','MAN1C1'],
        ['CTTN','IGK@'],
        ['CUX1','HBB'],
        ['CWC25','LSINCT5'],
        ['CWC25','OSBPL2'],
        ['CX3CL1','PRH1-PRR4'],
        ['CX3CL1','PRH2'],
        ['CXCL16','CYB561A3'],
        ['CXCR2','WSB1'],
        ['CXCR4','EEF1A1'],
        ['CXCR4','HNRNPA2B1'],
        ['CXXC5','TG'],
        ['CYBA','EML4'],
        ['CYP1A2','MICAL2'],
        ['CYP3A4','HECW1'],
        ['CYP4F34P','CYP4F35P'],
        ['CYP4V2','NSD2'],
        ['DA750114','TRA@'],
        ['DCTN2','PRB2'],
        ['DCTN5','GTDC1'],
        ['DDOST','SMR3B'],
        ['DDOST','TAS2R4'],
        ['DDX5','PTMA'],
        ['DENND3','HBA2'],
        ['DGCR2','PRH1-PRR4'],
        ['DGCR2','PRH2'],
        ['DGCR6','XXBAC-B444P24.13'],
        ['DGKZ','PRB3'],
        ['DHODH','MYSM1'],
        ['DHRS3','MAPK1'],
        ['DHX9','GLUL'],
        ['DHX9','NPL'],
        ['DIABLO','TG'],
        ['DIS3L2','SMR3B'],
        ['DKK3','MYH7'],
        ['DMBT1','IGH@'],
        ['DMBT1','PRB2'],
        ['DNAJB2','PRH1'],
        ['DNAJB6','MPO'],
        ['DNAJC25-GNG10','UGCG'],
        ['DNMT1','LYZ'],
        ['DPCR1','IGH@'],
        ['DPCR1','MUC5AC'],
        ['DPH5','TG'],
        ['DPYSL3','KRT13'],
        ['DSC3','SKP2'],
        ['DSP','EEF1A1'],
        ['DUXAP8','LINC01297'],
        ['DYM','STATH'],
        ['EBLN3P','ZCCHC7'],
        ['ECI2','PRSS1'],
        ['EEF1A1','FOS'],
        ['EEF1A1','HBA2'],
        ['EEF1A1','HMGB1'],
        ['EEF1A1','ITGB1'],
        ['EEF1A1','NFKBIZ'],
        ['EEF1A1','RPS4X'],
        ['EEF1A1','SLC4A1'],
        ['EEF1A1','SYNCRIP'],
        ['EEF1A1','TG'],
        ['EEF1A1','UBC'],
        ['EEF1A1','XPR1'],
        ['EEF1D','TG'],
        ['EEF2','IGKC'],
        ['EEF2','PRH1-PRR4'],
        ['EFCAB10','YBX1'],
        ['EGFR','ZG16B'],
        ['EIF3B','HTN3'],
        ['EIF3E','HTN1'],
        ['EIF3E','TRB@'],
        ['EIF4B','KCTD20'],
        ['EIF4E','PIK3C2A'],
        ['EIF4E3','FOXP1'],
        ['ELANE','HBA2'],
        ['ELK4','SLC45A3'],
        ['ELOVL6','MUC5AC'],
        ['EMX2OS','TRA@'],
        ['ENAH','LYZ'],
        ['EPOR','SLC16A12'],
        ['ERBB2','GRB7'],
        ['ERVW-1','LINC01239'],
        ['ETV6','PIGR'],
        ['EWSR1','KRT10'],
        ['EWSR1','LYZ'],
        ['FAM129A','HTN3'],
        ['FAM134C','TEDDM1'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM157C','RPL23AP53'],
        ['FAM20B','TFDP2'],
        ['FARP1','KRT10'],
        ['FBXL18','TTLL3'],
        ['FBXL5','HBB'],
        ['FCGBP','IGH@'],
        ['FCGBP','IGHA2'],
        ['FCGBP','SMR3B'],
        ['FGD5-AS1','LINC00969'],
        ['FGFR2','PNLIP'],
        ['FGFR2','TMPO'],
        ['FGFR4','ZNF346'],
        ['FICD','ISCU'],
        ['FKRP','TG'],
        ['FLJ22447','PRKCH'],
        ['FLOT1','SMR3B'],
        ['FMO4','TOP1'],
        ['FOSB','PPM1N'],
        ['FOXJ2','TG'],
        ['FOXP1','GPRC5A'],
        ['FOXP1','HECA'],
        ['FOXP1','IFNAR2'],
        ['FOXP4','PRH1'],
        ['FRG1BP','FRG1HP'],
        ['FRG1HP','FRG1JP'],
        ['FRG2C','FRG2EP'],
        ['FTH1','PRB3'],
        ['FUS','PTMA'],
        ['GABPB1-AS1','NSD2'],
        ['GALNT10','SFTPA1'],
        ['GAPDH','H19'],
        ['GAPDH','MUC7'],
        ['GAPDH','MYO1A'],
        ['GAPDH','PRH1-PRR4'],
        ['GATM-AS1','SMG1'],
        ['GBA','TG'],
        ['GCC2-AS1','PLGLB1'],
        ['GEMIN6','TSPAN14'],
        ['GGT1','IGSF3'],
        ['GINM1','RN7SL470P'],
        ['GLG1','HTN3'],
        ['GLTP','MALL'],
        ['GLTP','NEK5'],
        ['GLUL','PTMA'],
        ['GNAS','IGF2'],
        ['GNAS','IGH@'],
        ['GNAS','RGS5'],
        ['GNE','PRH1-PRR4'],
        ['GOLT1A','KISS1'],
        ['GP2','GSN'],
        ['GP2','MAN1B1-AS1'],
        ['GP2','MEG3'],
        ['GPR155','OTUB1'],
        ['GPRC5A','MIR100HG'],
        ['GRK2','IGH@'],
        ['GSE1','SLC7A5'],
        ['GSN','SMR3B'],
        ['GSN','TRB@'],
        ['GSTM4','MUC7'],
        ['GUSBP11','LYZ'],
        ['GYG2','TG'],
        ['GYPA','HBB'],
        ['H19','NRP1'],
        ['H19','PFN1'],
        ['H19','PLAC4'],
        ['H19','PLPPR2'],
        ['H19','TBC1D14'],
        ['H19','TPT1'],
        ['H1F0','PRB1'],
        ['H1F0','PRH1-PRR4'],
        ['HBB','HGS'],
        ['HBB','MICAL2'],
        ['HBB','MYH10'],
        ['HBB','MYO9B'],
        ['HBB','NSUN3'],
        ['HBB','PAFAH1B1'],
        ['HBB','PHLDA1'],
        ['HBB','PLXND1'],
        ['HBB','PTMA'],
        ['HBB','RBMS1'],
        ['HBB','RPS27'],
        ['HBB','RPS3'],
        ['HBB','SDHA'],
        ['HBB','SFI1'],
        ['HBB','SMARCA2'],
        ['HBB','SP100'],
        ['HBB','TACC3'],
        ['HBB','TSC2'],
        ['HBB','ZNF726'],
        ['HERC1','TG'],
        ['HHLA3','TESPA1'],
        ['HKR1','LINC00665'],
        ['HKR1','ZNF790-AS1'],
        ['HMGB1','PNLIP'],
        ['HMGB2','TMSB4X'],
        ['HMGN2','SEPT5'],
        ['HNRNPA1','KRT4'],
        ['HNRNPA1L2','MRPS31P4'],
        ['HNRNPA3','SFTPC'],
        ['HNRNPH1','PTP4A1'],
        ['HNRNPUL1','SMR3B'],
        ['HP','PLG'],
        ['HRG','ORM1'],
        ['HSBP1L1','ZNF554'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPA5','MPO'],
        ['HSPA6','HSPA7'],
        ['HSPE1','MOB4'],
        ['HTN1','NAALADL2'],
        ['HTN1','PDIA6'],
        ['HTN1','PLA2R1'],
        ['HTN1','ZG16B'],
        ['HTN3','LARP1'],
        ['HTN3','LRRC26'],
        ['HTN3','MGP'],
        ['HTN3','PRH2'],
        ['IGF2','NAP1L1'],
        ['IGFBP5','RPL30'],
        ['IGH@','LINC00926'],
        ['IGH@','MUC1'],
        ['IGH@','MUC2'],
        ['IGH@','MXRA7'],
        ['IGH@','NDRG1'],
        ['IGH@','NFKB2'],
        ['IGH@','NPM1'],
        ['IGH@','PCSK7'],
        ['IGH@','PRH2'],
        ['IGH@','PRR4'],
        ['IGH@','PTMA'],
        ['IGH@','PTRF'],
        ['IGH@','RAP1GAP'],
        ['IGH@','SDC1'],
        ['IGH@','SFTPC'],
        ['IGH@','SLC25A6'],
        ['IGH@','SRRM2'],
        ['IGH@','TAGLN2'],
        ['IGH@','TRA@'],
        ['IGH@','ZFP36L1'],
        ['IGHA1','LTF'],
        ['IGHA1','MUC1'],
        ['IGHA1','MVP'],
        ['IGHA1','MYO1A'],
        ['IGHA1','PFN1'],
        ['IGK@','KNL1'],
        ['IGK@','LINC01123'],
        ['IGK@','PRH2'],
        ['IGK@','SAMHD1'],
        ['IGK@','STATH'],
        ['IGK@','TRA@'],
        ['IGLL5','SMR3B'],
        ['IL3RA','YIPF5'],
        ['IL6ST','LYZ'],
        ['IL6ST','PRH1-PRR4'],
        ['ITGB4','SMR3B'],
        ['JADRR','KCNE3'],
        ['KANSL1','LRRC37A3'],
        ['KDM2A','NCALD'],
        ['KIAA1217','SMR3B'],
        ['KIAA2012','PEX26'],
        ['KIF13B','SMR3B'],
        ['KIF1C','SMR3B'],
        ['KLK3','PLPP3'],
        ['KLK4','KLKP1'],
        ['KMT2C','TPTE'],
        ['KRT13','MUC21'],
        ['KRT13','PFKL'],
        ['KRT13','RGS12'],
        ['KRT6A','MYH11'],
        ['KYNU','ZNF528-AS1'],
        ['LENG8','PRSS1'],
        ['LINC00378','LINC01442'],
        ['LINC00491','LINC01807'],
        ['LINC00665','ZNF790-AS1'],
        ['LINC01251','TRB@'],
        ['LINC01347','NUPR2'],
        ['LINC01641','RPS18'],
        ['LLFOS-48D6.2','PTMA'],
        ['LMX1B','PAPOLG'],
        ['LRIG2','ZNF106'],
        ['LRRC37A3','NSF'],
        ['LTBR','SPEF2'],
        ['LYZ','MAU2'],
        ['LYZ','NHSL2'],
        ['LYZ','NIPAL2'],
        ['LYZ','PAM'],
        ['LYZ','PAXBP1-AS1'],
        ['LYZ','PRAM1'],
        ['LYZ','SAMD4B'],
        ['LYZ','SETX'],
        ['LYZ','SPATS2L'],
        ['LYZ','TAPBP'],
        ['LYZ','TATDN2'],
        ['LYZ','TMEM184C'],
        ['LYZ','TNS3'],
        ['LYZ','TRB@'],
        ['LYZ','UBXN2A'],
        ['LYZ','WWOX'],
        ['LYZ','XIST'],
        ['MATR3','PAIP2'],
        ['MBNL1','SMR3B'],
        ['MC2R','SH3PXD2A'],
        ['MDM4','RAB30-AS1'],
        ['MDM4','TRA@'],
        ['MED9','SLC37A2'],
        ['MEG3','MYH9'],
        ['MEIS1','STATH'],
        ['METTL21B','TSFM'],
        ['MFSD13A','SLC25A29'],
        ['MINPP1','NUTM2A-AS1'],
        ['MN1','NCOA3'],
        ['MPO','RPS21'],
        ['MRPS10','PPM1B'],
        ['MS4A10','SP140L'],
        ['MS4A7','TCTN1'],
        ['MSMB','NCOA4'],
        ['MT1E','MT1L'],
        ['MTMR9','YIPF5'],
        ['MUC7','NDRG2'],
        ['MUC7','PTPRF'],
        ['MUC7','SLC35E2B'],
        ['MYH11','PABPC1'],
        ['MYH6','PLIN4'],
        ['MYH6','ZNF106'],
        ['MYH7','PSMC4'],
        ['MYH9','RPL13A'],
        ['MYL3','PRSS44'],
        ['NAA35','RPL7L1'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NBPF13P','PDE4DIP'],
        ['NCOA6','PIGU'],
        ['NFIX','PRB3'],
        ['NFIX','PRH2'],
        ['NORAD','TRB@'],
        ['NOTCH2','SMR3B'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NPPA','RAPGEF1'],
        ['NSUN5','TRIM50'],
        ['NSUN5','TRIM73'],
        ['NUDCD3','TG'],
        ['NUDT4','PNLIP'],
        ['NUMA1','SMR3B'],
        ['NUP188','PHYHD1'],
        ['OAZ2','SIK2'],
        ['OCIAD1','TG'],
        ['ODAM','PRH1-PRR4'],
        ['ODAM','PRH2'],
        ['OPRPN','SMR3B'],
        ['PABPC1','RPS19'],
        ['PAFAH1B2','SUGP2'],
        ['PBX1','PRB2'],
        ['PBXIP1','PMVK'],
        ['PCBP1-AS1','STATH'],
        ['PCBP2','PHF7'],
        ['PCDHB13','PCDHB8'],
        ['PDE1C','SNX1'],
        ['PDXDC1','PRSS1'],
        ['PDXDC1','SMR3B'],
        ['PEX26','PLIN3'],
        ['PIGN','PNLIP'],
        ['PIGR','TRA@'],
        ['PIK3R1','PRH1-PRR4'],
        ['PIM3','SCO2'],
        ['PKD1','PRB3'],
        ['PKP4','PRB3'],
        ['PLCB1','STATH'],
        ['PLEKHM1','TG'],
        ['PLIN1','WWOX'],
        ['PMS2','POLR2J3'],
        ['PNLIP','SMIM19'],
        ['PNLIP','TMSB4X'],
        ['POLR2J4','RASA4'],
        ['PPIP5K1','STRC'],
        ['PPP1R12B','ZNF623'],
        ['PRB1','PRSS8'],
        ['PRB2','ZG16B'],
        ['PRH1-PRR4','PRR27'],
        ['PRH1-PRR4','PSMF1'],
        ['PRH1-PRR4','RBM47'],
        ['PRH1-PRR4','RPL3'],
        ['PRH1-PRR4','SFRP1'],
        ['PRH1-PRR4','SIX4'],
        ['PRH1-PRR4','SLC20A2'],
        ['PRH1-PRR4','SLC6A8'],
        ['PRH1-PRR4','SORL1'],
        ['PRH1-PRR4','SQSTM1'],
        ['PRH2','RBM47'],
        ['PRH2','SFRP1'],
        ['PRH2','SLC6A8'],
        ['PRH2','SQSTM1'],
        ['PRPF8','PRSS1'],
        ['PRRC2A','SMR3B'],
        ['PRSS1','ZNF706'],
        ['PSD4','SMR3B'],
        ['PSPC1','ZMYM5'],
        ['RAB21','RNF24'],
        ['RAB2B','TOP3A'],
        ['RACK1','TRIM52'],
        ['RASSF8','SSPN'],
        ['RBM25','S100A9'],
        ['RCOR3','TG'],
        ['RFWD3','RSL1D1'],
        ['RHOA','TKT'],
        ['RHOH','SLC31A1'],
        ['RIPK1','SERPINB9'],
        ['RNF19A','TG'],
        ['RNF4','TG'],
        ['RPL14','SRP14'],
        ['RPL14','VSIG1'],
        ['RPL15','SMR3B'],
        ['RPL15','TG'],
        ['RPL28','TMEM190'],
        ['RPL37A','SRGN'],
        ['RPL38','TTYH2'],
        ['RPL4','TG'],
        ['RPL7L1','WBP2NL'],
        ['RPS11','SMR3B'],
        ['RPS14','S100A9'],
        ['RPS19','TG'],
        ['RPS25','SMR3B'],
        ['S100A9','YWHAZ'],
        ['SAMD5','SASH1'],
        ['SCNN1A','TNFRSF1A'],
        ['SELENOP','TPT1'],
        ['SENP1','TG'],
        ['SERF1A','SMN1'],
        ['SFI1','SMR3B'],
        ['SFN','TNS1'],
        ['SFTPA2','SLK'],
        ['SIDT2','TAGLN'],
        ['SLC1A4','TG'],
        ['SLC5A5','SMR3B'],
        ['SLC6A16','TEAD2'],
        ['SLC8A1','TPM1'],
        ['SLC9A7','VPS41'],
        ['SMR3B','SNTB1'],
        ['SMR3B','STAT6'],
        ['SMR3B','TBL1XR1'],
        ['SMR3B','TGOLN2'],
        ['SMR3B','TOB1'],
        ['SMR3B','TSIX'],
        ['SMR3B','USP34'],
        ['SMR3B','VIM'],
        ['SQSTM1','STEAP2'],
        ['STAG3','WWOX'],
        ['STAT3','TRA@'],
        ['STATH','WASHC4'],
        ['TBC1D29','USP6'],
        ['TBCEL','TECTA'],
        ['TDRP','TG'],
        ['TG','TMCC3'],
        ['TG','TPCN1'],
        ['TG','TSPAN14'],
        ['TG','USP39'],
        ['TG','VPS45'],
        ['TMEM45B','TRA@'],
        ['TRA@','TRIM69'],
        ['TRB@','ZFP36L1'],
        ['TUBB7P','TUBB8'],
        ['VPS53','WDR70'],
        ['ZNF440','ZNF69'],
        ['ZNF525','ZNF845'],
        ['ZNF702P','ZNF816-ZNF321P'],
        ['AC004520.1','HNRNPA2B1'],
        ['AC004980.4','AC211486.3'],
        ['AC005332.7','AMZ2'],
        ['AC005336.1','IPMK'],
        ['AC006064.4','EEF1A1'],
        ['AC008083.3','PCED1B'],
        ['AC009065.3','PDXDC1'],
        ['AC010273.2','LINC02198'],
        ['AC010329.1','UBXN8'],
        ['AC011450.1','SLC6A16'],
        ['AC011603.3','MBP'],
        ['AC011611.5','DENR'],
        ['AC012485.1','TARDBP'],
        ['AC017083.4','C1D'],
        ['AC017083.4','CNRIP1'],
        ['AC018754.1','CCNH'],
        ['AC019131.2','ADH5'],
        ['AC020656.1','CCL5'],
        ['AC022400.3','PARG'],
        ['AC069208.1','SOX5'],
        ['AC074250.2','AFM'],
        ['AC079921.2','KLHL5'],
        ['AC080162.1','IAH1'],
        ['AC084082.1','TRIM55'],
        ['AC087477.2','NR2F2'],
        ['AC091057.2','GOLGA8N'],
        ['AC092691.1','LSAMP'],
        ['AC092718.3','NDUFS1'],
        ['AC093142.1','RPL34'],
        ['AC093484.4','UBC'],
        ['AC093536.1','EBPL'],
        ['AC100771.1','RPL37A'],
        ['AC106795.2','THOC3'],
        ['AC116535.2','CTR9'],
        ['AC117409.1','PSMC2'],
        ['AC117440.1','MTCH2'],
        ['AC127502.2','AC135983.2'],
        ['AC133065.2','CIITA'],
        ['AC133555.5','SMG1'],
        ['AC134879.2','IGK@'],
        ['AC135983.2','WHAMM'],
        ['AC136944.4','DUSP22'],
        ['AC138207.8','LRRC37B'],
        ['AC138207.8','SMURF2'],
        ['AC145124.1','FAM85B'],
        ['AC243967.3','LYPD4'],
        ['AC244517.11','TAF7'],
        ['AC245060.4','IGLV5-52'],
        ['ADAMTS4','TRA@'],
        ['AF305872.1','RPL21'],
        ['AIG1','AL136116.3'],
        ['AL035587.1','CNPY3'],
        ['AL049839.2','FGB'],
        ['AL049839.2','SERPINA1'],
        ['AL117335.1','AL121760.1'],
        ['AL121761.1','RIN2'],
        ['AL121936.1','NORAD'],
        ['AL133415.1','VIM-AS1'],
        ['AL136982.4','BMS1'],
        ['AL139174.1','MMADHC'],
        ['AL160313.1','HHIPL1'],
        ['AL161782.1','LINC01507'],
        ['AL357078.1','JAK1'],
        ['AL590762.3','ZCRB1'],
        ['ANAPC1','RMND5A'],
        ['ANO1','AP003555.3'],
        ['AP000790.1','YWHAZ'],
        ['AP001528.2','FZD4'],
        ['AP003084.1','EED'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARNT','MYH11'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BMPR1A','MIR100HG'],
        ['BRS3','HTATSF1'],
        ['C19MC','RPLP1'],
        ['CATSPER2','PPIP5K1'],
        ['COQ8B','NUMBL'],
        ['CTSB','PIEZO1'],
        ['CTSB','SLC35E3'],
        ['CTSS','MIR100HG'],
        ['FAM157B','FBXO25'],
        ['GSN','PECAM1'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['MECOM','RPL22'],
        ['NAIP','OCLN'],
        ['NME4','Z97634.1'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['RPS23','ZNF888'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['AC004980.4','AC091390.4'],
        ['AC004980.4','PMS2'],
        ['AC004980.4','PRKRIP1'],
        ['AC004980.4','UPK3B'],
        ['AC005332.7','AMZ2'],
        ['AC006548.3','IGH@'],
        ['AC006548.3','TRA@'],
        ['AC007342.4','RBL2'],
        ['AC008993.1','AL928970.1'],
        ['AC009041.2','LMBR1'],
        ['AC009961.1','BAZ2B'],
        ['AC010273.2','LINC02198'],
        ['AC011481.3','LINC01602'],
        ['AC022400.5','LINC00843'],
        ['AC023157.1','IFITM3'],
        ['AC027290.2','OSBPL2'],
        ['AC068152.1','ARL17A'],
        ['AC069208.1','SOX5'],
        ['AC074043.1','PTPN11'],
        ['AC079921.2','KLHL5'],
        ['AC087500.1','RABEP1'],
        ['AC091057.2','GOLGA8N'],
        ['AC091390.2','PMS2'],
        ['AC092683.1','TRA@'],
        ['AC108024.1','KCNE3'],
        ['AC126544.2','BRWD1'],
        ['AC126755.3','AC136443.3'],
        ['AC135983.2','WHAMM'],
        ['AC138393.1','ZSCAN16-AS1'],
        ['AC138932.3','KIAA2013'],
        ['AC245060.5','IGK@'],
        ['AC245060.6','IGK@'],
        ['AL035701.1','CLIC5'],
        ['AL121761.1','RIN2'],
        ['AL121790.1','MIPOL1'],
        ['AL157871.1','YY1'],
        ['AL161457.2','FRG1JP'],
        ['AL161669.1','EXOC3L4'],
        ['AL450992.1','NOTCH2NL'],
        ['AL513314.2','LINC01002'],
        ['AL589843.2','MKRN1'],
        ['AL590867.1','TRA@'],
        ['AL691447.2','MFSD14C'],
        ['ALK','DA750114'],
        ['AP001372.3','SERPINA10'],
        ['AP001372.3','ZNF726'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['AZGP1','GJC3'],
        ['BACH2','TRA@'],
        ['BANK1','DA750114'],
        ['BANK1','IGH@'],
        ['BANK1','TRA@'],
        ['CDC42EP2','POLA2'],
        ['COQ8B','NUMBL'],
        ['DA750114','LINC00506'],
        ['DA750114','LMO7'],
        ['FAM157B','FBXO25'],
        ['FCRL4','IGH@'],
        ['HECW2','LINC00381'],
        ['HELLPAR','TRA@'],
        ['IGH@','LINC00506'],
        ['IGH@','MIR100HG'],
        ['IGH@','TMEM38B'],
        ['IGH@','TRA@'],
        ['IGH@','WWOX'],
        ['IGK@','MIR100HG'],
        ['KANSL1','LRRC37A3'],
        ['LINC00504','TRA@'],
        ['LINC01655','LINC02257'],
        ['LRRC37A3','NSF'],
        ['LRRK1','OLMALINC'],
        ['MIR100HG','TRA@'],
        ['MSMB','NCOA4'],
        ['MUC2','MUC5AC'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['POM121','PRKRIP1'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SOD2','TRA@'],
        ['TMEM38B','TRA@'],
        ['TMEM38B','TRB@'],
        ['TRA@','Z99755.3'],
        ['AC006548.3','DA750114'],
        ['AC016747.4','AHSA2'],
        ['AC044860.1','AC243562.1'],
        ['AC091390.4','POM121'],
        ['AC093752.1','AC138393.1'],
        ['AFMID','AP001372.3'],
        ['AGAP5','BMS1P4'],
        ['AL450992.1','NBPF15'],
        ['ALK','TRA@'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['AZGP1','GJC3'],
        ['BACH2','DA750114'],
        ['CDC42EP2','POLA2'],
        ['CHST11','DA750114'],
        ['COQ8B','NUMBL'],
        ['CYP4X1','CYP4Z2P'],
        ['DA750114','HELLPAR'],
        ['DA750114','MIR100HG'],
        ['DA750114','SOD2'],
        ['DCUN1D1','MCCC1'],
        ['FAM157B','FBXO25'],
        ['FCRL4','IGH@'],
        ['FOXP1','IGH@'],
        ['IGH@','MIR100HG'],
        ['IGH@','TRA@'],
        ['IGH@','WWOX'],
        ['IGK@','LINC00504'],
        ['KANSL1','LRRC37A3'],
        ['LRRC37A3','NSF'],
        ['MECOM','RPL22'],
        ['MSMB','NCOA4'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['PTBP3','SUSD1'],
        ['RPAP2','TRA@'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['ABBA01031661.1','ABBA01031674.1'],
        ['ABCA13','AC027088.1'],
        ['ABCA13','NOX5'],
        ['ABCA13','TRA@'],
        ['AC003102.2','YIPF5'],
        ['AC004706.3','FOXP1'],
        ['AC004980.4','AC091390.2'],
        ['AC005224.3','KCNK6'],
        ['AC005244.1','DYNLT1'],
        ['AC005832.2','CCDC181'],
        ['AC006548.3','CCDC90B'],
        ['AC006548.3','DA750114'],
        ['AC006548.3','IGK@'],
        ['AC007038.1','RPE'],
        ['AC007342.3','MPHOSPH10'],
        ['AC007529.2','MGST1'],
        ['AC007688.2','AC023790.2'],
        ['AC008158.1','POLRMT'],
        ['AC009093.2','BANP'],
        ['AC009318.1','ERGIC2'],
        ['AC010186.2','CLEC2D'],
        ['AC010329.1','AC061975.7'],
        ['AC010329.1','AL392046.2'],
        ['AC010615.1','ZNF738'],
        ['AC011997.1','PLCB1'],
        ['AC016588.2','AL078621.3'],
        ['AC016745.1','AL353608.2'],
        ['AC016747.4','AHSA2'],
        ['AC022400.5','PARG'],
        ['AC022400.5','TIMM23B'],
        ['AC023538.1','SMC5'],
        ['AC025161.1','RANBP1'],
        ['AC026412.1','PDCD6'],
        ['AC027020.2','PRKX'],
        ['AC037198.1','FSIP1'],
        ['AC037487.4','SLC16A6'],
        ['AC044860.1','AC243562.1'],
        ['AC068152.1','AC091132.1'],
        ['AC068587.6','AC127526.4'],
        ['AC068587.6','LINC00937'],
        ['AC073073.1','RNF181'],
        ['AC073343.2','ZNF316'],
        ['AC087473.1','TRA@'],
        ['AC087473.1','USP33'],
        ['AC091132.1','ARL17B'],
        ['AC091390.4','POM121'],
        ['AC091390.4','POM121C'],
        ['AC091564.2','TAF10'],
        ['AC093752.1','AC138393.1'],
        ['AC098831.1','PYURF'],
        ['AC099805.1','EYA1'],
        ['AC104819.3','SH3D19'],
        ['AC104837.1','SNW1'],
        ['AC106791.1','KLHL3'],
        ['AC110079.1','AC118758.3'],
        ['AC117386.2','TRA@'],
        ['AC122133.1','SEPT7'],
        ['AC131254.2','RBM47'],
        ['AC134511.1','GRIP1'],
        ['AC139795.1','SIMC1'],
        ['AC211486.3','PMS2'],
        ['AC211486.3','UPK3B'],
        ['ACTN4','EIF3K'],
        ['ADAM32','BANK1'],
        ['AFMID','AP001372.3'],
        ['AGAP5','BMS1P4'],
        ['AKR1B15','DA750114'],
        ['AL022238.4','TNRC6B'],
        ['AL121761.1','GABRP'],
        ['AL133410.1','RGP1'],
        ['AL139260.3','RHBDL2'],
        ['AL157791.1','COIL'],
        ['AL157834.2','PDLIM1'],
        ['AL365357.1','RPS14'],
        ['AL390728.5','ZNF496'],
        ['AL450992.1','NBPF15'],
        ['AL513283.1','MYO1D'],
        ['AL645929.1','AL671883.3'],
        ['AL845552.1','HMGN1'],
        ['ALG14','CNN3'],
        ['ALK','IGK@'],
        ['ALK','TRA@'],
        ['AMN1','KIAA1551'],
        ['ANAPC1','RMND5A'],
        ['ANKEF1','PDE1C'],
        ['ANKRD20A21P','CR381670.1'],
        ['ANKRD30B','AP005901.1'],
        ['AP000345.3','DRICH1'],
        ['AP000542.3','DA750114'],
        ['AP001528.1','TMEM135'],
        ['APP','FAM172A'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ASL','CRCP'],
        ['ASTN2','DA750114'],
        ['ASTN2','IGH@'],
        ['ASTN2','TRA@'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BABAM2','RBKS'],
        ['BACH2','DA750114'],
        ['BACH2','IGH@'],
        ['BACH2','IGK@'],
        ['BANK1','IGH@'],
        ['BANK1','TRB@'],
        ['BBOX1-AS1','SLC5A12'],
        ['BCAP29','DUS4L'],
        ['BGLAP','PMF1'],
        ['BX322639.1','ZNF355P'],
        ['C16ORF62','CCP110'],
        ['C19ORF33','SPINT2'],
        ['CALML4','CLN6'],
        ['CARF','NBEAL1'],
        ['CASR','LPP'],
        ['CATSPER2','PPIP5K1'],
        ['CAVIN1','STAT3'],
        ['CCDC170','ESR1'],
        ['CD163','DA750114'],
        ['CDC42EP2','POLA2'],
        ['CDK5RAP2','MEGF9'],
        ['CHIC2','FIP1L1'],
        ['CHRNA9','LINC02265'],
        ['CHST11','DA750114'],
        ['CHST11','IGH@'],
        ['CHST11','TRA@'],
        ['CLEC16A','EMP2'],
        ['CLIC6','MIR100HG'],
        ['CNKSR3','IPCEF1'],
        ['COL14A1','DEPTOR'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CR786580.1','LINC01410'],
        ['CRLF2','CSF2RA'],
        ['CTNND1','TMEM135'],
        ['CUX1','PRKRIP1'],
        ['CXADR','FP475955.3'],
        ['CYP2G1P','CYP2G2P'],
        ['CYP4X1','CYP4Z2P'],
        ['D2HGDH','GAL3ST2'],
        ['DA750114','HELLPAR'],
        ['DA750114','IGH@'],
        ['DA750114','IGK@'],
        ['DA750114','LINC00504'],
        ['DA750114','MIPOL1'],
        ['DA750114','MIR100HG'],
        ['DA750114','SOD2'],
        ['DA750114','SULT1B1'],
        ['DA750114','TMEM45A'],
        ['DCUN1D1','MCCC1'],
        ['DDX11','DDX12P'],
        ['DHX35','FAM83D'],
        ['DNTTIP2','TRA@'],
        ['EBF1','IGH@'],
        ['EIF4E3','FOXP1'],
        ['EIF5','MARK3'],
        ['ELK4','SLC45A3'],
        ['ENTPD4','TRA@'],
        ['ENTPD5','FAM161B'],
        ['FAM129C','PGLS'],
        ['FAM157B','FBXO25'],
        ['FAM177A1','Z99943.2'],
        ['FAM47E','PTGR1'],
        ['FBXL17','PJA2'],
        ['FCRL4','IGH@'],
        ['FER','MAN2A1'],
        ['FOXP1','IGH@'],
        ['FOXP1','IGK@'],
        ['GNAS','OVOL1'],
        ['GSR','GTF2E2'],
        ['GTF2H5','SORBS2'],
        ['H6PD','SPSB1'],
        ['HORMAD1','SLC22A20'],
        ['HSP90AB1','SLC29A1'],
        ['IGH@','KCNK6'],
        ['IGH@','MIR100HG'],
        ['IGH@','RPAP2'],
        ['IGH@','SOD2'],
        ['IGH@','TRA@'],
        ['IGH@','WWOX'],
        ['IGH@','Z99755.3'],
        ['IGK@','KMT2C'],
        ['IGK@','LINC00504'],
        ['IGK@','LINC01106'],
        ['IGK@','TMEM38B'],
        ['INPP1','MFSD6'],
        ['ITM2B','RB1'],
        ['KANSL1','LRRC37A3'],
        ['KCNK6','TRA@'],
        ['KCNQ1OT1','TRA@'],
        ['LAPTM4A','PUM2'],
        ['LINC01091','LINC02435'],
        ['LRRC37A3','NSF'],
        ['MACC1','TAC4'],
        ['MECOM','RPL22'],
        ['MIPOL1','TRA@'],
        ['MIR100HG','TRB@'],
        ['MSL2','STAG1'],
        ['MSMB','NCOA4'],
        ['MTSS1','TATDN1'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NEAT1','TRA@'],
        ['NFATC3','PLA2G15'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NUTM2A','NUTM2B'],
        ['PDCD2L','UBA2'],
        ['PIGR','SMU1'],
        ['POM121C','PRKRIP1'],
        ['PRMT7','SLC7A6'],
        ['PSPC1','ZMYM5'],
        ['PTBP3','SUSD1'],
        ['PVRIG','STAG3L5P-PVRIG2P-PILRB'],
        ['RASSF8','SSPN'],
        ['RLN1','RLN2'],
        ['RNF125','RNF138'],
        ['RPAP2','TRA@'],
        ['RPL28','TMEM190'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SLC25A32','UBR5'],
        ['STK3','VPS13B'],
        ['TMCO2','WDR60'],
        ['TRB@','XIAP'],
        ['UBA2','WTIP'],
        ['USP32','USP6'],
        ['USP34','XPO1'],
        ['ZNF439','ZNF69'],
        ['ABBA01031661.1','ABBA01031674.1'],
        ['ABCA13','AC027088.1'],
        ['ABCA13','NOX5'],
        ['ABCA13','TRA@'],
        ['AC003102.2','YIPF5'],
        ['AC004706.3','FOXP1'],
        ['AC004980.4','AC091390.2'],
        ['AC005224.3','KCNK6'],
        ['AC005244.1','DYNLT1'],
        ['AC005832.2','CCDC181'],
        ['AC006548.3','CCDC90B'],
        ['AC006548.3','DA750114'],
        ['AC006548.3','IGK@'],
        ['AC007038.1','RPE'],
        ['AC007342.3','MPHOSPH10'],
        ['AC007529.2','MGST1'],
        ['AC007688.2','AC023790.2'],
        ['AC008158.1','POLRMT'],
        ['AC009093.2','BANP'],
        ['AC009318.1','ERGIC2'],
        ['AC010186.2','CLEC2D'],
        ['AC010329.1','AC061975.7'],
        ['AC010329.1','AL392046.2'],
        ['AC010615.1','ZNF738'],
        ['AC011997.1','PLCB1'],
        ['AC016588.2','AL078621.3'],
        ['AC016745.1','AL353608.2'],
        ['AC016747.4','AHSA2'],
        ['AC022400.5','PARG'],
        ['AC022400.5','TIMM23B'],
        ['AC023538.1','SMC5'],
        ['AC025161.1','RANBP1'],
        ['AC026412.1','PDCD6'],
        ['AC027020.2','PRKX'],
        ['AC037198.1','FSIP1'],
        ['AC037487.4','SLC16A6'],
        ['AC044860.1','AC243562.1'],
        ['AC068152.1','AC091132.1'],
        ['AC068587.6','AC127526.4'],
        ['AC068587.6','LINC00937'],
        ['AC073073.1','RNF181'],
        ['AC073343.2','ZNF316'],
        ['AC087473.1','TRA@'],
        ['AC087473.1','USP33'],
        ['AC091132.1','ARL17B'],
        ['AC091390.4','POM121'],
        ['AC091390.4','POM121C'],
        ['AC091564.2','TAF10'],
        ['AC093752.1','AC138393.1'],
        ['AC098831.1','PYURF'],
        ['AC099805.1','EYA1'],
        ['AC104819.3','SH3D19'],
        ['AC104837.1','SNW1'],
        ['AC106791.1','KLHL3'],
        ['AC110079.1','AC118758.3'],
        ['AC113398.1','RPL28'],
        ['AC117386.2','TRA@'],
        ['AC122133.1','SEPT7'],
        ['AC131254.2','RBM47'],
        ['AC134511.1','GRIP1'],
        ['AC139795.1','SIMC1'],
        ['AC211429.1','PRKRIP1'],
        ['AC211486.3','PMS2'],
        ['AC211486.3','UPK3B'],
        ['ACTN4','EIF3K'],
        ['ADAM32','BANK1'],
        ['AFMID','AP001372.3'],
        ['AGAP5','BMS1P4'],
        ['AKR1B15','DA750114'],
        ['AL022238.4','TNRC6B'],
        ['AL121761.1','GABRP'],
        ['AL133410.1','RGP1'],
        ['AL139260.3','RHBDL2'],
        ['AL157791.1','COIL'],
        ['AL157834.2','PDLIM1'],
        ['AL365357.1','RPS14'],
        ['AL390728.5','ZNF496'],
        ['AL450992.1','NBPF15'],
        ['AL513283.1','MYO1D'],
        ['AL645929.1','AL671883.3'],
        ['AL845552.1','HMGN1'],
        ['ALG14','CNN3'],
        ['ALK','IGK@'],
        ['ALK','TRA@'],
        ['AMN1','KIAA1551'],
        ['ANAPC1','RMND5A'],
        ['ANKEF1','PDE1C'],
        ['ANKRD20A21P','CR381670.1'],
        ['ANKRD30B','AP005901.1'],
        ['AP000345.3','DRICH1'],
        ['AP000542.3','DA750114'],
        ['AP001528.1','TMEM135'],
        ['APP','FAM172A'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ASL','CRCP'],
        ['ASTN2','DA750114'],
        ['ASTN2','IGH@'],
        ['ASTN2','TRA@'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['BABAM2','RBKS'],
        ['BACH2','DA750114'],
        ['BACH2','IGH@'],
        ['BACH2','IGK@'],
        ['BANK1','IGH@'],
        ['BANK1','TRB@'],
        ['BBOX1-AS1','SLC5A12'],
        ['BCAP29','DUS4L'],
        ['BGLAP','PMF1'],
        ['BPTF','LRRC37A2'],
        ['BX322639.1','ZNF355P'],
        ['C16ORF62','CCP110'],
        ['C19ORF33','SPINT2'],
        ['CALML4','CLN6'],
        ['CARF','NBEAL1'],
        ['CASR','LPP'],
        ['CATSPER2','PPIP5K1'],
        ['CAVIN1','STAT3'],
        ['CCDC170','ESR1'],
        ['CD163','DA750114'],
        ['CDC42EP2','POLA2'],
        ['CDK11B','SLC35E2'],
        ['CDK5RAP2','MEGF9'],
        ['CEP170','RAD51B'],
        ['CHIC2','FIP1L1'],
        ['CHRNA9','LINC02265'],
        ['CHST11','DA750114'],
        ['CHST11','IGH@'],
        ['CHST11','TRA@'],
        ['CITED1','HDAC8'],
        ['CLEC16A','EMP2'],
        ['CLIC6','MIR100HG'],
        ['CNKSR3','IPCEF1'],
        ['COL14A1','DEPTOR'],
        ['COLQ','HACL1'],
        ['COQ8B','NUMBL'],
        ['CR786580.1','LINC01410'],
        ['CRLF2','CSF2RA'],
        ['CTNND1','TMEM135'],
        ['CUX1','PRKRIP1'],
        ['CXADR','FP475955.3'],
        ['CYP2G1P','CYP2G2P'],
        ['CYP4X1','CYP4Z2P'],
        ['D2HGDH','GAL3ST2'],
        ['DA750114','HELLPAR'],
        ['DA750114','IGH@'],
        ['DA750114','IGK@'],
        ['DA750114','LINC00504'],
        ['DA750114','MIPOL1'],
        ['DA750114','MIR100HG'],
        ['DA750114','SOD2'],
        ['DA750114','SULT1B1'],
        ['DA750114','TMEM45A'],
        ['DCUN1D1','MCCC1'],
        ['DDX11','DDX12P'],
        ['DHX35','FAM83D'],
        ['DNTTIP2','TRA@'],
        ['EBF1','IGH@'],
        ['EIF4E3','FOXP1'],
        ['EIF5','MARK3'],
        ['ELK4','SLC45A3'],
        ['ENTPD4','TRA@'],
        ['ENTPD5','FAM161B'],
        ['FAM129C','PGLS'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM177A1','Z99943.2'],
        ['FAM47E','PTGR1'],
        ['FBXL17','PJA2'],
        ['FCRL4','IGH@'],
        ['FER','MAN2A1'],
        ['FOXP1','IGH@'],
        ['FOXP1','IGK@'],
        ['FRK','TRA@'],
        ['GNAS','OVOL1'],
        ['GSR','GTF2E2'],
        ['GTF2H5','SORBS2'],
        ['H6PD','SPSB1'],
        ['HORMAD1','SLC22A20'],
        ['HSP90AB1','SLC29A1'],
        ['IGH@','KCNK6'],
        ['IGH@','MIR100HG'],
        ['IGH@','RPAP2'],
        ['IGH@','SOD2'],
        ['IGH@','TRA@'],
        ['IGH@','WWOX'],
        ['IGH@','Z99755.3'],
        ['IGK@','KMT2C'],
        ['IGK@','LINC00504'],
        ['IGK@','LINC01106'],
        ['IGK@','TMEM38B'],
        ['INPP1','MFSD6'],
        ['ITM2B','RB1'],
        ['KANSL1','LRRC37A3'],
        ['KCNK6','TRA@'],
        ['KCNQ1OT1','TRA@'],
        ['LAPTM4A','PUM2'],
        ['LINC01091','LINC02435'],
        ['LRRC37A3','NSF'],
        ['MACC1','TAC4'],
        ['MECOM','RPL22'],
        ['MIPOL1','TRA@'],
        ['MIR100HG','TRB@'],
        ['MSL2','STAG1'],
        ['MSMB','NCOA4'],
        ['MTSS1','TATDN1'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NBPF10','NOTCH2NL'],
        ['NEAT1','TRA@'],
        ['NFATC3','PLA2G15'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NUTM2A','NUTM2B'],
        ['PDCD2L','UBA2'],
        ['PIGR','SMU1'],
        ['POM121C','PRKRIP1'],
        ['PRMT7','SLC7A6'],
        ['PSPC1','ZMYM5'],
        ['PTBP3','SUSD1'],
        ['PVRIG','STAG3L5P-PVRIG2P-PILRB'],
        ['RASSF8','SSPN'],
        ['RLN1','RLN2'],
        ['RNF125','RNF138'],
        ['RPAP2','TRA@'],
        ['RPL28','TMEM190'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SLC25A32','UBR5'],
        ['SMAD2','TRA@'],
        ['STK3','VPS13B'],
        ['TBCEL','TECTA'],
        ['TMCO2','WDR60'],
        ['TRB@','XIAP'],
        ['UBA2','WTIP'],
        ['USP32','USP6'],
        ['USP34','XPO1'],
        ['ZNF439','ZNF69'],
        ['ABR','CLUH'],
        ['AC002116.2','WDR62'],
        ['AC004846.2','NUMB'],
        ['AC004980.3','AC091390.2'],
        ['AC004980.3','AC211486.3'],
        ['AC005258.1','CSNK1G2'],
        ['AC005258.1','DOT1L'],
        ['AC005258.1','SGTA'],
        ['AC005332.7','BPTF'],
        ['AC005540.1','RPL14'],
        ['AC005674.2','WDR1'],
        ['AC006001.3','RABGEF1'],
        ['AC006064.4','HBB'],
        ['AC008750.8','NKG7'],
        ['AC008764.4','SLC35E1'],
        ['AC009275.1','FGD4'],
        ['AC013394.1','B2M'],
        ['AC016831.1','LINC-PINT'],
        ['AC016876.2','LYZ'],
        ['AC016876.2','MPDU1'],
        ['AC020656.1','PRKAR2A'],
        ['AC020656.1','SLC25A37'],
        ['AC020656.1','SLC43A2'],
        ['AC020656.1','SLC4A1'],
        ['AC025171.2','ZNF131'],
        ['AC025262.1','TBC1D30'],
        ['AC026150.1','GOLGA8M'],
        ['AC026412.1','SDHA'],
        ['AC055713.1','WSB1'],
        ['AC067930.6','HBB'],
        ['AC068152.1','ARL17B'],
        ['AC079316.1','AL139353.3'],
        ['AC087190.4','LYZ'],
        ['AC091390.2','AC211486.3'],
        ['AC091390.3','STAG3'],
        ['AC091390.4','AC211429.1'],
        ['AC091390.4','AC211486.3'],
        ['AC092821.1','AC141557.1'],
        ['AC093162.2','RETSAT'],
        ['AC093484.4','HBA2'],
        ['AC093484.4','HBB'],
        ['AC093752.1','SEPT7'],
        ['AC095350.2','RIMBP2'],
        ['AC096664.2','RPL27'],
        ['AC103702.2','HOXB9'],
        ['AC105046.1','EGR3'],
        ['AC107983.2','USP32'],
        ['AC107983.2','USP6'],
        ['AC108010.1','TLK2'],
        ['AC116366.1','IRF1'],
        ['AC119396.1','ARHGEF18'],
        ['AC125257.1','KLHL11'],
        ['AC127526.4','LINC00937'],
        ['AC132008.2','SDHA'],
        ['AC133065.6','CIITA'],
        ['AC136475.10','SRP19'],
        ['AC138393.1','SEPT7'],
        ['AC138409.2','AP000347.1'],
        ['AC138811.2','SMG1'],
        ['AC138932.1','NPIPA8'],
        ['AC145098.1','DOK3'],
        ['AC245100.4','LINC00869'],
        ['AC245128.3','AL357055.3'],
        ['AC245128.3','ORC4'],
        ['AC245884.4','LYZ'],
        ['AC247036.4','IGH@'],
        ['AC247036.6','IGH@'],
        ['ACCS','EXT2'],
        ['ACSL1','CENPU'],
        ['ACTB','CCNG2'],
        ['ACTB','YWHAZ'],
        ['ACVR1B','GRASP'],
        ['ADM','RIN3'],
        ['AKNA','AL138895.1'],
        ['AKT2','C19ORF47'],
        ['AL035587.1','GNMT'],
        ['AL109811.3','TARDBP'],
        ['AL121578.3','SYTL5'],
        ['AL132780.3','PRMT5'],
        ['AL136133.1','RPL22'],
        ['AL139022.1','FNTB'],
        ['AL139383.1','STARD13'],
        ['AL158154.2','UPP1'],
        ['AL160408.2','HBB'],
        ['AL360181.3','SCART1'],
        ['AL360270.1','CD53'],
        ['AL606491.1','LDLRAP1'],
        ['AL645929.1','AL671277.1'],
        ['AL662899.1','LY6G5C'],
        ['AL928768.1','IGHA1'],
        ['ALDOA','PPP4C'],
        ['ANAPC1','RMND5A'],
        ['ANKAR','ASNSD1'],
        ['ANP32B','TMOD1'],
        ['ANXA1','ZNF207'],
        ['AP000346.2','AP000347.1'],
        ['AP000346.2','AP000347.2'],
        ['AP000924.1','FDX1'],
        ['AP000943.3','MRE11'],
        ['AP001931.1','TMX2'],
        ['APBB1IP','FAM238A'],
        ['ARHGAP18','DLGAP5'],
        ['ARHGAP26','NR3C1'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARPC4','OGG1'],
        ['AS3MT','CCND3'],
        ['ATG16L1','INPP5D'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['AZIN1','KLF10'],
        ['B2M','FUS'],
        ['B2M','MED13L'],
        ['B2M','SF1'],
        ['B2M','UBE3A'],
        ['B3GNTL1','METRNL'],
        ['BCAP29','DUS4L'],
        ['BCL6','NAMPT'],
        ['BEST1','HBB'],
        ['BPI','CFL1'],
        ['BSG','ELANE'],
        ['BSG','R3HDM4'],
        ['BSG','SBNO2'],
        ['BSG','TCF3'],
        ['BSG','UBB'],
        ['C15ORF48','SPATA5L1'],
        ['C15ORF57','CBX3'],
        ['C18ORF8','CAPZA2'],
        ['C19MC','RPLP1'],
        ['C2ORF48','RRM2'],
        ['CASTOR2','FURIN'],
        ['CATSPER2','PPIP5K1'],
        ['CCNG2','OAZ1'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CDC42','ZEB2'],
        ['CDK2','RAB5B'],
        ['CITED1','HDAC8'],
        ['CNN2','RNF126'],
        ['COX19','LINC01002'],
        ['CPM','LYZ'],
        ['CREB3','GRB2'],
        ['CTBP1','MAEA'],
        ['CTDP1','LINC00969'],
        ['CTDP1','PQLC1'],
        ['CTSC','RAB38'],
        ['CTSD','LSP1'],
        ['CUX1','KLF10'],
        ['CXCR4','RARA'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZEB2'],
        ['CYBA','WWOX'],
        ['CYBB','XK'],
        ['DCUN1D1','MCCC1'],
        ['DDX39A','JMJD1C'],
        ['DDX5','EIF4A1'],
        ['DDX5','FUS'],
        ['DDX5','HMGN2'],
        ['DDX5','NFE2L1'],
        ['DDX5','TEX2'],
        ['DDX5','TPD52L2'],
        ['DGCR8','TANGO2'],
        ['DHX9','NPL'],
        ['DLEU2','HMGB1'],
        ['DNAJC7','LYZ'],
        ['DOCK9','STK24'],
        ['DOT1L','PTBP1'],
        ['DOT1L','ZBTB7A'],
        ['EEF1A1','HBA1'],
        ['EIF2AK1','S100A9'],
        ['EIF4E3','FOXP1'],
        ['ELANE','LINC01578'],
        ['EPB42','LYZ'],
        ['EPN2','UBB'],
        ['FAM110A','JAZF1'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FBXO16','HBB'],
        ['FCAR','LENG8'],
        ['FES','MAN2A2'],
        ['FNIP1','PDE4B'],
        ['FUS','KCTD5'],
        ['GALK2','SUPT16H'],
        ['GAS7','RCVRN'],
        ['GATC','KDM2A'],
        ['GCA','SAMSN1'],
        ['GNAS','SENP3'],
        ['GSN','S100A9'],
        ['HBA1','RPS27'],
        ['HBB','NLRC5'],
        ['HBB','SEPT2'],
        ['HMGB1','HNRNPH1'],
        ['HNRNPD','RCC1'],
        ['HNRNPDL','TUBA1B'],
        ['HNRNPUL1','MAEA'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['IGH@','NFKB2'],
        ['IGH@','ZFP36L1'],
        ['INPP5A','PWWP2B'],
        ['ITGB2','POFUT2'],
        ['JARID2','LINC01578'],
        ['JAZF1','ZNF777'],
        ['JKAMP','PCNX4'],
        ['KANSL1','LRRC37A3'],
        ['KANSL3','TMCO4'],
        ['KDM2A','SF1'],
        ['KDM4B','SAFB2'],
        ['KDM7A','LAPTM5'],
        ['KIFC1','RNH1'],
        ['KLF2','OAZ1'],
        ['KLHL36','ZDHHC7'],
        ['KLK4','KLKP1'],
        ['LAPTM5','RIPOR1'],
        ['LARP1','NR3C1'],
        ['LINC00886','TIPARP-AS1'],
        ['LINC01578','YPEL5'],
        ['LPGAT1','RPS6KC1'],
        ['LRRC37A3','NSF'],
        ['LYZ','PIGQ'],
        ['MAN2A1','UIMC1'],
        ['MANBAL','SRC'],
        ['METRNL','RALGDS'],
        ['MICAL2','TEAD1'],
        ['MPO','TBC1D9B'],
        ['MTSS1L','VAC14'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NCOR2','UBC'],
        ['NDEL1','UBB'],
        ['NFATC3','PLA2G15'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NQO2','PABPC1L'],
        ['OAZ1','RAC2'],
        ['PACSIN2','TOB1'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PMS2','POLR2J3'],
        ['PPP1CB','YPEL5'],
        ['PRKAG2','RHEB'],
        ['PTBP1','RNF126'],
        ['PTBP1','SBNO2'],
        ['PTMA','SRSF5'],
        ['RBM38','SUMO2'],
        ['RNF139','TRMT12'],
        ['SDC2','VMP1'],
        ['SDF4','TNFRSF4'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SLC19A1','SUMO3'],
        ['SLC25A3','TOM1'],
        ['ST3GAL1','UBB'],
        ['STK10','UBE2D2'],
        ['TFDP1','TMCO3'],
        ['TFDP2','XRN1'],
        ['ABCA10','GFOD1'],
        ['ABL1','EXOSC2'],
        ['ABR','CLUH'],
        ['AC002116.2','WDR62'],
        ['AC004080.5','HOXA5'],
        ['AC004846.2','NUMB'],
        ['AC004865.2','PSMB2'],
        ['AC004889.1','AC005304.2'],
        ['AC004980.3','AC091390.2'],
        ['AC004980.3','AC211486.3'],
        ['AC004980.3','PMS2'],
        ['AC005154.1','GGCT'],
        ['AC005258.1','ACTB'],
        ['AC005258.1','CSNK1G2'],
        ['AC005258.1','DOT1L'],
        ['AC005258.1','KLF16'],
        ['AC005258.1','PTMA'],
        ['AC005258.1','SGTA'],
        ['AC005332.7','BPTF'],
        ['AC005540.1','RPL14'],
        ['AC005674.2','WDR1'],
        ['AC005740.1','NDUFS1'],
        ['AC006001.3','RABGEF1'],
        ['AC006064.4','HBB'],
        ['AC006269.1','MMD'],
        ['AC008105.3','AC138150.2'],
        ['AC008575.1','AC136475.10'],
        ['AC008750.8','NKG7'],
        ['AC008753.1','RPLP1'],
        ['AC008764.4','SLC35E1'],
        ['AC008870.1','GTDC1'],
        ['AC009275.1','FGD4'],
        ['AC010186.2','DCK'],
        ['AC010186.4','GOT2'],
        ['AC010203.1','DEPDC4'],
        ['AC010326.4','ZNF587'],
        ['AC010542.1','CMTM4'],
        ['AC010542.5','CMTM4'],
        ['AC011445.1','GMFG'],
        ['AC011468.4','TMEM50B'],
        ['AC012441.2','AC018638.5'],
        ['AC013394.1','B2M'],
        ['AC015813.1','SRSF1'],
        ['AC015922.4','IL6ST'],
        ['AC016831.1','LINC-PINT'],
        ['AC016876.2','EIF4A2'],
        ['AC016876.2','LYZ'],
        ['AC016876.2','MPDU1'],
        ['AC018688.1','ZNF285'],
        ['AC020656.1','CTSG'],
        ['AC020656.1','FBXO6'],
        ['AC020656.1','PKM'],
        ['AC020656.1','PRKAR2A'],
        ['AC020656.1','RPL8'],
        ['AC020656.1','S100A8'],
        ['AC020656.1','SLC25A37'],
        ['AC020656.1','SLC2A3'],
        ['AC020656.1','SLC43A2'],
        ['AC020656.1','SLC4A1'],
        ['AC022400.3','BMS1'],
        ['AC024587.1','USP7'],
        ['AC025171.2','ZNF131'],
        ['AC025262.1','TBC1D30'],
        ['AC026150.1','GOLGA8M'],
        ['AC026412.1','SDHA'],
        ['AC027031.2','OXR1'],
        ['AC036108.3','SYNM'],
        ['AC055713.1','WSB1'],
        ['AC058791.1','LINC00513'],
        ['AC067930.6','HBB'],
        ['AC068152.1','ARL17B'],
        ['AC068831.3','CERK'],
        ['AC073264.3','AC099548.2'],
        ['AC078842.1','PTN'],
        ['AC079316.1','AL139353.3'],
        ['AC083862.1','RNF14'],
        ['AC084032.1','CNOT2'],
        ['AC087190.4','LYZ'],
        ['AC091390.2','AC211486.3'],
        ['AC091390.3','STAG3'],
        ['AC091390.4','AC211429.1'],
        ['AC091390.4','AC211486.3'],
        ['AC091390.4','PMS2'],
        ['AC091849.2','SDHA'],
        ['AC092376.1','MAF'],
        ['AC092718.8','NDUFS1'],
        ['AC092821.1','AC141557.1'],
        ['AC092902.2','FAM85B'],
        ['AC093162.2','RETSAT'],
        ['AC093392.1','SEPT7'],
        ['AC093484.4','HBA2'],
        ['AC093484.4','HBB'],
        ['AC093525.2','DDX5'],
        ['AC093752.1','SEPT7'],
        ['AC095350.2','RIMBP2'],
        ['AC096664.2','RPL27'],
        ['AC099548.2','TCAF2'],
        ['AC099794.1','ATG4C'],
        ['AC100861.1','CHMP7'],
        ['AC103702.2','HOXB9'],
        ['AC105046.1','EGR3'],
        ['AC107983.2','USP32'],
        ['AC107983.2','USP6'],
        ['AC108010.1','TLK2'],
        ['AC113189.4','TMEM102'],
        ['AC116366.1','IRF1'],
        ['AC117386.2','CHURC1'],
        ['AC119396.1','ARHGEF18'],
        ['AC125257.1','KLHL11'],
        ['AC125494.1','RPL14'],
        ['AC126120.1','PPIA'],
        ['AC127526.4','LINC00937'],
        ['AC132008.2','SDHA'],
        ['AC133065.6','CIITA'],
        ['AC134772.1','SHISA5'],
        ['AC136475.10','SRP19'],
        ['AC136944.2','IGK@'],
        ['AC138035.1','AL627309.5'],
        ['AC138207.8','NF1'],
        ['AC138393.1','SEPT7'],
        ['AC138409.2','AP000347.1'],
        ['AC138811.2','SMG1'],
        ['AC138932.1','NPIPA8'],
        ['AC142381.1','IGH@'],
        ['AC145098.1','DOK3'],
        ['AC211486.3','PRKRIP1'],
        ['AC243562.1','GOLGA2P3Y'],
        ['AC244517.10','PCDHB9'],
        ['AC245100.4','LINC00869'],
        ['AC245128.3','AL357055.3'],
        ['AC245128.3','ORC4'],
        ['AC245369.3','IGH@'],
        ['AC245369.3','IGHA2'],
        ['AC245369.3','IGHG1'],
        ['AC245369.7','IGH@'],
        ['AC245884.4','LYZ'],
        ['AC247036.4','IGH@'],
        ['AC247036.6','IGH@'],
        ['ACBD3','ZMYM4'],
        ['ACCS','EXT2'],
        ['ACSL1','CENPU'],
        ['ACSS1','APMAP'],
        ['ACTB','CCNG2'],
        ['ACTB','CNN2'],
        ['ACTB','CXCR4'],
        ['ACTB','FAM46A'],
        ['ACTB','HMGB2'],
        ['ACTB','HNRNPK'],
        ['ACTB','YWHAZ'],
        ['ACTN4','EIF3K'],
        ['ACVR1B','GRASP'],
        ['ADAM23','FAM237A'],
        ['ADAP1','MAFK'],
        ['ADGRG7','TFG'],
        ['ADM','RIN3'],
        ['ADRM1','SS18L1'],
        ['AGAP5','BMS1P4'],
        ['AKNA','AL138895.1'],
        ['AKT2','C19ORF47'],
        ['AL031282.2','SLC35E2B'],
        ['AL031587.6','AL627309.5'],
        ['AL035587.1','GNMT'],
        ['AL035701.1','ENPP5'],
        ['AL109804.1','MAVS'],
        ['AL109809.1','AL117335.1'],
        ['AL109809.1','AL121760.1'],
        ['AL109811.3','TARDBP'],
        ['AL121578.3','SYTL5'],
        ['AL132780.3','PRMT5'],
        ['AL133353.1','SLC25A28'],
        ['AL136133.1','RPL22'],
        ['AL136982.1','BMS1'],
        ['AL137145.2','PFKFB3'],
        ['AL139022.1','FNTB'],
        ['AL139220.2','KLF17'],
        ['AL139353.3','CHST11'],
        ['AL139383.1','STARD13'],
        ['AL158154.2','UPP1'],
        ['AL158212.1','VTI1A'],
        ['AL160408.2','HBB'],
        ['AL353148.1','LYZ'],
        ['AL356235.1','ZC3H12B'],
        ['AL360181.3','SCART1'],
        ['AL360270.1','CD53'],
        ['AL365203.3','ITGB1'],
        ['AL390036.1','VAV3-AS1'],
        ['AL390860.1','TRIM58'],
        ['AL392172.1','DISP1'],
        ['AL512662.2','BMS1'],
        ['AL583722.1','INF2'],
        ['AL591684.2','BMS1'],
        ['AL606491.1','LDLRAP1'],
        ['AL645929.1','AL671277.1'],
        ['AL662899.1','LY6G5C'],
        ['AL671277.1','AL671883.3'],
        ['AL671710.1','ZBED4'],
        ['AL928768.1','IGHA1'],
        ['ALDOA','PPP4C'],
        ['ALG14','CNN3'],
        ['AMD1','EEF1A1'],
        ['ANAPC1','RMND5A'],
        ['ANKAR','ASNSD1'],
        ['ANKEF1','TACC2'],
        ['ANP32B','TMOD1'],
        ['ANXA1','MAGT1'],
        ['ANXA1','ZNF207'],
        ['ANXA5','CLEC4E'],
        ['AP000317.1','SLC5A3'],
        ['AP000346.1','IGLL3P'],
        ['AP000346.2','AP000347.1'],
        ['AP000346.2','AP000347.2'],
        ['AP000356.3','BCR'],
        ['AP000924.1','FDX1'],
        ['AP000943.3','MRE11'],
        ['AP000943.4','FUT4'],
        ['AP001931.1','TMX2'],
        ['AP3S2','YPEL5'],
        ['APBB1IP','FAM238A'],
        ['APBB2','NR3C2'],
        ['ARHGAP18','DLGAP5'],
        ['ARHGAP26','NR3C1'],
        ['ARID4B','B2M'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARPC4','OGG1'],
        ['ARRDC3','BCLAF1'],
        ['AS3MT','CCND3'],
        ['ASXL1','NOL4L'],
        ['ATG16L1','INPP5D'],
        ['ATP5I','MFSD7'],
        ['ATP6V0B','DPH2'],
        ['ATP6V0C','EEF1A1'],
        ['AZGP1','GJC3'],
        ['AZIN1','KLF10'],
        ['B2M','FUS'],
        ['B2M','G3BP2'],
        ['B2M','IGH@'],
        ['B2M','MED13L'],
        ['B2M','PFKFB3'],
        ['B2M','RUNX1'],
        ['B2M','SF1'],
        ['B2M','SUB1'],
        ['B2M','UBC'],
        ['B2M','UBE3A'],
        ['B3GNT5','KLHL24'],
        ['B3GNT5','TIPARP'],
        ['B3GNTL1','METRNL'],
        ['BANK1','IGH@'],
        ['BCAP29','DUS4L'],
        ['BCAS3','TG'],
        ['BCL2L11','CXCR4'],
        ['BCL6','NAMPT'],
        ['BCL7A','RBM38'],
        ['BEST1','HBB'],
        ['BLOC1S6','SQOR'],
        ['BMI1','DDX5'],
        ['BPI','CFL1'],
        ['BPTF','LRRC37A2'],
        ['BRD1','C22ORF34'],
        ['BRD2','SRSF5'],
        ['BSG','ELANE'],
        ['BSG','OAZ1'],
        ['BSG','R3HDM4'],
        ['BSG','SBNO2'],
        ['BSG','TCF3'],
        ['BSG','UBB'],
        ['C12ORF73','HSP90B1'],
        ['C15ORF48','SPATA5L1'],
        ['C15ORF57','CBX3'],
        ['C18ORF8','CAPZA2'],
        ['C19MC','RPLP1'],
        ['C19ORF33','SPINT2'],
        ['C2CD3','UCP3'],
        ['C2ORF48','RRM2'],
        ['C6ORF106','SPDEF'],
        ['CABIN1','CBFA2T2'],
        ['CABP7','NF2'],
        ['CASTOR2','FURIN'],
        ['CATSPER2','PPIP5K1'],
        ['CBWD2','DOCK8'],
        ['CCNG2','OAZ1'],
        ['CCNG2','VAV1'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CDC34','GZMM'],
        ['CDC34','PTMA'],
        ['CDC42','SLC25A37'],
        ['CDC42','ZEB2'],
        ['CDC42EP2','POLA2'],
        ['CDK11B','SLC35E2'],
        ['CDK2','RAB5B'],
        ['CDR1','LINC00632'],
        ['CFL1','MKNK2'],
        ['CFL1','SF1'],
        ['CITED1','HDAC8'],
        ['CLIC2','F8'],
        ['CLK1','CXCR4'],
        ['CNN2','FURIN'],
        ['CNN2','RNF126'],
        ['CNN2','SBNO2'],
        ['COLQ','HACL1'],
        ['COMMD10','FBXL17'],
        ['COMMD2','WWTR1'],
        ['COQ8B','NUMBL'],
        ['COX19','LINC01002'],
        ['COX7A2','MINDY3'],
        ['CPM','LYZ'],
        ['CRCP','TPST1'],
        ['CREB3','GRB2'],
        ['CREBRF','NSD1'],
        ['CRLF2','LYZ'],
        ['CTBP1','MAEA'],
        ['CTBP1','UVSSA'],
        ['CTDP1','LINC00969'],
        ['CTDP1','NFATC1'],
        ['CTDP1','PQLC1'],
        ['CTSC','RAB38'],
        ['CTSD','LSP1'],
        ['CUX1','KLF10'],
        ['CXCR4','METRNL'],
        ['CXCR4','MYO15B'],
        ['CXCR4','RARA'],
        ['CXCR4','RBM38'],
        ['CXCR4','SF1'],
        ['CXCR4','THAP4'],
        ['CXCR4','UBC'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZBTB18'],
        ['CXCR4','ZEB2'],
        ['CYBA','WWOX'],
        ['CYBB','XK'],
        ['CYTIP','ERMN'],
        ['DAND5','NFIX'],
        ['DAZAP1','GLUL'],
        ['DCUN1D1','MCCC1'],
        ['DDX39A','JMJD1C'],
        ['DDX3X','EEF1A1'],
        ['DDX3X','UBC'],
        ['DDX42','MAP3K3'],
        ['DDX5','EIF4A1'],
        ['DDX5','FUS'],
        ['DDX5','GRK2'],
        ['DDX5','HMGN2'],
        ['DDX5','HNRNPU'],
        ['DDX5','METRNL'],
        ['DDX5','NFE2L1'],
        ['DDX5','OAZ1'],
        ['DDX5','TEX2'],
        ['DDX5','TPD52L2'],
        ['DDX5','UGCG'],
        ['DGCR8','TANGO2'],
        ['DHX35','FAM83D'],
        ['DHX9','NPL'],
        ['DLD','HNRNPK'],
        ['DLEU2','HMGB1'],
        ['DNAJC7','LYZ'],
        ['DOCK9','STK24'],
        ['DOT1L','PTBP1'],
        ['DOT1L','ZBTB7A'],
        ['EAF2','SLC15A2'],
        ['EEF1A1','HBA1'],
        ['EEF1A1','PTMA'],
        ['EEF1A1','TPM3'],
        ['EFCAB2','KIF26B'],
        ['EFCAB7','PGM1'],
        ['EIF1AX','MAP7D2'],
        ['EIF1AX','RPS6KA3'],
        ['EIF2AK1','S100A9'],
        ['EIF4E3','FOXP1'],
        ['EIF4G2','PTMA'],
        ['ELANE','HBD'],
        ['ELANE','LINC01578'],
        ['ELANE','SF1'],
        ['ENTPD5','FAM161B'],
        ['EPB42','LYZ'],
        ['EPN2','UBB'],
        ['ERBB2','GRB7'],
        ['FAM110A','JAZF1'],
        ['FAM133B','FAM133CP'],
        ['FAM135A','MAP3K4'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM53A','XRRA1'],
        ['FAM66A','FAM66C'],
        ['FAM86B1','FAM86B3P'],
        ['FBXL5','HMGB1'],
        ['FBXO16','HBB'],
        ['FCAR','LENG8'],
        ['FES','MAN2A2'],
        ['FITM2','OSER1'],
        ['FNIP1','PDE4B'],
        ['FOXP1','RYBP'],
        ['FRG1','FRG1HP'],
        ['FRMD6-AS2','LINC02310'],
        ['FUS','KCTD5'],
        ['G3BP2','PTP4A1'],
        ['GABPB1','PTP4A1'],
        ['GALK2','SUPT16H'],
        ['GAS7','RCVRN'],
        ['GATC','KDM2A'],
        ['GCA','SAMSN1'],
        ['GGT1','IGSF3'],
        ['GLUL','KCNAB2'],
        ['GNAS','SENP3'],
        ['GOLT1A','KISS1'],
        ['GPR156','LRRC58'],
        ['GSN','MMP25'],
        ['GSN','S100A9'],
        ['HAPLN3','MFGE8'],
        ['HBA1','RPS27'],
        ['HBA2','TUBB4B'],
        ['HBB','MOB3A'],
        ['HBB','NLRC5'],
        ['HBB','SEPT2'],
        ['HBB','VTI1A'],
        ['HBD','LTF'],
        ['HIF1A','PTP4A1'],
        ['HIF1A','RNPC3'],
        ['HMGB1','HNRNPH1'],
        ['HMGB1','MXD4'],
        ['HMGB1','SRSF5'],
        ['HMGB2','IVNS1ABP'],
        ['HMGN2','NECAB3'],
        ['HNRNPA2B1','UBB'],
        ['HNRNPC','PTP4A1'],
        ['HNRNPD','RCC1'],
        ['HNRNPDL','TUBA1B'],
        ['HNRNPH1','ITPRIP'],
        ['HNRNPK','NFIL3'],
        ['HNRNPK','UBQLN1'],
        ['HNRNPUL1','MAEA'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPB11','YIPF1'],
        ['HSPE1','MOB4'],
        ['IFT122','RYK'],
        ['IGH@','LAPTM5'],
        ['IGH@','MIR100HG'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','PPP6R2'],
        ['IGH@','ZFP36L1'],
        ['INPP5A','PWWP2B'],
        ['IQGAP1','ZNF774'],
        ['ITGAV','PXK'],
        ['ITGB2','POFUT2'],
        ['JARID2','LINC01578'],
        ['JARID2','MGAT4A'],
        ['JAZF1','ZNF777'],
        ['JKAMP','PCNX4'],
        ['KANSL1','LRRC37A3'],
        ['KANSL3','RIPK4'],
        ['KANSL3','TMCO4'],
        ['KCNK7','MAP3K11'],
        ['KDM2A','SF1'],
        ['KDM4B','SAFB2'],
        ['KDM7A','LAPTM5'],
        ['KIFC1','RNH1'],
        ['KLF2','OAZ1'],
        ['KLHL36','ZDHHC7'],
        ['KLK4','KLKP1'],
        ['KMT2C','TPTE'],
        ['KMT5A','RFLNA'],
        ['LAPTM5','RIPOR1'],
        ['LARP1','NR3C1'],
        ['LINC00886','TIPARP-AS1'],
        ['LINC01578','YPEL5'],
        ['LINC01578','ZBTB7A'],
        ['LINC02246','NRIP1'],
        ['LPGAT1','RPS6KC1'],
        ['LRP2BP','UFSP2'],
        ['LRPAP1','WDR1'],
        ['LRRC37A16P','LRRC37B'],
        ['LRRC37A3','NSF'],
        ['LYST','SETX'],
        ['LYZ','MORF4L1'],
        ['LYZ','PIGQ'],
        ['LYZ','SYNE3'],
        ['LYZ','WAC-AS1'],
        ['MAD1L1','MAFK'],
        ['MAN2A1','UIMC1'],
        ['MANBAL','SRC'],
        ['MAPKBP1','MGA'],
        ['MARCKS','PTMA'],
        ['METRNL','RALGDS'],
        ['MGRN1','NUDT16L1'],
        ['MICAL2','TEAD1'],
        ['MIR100HG','PIGR'],
        ['MIR22HG','TLCD2'],
        ['MPO','TBC1D9B'],
        ['MTSS1','TATDN1'],
        ['MTSS1L','VAC14'],
        ['MUC2','MUC5B'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NCOA6','PIGU'],
        ['NCOR2','UBC'],
        ['NDEL1','UBB'],
        ['NFATC1','PQLC1'],
        ['NFATC3','PLA2G15'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NQO2','PABPC1L'],
        ['NSMAF','SPTLC3'],
        ['OAZ1','PFKFB3'],
        ['OAZ1','PTBP1'],
        ['OAZ1','RAC2'],
        ['OAZ1','SEPT5'],
        ['OAZ1','TCF3'],
        ['OAZ1','UBTF'],
        ['PACSIN2','TOB1'],
        ['PCP4L1','SDHC'],
        ['PEMT','UBB'],
        ['PFKFB3','S100A9'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PINK1-AS','TAS2R4'],
        ['PITPNA','YWHAE'],
        ['PLSCR4','RNF13'],
        ['PMS2','POLR2J3'],
        ['PPP1CB','YPEL5'],
        ['PRDX1','TESK2'],
        ['PRKAG2','RHEB'],
        ['PRKCH','PRPS2'],
        ['PRSS3','TMEM45B'],
        ['PTBP1','RNF126'],
        ['PTBP1','SBNO2'],
        ['PTCSC2','RNF144B'],
        ['PTMA','SRSF5'],
        ['PTMA','WDR1'],
        ['PTMA','ZEB2'],
        ['PTP4A1','SRSF5'],
        ['PXK','RPP14'],
        ['PXYLP1','SPSB4'],
        ['RAB35','THEMIS2'],
        ['RASA3','TFDP1'],
        ['RBM38','SUMO2'],
        ['RBMS2','SPRYD4'],
        ['RHOF','UBC'],
        ['RIPK1','SERPINB9'],
        ['RNASET2','SRGN'],
        ['RNF139','TRMT12'],
        ['RPL14','SRP14'],
        ['RPL38','TTYH2'],
        ['RPS9','UBB'],
        ['SAMD5','SASH1'],
        ['SAMSN1','USP25'],
        ['SCNN1A','TNFRSF1A'],
        ['SDC2','VMP1'],
        ['SDF4','TNFRSF4'],
        ['SENP3-EIF4A1','TNFSF13'],
        ['SERF1A','SMN1'],
        ['SH3KBP1','TYROBP'],
        ['SIDT2','TAGLN'],
        ['SLC19A1','SUMO3'],
        ['SLC25A3','TOM1'],
        ['SLC2A11','SMARCB1'],
        ['SMG1','ZNF623'],
        ['ST3GAL1','UBB'],
        ['STK10','UBE2D2'],
        ['TBCEL','TECTA'],
        ['TFDP1','TMCO3'],
        ['TFDP1','ZEB2'],
        ['TFDP2','XRN1'],
        ['TMEM225B','ZNF655'],
        ['USP34','XPO1'],
        ['YPEL5','ZEB2'],
        ['Z93930.2','ZNRF3'],
        ['A4GNT','DZIP1L'],
        ['AATF','TK2'],
        ['AB015752.1','TMEM64'],
        ['ABCA10','GFOD1'],
        ['ABHD2','AC013565.1'],
        ['ABHD2','AC020656.1'],
        ['ABL1','EXOSC2'],
        ['ABR','CLUH'],
        ['AC002116.2','WDR62'],
        ['AC002398.2','DES'],
        ['AC002398.2','MYL2'],
        ['AC002480.4','STEAP1B'],
        ['AC003973.2','LINC01859'],
        ['AC004080.5','HOXA5'],
        ['AC004160.1','PHF14'],
        ['AC004241.5','BGN'],
        ['AC004381.1','THUMPD1'],
        ['AC004471.2','DGCR2'],
        ['AC004477.1','COPZ2'],
        ['AC004846.2','NUMB'],
        ['AC004865.2','PSMB2'],
        ['AC004889.1','AC005304.2'],
        ['AC004889.1','ARHGEF5'],
        ['AC004895.1','USP42'],
        ['AC004908.2','LINC00115'],
        ['AC004951.2','RASA4'],
        ['AC004951.4','RASA4'],
        ['AC004967.1','OR7E38P'],
        ['AC004980.3','AC091390.2'],
        ['AC004980.3','AC211486.3'],
        ['AC004980.3','PMS2'],
        ['AC005005.4','TXNDC15'],
        ['AC005013.1','AC005162.3'],
        ['AC005035.1','IL1R2'],
        ['AC005077.2','LINC00174'],
        ['AC005091.1','HIBADH'],
        ['AC005154.1','GGCT'],
        ['AC005154.3','GGCT'],
        ['AC005258.1','ACTB'],
        ['AC005258.1','CSNK1G2'],
        ['AC005258.1','DOT1L'],
        ['AC005258.1','KLF16'],
        ['AC005258.1','PTMA'],
        ['AC005258.1','SGTA'],
        ['AC005332.7','BPTF'],
        ['AC005410.1','RPL21'],
        ['AC005479.2','NPC2'],
        ['AC005487.1','DNAJB9'],
        ['AC005540.1','RPL14'],
        ['AC005562.1','AC138207.8'],
        ['AC005586.2','LRRC61'],
        ['AC005674.2','WDR1'],
        ['AC005703.1','PMP22'],
        ['AC005740.1','NDUFS1'],
        ['AC005775.1','ODF3L2'],
        ['AC005832.1','NDUFA9'],
        ['AC005856.1','AC005884.1'],
        ['AC006001.3','RABGEF1'],
        ['AC006064.4','HBB'],
        ['AC006064.4','NPPA'],
        ['AC006269.1','MMD'],
        ['AC006333.1','NDUFA5'],
        ['AC006460.2','NEMP2'],
        ['AC006946.2','CECR7'],
        ['AC007192.1','AC020656.1'],
        ['AC007229.1','HIKESHI'],
        ['AC007388.1','AC020656.1'],
        ['AC007570.1','KCTD10'],
        ['AC007663.1','PRODH'],
        ['AC007663.4','DGCR6'],
        ['AC007923.4','SDHA'],
        ['AC008105.3','AC138150.2'],
        ['AC008132.1','CALM1'],
        ['AC008543.1','ZNF627'],
        ['AC008555.5','LINC01801'],
        ['AC008575.1','AC136475.10'],
        ['AC008750.8','NKG7'],
        ['AC008753.1','RPLP1'],
        ['AC008760.2','SERPING1'],
        ['AC008764.1','EPS15L1'],
        ['AC008764.4','SLC35E1'],
        ['AC008770.1','AC011455.2'],
        ['AC008770.1','ZNF625-ZNF20'],
        ['AC008808.2','LINC02106'],
        ['AC008870.1','GTDC1'],
        ['AC009107.1','CCDC113'],
        ['AC009126.1','ERAP1'],
        ['AC009135.2','ASNS'],
        ['AC009171.2','SRL'],
        ['AC009275.1','FGD4'],
        ['AC009292.2','PIAS1'],
        ['AC009309.1','MRPL35'],
        ['AC009948.1','DFNB59'],
        ['AC010186.2','DCK'],
        ['AC010186.2','GOT2'],
        ['AC010186.4','GOT2'],
        ['AC010203.1','DEPDC4'],
        ['AC010325.1','C19ORF48'],
        ['AC010326.4','ZNF587'],
        ['AC010327.1','ACTA1'],
        ['AC010327.1','DES'],
        ['AC010327.1','NPPA-AS1'],
        ['AC010542.1','CMTM4'],
        ['AC010542.5','CMTM4'],
        ['AC010643.1','TEAD2'],
        ['AC010735.2','RHBDD1'],
        ['AC011379.2','CYSTM1'],
        ['AC011410.1','SMIM23'],
        ['AC011445.1','GMFG'],
        ['AC011468.4','TMEM50B'],
        ['AC011487.3','OSTC'],
        ['AC011491.3','ACER1'],
        ['AC011591.1','SFTPB'],
        ['AC011767.1','GOLGA6L2'],
        ['AC011825.1','RPS15A'],
        ['AC011840.1','KRT42P'],
        ['AC012170.2','USP50'],
        ['AC012435.2','UBL7-AS1'],
        ['AC012441.2','AC018638.5'],
        ['AC012442.1','CHCHD5'],
        ['AC013244.1','DIP2B'],
        ['AC013394.1','B2M'],
        ['AC013457.1','AP000688.2'],
        ['AC015468.2','LINC02153'],
        ['AC015689.2','RDX'],
        ['AC015802.4','ST6GALNAC2'],
        ['AC015802.6','CYGB'],
        ['AC015813.1','SRSF1'],
        ['AC015849.1','TAF15'],
        ['AC015849.5','LRRC37A9P'],
        ['AC015922.4','IL6ST'],
        ['AC016044.1','ONECUT1'],
        ['AC016257.1','ALDH1L2'],
        ['AC016831.1','LINC-PINT'],
        ['AC016876.2','EIF4A2'],
        ['AC016876.2','IGK@'],
        ['AC016876.2','LYZ'],
        ['AC016876.2','MPDU1'],
        ['AC016907.2','ALK'],
        ['AC017091.1','AC097501.1'],
        ['AC017104.5','LINC00471'],
        ['AC018521.5','SP2-AS1'],
        ['AC018616.1','BEST1'],
        ['AC018638.5','AL606534.4'],
        ['AC018688.1','ZNF285'],
        ['AC018695.3','COX4I1'],
        ['AC018797.2','CISD2'],
        ['AC019349.1','CRNN'],
        ['AC019349.1','GSN'],
        ['AC019349.1','KRT4'],
        ['AC019349.1','KRT6A'],
        ['AC019349.1','S100A9'],
        ['AC019349.1','SPINK5'],
        ['AC020656.1','AKR1B10'],
        ['AC020656.1','CEP68'],
        ['AC020656.1','CLDN18'],
        ['AC020656.1','CTSG'],
        ['AC020656.1','DEPDC4'],
        ['AC020656.1','FAXDC2'],
        ['AC020656.1','FBXO6'],
        ['AC020656.1','HSD17B12'],
        ['AC020656.1','IGK@'],
        ['AC020656.1','IL3RA'],
        ['AC020656.1','LINC01285'],
        ['AC020656.1','MBTD1'],
        ['AC020656.1','MCAM'],
        ['AC020656.1','MPPED2'],
        ['AC020656.1','PEAK1'],
        ['AC020656.1','PGC'],
        ['AC020656.1','PKM'],
        ['AC020656.1','POMGNT1'],
        ['AC020656.1','PRKAR2A'],
        ['AC020656.1','RPL8'],
        ['AC020656.1','S100A8'],
        ['AC020656.1','SFTPB'],
        ['AC020656.1','SLC25A37'],
        ['AC020656.1','SLC2A3'],
        ['AC020656.1','SLC43A2'],
        ['AC020656.1','SLC4A1'],
        ['AC020656.1','TARS2'],
        ['AC020656.1','WWOX'],
        ['AC020765.2','NUPR1'],
        ['AC020915.2','ZSCAN22'],
        ['AC020928.1','ZNF567'],
        ['AC020978.4','ESRP2'],
        ['AC021171.1','FAM133B'],
        ['AC022028.2','GRID1-AS1'],
        ['AC022145.1','STK33'],
        ['AC022217.1','FAM58A'],
        ['AC022400.3','BMS1'],
        ['AC022400.5','BMS1'],
        ['AC022733.1','AC087518.1'],
        ['AC023510.1','CASC1'],
        ['AC023644.1','BEST1'],
        ['AC024084.1','AC078845.1'],
        ['AC024560.3','AC093642.1'],
        ['AC024575.1','TMEM205'],
        ['AC024587.1','USP7'],
        ['AC024592.2','LINC01356'],
        ['AC024937.3','AC026412.1'],
        ['AC024937.3','PDCD6'],
        ['AC025048.2','AC243829.2'],
        ['AC025171.2','ZNF131'],
        ['AC025262.1','TBC1D30'],
        ['AC025279.1','SNX29'],
        ['AC025566.1','LINC02032'],
        ['AC025884.1','RERE'],
        ['AC026150.1','GOLGA8M'],
        ['AC026412.1','SDHA'],
        ['AC026471.2','AHSP'],
        ['AC027031.2','OXR1'],
        ['AC027343.1','LINC02123'],
        ['AC027559.1','AC090825.1'],
        ['AC034205.2','SLC36A2'],
        ['AC036108.3','SYNM'],
        ['AC040170.1','LINC01081'],
        ['AC044810.2','OVCH2'],
        ['AC048337.1','DGKD'],
        ['AC048338.1','CLIP1'],
        ['AC053503.4','KRT13'],
        ['AC053503.4','MYH6'],
        ['AC053503.4','MYH7'],
        ['AC053503.4','MYL2'],
        ['AC053503.4','NPPA'],
        ['AC053503.4','TNNI3'],
        ['AC053503.4','TPM1'],
        ['AC055713.1','WSB1'],
        ['AC058791.1','LINC00513'],
        ['AC060780.1','AC096637.2'],
        ['AC063926.3','PIWIL1'],
        ['AC063944.1','LINC00882'],
        ['AC063979.2','LINC02056'],
        ['AC067930.6','ANKRD19P'],
        ['AC067930.6','HBB'],
        ['AC068152.1','ARL17B'],
        ['AC068446.1','PAK2'],
        ['AC068491.1','PAFAH1B1'],
        ['AC068580.4','ALB'],
        ['AC068768.1','MPHOSPH9'],
        ['AC068831.3','CERK'],
        ['AC069281.1','AGFG2'],
        ['AC073264.3','AC099548.2'],
        ['AC073288.2','SKIL'],
        ['AC073578.1','LINC01257'],
        ['AC074051.2','ASNS'],
        ['AC074099.1','LINC01876'],
        ['AC078842.1','PTN'],
        ['AC079193.2','MICU3'],
        ['AC079228.1','RTN3'],
        ['AC079316.1','AL139353.3'],
        ['AC079848.1','AC108134.3'],
        ['AC083862.1','RNF14'],
        ['AC083964.2','TDRP'],
        ['AC084024.3','ASH2L'],
        ['AC084032.1','CNOT2'],
        ['AC087071.2','SSMEM1'],
        ['AC087190.4','LYZ'],
        ['AC087463.4','PWRN1'],
        ['AC087463.4','PWRN3'],
        ['AC087481.2','FAN1'],
        ['AC087481.2','HERC2'],
        ['AC087672.3','LY96'],
        ['AC087897.2','CPNE8'],
        ['AC090001.1','NTN4'],
        ['AC090114.3','TLK2'],
        ['AC090409.2','AC105094.2'],
        ['AC091167.3','GDPGP1'],
        ['AC091390.2','AC211486.3'],
        ['AC091390.3','STAG3'],
        ['AC091390.3','STAG3L5P'],
        ['AC091390.3','STAG3L5P-PVRIG2P-PILRB'],
        ['AC091390.4','AC211429.1'],
        ['AC091390.4','AC211486.3'],
        ['AC091390.4','PMS2'],
        ['AC091849.2','SDHA'],
        ['AC092079.1','DDX18'],
        ['AC092162.2','SPOPL'],
        ['AC092284.1','CRLF3'],
        ['AC092376.1','MAF'],
        ['AC092506.1','THAP3'],
        ['AC092718.8','CMC2'],
        ['AC092718.8','NDUFS1'],
        ['AC092746.1','CLEC4E'],
        ['AC092747.2','MED21'],
        ['AC092813.2','ST6GALNAC3'],
        ['AC092821.1','AC141557.1'],
        ['AC092821.1','TPT1-AS1'],
        ['AC092821.2','RANBP6'],
        ['AC092902.2','FAM85B'],
        ['AC093157.2','SLC30A7'],
        ['AC093162.2','RETSAT'],
        ['AC093392.1','SEPT7'],
        ['AC093484.4','ANKRD1'],
        ['AC093484.4','HBA2'],
        ['AC093484.4','HBB'],
        ['AC093525.2','DDX5'],
        ['AC093627.3','HABP4'],
        ['AC093720.1','UGT2A3'],
        ['AC093752.1','SEPT7'],
        ['AC093895.1','SPARCL1'],
        ['AC095038.1','ZNF727'],
        ['AC095350.2','RIMBP2'],
        ['AC096664.2','RPL27'],
        ['AC097372.3','LINC01095'],
        ['AC097374.1','AL513478.2'],
        ['AC098484.2','PPIH'],
        ['AC098798.1','PANCR'],
        ['AC098848.1','CCDC68'],
        ['AC098850.4','AC107983.2'],
        ['AC098850.4','USP32'],
        ['AC098850.5','NOS2'],
        ['AC098850.5','USP6'],
        ['AC099329.1','CCDC13'],
        ['AC099508.2','ZFP1'],
        ['AC099548.2','TCAF2'],
        ['AC099794.1','ATG4C'],
        ['AC100821.2','TCEA1'],
        ['AC100861.1','CHMP7'],
        ['AC103702.2','HOXB9'],
        ['AC103957.1','PRAG1'],
        ['AC104109.3','LINC00174'],
        ['AC104758.1','COMMD4'],
        ['AC105046.1','EGR3'],
        ['AC105411.1','LINC01227'],
        ['AC106795.3','AC139493.2'],
        ['AC106886.5','MYH11'],
        ['AC107983.2','USP32'],
        ['AC107983.2','USP6'],
        ['AC108010.1','TLK2'],
        ['AC108097.1','LINC01227'],
        ['AC108488.1','ADI1'],
        ['AC108688.1','DNAJB6'],
        ['AC108734.3','RNF13'],
        ['AC108751.4','TM4SF1'],
        ['AC109466.1','CCDC7'],
        ['AC110079.2','NPR1'],
        ['AC111149.1','RDH10-AS1'],
        ['AC112719.2','ART3'],
        ['AC113189.4','TMEM102'],
        ['AC113189.6','C17ORF74'],
        ['AC113189.6','TMEM102'],
        ['AC114284.1','PRR16'],
        ['AC114501.2','INTS4'],
        ['AC114781.2','COQ2'],
        ['AC116050.1','IGK@'],
        ['AC116366.1','IRF1'],
        ['AC116651.1','FBXL5'],
        ['AC117386.2','CHURC1'],
        ['AC117386.2','TGOLN2'],
        ['AC118758.3','LINC00174'],
        ['AC119396.1','ARHGEF18'],
        ['AC119403.1','ZNF77'],
        ['AC120057.3','NPPA-AS1'],
        ['AC123767.1','ADAM5'],
        ['AC123912.3','ZNF100'],
        ['AC125257.1','KLHL11'],
        ['AC125494.1','RPL14'],
        ['AC125634.1','AL353626.3'],
        ['AC126120.1','PPIA'],
        ['AC126335.1','IGK@'],
        ['AC126544.1','HARS'],
        ['AC127502.2','LYZ'],
        ['AC127526.4','LINC00937'],
        ['AC131160.1','AL136116.3'],
        ['AC132008.2','SDHA'],
        ['AC133065.6','CIITA'],
        ['AC133919.3','AL606534.5'],
        ['AC133961.1','SEL1L3'],
        ['AC134772.1','SHISA5'],
        ['AC134878.2','CR381653.1'],
        ['AC134878.2','TEKT4'],
        ['AC135983.1','CHRFAM7A'],
        ['AC136475.10','SRP19'],
        ['AC136628.2','C5ORF60'],
        ['AC136698.1','AC243562.3'],
        ['AC136698.1','LINC00933'],
        ['AC136944.2','IGK@'],
        ['AC136944.3','CHEK2'],
        ['AC137056.1','AC141273.1'],
        ['AC138035.1','AL627309.5'],
        ['AC138207.8','NF1'],
        ['AC138356.3','ERLIN2'],
        ['AC138393.1','SEPT7'],
        ['AC138409.2','AP000347.1'],
        ['AC138409.2','C1QTNF3'],
        ['AC138409.2','C1QTNF3-AMACR'],
        ['AC138409.2','WDR70'],
        ['AC138649.1','PDCD6IP'],
        ['AC138811.2','SMG1'],
        ['AC138932.1','NPIPA8'],
        ['AC139256.1','SNX29'],
        ['AC139491.5','FAM153B'],
        ['AC140481.3','FAM95B1'],
        ['AC141586.1','KCTD5'],
        ['AC142381.1','IGH@'],
        ['AC145098.1','DOK3'],
        ['AC211486.3','CCDC146'],
        ['AC211486.3','PRKRIP1'],
        ['AC233266.2','TMEM128'],
        ['AC233702.10','FLJ36000'],
        ['AC233702.7','FLJ36000'],
        ['AC239860.1','NBPF13P'],
        ['AC241584.1','SSB'],
        ['AC242376.2','HERC2'],
        ['AC243562.1','GOLGA2P3Y'],
        ['AC243562.3','DNM1'],
        ['AC243829.4','CCL4'],
        ['AC244205.1','IGKV4-1'],
        ['AC244226.1','IGH@'],
        ['AC244517.10','PCDHB9'],
        ['AC244517.2','MALL'],
        ['AC245100.4','LINC00869'],
        ['AC245128.3','AL357055.3'],
        ['AC245128.3','ORC4'],
        ['AC245369.1','IGH@'],
        ['AC245369.3','IGH@'],
        ['AC245369.3','IGHA2'],
        ['AC245369.3','IGHG1'],
        ['AC245369.7','IGH@'],
        ['AC245884.11','LILRB3'],
        ['AC245884.4','LYZ'],
        ['AC246787.2','FAM30A'],
        ['AC247036.4','IGH@'],
        ['AC247036.6','IGH@'],
        ['AC253536.1','GSTTP1'],
        ['ACADSB','MCFD2'],
        ['ACAT1','CUL5'],
        ['ACBD3','ZMYM4'],
        ['ACCS','EXT2'],
        ['ACSL1','CENPU'],
        ['ACSS1','APMAP'],
        ['ACTB','CCNG2'],
        ['ACTB','CNN2'],
        ['ACTB','CXCR4'],
        ['ACTB','FAM46A'],
        ['ACTB','HMGB2'],
        ['ACTB','HNRNPK'],
        ['ACTB','YWHAZ'],
        ['ACTG1','TINAGL1'],
        ['ACTN4','EIF3K'],
        ['ACTR10','TG'],
        ['ACVR1B','GRASP'],
        ['ACVR2B','EXOG'],
        ['ADAM23','FAM237A'],
        ['ADAMTS6','CENPK'],
        ['ADAMTS7','AL049757.1'],
        ['ADAP1','MAFK'],
        ['ADAP1','SUN1'],
        ['ADGRF5','AL512625.3'],
        ['ADGRG7','TFG'],
        ['ADK','HIBCH'],
        ['ADM','RIN3'],
        ['ADRM1','SS18L1'],
        ['AF001548.2','KRT13'],
        ['AF064860.2','IGSF5'],
        ['AFAP1','ANXA7'],
        ['AFDN-AS1','LINC01558'],
        ['AFP','ALB'],
        ['AGAP4','LINC00843'],
        ['AGAP5','BMS1P4'],
        ['AGGF1','ZBED3-AS1'],
        ['AGMAT','AL121992.1'],
        ['AIM2','CADM3-AS1'],
        ['AKNA','AL138895.1'],
        ['AKT2','C19ORF47'],
        ['AL008628.1','FAM120B'],
        ['AL021707.2','FAM227A'],
        ['AL021920.2','EIF1AX'],
        ['AL022322.2','POC1B-GALNT4'],
        ['AL022324.2','PHF10'],
        ['AL024509.3','CMAHP'],
        ['AL031282.2','MALL'],
        ['AL031282.2','SLC35E2B'],
        ['AL031289.1','RIMS3'],
        ['AL031587.6','AL627309.5'],
        ['AL031587.6','MICALL1'],
        ['AL035420.1','EPB41L1'],
        ['AL035425.3','IRS4'],
        ['AL035587.1','GNMT'],
        ['AL035701.1','ENPP5'],
        ['AL049795.1','TMEM234'],
        ['AL049830.3','STRN3'],
        ['AL049836.2','LINC02314'],
        ['AL049869.3','ZBTB25'],
        ['AL050303.3','LINC01087'],
        ['AL050303.3','LINC01297'],
        ['AL050320.1','ISM1-AS1'],
        ['AL080317.1','MFSD4B'],
        ['AL096870.2','AL136295.7'],
        ['AL109804.1','MAVS'],
        ['AL109809.1','AL117335.1'],
        ['AL109809.1','AL121760.1'],
        ['AL109811.3','TARDBP'],
        ['AL109954.2','CSTL1'],
        ['AL117350.1','RAB4A'],
        ['AL121578.3','SYTL5'],
        ['AL121656.1','LTBP1'],
        ['AL121987.2','DCAF8'],
        ['AL132639.4','TRAPPC6B'],
        ['AL132656.2','NUTM2A-AS1'],
        ['AL132780.3','PRMT5'],
        ['AL133338.2','MAPK1IP1L'],
        ['AL133353.1','SLC25A28'],
        ['AL133371.1','AL355922.5'],
        ['AL133410.3','TMEM8B'],
        ['AL133481.2','AL359195.2'],
        ['AL136133.1','RPL22'],
        ['AL136309.4','C6ORF201'],
        ['AL136520.1','SRSF10'],
        ['AL136982.1','BMS1'],
        ['AL136982.6','BMS1'],
        ['AL137003.1','ATXN1'],
        ['AL137145.2','PFKFB3'],
        ['AL137786.1','PAPOLA'],
        ['AL137796.1','C1ORF220'],
        ['AL137856.1','AP4B1-AS1'],
        ['AL138963.3','APOH'],
        ['AL138963.3','HP'],
        ['AL139022.1','FNTB'],
        ['AL139142.1','BX248409.2'],
        ['AL139220.2','KLF17'],
        ['AL139300.2','APOPT1'],
        ['AL139353.3','CHST11'],
        ['AL139383.1','LINC02344'],
        ['AL139383.1','STARD13'],
        ['AL139384.2','TUBGCP3'],
        ['AL139412.1','C1ORF61'],
        ['AL139819.1','PKD2L1'],
        ['AL157714.2','RXRG'],
        ['AL157834.1','CYP2C8'],
        ['AL158066.1','MRPS31P4'],
        ['AL158154.2','UPP1'],
        ['AL158212.1','VTI1A'],
        ['AL160408.2','HBB'],
        ['AL162258.2','DES'],
        ['AL162414.1','CLPB'],
        ['AL353148.1','LYZ'],
        ['AL353608.3','PIP5K1B'],
        ['AL353743.1','KIF27'],
        ['AL354710.2','GAPVD1'],
        ['AL354726.1','HEMGN'],
        ['AL354751.1','PTPRVP'],
        ['AL354760.1','ST7L'],
        ['AL354892.2','TCTE3'],
        ['AL354956.1','TCAM1P'],
        ['AL355376.1','LINC01517'],
        ['AL356235.1','ZC3H12B'],
        ['AL356585.4','ANKRD36B'],
        ['AL356867.1','SPRR3'],
        ['AL358790.1','JPX'],
        ['AL359075.1','SEC16B'],
        ['AL359715.2','BCKDHB'],
        ['AL360181.3','SCART1'],
        ['AL360270.1','CD53'],
        ['AL360295.1','C1ORF168'],
        ['AL365203.3','ITGB1'],
        ['AL365277.1','PPIE'],
        ['AL390036.1','VAV3-AS1'],
        ['AL390195.1','OVGP1'],
        ['AL390726.6','ANKRD18B'],
        ['AL390728.4','RPGRIP1L'],
        ['AL390860.1','TRIM58'],
        ['AL390961.2','FAM238B'],
        ['AL390961.2','FAM238C'],
        ['AL392046.1','CUL2'],
        ['AL392172.1','DISP1'],
        ['AL445238.1','OR7E156P'],
        ['AL445472.1','KLF8'],
        ['AL445623.2','ELAVL2'],
        ['AL450124.1','FAM182B'],
        ['AL450322.2','MYH11'],
        ['AL451085.2','CKS1B'],
        ['AL512625.3','GLIDR'],
        ['AL512662.2','BMS1'],
        ['AL513314.2','LINC00174'],
        ['AL513478.2','PARG'],
        ['AL583722.1','INF2'],
        ['AL589935.1','LINC00472'],
        ['AL590705.5','CCDC180'],
        ['AL590787.1','MTMR6'],
        ['AL591242.1','PLA2G7'],
        ['AL591684.2','BMS1'],
        ['AL591806.3','CCND3'],
        ['AL606491.1','LDLRAP1'],
        ['AL645929.1','AL671277.1'],
        ['AL645937.4','AL672167.2'],
        ['AL662899.1','LY6G5C'],
        ['AL671277.1','AL671883.3'],
        ['AL671710.1','ZBED4'],
        ['AL807752.7','COL1A1'],
        ['AL845552.2','WDR25'],
        ['AL928768.1','IGHA1'],
        ['ALB','ORM2'],
        ['ALB','PRDX1'],
        ['ALDH2','ANKHD1'],
        ['ALDOA','PPP4C'],
        ['ALG14','CNN3'],
        ['ALG1L2','ALG1L6P'],
        ['ALKBH5','LLGL1'],
        ['ALMS1','SMOC2'],
        ['ALOX12-AS1','MIR497HG'],
        ['AMD1','EEF1A1'],
        ['AMMECR1L','NUTM2B-AS1'],
        ['ANAPC1','RMND5A'],
        ['ANK2','CAMK2D'],
        ['ANKAR','ASNSD1'],
        ['ANKAR','ZDHHC7'],
        ['ANKEF1','TACC2'],
        ['ANKRD18CP','BX664727.3'],
        ['ANKRD20A11P','AP005212.5'],
        ['ANKRD20A17P','CR381670.1'],
        ['ANO10','NEK7'],
        ['ANP32B','IGH@'],
        ['ANP32B','TMOD1'],
        ['ANP32B','TNNT2'],
        ['ANXA1','MAGT1'],
        ['ANXA1','TAGLN'],
        ['ANXA1','ZNF207'],
        ['ANXA5','CLEC4E'],
        ['AP000317.1','MRPS6'],
        ['AP000317.1','SLC5A3'],
        ['AP000344.1','LINC01659'],
        ['AP000346.1','AP000347.1'],
        ['AP000346.1','AP000347.2'],
        ['AP000346.1','IGLL3P'],
        ['AP000346.2','AP000347.1'],
        ['AP000346.2','AP000347.2'],
        ['AP000356.2','POM121L7P'],
        ['AP000356.2','USP18'],
        ['AP000356.3','BCR'],
        ['AP000473.1','MIR99AHG'],
        ['AP000781.2','TG'],
        ['AP000880.1','NCAM1-AS1'],
        ['AP000924.1','FDX1'],
        ['AP000936.1','PCSK7'],
        ['AP000943.3','MRE11'],
        ['AP000943.4','FUT4'],
        ['AP001005.4','TUBB8'],
        ['AP001107.1','EMP2'],
        ['AP001350.2','ZFP91-CNTF'],
        ['AP001931.1','TMX2'],
        ['AP001979.2','SPATA19'],
        ['AP002008.1','BTG4'],
        ['AP002748.1','CTSF'],
        ['AP002884.4','IL18'],
        ['AP003066.1','AP003781.1'],
        ['AP003354.2','ATP6V1C1'],
        ['AP003400.1','NOX4'],
        ['AP005329.1','MYOM1'],
        ['AP005435.2','EMB'],
        ['AP005671.1','EPB41L3'],
        ['AP3S1','COMMD10'],
        ['AP3S2','YPEL5'],
        ['APBB1IP','FAM238A'],
        ['APBB2','NR3C2'],
        ['APBB3','SRA1'],
        ['APOL3','MIR100HG'],
        ['ARHGAP11B','OTUD7A'],
        ['ARHGAP18','DLGAP5'],
        ['ARHGAP18','TMEM244'],
        ['ARHGAP26','NR3C1'],
        ['ARHGAP27P1-BPTFP1-KPNA2P3','KPNA2'],
        ['ARHGEF5','OR2A1-AS1'],
        ['ARHGEF9','LINC01278'],
        ['ARID4B','B2M'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARL6IP1','RPS15A'],
        ['ARL6IP6','TRNT1'],
        ['ARMC6','SLC25A42'],
        ['ARPC4','OGG1'],
        ['ARRDC3','BCLAF1'],
        ['ART4','SMCO3'],
        ['AS3MT','CCND3'],
        ['ASL','CRCP'],
        ['ASXL1','NOL4L'],
        ['ATAD1','PTEN'],
        ['ATG16L1','INPP5D'],
        ['ATP10D','USP32'],
        ['ATP13A3','UBR3'],
        ['ATP5C1','ITIH2'],
        ['ATP5H','TG'],
        ['ATP5I','MFSD7'],
        ['ATP6V0B','DPH2'],
        ['ATP6V0C','EEF1A1'],
        ['ATP7B','LINC00282'],
        ['ATP8A2','NUP58'],
        ['AZGP1','GJC3'],
        ['AZIN1','KLF10'],
        ['B2M','FBN1'],
        ['B2M','FUS'],
        ['B2M','G3BP2'],
        ['B2M','HSPA9'],
        ['B2M','IGH@'],
        ['B2M','MED13L'],
        ['B2M','PFKFB3'],
        ['B2M','RUNX1'],
        ['B2M','SF1'],
        ['B2M','SUB1'],
        ['B2M','UBC'],
        ['B2M','UBE3A'],
        ['B3GNT5','KLHL24'],
        ['B3GNT5','TIPARP'],
        ['B3GNTL1','METRNL'],
        ['BABAM2','RBKS'],
        ['BANK1','IGH@'],
        ['BASP1','DSE'],
        ['BCAP29','DUS4L'],
        ['BCAS3','TG'],
        ['BCKDHA','TMEM91'],
        ['BCL2L11','CXCR4'],
        ['BCL6','NAMPT'],
        ['BCL7A','RBM38'],
        ['BEST1','HBB'],
        ['BEST1','MYH6'],
        ['BGLAP','PMF1'],
        ['BICD2','IPPK'],
        ['BLOC1S6','SQOR'],
        ['BMI1','DDX5'],
        ['BPI','CFL1'],
        ['BPIFC','RTCB'],
        ['BPTF','LRRC37A2'],
        ['BRD1','C22ORF34'],
        ['BRD2','SRSF5'],
        ['BRD3','TG'],
        ['BRD4','TG'],
        ['BRK1','VHL'],
        ['BSDC1','FAM229A'],
        ['BSG','ELANE'],
        ['BSG','OAZ1'],
        ['BSG','R3HDM4'],
        ['BSG','SBNO2'],
        ['BSG','TCF3'],
        ['BSG','UBB'],
        ['BX072579.2','LINC00452'],
        ['BX276092.10','BX276092.7'],
        ['BX284632.1','IGK@'],
        ['BX547991.1','LINC00864'],
        ['C10ORF90','LINC00601'],
        ['C11ORF57','DLAT'],
        ['C11ORF70','YAP1'],
        ['C12ORF73','HSP90B1'],
        ['C14ORF166','GNG2'],
        ['C15ORF48','SPATA5L1'],
        ['C15ORF57','CBX3'],
        ['C16ORF87','COL14A1'],
        ['C18ORF8','CAPZA2'],
        ['C19MC','RPLP1'],
        ['C19ORF33','SPINT2'],
        ['C1ORF146','KIAA1107'],
        ['C1ORF194','UQCR10'],
        ['C1ORF43','MB'],
        ['C1ORF61','IFNAR2'],
        ['C20ORF202','PSMF1'],
        ['C2CD3','UCP3'],
        ['C2ORF48','RRM2'],
        ['C2ORF72','SPATA3'],
        ['C2ORF81','HMGA1'],
        ['C2ORF81','RTKN'],
        ['C3ORF67','FAM3D'],
        ['C4ORF51','CALB1'],
        ['C6ORF106','SPDEF'],
        ['C9ORF24','KIF24'],
        ['CABIN1','CBFA2T2'],
        ['CABP7','NF2'],
        ['CACNA1C','LINC02371'],
        ['CALCRL','NBEAL1'],
        ['CALM2','PIGF'],
        ['CALM3','MYL4'],
        ['CALU','FAM71F1'],
        ['CASTOR2','FURIN'],
        ['CATSPER2','PPIP5K1'],
        ['CBWD2','DOCK8'],
        ['CCDC117','Z93930.2'],
        ['CCDC175','L3HYPDH'],
        ['CCDC28A','ECT2L'],
        ['CCDC82','JRKL-AS1'],
        ['CCM2L','HCK'],
        ['CCNG2','OAZ1'],
        ['CCNG2','VAV1'],
        ['CD302','TRIP12'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CD8B','CD8B2'],
        ['CDC34','GZMM'],
        ['CDC34','PTMA'],
        ['CDC42','SLC25A37'],
        ['CDC42','ZEB2'],
        ['CDC42EP2','POLA2'],
        ['CDC42EP4','CPSF4L'],
        ['CDHR4','IP6K1'],
        ['CDK10','SPATA33'],
        ['CDK11B','SLC35E2'],
        ['CDK2','RAB5B'],
        ['CDK5RAP3','PNPO'],
        ['CDR1','LINC00632'],
        ['CDRT15L2','TBC1D26'],
        ['CEL','CELP'],
        ['CELSR3','NCKIPSD'],
        ['CEP152','FBN1'],
        ['CEP192','SEH1L'],
        ['CFAP157','STXBP1'],
        ['CFL1','MKNK2'],
        ['CFL1','SF1'],
        ['CHST11','IGH@'],
        ['CHSY3','ROCK2'],
        ['CITED1','HDAC8'],
        ['CLCF1','POLD4'],
        ['CLEC2D','USP24'],
        ['CLIC2','F8'],
        ['CLK1','CXCR4'],
        ['CLTC','VMP1'],
        ['CNBD2','PHF20'],
        ['CNN2','FURIN'],
        ['CNN2','RNF126'],
        ['CNN2','SBNO2'],
        ['CNTROB','PET100'],
        ['COL14A1','DEPTOR'],
        ['COL16A1','DES'],
        ['COL18A1','COL3A1'],
        ['COLQ','HACL1'],
        ['COMMD10','FBXL17'],
        ['COMMD2','WWTR1'],
        ['COQ4','SLC27A4'],
        ['COQ8B','NUMBL'],
        ['COX19','LINC01002'],
        ['COX6B1','ETV2'],
        ['COX6C','HMGB1'],
        ['COX7A2','MINDY3'],
        ['CPM','LYZ'],
        ['CPSF6','LYZ'],
        ['CRCP','TPST1'],
        ['CREB3','GRB2'],
        ['CREBRF','NSD1'],
        ['CRELD1','PRRT3-AS1'],
        ['CRLF2','LYZ'],
        ['CRYBB2','Z99916.1'],
        ['CRYBG2','ZNF683'],
        ['CTBP1','MAEA'],
        ['CTBP1','UVSSA'],
        ['CTBP1-AS2','MAEA'],
        ['CTCF','FAT1'],
        ['CTDP1','LINC00969'],
        ['CTDP1','NFATC1'],
        ['CTDP1','PQLC1'],
        ['CTDSP1','DNM3OS'],
        ['CTH','HHLA3'],
        ['CTNNA1','MATR3'],
        ['CTNND1','MAGED1'],
        ['CTNND2','MEX3C'],
        ['CTSB','NUCB2'],
        ['CTSC','RAB38'],
        ['CTSD','LSP1'],
        ['CU633967.1','CU638689.4'],
        ['CU634019.6','CU638689.4'],
        ['CUX1','KLF10'],
        ['CWF19L2','TMEM41A'],
        ['CXCR4','METRNL'],
        ['CXCR4','MYO15B'],
        ['CXCR4','RARA'],
        ['CXCR4','RBM38'],
        ['CXCR4','SF1'],
        ['CXCR4','THAP4'],
        ['CXCR4','UBC'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZBTB18'],
        ['CXCR4','ZEB2'],
        ['CYBA','WWOX'],
        ['CYBB','XK'],
        ['CYTIP','ERMN'],
        ['DAND5','NFIX'],
        ['DAZAP1','GLUL'],
        ['DCN','UBE2N'],
        ['DCTN5','UBFD1'],
        ['DCUN1D1','MCCC1'],
        ['DDX27','PARP15'],
        ['DDX39A','JMJD1C'],
        ['DDX3X','EEF1A1'],
        ['DDX3X','UBC'],
        ['DDX42','MAP3K3'],
        ['DDX5','EIF4A1'],
        ['DDX5','FUS'],
        ['DDX5','GRK2'],
        ['DDX5','HMGN2'],
        ['DDX5','HNRNPU'],
        ['DDX5','METRNL'],
        ['DDX5','NFE2L1'],
        ['DDX5','OAZ1'],
        ['DDX5','TEX2'],
        ['DDX5','TPD52L2'],
        ['DDX5','UGCG'],
        ['DES','KRT7'],
        ['DGCR8','TANGO2'],
        ['DHCR24','KRT4'],
        ['DHRS4-AS1','NRL'],
        ['DHRS7C','GSG1L2'],
        ['DHX35','FAM83D'],
        ['DHX37','UBC'],
        ['DHX9','NPL'],
        ['DIP2B','TMBIM6'],
        ['DLD','HNRNPK'],
        ['DLEU2','HMGB1'],
        ['DLEU2','SPRYD7'],
        ['DNAAF3','SYT5'],
        ['DNAJA1','POLR2A'],
        ['DNAJB6','LYZ'],
        ['DNAJC7','LYZ'],
        ['DOC2B','LINC02091'],
        ['DOCK1','FAM24B'],
        ['DOCK9','STK24'],
        ['DOT1L','PTBP1'],
        ['DOT1L','ZBTB7A'],
        ['DPH7','PNPLA7'],
        ['DPP9','TG'],
        ['DSC3','TMEM269'],
        ['DTD1','MKL2'],
        ['E2F1','PXMP4'],
        ['EAF2','SLC15A2'],
        ['EEF1A1','HBA1'],
        ['EEF1A1','PTMA'],
        ['EEF1A1','TG'],
        ['EEF1A1','TPM3'],
        ['EEF1A2','MYH7'],
        ['EEF1AKMT3','TSFM'],
        ['EEF2','IGK@'],
        ['EFCAB2','KIF26B'],
        ['EFCAB7','PGM1'],
        ['EGF','SEC24B'],
        ['EIF1AX','MAP7D2'],
        ['EIF1AX','RPS6KA3'],
        ['EIF2AK1','S100A9'],
        ['EIF2S1','RBM25'],
        ['EIF4E3','FOXP1'],
        ['EIF4E3','PROK2'],
        ['EIF4G2','PTMA'],
        ['ELANE','HBD'],
        ['ELANE','LINC01578'],
        ['ELANE','SF1'],
        ['EMC2','NUDCD1'],
        ['EMP2','MIR100HG'],
        ['ENO2','MYBPC3'],
        ['ENTPD5','FAM161B'],
        ['EPB41L3','ZBTB14'],
        ['EPB42','LYZ'],
        ['EPN2','UBB'],
        ['ERBB2','GRB7'],
        ['ERVK-28','ZNF585A'],
        ['F5','SELP'],
        ['FADS1','TMEM258'],
        ['FAM110A','JAZF1'],
        ['FAM129C','PGLS'],
        ['FAM133B','FAM133CP'],
        ['FAM135A','MAP3K4'],
        ['FAM135A','SDHAF4'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM160B1','TRUB1'],
        ['FAM53A','XRRA1'],
        ['FAM53B','MTHFS'],
        ['FAM53B','TSPAN3'],
        ['FAM66A','FAM66C'],
        ['FAM85B','LINC02018'],
        ['FAM86B1','FAM86B3P'],
        ['FBXL5','HMGB1'],
        ['FBXO16','HBB'],
        ['FBXO25','TDRP'],
        ['FCAR','LENG8'],
        ['FES','MAN2A2'],
        ['FGD5','TPRXL'],
        ['FGFR4','ZNF346'],
        ['FITM2','OSER1'],
        ['FLJ22447','PRKCH'],
        ['FNIP1','PDE4B'],
        ['FOLR3','LRTOMT'],
        ['FOXK2','SERPINE2'],
        ['FOXO1','NUP98'],
        ['FOXP1','RYBP'],
        ['FP700111.2','IGK@'],
        ['FRG1','FRG1HP'],
        ['FRMD6-AS2','LINC02310'],
        ['FUS','KCTD5'],
        ['G2E3','SCFD1'],
        ['G3BP2','PTP4A1'],
        ['GABPB1','PTP4A1'],
        ['GALK2','SUPT16H'],
        ['GAS7','RCVRN'],
        ['GATC','KDM2A'],
        ['GBP3','KYAT3'],
        ['GCA','SAMSN1'],
        ['GCA','TG'],
        ['GCC1','TG'],
        ['GGT1','IGSF3'],
        ['GLE1','ODF2'],
        ['GLUL','KCNAB2'],
        ['GNAS','SENP3'],
        ['GNG7','TPM1'],
        ['GOLT1A','KISS1'],
        ['GPR156','LRRC58'],
        ['GPRC5A','OGFRL1'],
        ['GPT','PPP1R16A'],
        ['GPX3','KIAA0319L'],
        ['GRAMD1A','TG'],
        ['GRSF1','RSRP1'],
        ['GS1-124K5.4','TPST1'],
        ['GSN','MMP25'],
        ['GSN','S100A9'],
        ['GUCA1C','MORC1'],
        ['GYPB','SAV1'],
        ['H6PD','SPSB1'],
        ['HACD1','PLS3'],
        ['HAPLN3','MFGE8'],
        ['HAS3','UTP4'],
        ['HBA1','RPS27'],
        ['HBA2','TUBB4B'],
        ['HBB','MOB3A'],
        ['HBB','NLRC5'],
        ['HBB','SEPT2'],
        ['HBB','VTI1A'],
        ['HBD','LTF'],
        ['HBP1','IMMP2L'],
        ['HIF1A','PTP4A1'],
        ['HIF1A','RNPC3'],
        ['HIF1A','STAG2'],
        ['HIPK1','OLFML3'],
        ['HKR1','TG'],
        ['HMBS','SBNO2'],
        ['HMGB1','HNRNPH1'],
        ['HMGB1','MXD4'],
        ['HMGB1','SRSF5'],
        ['HMGB2','IVNS1ABP'],
        ['HMGN2','NECAB3'],
        ['HNRNPA2B1','UBB'],
        ['HNRNPC','PTP4A1'],
        ['HNRNPD','RCC1'],
        ['HNRNPDL','TUBA1B'],
        ['HNRNPH1','ITPRIP'],
        ['HNRNPH1','TG'],
        ['HNRNPK','NFIL3'],
        ['HNRNPK','UBQLN1'],
        ['HNRNPUL1','MAEA'],
        ['HOOK3','PPM1A'],
        ['HORMAD1','SLC22A20'],
        ['HSD17B14','PLEKHA4'],
        ['HSD17B4','TNFAIP8'],
        ['HSP90AB1','SLC29A1'],
        ['HSPA9','UBE2D2'],
        ['HSPB11','YIPF1'],
        ['HSPE1','MOB4'],
        ['IFT122','RYK'],
        ['IGH@','LAPTM5'],
        ['IGH@','MIR100HG'],
        ['IGH@','MUC1'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','PPP6R2'],
        ['IGH@','TRA@'],
        ['IGH@','ZFP36L1'],
        ['IGSF10','MED12L'],
        ['IL9R','RPL23AP82'],
        ['IMP3','SNUPN'],
        ['INPP4B','PLCZ1'],
        ['INPP5A','PWWP2B'],
        ['IQCG','POLG2'],
        ['IQGAP1','ZNF774'],
        ['ITGAV','PXK'],
        ['ITGB2','POFUT2'],
        ['ITGB6','PPP4R1'],
        ['ITM2B','RB1'],
        ['ITPKC','SNRPA'],
        ['IVD','KNSTRN'],
        ['JARID2','LINC01578'],
        ['JARID2','MGAT4A'],
        ['JAZF1','ZNF777'],
        ['JKAMP','PCNX4'],
        ['KANSL1','LINC02210'],
        ['KANSL1','LRRC37A3'],
        ['KANSL3','RIPK4'],
        ['KANSL3','TMCO4'],
        ['KAT2B','SGO1-AS1'],
        ['KCNK7','MAP3K11'],
        ['KDM2A','SF1'],
        ['KDM4B','SAFB2'],
        ['KDM7A','LAPTM5'],
        ['KHDRBS1','KIAA0319L'],
        ['KIAA1024','TMED3'],
        ['KIAA1147','WEE2-AS1'],
        ['KIF15','ZNF501'],
        ['KIF1C','KRT13'],
        ['KIFC1','RNH1'],
        ['KLF2','OAZ1'],
        ['KLHL36','ZDHHC7'],
        ['KLK4','KLKP1'],
        ['KMT2C','TPTE'],
        ['KMT2E','SRSF7'],
        ['KMT5A','RFLNA'],
        ['KRT13','SCP2'],
        ['KRT4','LY6D'],
        ['KRT8','VIM'],
        ['LACTB2-AS1','XKR9'],
        ['LAMA3','RBBP8'],
        ['LAMTOR1','NUMA1'],
        ['LAPTM5','RIPOR1'],
        ['LARP1','NR3C1'],
        ['LATS1','NUP43'],
        ['LDAH','PUM2'],
        ['LGMN','MYH11'],
        ['LINC00467','TRAF5'],
        ['LINC00535','UBE2V2'],
        ['LINC00886','TIPARP-AS1'],
        ['LINC01068','NDFIP2'],
        ['LINC01169','SMAD6'],
        ['LINC01372','MEF2C'],
        ['LINC01470','NEDD1'],
        ['LINC01578','YPEL5'],
        ['LINC01578','ZBTB7A'],
        ['LINC01608','LINC01609'],
        ['LINC01619','LINC02391'],
        ['LINC01705','LINC02257'],
        ['LINC01872','SIGLECL1'],
        ['LINC02219','PIK3R1'],
        ['LINC02246','NRIP1'],
        ['LINC02345','TMCO5A'],
        ['LPAR2','PBX4'],
        ['LPGAT1','RPS6KC1'],
        ['LRP12','PKP4'],
        ['LRP2BP','UFSP2'],
        ['LRP5','LRP5L'],
        ['LRPAP1','WDR1'],
        ['LRRC37A16P','LRRC37B'],
        ['LRRC37A3','NSF'],
        ['LRRC59','MRPL27'],
        ['LY6K','THEM6'],
        ['LYST','SETX'],
        ['LYZ','MARF1'],
        ['LYZ','MORF4L1'],
        ['LYZ','PIGQ'],
        ['LYZ','SYNE3'],
        ['LYZ','WAC-AS1'],
        ['LYZ','Z99129.3'],
        ['MAD1L1','MAFK'],
        ['MAML2','MTMR2'],
        ['MAN2A1','UIMC1'],
        ['MANBA','UBE2D3'],
        ['MANBAL','SRC'],
        ['MAP3K8','PRKG1'],
        ['MAPKBP1','MGA'],
        ['MARCKS','PTMA'],
        ['MAST2','TNNT2'],
        ['MECOM','RPL22'],
        ['MEF2C','TMEM161B'],
        ['METRNL','RALGDS'],
        ['MGRN1','NUDT16L1'],
        ['MICAL2','TEAD1'],
        ['MINOS1','MMD'],
        ['MINPP1','NUTM2A-AS1'],
        ['MIR100HG','PIGR'],
        ['MIR100HG','SSR3'],
        ['MIR22HG','TLCD2'],
        ['MLEC','UNC119B'],
        ['MLIP','TINAG'],
        ['MOB1B','VLDLR-AS1'],
        ['MPO','TBC1D9B'],
        ['MRC1','TMEM236'],
        ['MSMB','NCOA4'],
        ['MSRB3','TMBIM4'],
        ['MTA3','THADA'],
        ['MTHFD1L','PLEKHG1'],
        ['MTHFSD','STIM2'],
        ['MTMR10','TRPM1'],
        ['MTSS1','TATDN1'],
        ['MTSS1L','VAC14'],
        ['MTURN','PLEKHA8'],
        ['MUC2','MUC5B'],
        ['MYH11','SCARB2'],
        ['MYH6','TIMP3'],
        ['MYH6','TPT1'],
        ['MYL3','TNNC1'],
        ['MYO10','TSPAN18'],
        ['MYO5C','NDUFS6'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NCOA6','PIGU'],
        ['NCOR2','UBC'],
        ['NDEL1','UBB'],
        ['NECAP2','SZRD1'],
        ['NFATC1','PQLC1'],
        ['NFATC3','PLA2G15'],
        ['NIN','SAV1'],
        ['NOXRED1','TMED8'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB15','PDXDC2P-NPIPB14P'],
        ['NPIPB5','SMG1'],
        ['NPPA','SORBS2'],
        ['NQO2','PABPC1L'],
        ['NRAV','SRSF9'],
        ['NRBP1','TG'],
        ['NSMAF','SPTLC3'],
        ['NUCB1-AS1','TULP2'],
        ['NUP188','PHYHD1'],
        ['NUP210','STOM'],
        ['NUSAP1','OIP5-AS1'],
        ['OAZ1','PFKFB3'],
        ['OAZ1','PTBP1'],
        ['OAZ1','RAC2'],
        ['OAZ1','SEPT5'],
        ['OAZ1','TCF3'],
        ['OAZ1','UBTF'],
        ['OSBPL8','UBAC2'],
        ['PACSIN2','TOB1'],
        ['PAN2','STAT2'],
        ['PATJ','TG'],
        ['PBXIP1','PMVK'],
        ['PCM1','SPIDR'],
        ['PCP4L1','SDHC'],
        ['PEMT','UBB'],
        ['PFKFB3','S100A9'],
        ['PIAS2','ST8SIA5'],
        ['PID1','SFTPA1'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PINK1-AS','TAS2R4'],
        ['PITPNA','YWHAE'],
        ['PITPNC1','PRKCA'],
        ['PKHD1L1','STX8'],
        ['PLA2G15','WBP1L'],
        ['PLCG1','TG'],
        ['PLEKHB2','ZNF727'],
        ['PLSCR4','RNF13'],
        ['PMS2','POLR2J3'],
        ['PPM1B','SLC3A1'],
        ['PPM1H','USP15'],
        ['PPP1CB','WSB1'],
        ['PPP1CB','YPEL5'],
        ['PPP1R12C','TSPAN4'],
        ['PRC1','VPS33B'],
        ['PRDX1','TESK2'],
        ['PRKAG2','PSD3'],
        ['PRKAG2','RHEB'],
        ['PRKCH','PRPS2'],
        ['PRKCZ','SKI'],
        ['PRKRIP1','RBAK'],
        ['PRMT7','SLC7A6'],
        ['PRSS3','TMEM45B'],
        ['PRSS51','TNKS'],
        ['PSMA5','SORT1'],
        ['PSMF1','SIPA1L1'],
        ['PSPC1','ZMYM5'],
        ['PTBP1','RNF126'],
        ['PTBP1','SBNO2'],
        ['PTBP3','SUSD1'],
        ['PTCSC2','RNF144B'],
        ['PTMA','SRSF5'],
        ['PTMA','WDR1'],
        ['PTMA','ZEB2'],
        ['PTP4A1','SRSF5'],
        ['PVRIG2P','STAG3'],
        ['PXK','RPP14'],
        ['PXYLP1','SPSB4'],
        ['R3HDM1','TRAPPC12'],
        ['RAB11B','SEPHS1'],
        ['RAB35','THEMIS2'],
        ['RARA','RCN1'],
        ['RASA3','TFDP1'],
        ['RASSF8','SSPN'],
        ['RBAK','RNF216'],
        ['RBM38','SUMO2'],
        ['RBMS2','SPRYD4'],
        ['RBMS2','TG'],
        ['RECQL5','SENP6'],
        ['RHOF','UBC'],
        ['RIPK1','SERPINB9'],
        ['RNASET2','SRGN'],
        ['RNF125','RNF138'],
        ['RNF139','TRMT12'],
        ['RNF151','TBL3'],
        ['RNF31','THADA'],
        ['RPL13','SPG7'],
        ['RPL14','SRP14'],
        ['RPL27','RUNDC1'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['RPS18','TG'],
        ['RPS19','SNUPN'],
        ['RPS9','UBB'],
        ['RWDD3','TMEM56'],
        ['SAMD5','SASH1'],
        ['SAMSN1','USP25'],
        ['SCAF4','SERGEF'],
        ['SCARB1','UBC'],
        ['SCNN1A','TNFRSF1A'],
        ['SDC2','VMP1'],
        ['SDF4','TNFRSF4'],
        ['SENP3-EIF4A1','TNFSF13'],
        ['SERF1A','SMN1'],
        ['SH3KBP1','TYROBP'],
        ['SIDT2','TAGLN'],
        ['SIRT2','XPNPEP1'],
        ['SLC19A1','SUMO3'],
        ['SLC20A2','WDR45'],
        ['SLC25A3','TOM1'],
        ['SLC2A11','SMARCB1'],
        ['SLC33A1','UBP1'],
        ['SLC6A16','TEAD2'],
        ['SLFN11','TG'],
        ['SMAP2','ZFP69B'],
        ['SMARCA2','TG'],
        ['SMG1','ZNF623'],
        ['SNRPB','SNURF'],
        ['SNRPN','WDR4'],
        ['SNTB2','VPS4A'],
        ['SNX29','TG'],
        ['SPATA3','ZNF483'],
        ['ST3GAL1','UBB'],
        ['STK10','UBE2D2'],
        ['STK4-AS1','TOMM34'],
        ['TAMM41','VGLL4'],
        ['TBCEL','TECTA'],
        ['TDRD10','UBE2Q1-AS1'],
        ['TFDP1','TMCO3'],
        ['TFDP1','ZEB2'],
        ['TFDP2','XRN1'],
        ['TG','TIE1'],
        ['TG','ZNF12'],
        ['TMEM225B','ZNF655'],
        ['TMEM225B','ZSCAN25'],
        ['TOP1','TPRXL'],
        ['TPM1','UQCRB'],
        ['TPM1','XIRP2'],
        ['TTC32','WDR35'],
        ['UBA2','WTIP'],
        ['USP34','XPO1'],
        ['VRK2','WDPCP'],
        ['YPEL5','ZEB2'],
        ['Z93930.2','ZNRF3'],
        ['ZC2HC1A','ZFX'],
        ['ZDHHC12','ZER1'],
        ['ZFP30','ZNF571'],
        ['ZNF32-AS3','ZNF485'],
        ['ZNF616','ZNF836'],
        ['ZNF746','ZNF767P'],
        ['ZNF780A','ZNF780B'],
        ['ZNF815P','ZNF890P'],
        ['DUS4L','BCAP29'],
        ['A4GNT','DZIP1L'],
        ['AATF','TK2'],
        ['AB015752.1','TMEM64'],
        ['ABAT','CEL'],
        ['ABCA10','GFOD1'],
        ['ABCF2','ICA1'],
        ['ABHD2','AC013565.1'],
        ['ABHD2','AC020656.1'],
        ['ABL1','EXOSC2'],
        ['ABR','AC018630.6'],
        ['ABR','CLUH'],
        ['AC002116.2','WDR62'],
        ['AC002398.2','DES'],
        ['AC002398.2','MYL2'],
        ['AC002480.4','STEAP1B'],
        ['AC003973.2','LINC01859'],
        ['AC004066.2','PPA2'],
        ['AC004080.5','HOXA5'],
        ['AC004160.1','PHF14'],
        ['AC004241.5','BGN'],
        ['AC004381.1','THUMPD1'],
        ['AC004471.2','DGCR2'],
        ['AC004477.1','COPZ2'],
        ['AC004846.2','NUMB'],
        ['AC004865.2','PSMB2'],
        ['AC004889.1','AC005304.2'],
        ['AC004889.1','ARHGEF5'],
        ['AC004895.1','USP42'],
        ['AC004908.2','LINC00115'],
        ['AC004951.2','RASA4'],
        ['AC004951.4','RASA4'],
        ['AC004967.1','OR7E38P'],
        ['AC004980.3','AC091390.2'],
        ['AC004980.3','AC211486.3'],
        ['AC004980.3','PMS2'],
        ['AC005005.4','TXNDC15'],
        ['AC005013.1','AC005162.3'],
        ['AC005035.1','IL1R2'],
        ['AC005077.2','LINC00174'],
        ['AC005091.1','HIBADH'],
        ['AC005154.1','GGCT'],
        ['AC005154.3','GGCT'],
        ['AC005258.1','ACTB'],
        ['AC005258.1','CSNK1G2'],
        ['AC005258.1','DOT1L'],
        ['AC005258.1','KLF16'],
        ['AC005258.1','PTMA'],
        ['AC005258.1','SGTA'],
        ['AC005332.7','BPTF'],
        ['AC005410.1','RPL21'],
        ['AC005479.2','NPC2'],
        ['AC005487.1','DNAJB9'],
        ['AC005540.1','RPL14'],
        ['AC005562.1','AC138207.8'],
        ['AC005586.2','LRRC61'],
        ['AC005674.2','WDR1'],
        ['AC005703.1','PMP22'],
        ['AC005740.1','NDUFS1'],
        ['AC005775.1','ODF3L2'],
        ['AC005832.1','NDUFA9'],
        ['AC005856.1','AC005884.1'],
        ['AC005993.1','RGS6'],
        ['AC006001.3','RABGEF1'],
        ['AC006064.4','HBB'],
        ['AC006064.4','NPPA'],
        ['AC006262.3','IGFL2-AS1'],
        ['AC006269.1','MMD'],
        ['AC006333.1','NDUFA5'],
        ['AC006460.2','NEMP2'],
        ['AC006946.2','CECR7'],
        ['AC007192.1','AC020656.1'],
        ['AC007207.2','AC027288.1'],
        ['AC007207.2','AL117378.1'],
        ['AC007229.1','HIKESHI'],
        ['AC007388.1','AC020656.1'],
        ['AC007570.1','KCTD10'],
        ['AC007663.1','PRODH'],
        ['AC007663.4','DGCR6'],
        ['AC007923.4','SDHA'],
        ['AC008105.3','AC138150.2'],
        ['AC008132.1','CALM1'],
        ['AC008543.1','ZNF627'],
        ['AC008555.5','LINC01801'],
        ['AC008575.1','AC136475.10'],
        ['AC008750.8','NKG7'],
        ['AC008753.1','RPLP1'],
        ['AC008760.2','SERPING1'],
        ['AC008764.1','EPS15L1'],
        ['AC008764.4','SLC35E1'],
        ['AC008770.1','AC011455.2'],
        ['AC008770.1','ZNF625-ZNF20'],
        ['AC008808.2','LINC02106'],
        ['AC008870.1','GTDC1'],
        ['AC009078.2','PRSS1'],
        ['AC009078.2','PRSS2'],
        ['AC009078.2','TRB@'],
        ['AC009107.1','CCDC113'],
        ['AC009126.1','ERAP1'],
        ['AC009135.2','ASNS'],
        ['AC009163.5','CHST5'],
        ['AC009171.2','SRL'],
        ['AC009275.1','FGD4'],
        ['AC009292.2','PIAS1'],
        ['AC009309.1','MRPL35'],
        ['AC009948.1','DFNB59'],
        ['AC010186.2','DCK'],
        ['AC010186.2','GOT2'],
        ['AC010186.4','GOT2'],
        ['AC010203.1','DEPDC4'],
        ['AC010325.1','C19ORF48'],
        ['AC010326.4','ZNF587'],
        ['AC010327.1','ACTA1'],
        ['AC010327.1','DES'],
        ['AC010327.1','NPPA-AS1'],
        ['AC010542.1','CMTM4'],
        ['AC010542.5','CMTM4'],
        ['AC010643.1','TEAD2'],
        ['AC010735.2','RHBDD1'],
        ['AC011379.2','CYSTM1'],
        ['AC011410.1','SMIM23'],
        ['AC011445.1','GMFG'],
        ['AC011446.2','PRB1'],
        ['AC011468.4','TMEM50B'],
        ['AC011487.1','ZNF765'],
        ['AC011487.3','OSTC'],
        ['AC011491.3','ACER1'],
        ['AC011523.1','KLK3'],
        ['AC011591.1','SFTPB'],
        ['AC011767.1','GOLGA6L2'],
        ['AC011825.1','RPS15A'],
        ['AC011840.1','KRT42P'],
        ['AC012170.2','USP50'],
        ['AC012435.2','UBL7-AS1'],
        ['AC012441.2','AC018638.5'],
        ['AC012442.1','CHCHD5'],
        ['AC013244.1','DIP2B'],
        ['AC013394.1','B2M'],
        ['AC013457.1','AP000688.2'],
        ['AC015468.2','LINC02153'],
        ['AC015689.2','RDX'],
        ['AC015691.1','HBE1'],
        ['AC015802.4','ST6GALNAC2'],
        ['AC015802.6','CYGB'],
        ['AC015813.1','SRSF1'],
        ['AC015849.1','TAF15'],
        ['AC015849.5','LRRC37A9P'],
        ['AC015922.4','IL6ST'],
        ['AC016044.1','ONECUT1'],
        ['AC016257.1','ALDH1L2'],
        ['AC016730.1','AC073062.1'],
        ['AC016831.1','LINC-PINT'],
        ['AC016876.2','EIF4A2'],
        ['AC016876.2','IGK@'],
        ['AC016876.2','LYZ'],
        ['AC016876.2','MPDU1'],
        ['AC016907.2','ALK'],
        ['AC017091.1','AC097501.1'],
        ['AC017104.5','LINC00471'],
        ['AC018521.5','SP2-AS1'],
        ['AC018616.1','BEST1'],
        ['AC018630.6','AC020656.1'],
        ['AC018630.6','AMY2B'],
        ['AC018630.6','CA6'],
        ['AC018630.6','CCL28'],
        ['AC018630.6','CD44'],
        ['AC018630.6','CD74'],
        ['AC018630.6','CST1'],
        ['AC018630.6','CST4'],
        ['AC018630.6','CX3CL1'],
        ['AC018630.6','DSG2'],
        ['AC018630.6','EEF1A1'],
        ['AC018630.6','EEF2'],
        ['AC018630.6','GAPDH'],
        ['AC018630.6','GNAS'],
        ['AC018630.6','GNE'],
        ['AC018630.6','H1F0'],
        ['AC018630.6','IGH@'],
        ['AC018630.6','IGK@'],
        ['AC018630.6','IL6ST'],
        ['AC018630.6','ITPR1'],
        ['AC018630.6','LPO'],
        ['AC018630.6','LYZ'],
        ['AC018630.6','MLLT6'],
        ['AC018630.6','MUC7'],
        ['AC018630.6','MYH9'],
        ['AC018630.6','ODAM'],
        ['AC018630.6','PER1'],
        ['AC018630.6','PHLDA1'],
        ['AC018630.6','PIGR'],
        ['AC018630.6','PIK3R1'],
        ['AC018630.6','PRB1'],
        ['AC018630.6','PRB2'],
        ['AC018630.6','PRB3'],
        ['AC018630.6','PRB4'],
        ['AC018630.6','PRR27'],
        ['AC018630.6','PSMF1'],
        ['AC018630.6','RBM47'],
        ['AC018630.6','RPL3'],
        ['AC018630.6','SFRP1'],
        ['AC018630.6','SIX4'],
        ['AC018630.6','SLC6A8'],
        ['AC018630.6','SMR3B'],
        ['AC018630.6','SND1'],
        ['AC018630.6','SORL1'],
        ['AC018630.6','SQSTM1'],
        ['AC018630.6','STATH'],
        ['AC018630.6','ZG16B'],
        ['AC018638.5','AL606534.4'],
        ['AC018688.1','ZNF285'],
        ['AC018695.3','COX4I1'],
        ['AC018797.2','CISD2'],
        ['AC019349.1','CRNN'],
        ['AC019349.1','GSN'],
        ['AC019349.1','KRT4'],
        ['AC019349.1','KRT6A'],
        ['AC019349.1','S100A9'],
        ['AC019349.1','SPINK5'],
        ['AC020656.1','AHNAK'],
        ['AC020656.1','AKR1B10'],
        ['AC020656.1','CEP68'],
        ['AC020656.1','CLDN18'],
        ['AC020656.1','CRISP3'],
        ['AC020656.1','CTSG'],
        ['AC020656.1','DEPDC4'],
        ['AC020656.1','EFCAB11'],
        ['AC020656.1','FAXDC2'],
        ['AC020656.1','FBXO6'],
        ['AC020656.1','HSD17B12'],
        ['AC020656.1','IGK@'],
        ['AC020656.1','IL3RA'],
        ['AC020656.1','LINC01285'],
        ['AC020656.1','LPO'],
        ['AC020656.1','MBTD1'],
        ['AC020656.1','MCAM'],
        ['AC020656.1','MPPED2'],
        ['AC020656.1','PEAK1'],
        ['AC020656.1','PGC'],
        ['AC020656.1','PKM'],
        ['AC020656.1','POMGNT1'],
        ['AC020656.1','PRKAR2A'],
        ['AC020656.1','RPL8'],
        ['AC020656.1','S100A8'],
        ['AC020656.1','SFTPB'],
        ['AC020656.1','SLC25A37'],
        ['AC020656.1','SLC2A3'],
        ['AC020656.1','SLC43A2'],
        ['AC020656.1','SLC4A1'],
        ['AC020656.1','SMARCA4'],
        ['AC020656.1','SPTBN1'],
        ['AC020656.1','TAF15'],
        ['AC020656.1','TARS2'],
        ['AC020656.1','WWOX'],
        ['AC020765.2','NUPR1'],
        ['AC020915.2','ZSCAN22'],
        ['AC020928.1','ZNF567'],
        ['AC020978.4','ESRP2'],
        ['AC021171.1','FAM133B'],
        ['AC022028.2','GRID1-AS1'],
        ['AC022145.1','STK33'],
        ['AC022217.1','FAM58A'],
        ['AC022400.3','BMS1'],
        ['AC022400.5','BMS1'],
        ['AC022733.1','AC087518.1'],
        ['AC023055.1','AC073063.1'],
        ['AC023510.1','CASC1'],
        ['AC023644.1','BEST1'],
        ['AC024084.1','AC078845.1'],
        ['AC024560.3','AC093642.1'],
        ['AC024575.1','TMEM205'],
        ['AC024587.1','USP7'],
        ['AC024592.2','LINC01356'],
        ['AC024937.3','AC026412.1'],
        ['AC024937.3','PDCD6'],
        ['AC025048.2','AC243829.2'],
        ['AC025171.2','ZNF131'],
        ['AC025262.1','TBC1D30'],
        ['AC025279.1','SNX29'],
        ['AC025566.1','LINC02032'],
        ['AC025580.3','SMG1'],
        ['AC025884.1','RERE'],
        ['AC026150.1','GOLGA8M'],
        ['AC026412.1','SDHA'],
        ['AC026471.2','AHSP'],
        ['AC027031.2','OXR1'],
        ['AC027343.1','LINC02123'],
        ['AC027559.1','AC090825.1'],
        ['AC034205.2','SLC36A2'],
        ['AC036108.3','SYNM'],
        ['AC040162.1','CELA2A'],
        ['AC040162.1','CELA2B'],
        ['AC040162.1','CTRB2'],
        ['AC040162.1','PRSS1'],
        ['AC040162.1','PRSS2'],
        ['AC040162.1','TRB@'],
        ['AC040170.1','LINC01081'],
        ['AC044810.2','OVCH2'],
        ['AC044860.1','GOLGA2P3Y'],
        ['AC048337.1','DGKD'],
        ['AC048338.1','CLIP1'],
        ['AC053503.4','KRT13'],
        ['AC053503.4','MYH6'],
        ['AC053503.4','MYH7'],
        ['AC053503.4','MYL2'],
        ['AC053503.4','NPPA'],
        ['AC053503.4','TNNI3'],
        ['AC053503.4','TPM1'],
        ['AC055713.1','WSB1'],
        ['AC058791.1','LINC00513'],
        ['AC060780.1','AC096637.2'],
        ['AC061975.7','AL035661.1'],
        ['AC063926.3','PIWIL1'],
        ['AC063944.1','LINC00882'],
        ['AC063979.2','LINC02056'],
        ['AC067930.6','ANKRD19P'],
        ['AC067930.6','HBB'],
        ['AC068152.1','ARL17B'],
        ['AC068446.1','PAK2'],
        ['AC068491.1','PAFAH1B1'],
        ['AC068580.4','ALB'],
        ['AC068580.4','IGF2'],
        ['AC068580.4','PRB1'],
        ['AC068580.4','PRB2'],
        ['AC068580.4','SMR3B'],
        ['AC068768.1','MPHOSPH9'],
        ['AC068831.3','CERK'],
        ['AC069281.1','AGFG2'],
        ['AC073264.3','AC099548.2'],
        ['AC073288.2','SKIL'],
        ['AC073578.1','LINC01257'],
        ['AC074051.2','ASNS'],
        ['AC074099.1','LINC01876'],
        ['AC078842.1','PTN'],
        ['AC079193.2','MICU3'],
        ['AC079228.1','RTN3'],
        ['AC079316.1','AL139353.3'],
        ['AC079848.1','AC108134.3'],
        ['AC083862.1','RNF14'],
        ['AC083964.2','TDRP'],
        ['AC084024.3','ASH2L'],
        ['AC084032.1','CNOT2'],
        ['AC087071.2','SSMEM1'],
        ['AC087190.4','LYZ'],
        ['AC087463.4','PWRN1'],
        ['AC087463.4','PWRN3'],
        ['AC087481.2','FAN1'],
        ['AC087481.2','HERC2'],
        ['AC087672.3','LY96'],
        ['AC087897.2','CPNE8'],
        ['AC090001.1','NTN4'],
        ['AC090114.3','TLK2'],
        ['AC090409.2','AC105094.2'],
        ['AC091167.3','GDPGP1'],
        ['AC091390.2','AC211486.3'],
        ['AC091390.3','STAG3'],
        ['AC091390.3','STAG3L5P'],
        ['AC091390.3','STAG3L5P-PVRIG2P-PILRB'],
        ['AC091390.4','AC211429.1'],
        ['AC091390.4','AC211486.3'],
        ['AC091390.4','PMS2'],
        ['AC091849.2','SDHA'],
        ['AC092079.1','DDX18'],
        ['AC092162.2','SPOPL'],
        ['AC092284.1','CRLF3'],
        ['AC092376.1','MAF'],
        ['AC092506.1','THAP3'],
        ['AC092718.8','CMC2'],
        ['AC092718.8','NDUFS1'],
        ['AC092746.1','CLEC4E'],
        ['AC092747.2','MED21'],
        ['AC092813.2','ST6GALNAC3'],
        ['AC092821.1','AC141557.1'],
        ['AC092821.1','TPT1-AS1'],
        ['AC092821.2','RANBP6'],
        ['AC092902.2','FAM85B'],
        ['AC092920.1','LINC01208'],
        ['AC093157.2','SLC30A7'],
        ['AC093162.2','RETSAT'],
        ['AC093392.1','SEPT7'],
        ['AC093484.4','ANKRD1'],
        ['AC093484.4','HBA2'],
        ['AC093484.4','HBB'],
        ['AC093484.4','UBA52'],
        ['AC093525.2','DDX5'],
        ['AC093627.3','HABP4'],
        ['AC093720.1','UGT2A3'],
        ['AC093752.1','SEPT7'],
        ['AC093821.1','LINC01091'],
        ['AC093895.1','SPARCL1'],
        ['AC095038.1','ZNF727'],
        ['AC095350.2','RIMBP2'],
        ['AC096664.2','RPL27'],
        ['AC097372.3','LINC01095'],
        ['AC097374.1','AL513478.2'],
        ['AC097374.1','CR392039.5'],
        ['AC098484.2','PPIH'],
        ['AC098798.1','PANCR'],
        ['AC098848.1','CCDC68'],
        ['AC098850.4','AC107983.2'],
        ['AC098850.4','USP32'],
        ['AC098850.5','NOS2'],
        ['AC098850.5','USP6'],
        ['AC099329.1','CCDC13'],
        ['AC099508.2','ZFP1'],
        ['AC099548.2','TCAF1'],
        ['AC099548.2','TCAF2'],
        ['AC099794.1','ATG4C'],
        ['AC100821.2','TCEA1'],
        ['AC100861.1','CHMP7'],
        ['AC103702.2','HOXB9'],
        ['AC103957.1','PRAG1'],
        ['AC104109.3','LINC00174'],
        ['AC104581.1','DNAH14'],
        ['AC104758.1','COMMD4'],
        ['AC105046.1','EGR3'],
        ['AC105052.3','RASA4CP'],
        ['AC105411.1','LINC01227'],
        ['AC106795.3','AC139493.2'],
        ['AC106865.1','TLR2'],
        ['AC106886.5','MYH11'],
        ['AC107983.2','USP32'],
        ['AC107983.2','USP6'],
        ['AC108010.1','TLK2'],
        ['AC108097.1','LINC01227'],
        ['AC108488.1','ADI1'],
        ['AC108688.1','DNAJB6'],
        ['AC108734.3','RNF13'],
        ['AC108751.4','TM4SF1'],
        ['AC109466.1','CCDC7'],
        ['AC110079.2','NPR1'],
        ['AC111149.1','RDH10-AS1'],
        ['AC112719.2','ART3'],
        ['AC113189.4','TMEM102'],
        ['AC113189.6','C17ORF74'],
        ['AC113189.6','TMEM102'],
        ['AC114284.1','PRR16'],
        ['AC114501.2','INTS4'],
        ['AC114781.2','COQ2'],
        ['AC116050.1','IGK@'],
        ['AC116366.1','IRF1'],
        ['AC116563.1','AC131956.2'],
        ['AC116651.1','FBXL5'],
        ['AC117386.2','CHURC1'],
        ['AC117386.2','TGOLN2'],
        ['AC118758.3','LINC00174'],
        ['AC119396.1','ARHGEF18'],
        ['AC119403.1','ZNF77'],
        ['AC120057.3','NPPA-AS1'],
        ['AC123767.1','ADAM5'],
        ['AC123912.3','ZNF100'],
        ['AC125257.1','KLHL11'],
        ['AC125494.1','RPL14'],
        ['AC125634.1','AL353626.3'],
        ['AC126120.1','PPIA'],
        ['AC126335.1','IGK@'],
        ['AC126544.1','HARS'],
        ['AC127502.2','LYZ'],
        ['AC127526.4','LINC00937'],
        ['AC131160.1','AL136116.3'],
        ['AC132008.2','SDHA'],
        ['AC133065.6','CIITA'],
        ['AC133919.3','AL606534.5'],
        ['AC133961.1','SEL1L3'],
        ['AC134772.1','SHISA5'],
        ['AC134878.2','CR381653.1'],
        ['AC134878.2','TEKT4'],
        ['AC135983.1','CHRFAM7A'],
        ['AC136475.10','SRP19'],
        ['AC136628.2','C5ORF60'],
        ['AC136698.1','AC243562.3'],
        ['AC136698.1','LINC00933'],
        ['AC136944.2','IGK@'],
        ['AC136944.3','CHEK2'],
        ['AC137056.1','AC141273.1'],
        ['AC138035.1','AL627309.5'],
        ['AC138207.8','NF1'],
        ['AC138356.3','ERLIN2'],
        ['AC138393.1','SEPT7'],
        ['AC138409.2','AP000347.1'],
        ['AC138409.2','C1QTNF3'],
        ['AC138409.2','C1QTNF3-AMACR'],
        ['AC138409.2','WDR70'],
        ['AC138649.1','PDCD6IP'],
        ['AC138811.2','SMG1'],
        ['AC138915.4','CISD2'],
        ['AC138932.1','NPIPA8'],
        ['AC139256.1','SNX29'],
        ['AC139491.5','FAM153B'],
        ['AC140481.3','FAM95B1'],
        ['AC141586.1','KCTD5'],
        ['AC142381.1','IGH@'],
        ['AC145098.1','DOK3'],
        ['AC211486.3','CCDC146'],
        ['AC211486.3','POM121C'],
        ['AC211486.3','PRKRIP1'],
        ['AC233266.2','TMEM128'],
        ['AC233702.10','FLJ36000'],
        ['AC233702.7','FLJ36000'],
        ['AC239860.1','NBPF13P'],
        ['AC241584.1','SSB'],
        ['AC242376.2','HERC2'],
        ['AC243562.1','GOLGA2P3Y'],
        ['AC243562.3','DNM1'],
        ['AC243829.4','CCL4'],
        ['AC244205.1','IGKV4-1'],
        ['AC244226.1','IGH@'],
        ['AC244517.10','PCDHB9'],
        ['AC244517.2','MALL'],
        ['AC245100.4','LINC00869'],
        ['AC245128.3','AL357055.3'],
        ['AC245128.3','ORC4'],
        ['AC245369.1','IGH@'],
        ['AC245369.3','IGH@'],
        ['AC245369.3','IGHA2'],
        ['AC245369.3','IGHG1'],
        ['AC245369.7','IGH@'],
        ['AC245884.11','LILRA6'],
        ['AC245884.11','LILRB3'],
        ['AC245884.4','LYZ'],
        ['AC246787.2','FAM30A'],
        ['AC247036.4','IGH@'],
        ['AC247036.6','IGH@'],
        ['AC253536.1','GSTTP1'],
        ['ACADSB','MCFD2'],
        ['ACAT1','CUL5'],
        ['ACBD3','ZMYM4'],
        ['ACCS','EXT2'],
        ['ACE2','PIR'],
        ['ACER3','POP1'],
        ['ACSL1','CENPU'],
        ['ACSS1','APMAP'],
        ['ACTB','CCNG2'],
        ['ACTB','CLPS'],
        ['ACTB','CNN2'],
        ['ACTB','CXCR4'],
        ['ACTB','FAM46A'],
        ['ACTB','HMGB2'],
        ['ACTB','HNRNPK'],
        ['ACTB','NUCB1'],
        ['ACTB','PRH1'],
        ['ACTB','YWHAZ'],
        ['ACTG1','TINAGL1'],
        ['ACTN4','EIF3K'],
        ['ACTR10','TG'],
        ['ACVR1B','GRASP'],
        ['ACVR2B','EXOG'],
        ['ADAM23','FAM237A'],
        ['ADAMTS6','CENPK'],
        ['ADAMTS7','AL049757.1'],
        ['ADAP1','MAFK'],
        ['ADAP1','SUN1'],
        ['ADGRF5','AL512625.3'],
        ['ADGRG1','CLPS'],
        ['ADGRG7','TFG'],
        ['ADK','HIBCH'],
        ['ADM','RIN3'],
        ['ADRM1','SS18L1'],
        ['AF001548.2','KRT13'],
        ['AF064860.2','IGSF5'],
        ['AF165147.1','FXR1'],
        ['AFAP1','ANXA7'],
        ['AFDN-AS1','LINC01558'],
        ['AFF1','PTPN13'],
        ['AFP','ALB'],
        ['AFTPH','LINC01805'],
        ['AGAP3','CUX1'],
        ['AGAP4','LINC00843'],
        ['AGAP5','BMS1P4'],
        ['AGGF1','ZBED3-AS1'],
        ['AGMAT','AL121992.1'],
        ['AIM2','CADM3-AS1'],
        ['AKAP13','CD2AP'],
        ['AKAP8L','SMR3B'],
        ['AKNA','AL138895.1'],
        ['AKT2','C19ORF47'],
        ['AL008628.1','FAM120B'],
        ['AL021707.2','FAM227A'],
        ['AL021920.2','EIF1AX'],
        ['AL022322.2','POC1B-GALNT4'],
        ['AL022324.2','PHF10'],
        ['AL024509.3','CMAHP'],
        ['AL031282.2','MALL'],
        ['AL031282.2','SLC35E2B'],
        ['AL031289.1','RIMS3'],
        ['AL031587.6','AL627309.5'],
        ['AL031587.6','LYZ'],
        ['AL031587.6','MICALL1'],
        ['AL034369.1','PNLIPRP2'],
        ['AL035420.1','EPB41L1'],
        ['AL035425.3','IRS4'],
        ['AL035587.1','GNMT'],
        ['AL035701.1','ENPP5'],
        ['AL049795.1','TMEM234'],
        ['AL049830.3','STRN3'],
        ['AL049836.2','LINC02314'],
        ['AL049869.3','ZBTB25'],
        ['AL050303.3','LINC01087'],
        ['AL050303.3','LINC01297'],
        ['AL050320.1','ISM1-AS1'],
        ['AL079341.1','TFG'],
        ['AL080317.1','MFSD4B'],
        ['AL096870.2','AL136295.7'],
        ['AL109804.1','MAVS'],
        ['AL109809.1','AL117335.1'],
        ['AL109809.1','AL121760.1'],
        ['AL109811.3','TARDBP'],
        ['AL109954.2','CSTL1'],
        ['AL117350.1','RAB4A'],
        ['AL121578.3','SYTL5'],
        ['AL121656.1','LTBP1'],
        ['AL121790.1','TTC6'],
        ['AL121987.2','DCAF8'],
        ['AL132639.4','TRAPPC6B'],
        ['AL132656.2','NUTM2A-AS1'],
        ['AL132780.3','PRMT5'],
        ['AL133338.2','MAPK1IP1L'],
        ['AL133353.1','SLC25A28'],
        ['AL133371.1','AL355922.5'],
        ['AL133410.3','TMEM8B'],
        ['AL133481.2','AL359195.2'],
        ['AL136133.1','RPL22'],
        ['AL136309.4','C6ORF201'],
        ['AL136520.1','SRSF10'],
        ['AL136982.1','BMS1'],
        ['AL136982.6','BMS1'],
        ['AL137003.1','ATXN1'],
        ['AL137145.2','PFKFB3'],
        ['AL137786.1','PAPOLA'],
        ['AL137796.1','C1ORF220'],
        ['AL137856.1','AP4B1-AS1'],
        ['AL138963.3','APOH'],
        ['AL138963.3','HP'],
        ['AL138963.3','PNLIP'],
        ['AL138963.3','PRB1'],
        ['AL138963.3','PRB3'],
        ['AL138963.3','PRSS1'],
        ['AL138963.3','SMR3B'],
        ['AL138963.3','TRB@'],
        ['AL139022.1','FNTB'],
        ['AL139142.1','BX248409.2'],
        ['AL139220.2','KLF17'],
        ['AL139300.2','APOPT1'],
        ['AL139353.3','CHST11'],
        ['AL139383.1','LINC02344'],
        ['AL139383.1','STARD13'],
        ['AL139384.2','TUBGCP3'],
        ['AL139412.1','C1ORF61'],
        ['AL139819.1','PKD2L1'],
        ['AL157714.2','RXRG'],
        ['AL157834.1','CYP2C8'],
        ['AL158066.1','MRPS31P4'],
        ['AL158154.2','UPP1'],
        ['AL158212.1','VTI1A'],
        ['AL160291.1','ARMC4'],
        ['AL160408.2','HBB'],
        ['AL162258.2','DES'],
        ['AL162414.1','CLPB'],
        ['AL353148.1','LYZ'],
        ['AL353608.3','PIP5K1B'],
        ['AL353743.1','KIF27'],
        ['AL354710.2','GAPVD1'],
        ['AL354726.1','HEMGN'],
        ['AL354751.1','PTPRVP'],
        ['AL354760.1','ST7L'],
        ['AL354892.2','TCTE3'],
        ['AL354956.1','TCAM1P'],
        ['AL355376.1','LINC01517'],
        ['AL356235.1','ZC3H12B'],
        ['AL356489.2','TRB@'],
        ['AL356585.4','ANKRD36B'],
        ['AL356867.1','SPRR3'],
        ['AL358790.1','JPX'],
        ['AL359075.1','SEC16B'],
        ['AL359715.2','BCKDHB'],
        ['AL360181.3','SCART1'],
        ['AL360270.1','CD53'],
        ['AL360295.1','C1ORF168'],
        ['AL365203.3','ITGB1'],
        ['AL365277.1','PPIE'],
        ['AL390036.1','VAV3-AS1'],
        ['AL390195.1','OVGP1'],
        ['AL390726.6','ANKRD18B'],
        ['AL390726.6','CYP4F26P'],
        ['AL390728.4','RPGRIP1L'],
        ['AL390860.1','TRIM58'],
        ['AL390961.2','FAM238B'],
        ['AL390961.2','FAM238C'],
        ['AL392046.1','CUL2'],
        ['AL392172.1','DISP1'],
        ['AL445238.1','OR7E156P'],
        ['AL445472.1','KLF8'],
        ['AL445623.2','ELAVL2'],
        ['AL450124.1','FAM182B'],
        ['AL450322.2','MYH11'],
        ['AL451085.2','CKS1B'],
        ['AL512310.12','NF1'],
        ['AL512625.3','GLIDR'],
        ['AL512662.2','BMS1'],
        ['AL513314.2','AL627309.5'],
        ['AL513314.2','LINC00174'],
        ['AL513478.2','PARG'],
        ['AL583722.1','INF2'],
        ['AL589935.1','LINC00472'],
        ['AL590705.5','CCDC180'],
        ['AL590787.1','MTMR6'],
        ['AL591242.1','PLA2G7'],
        ['AL591684.2','BMS1'],
        ['AL591806.3','CCND3'],
        ['AL606491.1','LDLRAP1'],
        ['AL645929.1','AL671277.1'],
        ['AL645937.4','AL672167.2'],
        ['AL662899.1','LY6G5C'],
        ['AL669831.4','LINC01881'],
        ['AL671277.1','AL671883.3'],
        ['AL671710.1','ZBED4'],
        ['AL807752.7','COL1A1'],
        ['AL845552.2','WDR25'],
        ['AL928768.1','IGHA1'],
        ['ALB','ORM2'],
        ['ALB','PRDX1'],
        ['ALDH2','ANKHD1'],
        ['ALDOA','PPP4C'],
        ['ALG14','CNN3'],
        ['ALG1L2','ALG1L6P'],
        ['ALKBH5','LLGL1'],
        ['ALMS1','SMOC2'],
        ['ALOX12-AS1','MIR497HG'],
        ['AMD1','EEF1A1'],
        ['AMMECR1L','NUTM2B-AS1'],
        ['AMY2A','CPA2'],
        ['AMY2B','PRSS3'],
        ['ANAPC1','RMND5A'],
        ['ANK2','CAMK2D'],
        ['ANKAR','ASNSD1'],
        ['ANKAR','ZDHHC7'],
        ['ANKEF1','TACC2'],
        ['ANKRD18CP','BX664727.3'],
        ['ANKRD20A11P','AP005212.5'],
        ['ANKRD20A17P','CR381670.1'],
        ['ANO10','NEK7'],
        ['ANP32B','IGH@'],
        ['ANP32B','TMOD1'],
        ['ANP32B','TNNT2'],
        ['ANXA1','MAGT1'],
        ['ANXA1','TAGLN'],
        ['ANXA1','ZNF207'],
        ['ANXA2','CELA2A'],
        ['ANXA2','CRYBG1'],
        ['ANXA2','RNF111'],
        ['ANXA5','CLEC4E'],
        ['AP000317.1','MRPS6'],
        ['AP000317.1','SLC5A3'],
        ['AP000344.1','LINC01659'],
        ['AP000346.1','AP000347.1'],
        ['AP000346.1','AP000347.2'],
        ['AP000346.1','IGLL3P'],
        ['AP000346.2','AP000347.1'],
        ['AP000346.2','AP000347.2'],
        ['AP000356.2','POM121L7P'],
        ['AP000356.2','USP18'],
        ['AP000356.3','BCR'],
        ['AP000473.1','MIR99AHG'],
        ['AP000781.2','IGF2'],
        ['AP000781.2','TG'],
        ['AP000880.1','NCAM1-AS1'],
        ['AP000924.1','FDX1'],
        ['AP000936.1','PCSK7'],
        ['AP000943.3','MRE11'],
        ['AP000943.4','FUT4'],
        ['AP001005.4','TUBB8'],
        ['AP001107.1','EMP2'],
        ['AP001350.2','ZFP91-CNTF'],
        ['AP001931.1','TMX2'],
        ['AP001979.2','SPATA19'],
        ['AP002008.1','BTG4'],
        ['AP002748.1','CTSF'],
        ['AP002884.4','IL18'],
        ['AP003066.1','AP003781.1'],
        ['AP003086.1','GAB2'],
        ['AP003354.2','ATP6V1C1'],
        ['AP003400.1','NOX4'],
        ['AP005329.1','MYOM1'],
        ['AP005435.2','EMB'],
        ['AP005671.1','EPB41L3'],
        ['AP3S1','COMMD10'],
        ['AP3S2','YPEL5'],
        ['APBB1IP','FAM238A'],
        ['APBB2','NR3C2'],
        ['APBB3','SRA1'],
        ['APOL3','MIR100HG'],
        ['APP','HTN3'],
        ['ARHGAP11B','OTUD7A'],
        ['ARHGAP18','DLGAP5'],
        ['ARHGAP18','TMEM244'],
        ['ARHGAP26','NR3C1'],
        ['ARHGAP27P1-BPTFP1-KPNA2P3','KPNA2'],
        ['ARHGEF17','P2RY6'],
        ['ARHGEF5','OR2A1-AS1'],
        ['ARHGEF9','LINC01278'],
        ['ARID4B','B2M'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARL6IP1','RPS15A'],
        ['ARL6IP6','TRNT1'],
        ['ARMC6','SLC25A42'],
        ['ARPC4','OGG1'],
        ['ARRDC3','BCLAF1'],
        ['ART4','SMCO3'],
        ['AS3MT','CCND3'],
        ['ASAP3','TCEA3'],
        ['ASL','CRCP'],
        ['ASXL1','NOL4L'],
        ['ATAD1','PTEN'],
        ['ATG16L1','INPP5D'],
        ['ATP10D','USP32'],
        ['ATP13A3','UBR3'],
        ['ATP1A1-AS1','LINC01762'],
        ['ATP5C1','ITIH2'],
        ['ATP5H','TG'],
        ['ATP5I','MFSD7'],
        ['ATP6V0B','DPH2'],
        ['ATP6V0C','EEF1A1'],
        ['ATP7B','LINC00282'],
        ['ATP8A2','NUP58'],
        ['AZGP1','GJC3'],
        ['AZIN1','KLF10'],
        ['B2M','FBN1'],
        ['B2M','FUS'],
        ['B2M','G3BP2'],
        ['B2M','HSPA9'],
        ['B2M','IGH@'],
        ['B2M','MED13L'],
        ['B2M','PFKFB3'],
        ['B2M','RUNX1'],
        ['B2M','SF1'],
        ['B2M','SUB1'],
        ['B2M','UBC'],
        ['B2M','UBE3A'],
        ['B3GNT5','KLHL24'],
        ['B3GNT5','TIPARP'],
        ['B3GNTL1','METRNL'],
        ['BABAM2','RBKS'],
        ['BANK1','IGH@'],
        ['BASP1','DSE'],
        ['BCAP29','DUS4L'],
        ['BCAS3','TG'],
        ['BCKDHA','TMEM91'],
        ['BCL2L11','CXCR4'],
        ['BCL6','NAMPT'],
        ['BCL7A','RBM38'],
        ['BEST1','HBB'],
        ['BEST1','MYH6'],
        ['BGLAP','PMF1'],
        ['BICD2','IPPK'],
        ['BLOC1S6','SQOR'],
        ['BMI1','DDX5'],
        ['BMPR1B','PDLIM5'],
        ['BPI','CFL1'],
        ['BPIFC','RTCB'],
        ['BPTF','LRRC37A2'],
        ['BRD1','C22ORF34'],
        ['BRD2','SRSF5'],
        ['BRD3','TG'],
        ['BRD4','TG'],
        ['BRK1','VHL'],
        ['BSDC1','FAM229A'],
        ['BSG','ELANE'],
        ['BSG','OAZ1'],
        ['BSG','R3HDM4'],
        ['BSG','SBNO2'],
        ['BSG','TCF3'],
        ['BSG','UBB'],
        ['BX072579.2','LINC00452'],
        ['BX276092.10','BX276092.7'],
        ['BX284632.1','IGK@'],
        ['BX547991.1','LINC00864'],
        ['C10ORF90','LINC00601'],
        ['C11ORF57','DLAT'],
        ['C11ORF70','YAP1'],
        ['C12ORF73','HSP90B1'],
        ['C14ORF166','GNG2'],
        ['C15ORF48','SPATA5L1'],
        ['C15ORF57','CBX3'],
        ['C16ORF87','COL14A1'],
        ['C18ORF8','CAPZA2'],
        ['C19MC','RPLP1'],
        ['C19ORF33','SPINT2'],
        ['C1ORF146','KIAA1107'],
        ['C1ORF194','UQCR10'],
        ['C1ORF43','MB'],
        ['C1ORF61','IFNAR2'],
        ['C20ORF202','PSMF1'],
        ['C2CD3','UCP3'],
        ['C2ORF48','RRM2'],
        ['C2ORF72','SPATA3'],
        ['C2ORF81','HMGA1'],
        ['C2ORF81','RTKN'],
        ['C3ORF67','FAM3D'],
        ['C4ORF51','CALB1'],
        ['C6ORF106','SPDEF'],
        ['C9ORF24','KIF24'],
        ['CA6','JCHAIN'],
        ['CA6','PRB3'],
        ['CA6','PRB4'],
        ['CABIN1','CBFA2T2'],
        ['CABP7','NF2'],
        ['CACNA1C','LINC02371'],
        ['CACNG1','CACNG4'],
        ['CACUL1','SND1'],
        ['CALCRL','NBEAL1'],
        ['CALM2','PIGF'],
        ['CALM2','STATH'],
        ['CALM3','MYL4'],
        ['CALU','FAM71F1'],
        ['CASTOR2','FURIN'],
        ['CATSPER2','PPIP5K1'],
        ['CAVIN1','STAT3'],
        ['CBWD2','DOCK8'],
        ['CCDC117','Z93930.2'],
        ['CCDC175','L3HYPDH'],
        ['CCDC28A','ECT2L'],
        ['CCDC82','JRKL-AS1'],
        ['CCM2L','HCK'],
        ['CCNG2','OAZ1'],
        ['CCNG2','VAV1'],
        ['CD302','TRIP12'],
        ['CD63','TMBIM6'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CD8B','CD8B2'],
        ['CDC34','GZMM'],
        ['CDC34','PTMA'],
        ['CDC42','SLC25A37'],
        ['CDC42','ZEB2'],
        ['CDC42EP2','POLA2'],
        ['CDC42EP4','CPSF4L'],
        ['CDHR4','IP6K1'],
        ['CDK10','SPATA33'],
        ['CDK11B','SLC35E2'],
        ['CDK19','REV3L'],
        ['CDK2','RAB5B'],
        ['CDK5RAP3','PNPO'],
        ['CDKN1A','RAB44'],
        ['CDR1','LINC00632'],
        ['CDRT15L2','TBC1D26'],
        ['CEL','CELP'],
        ['CELA2B','PDIA2'],
        ['CELSR3','NCKIPSD'],
        ['CEP152','FBN1'],
        ['CEP192','SEH1L'],
        ['CFAP157','STXBP1'],
        ['CFL1','MKNK2'],
        ['CFL1','SF1'],
        ['CHST11','IGH@'],
        ['CHSY3','ROCK2'],
        ['CITED1','HDAC8'],
        ['CLCF1','POLD4'],
        ['CLEC2D','USP24'],
        ['CLIC2','F8'],
        ['CLK1','CXCR4'],
        ['CLPS','CTRB2'],
        ['CLTC','VMP1'],
        ['CMTM7','CMTM8'],
        ['CNBD2','PHF20'],
        ['CNN2','FURIN'],
        ['CNN2','RNF126'],
        ['CNN2','SBNO2'],
        ['CNTROB','PET100'],
        ['COL14A1','DEPTOR'],
        ['COL16A1','DES'],
        ['COL18A1','COL3A1'],
        ['COL4A1','SLC29A1'],
        ['COLQ','HACL1'],
        ['COMMD10','FBXL17'],
        ['COMMD2','WWTR1'],
        ['COPA','HTN3'],
        ['COQ4','SLC27A4'],
        ['COQ8B','NUMBL'],
        ['COX19','LINC01002'],
        ['COX6B1','ETV2'],
        ['COX6C','HMGB1'],
        ['COX7A2','MINDY3'],
        ['CPM','LYZ'],
        ['CPSF6','LYZ'],
        ['CRCP','TPST1'],
        ['CREB3','GRB2'],
        ['CREBRF','NSD1'],
        ['CRELD1','PRRT3-AS1'],
        ['CRLF2','LYZ'],
        ['CRYBB2','Z99916.1'],
        ['CRYBG2','ZNF683'],
        ['CST1','STATH'],
        ['CTBP1','MAEA'],
        ['CTBP1','UVSSA'],
        ['CTBP1-AS2','MAEA'],
        ['CTCF','FAT1'],
        ['CTDP1','LINC00969'],
        ['CTDP1','NFATC1'],
        ['CTDP1','PQLC1'],
        ['CTDSP1','DNM3OS'],
        ['CTH','HHLA3'],
        ['CTNNA1','MATR3'],
        ['CTNND1','MAGED1'],
        ['CTNND2','MEX3C'],
        ['CTRB2','P4HB'],
        ['CTSB','NUCB2'],
        ['CTSC','RAB38'],
        ['CTSD','LSP1'],
        ['CU633967.1','CU638689.4'],
        ['CU634019.6','CU638689.4'],
        ['CUEDC1','MALAT1'],
        ['CUX1','KLF10'],
        ['CWF19L2','TMEM41A'],
        ['CXCR4','METRNL'],
        ['CXCR4','MYO15B'],
        ['CXCR4','RARA'],
        ['CXCR4','RBM38'],
        ['CXCR4','SF1'],
        ['CXCR4','THAP4'],
        ['CXCR4','UBC'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZBTB18'],
        ['CXCR4','ZEB2'],
        ['CYBA','WWOX'],
        ['CYBB','XK'],
        ['CYTIP','ERMN'],
        ['DAND5','NFIX'],
        ['DAZAP1','GLUL'],
        ['DCN','UBE2N'],
        ['DCTN5','UBFD1'],
        ['DCUN1D1','MCCC1'],
        ['DDX27','PARP15'],
        ['DDX39A','JMJD1C'],
        ['DDX3X','EEF1A1'],
        ['DDX3X','UBC'],
        ['DDX42','MAP3K3'],
        ['DDX5','EIF4A1'],
        ['DDX5','FUS'],
        ['DDX5','GRK2'],
        ['DDX5','HMGN2'],
        ['DDX5','HNRNPU'],
        ['DDX5','LIMS2'],
        ['DDX5','METRNL'],
        ['DDX5','NFE2L1'],
        ['DDX5','OAZ1'],
        ['DDX5','TEX2'],
        ['DDX5','TPD52L2'],
        ['DDX5','UGCG'],
        ['DEPDC1B','PDE4D'],
        ['DES','KRT7'],
        ['DGCR8','TANGO2'],
        ['DHCR24','KRT4'],
        ['DHRS4-AS1','NRL'],
        ['DHRS7C','GSG1L2'],
        ['DHX35','FAM83D'],
        ['DHX37','UBC'],
        ['DHX9','NPL'],
        ['DIP2A','LYZ'],
        ['DIP2B','TMBIM6'],
        ['DLD','HNRNPK'],
        ['DLEU2','HMGB1'],
        ['DLEU2','SPRYD7'],
        ['DLEU7-AS1','RNASEH2B'],
        ['DNAAF3','SYT5'],
        ['DNAJA1','POLR2A'],
        ['DNAJB6','LYZ'],
        ['DNAJC7','LYZ'],
        ['DOC2B','LINC02091'],
        ['DOCK1','FAM24B'],
        ['DOCK9','STK24'],
        ['DOT1L','PTBP1'],
        ['DOT1L','ZBTB7A'],
        ['DPH7','PNPLA7'],
        ['DPP9','TG'],
        ['DSC3','TMEM269'],
        ['DTD1','MKL2'],
        ['E2F1','PXMP4'],
        ['EAF2','SLC15A2'],
        ['EEF1A1','HBA1'],
        ['EEF1A1','MUC7'],
        ['EEF1A1','PTMA'],
        ['EEF1A1','TG'],
        ['EEF1A1','TPM3'],
        ['EEF1A2','MYH7'],
        ['EEF1AKMT3','TSFM'],
        ['EEF2','IGK@'],
        ['EFCAB2','KIF26B'],
        ['EFCAB7','PGM1'],
        ['EGF','SEC24B'],
        ['EGFR','H19'],
        ['EIF1AX','MAP7D2'],
        ['EIF1AX','RPS6KA3'],
        ['EIF2AK1','S100A9'],
        ['EIF2S1','RBM25'],
        ['EIF3F','PNLIP'],
        ['EIF4E3','FOXP1'],
        ['EIF4E3','PROK2'],
        ['EIF4G2','PTMA'],
        ['ELANE','HBD'],
        ['ELANE','LINC01578'],
        ['ELANE','SF1'],
        ['ELK4','SLC45A3'],
        ['EMC2','NUDCD1'],
        ['EMG1','MIR200CHG'],
        ['EMP2','MIR100HG'],
        ['ENO2','MYBPC3'],
        ['ENTPD5','FAM161B'],
        ['EPB41L3','ZBTB14'],
        ['EPB42','LYZ'],
        ['EPN2','UBB'],
        ['ERBB2','GRB7'],
        ['ERBB3','LYZ'],
        ['ERP44','UBQLN1'],
        ['ERVK-28','ZNF585A'],
        ['F5','SELP'],
        ['FADS1','TMEM258'],
        ['FAM110A','JAZF1'],
        ['FAM129C','PGLS'],
        ['FAM133B','FAM133CP'],
        ['FAM135A','MAP3K4'],
        ['FAM135A','SDHAF4'],
        ['FAM13A-AS1','HERC3'],
        ['FAM149B1','PTGR1'],
        ['FAM157B','FBXO25'],
        ['FAM160B1','TRUB1'],
        ['FAM174A','ST8SIA4'],
        ['FAM53A','XRRA1'],
        ['FAM53B','MTHFS'],
        ['FAM53B','TSPAN3'],
        ['FAM66A','FAM66C'],
        ['FAM85B','LINC02018'],
        ['FAM86B1','FAM86B3P'],
        ['FBXL5','HMGB1'],
        ['FBXO16','HBB'],
        ['FBXO25','TDRP'],
        ['FCAR','LENG8'],
        ['FES','MAN2A2'],
        ['FGD5','TPRXL'],
        ['FGFR4','ZNF346'],
        ['FITM2','OSER1'],
        ['FKBP5','PNLIP'],
        ['FLJ22447','PRKCH'],
        ['FLNB','PRB3'],
        ['FNIP1','PDE4B'],
        ['FOLR3','LRTOMT'],
        ['FOXK2','SERPINE2'],
        ['FOXO1','NUP98'],
        ['FOXP1','RYBP'],
        ['FP700111.2','IGK@'],
        ['FRG1','FRG1HP'],
        ['FRMD6-AS2','LINC02310'],
        ['FUS','KCTD5'],
        ['G2E3','SCFD1'],
        ['G3BP2','PTP4A1'],
        ['GABPB1','PTP4A1'],
        ['GALK2','SUPT16H'],
        ['GAS7','RCVRN'],
        ['GATC','KDM2A'],
        ['GBP3','KYAT3'],
        ['GCA','SAMSN1'],
        ['GCA','TG'],
        ['GCC1','TG'],
        ['GGT1','IGSF3'],
        ['GLE1','ODF2'],
        ['GLI3','GRB10'],
        ['GLTP','U52111.1'],
        ['GLUL','KCNAB2'],
        ['GNAS','SENP3'],
        ['GNE','SMR3B'],
        ['GNG7','TPM1'],
        ['GOLGA2','SMR3B'],
        ['GOLT1A','KISS1'],
        ['GPR156','LRRC58'],
        ['GPRC5A','OGFRL1'],
        ['GPT','PPP1R16A'],
        ['GPX3','KIAA0319L'],
        ['GRAMD1A','TG'],
        ['GRSF1','RSRP1'],
        ['GS1-124K5.4','TPST1'],
        ['GSN','MMP25'],
        ['GSN','S100A9'],
        ['GUCA1C','MORC1'],
        ['GULP1','SCN9A'],
        ['GYPB','SAV1'],
        ['H1F0','PRB2'],
        ['H3F3B','ZG16B'],
        ['H6PD','SPSB1'],
        ['HACD1','PLS3'],
        ['HAPLN3','MFGE8'],
        ['HAS3','UTP4'],
        ['HBA1','RPS27'],
        ['HBA2','TUBB4B'],
        ['HBB','MOB3A'],
        ['HBB','NLRC5'],
        ['HBB','SEPT2'],
        ['HBB','VTI1A'],
        ['HBD','LTF'],
        ['HBP1','IMMP2L'],
        ['HDLBP','KAT6A'],
        ['HIF1A','PTP4A1'],
        ['HIF1A','RNPC3'],
        ['HIF1A','STAG2'],
        ['HIPK1','OLFML3'],
        ['HKR1','TG'],
        ['HMBS','SBNO2'],
        ['HMGB1','HNRNPH1'],
        ['HMGB1','MXD4'],
        ['HMGB1','SRSF5'],
        ['HMGB2','IVNS1ABP'],
        ['HMGN2','NECAB3'],
        ['HNRNPA2B1','UBB'],
        ['HNRNPC','PTP4A1'],
        ['HNRNPD','RCC1'],
        ['HNRNPDL','TUBA1B'],
        ['HNRNPH1','ITPRIP'],
        ['HNRNPH1','TG'],
        ['HNRNPK','NFIL3'],
        ['HNRNPK','UBQLN1'],
        ['HNRNPUL1','MAEA'],
        ['HOOK3','PPM1A'],
        ['HORMAD1','SLC22A20'],
        ['HSD17B14','PLEKHA4'],
        ['HSD17B4','TNFAIP8'],
        ['HSP90AB1','SLC29A1'],
        ['HSPA9','UBE2D2'],
        ['HSPB11','YIPF1'],
        ['HSPE1','MOB4'],
        ['HTN1','MYH11'],
        ['HTN1','PRR27'],
        ['HTN1','SH3BGRL2'],
        ['HTN3','MAPK1'],
        ['HTN3','PRR27'],
        ['HTN3','PTK7'],
        ['IFT122','RYK'],
        ['IGH@','LAPTM5'],
        ['IGH@','MIR100HG'],
        ['IGH@','MUC1'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','PPP6R2'],
        ['IGH@','TRA@'],
        ['IGH@','ZFP36L1'],
        ['IGLC1','SMR3B'],
        ['IGSF10','MED12L'],
        ['IL9R','RPL23AP82'],
        ['IMP3','SNUPN'],
        ['INPP4B','PLCZ1'],
        ['INPP5A','PWWP2B'],
        ['IQCG','POLG2'],
        ['IQGAP1','ZNF774'],
        ['ITGA7','PPP1R1A'],
        ['ITGAV','PXK'],
        ['ITGB2','POFUT2'],
        ['ITGB6','PPP4R1'],
        ['ITM2B','RB1'],
        ['ITPKC','SNRPA'],
        ['IVD','KNSTRN'],
        ['JARID2','LINC01578'],
        ['JARID2','MGAT4A'],
        ['JAZF1','ZNF777'],
        ['JKAMP','PCNX4'],
        ['KANSL1','LINC02210'],
        ['KANSL1','LRRC37A3'],
        ['KANSL3','RIPK4'],
        ['KANSL3','TMCO4'],
        ['KAT2B','SGO1-AS1'],
        ['KCNK7','MAP3K11'],
        ['KDM2A','SF1'],
        ['KDM4B','SAFB2'],
        ['KDM7A','LAPTM5'],
        ['KHDRBS1','KIAA0319L'],
        ['KIAA1024','TMED3'],
        ['KIAA1147','WEE2-AS1'],
        ['KIF15','ZNF501'],
        ['KIF1C','KRT13'],
        ['KIFC1','RNH1'],
        ['KLF2','OAZ1'],
        ['KLHL36','ZDHHC7'],
        ['KLK4','KLKP1'],
        ['KMT2C','TPTE'],
        ['KMT2E','SRSF7'],
        ['KMT5A','RFLNA'],
        ['KRT13','SCP2'],
        ['KRT4','LY6D'],
        ['KRT8','VIM'],
        ['LACTB2-AS1','XKR9'],
        ['LAMA3','RBBP8'],
        ['LAMTOR1','NUMA1'],
        ['LAPTM5','RIPOR1'],
        ['LARP1','NR3C1'],
        ['LATS1','NUP43'],
        ['LDAH','PUM2'],
        ['LGMN','MYH11'],
        ['LINC00467','TRAF5'],
        ['LINC00535','UBE2V2'],
        ['LINC00886','TIPARP-AS1'],
        ['LINC01068','NDFIP2'],
        ['LINC01135','LINC01358'],
        ['LINC01169','SMAD6'],
        ['LINC01372','MEF2C'],
        ['LINC01470','NEDD1'],
        ['LINC01578','YPEL5'],
        ['LINC01578','ZBTB7A'],
        ['LINC01608','LINC01609'],
        ['LINC01619','LINC02391'],
        ['LINC01705','LINC02257'],
        ['LINC01872','SIGLECL1'],
        ['LINC02219','PIK3R1'],
        ['LINC02246','NRIP1'],
        ['LINC02345','TMCO5A'],
        ['LMO1','RIC3'],
        ['LPAR2','PBX4'],
        ['LPGAT1','RPS6KC1'],
        ['LRP12','PKP4'],
        ['LRP2BP','UFSP2'],
        ['LRP5','LRP5L'],
        ['LRPAP1','WDR1'],
        ['LRRC37A16P','LRRC37B'],
        ['LRRC37A3','NSF'],
        ['LRRC57','PLEKHA1'],
        ['LRRC59','MRPL27'],
        ['LY6K','THEM6'],
        ['LYST','SETX'],
        ['LYZ','MARF1'],
        ['LYZ','MORF4L1'],
        ['LYZ','PIGQ'],
        ['LYZ','SYNE3'],
        ['LYZ','WAC-AS1'],
        ['LYZ','Z99129.3'],
        ['MAD1L1','MAFK'],
        ['MAML2','MTMR2'],
        ['MAN2A1','UIMC1'],
        ['MAN2C1','PLA2G4F'],
        ['MANBA','UBE2D3'],
        ['MANBAL','SRC'],
        ['MAP3K8','PRKG1'],
        ['MAPKBP1','MGA'],
        ['MARCKS','PTMA'],
        ['MAST2','TNNT2'],
        ['MECOM','RPL22'],
        ['MEF2C','TMEM161B'],
        ['MEIS2','MUC7'],
        ['METRNL','RALGDS'],
        ['MGRN1','NUDT16L1'],
        ['MICAL2','TEAD1'],
        ['MINOS1','MMD'],
        ['MINPP1','NUTM2A-AS1'],
        ['MIR100HG','PIGR'],
        ['MIR100HG','SSR3'],
        ['MIR22HG','TLCD2'],
        ['MLEC','UNC119B'],
        ['MLIP','TINAG'],
        ['MOB1B','VLDLR-AS1'],
        ['MPO','TBC1D9B'],
        ['MRC1','TMEM236'],
        ['MSMB','NCOA4'],
        ['MSRB3','TMBIM4'],
        ['MTA3','THADA'],
        ['MTHFD1L','PLEKHG1'],
        ['MTHFSD','STIM2'],
        ['MTMR10','TRPM1'],
        ['MTSS1','TATDN1'],
        ['MTSS1L','VAC14'],
        ['MTURN','PLEKHA8'],
        ['MUC2','MUC5B'],
        ['MYH11','SCARB2'],
        ['MYH6','TIMP3'],
        ['MYH6','TPT1'],
        ['MYL3','TNNC1'],
        ['MYO10','TSPAN18'],
        ['MYO5C','NDUFS6'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NANP','NINL'],
        ['NCKAP1','SMR3B'],
        ['NCKAP5','R3HDM1'],
        ['NCOA6','PIGU'],
        ['NCOR2','UBC'],
        ['NDEL1','UBB'],
        ['NDUFV3','PKNOX1'],
        ['NECAP2','SZRD1'],
        ['NFATC1','PQLC1'],
        ['NFATC3','PLA2G15'],
        ['NIN','SAV1'],
        ['NMU','PDCL2'],
        ['NOXRED1','TMED8'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB15','PDXDC2P-NPIPB14P'],
        ['NPIPB5','SMG1'],
        ['NPPA','SORBS2'],
        ['NQO2','PABPC1L'],
        ['NRAV','SRSF9'],
        ['NRBP1','TG'],
        ['NSMAF','SPTLC3'],
        ['NUCB1-AS1','TULP2'],
        ['NUP188','PHYHD1'],
        ['NUP210','STOM'],
        ['NUSAP1','OIP5-AS1'],
        ['OAZ1','PFKFB3'],
        ['OAZ1','PTBP1'],
        ['OAZ1','RAC2'],
        ['OAZ1','SEPT5'],
        ['OAZ1','TCF3'],
        ['OAZ1','UBTF'],
        ['OSBPL8','UBAC2'],
        ['PACSIN2','TOB1'],
        ['PAN2','STAT2'],
        ['PATJ','TG'],
        ['PBXIP1','PMVK'],
        ['PCM1','SPIDR'],
        ['PCP4L1','SDHC'],
        ['PDZRN3','STATH'],
        ['PEMT','UBB'],
        ['PFDN1','SKP1'],
        ['PFKFB3','S100A9'],
        ['PIAS2','ST8SIA5'],
        ['PICALM','SLC38A1'],
        ['PID1','SFTPA1'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PINK1-AS','TAS2R4'],
        ['PITPNA','YWHAE'],
        ['PITPNC1','PRKCA'],
        ['PKHD1L1','STX8'],
        ['PLA2G15','WBP1L'],
        ['PLCG1','TG'],
        ['PLEKHB2','ZNF727'],
        ['PLSCR4','RNF13'],
        ['PMS2','POLR2J3'],
        ['PNLIP','STAG3L5P'],
        ['PNLIP','UBA52'],
        ['POLR2A','SMR3B'],
        ['PPM1B','SLC3A1'],
        ['PPM1H','USP15'],
        ['PPP1CB','WSB1'],
        ['PPP1CB','YPEL5'],
        ['PPP1R12C','TSPAN4'],
        ['PRC1','VPS33B'],
        ['PRDX1','TESK2'],
        ['PRH2','SASH1'],
        ['PRH2','TACC2'],
        ['PRKAG2','PSD3'],
        ['PRKAG2','RHEB'],
        ['PRKCH','PRPS2'],
        ['PRKCZ','SKI'],
        ['PRKRIP1','RBAK'],
        ['PRMT7','SLC7A6'],
        ['PRPF39','TOGARAM1'],
        ['PRSS3','TMEM45B'],
        ['PRSS42','PRSS50'],
        ['PRSS51','TNKS'],
        ['PSMA5','SORT1'],
        ['PSMB8','TAP1'],
        ['PSMF1','SIPA1L1'],
        ['PSPC1','ZMYM5'],
        ['PTBP1','RNF126'],
        ['PTBP1','SBNO2'],
        ['PTBP3','SUSD1'],
        ['PTCSC2','RNF144B'],
        ['PTMA','SRSF5'],
        ['PTMA','WDR1'],
        ['PTMA','ZEB2'],
        ['PTP4A1','SRSF5'],
        ['PTPN4','SP140L'],
        ['PVRIG2P','STAG3'],
        ['PXK','RPP14'],
        ['PXYLP1','SPSB4'],
        ['R3HDM1','TRAPPC12'],
        ['RAB11B','SEPHS1'],
        ['RAB35','THEMIS2'],
        ['RARA','RCN1'],
        ['RASA3','TFDP1'],
        ['RASSF8','SSPN'],
        ['RBAK','RNF216'],
        ['RBM38','SUMO2'],
        ['RBMS2','SPRYD4'],
        ['RBMS2','TG'],
        ['RDH11','VTI1B'],
        ['RECQL5','SENP6'],
        ['RHOF','UBC'],
        ['RHOQ','TMEM247'],
        ['RIPK1','SERPINB9'],
        ['RLN1','RLN2'],
        ['RNASET2','SRGN'],
        ['RNF125','RNF138'],
        ['RNF139','TRMT12'],
        ['RNF151','TBL3'],
        ['RNF31','THADA'],
        ['RPL13','SPG7'],
        ['RPL14','SRP14'],
        ['RPL27','RUNDC1'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['RPS18','TG'],
        ['RPS19','SNUPN'],
        ['RPS9','UBB'],
        ['RWDD3','TMEM56'],
        ['SAMD5','SASH1'],
        ['SAMSN1','USP25'],
        ['SCAF4','SERGEF'],
        ['SCARB1','UBC'],
        ['SCNN1A','TNFRSF1A'],
        ['SDC2','VMP1'],
        ['SDF4','TNFRSF4'],
        ['SENP3-EIF4A1','TNFSF13'],
        ['SERF1A','SMN1'],
        ['SH3KBP1','TYROBP'],
        ['SIDT2','TAGLN'],
        ['SIRT2','XPNPEP1'],
        ['SLC19A1','SUMO3'],
        ['SLC20A2','WDR45'],
        ['SLC25A3','TOM1'],
        ['SLC25A5','TRB@'],
        ['SLC2A11','SMARCB1'],
        ['SLC33A1','UBP1'],
        ['SLC39A8','TRB@'],
        ['SLC6A16','TEAD2'],
        ['SLFN11','TG'],
        ['SMAP2','ZFP69B'],
        ['SMARCA2','TG'],
        ['SMG1','ZNF623'],
        ['SMR3B','SPINT2'],
        ['SND1','ZNF704'],
        ['SNRPB','SNURF'],
        ['SNRPN','WDR4'],
        ['SNTB2','VPS4A'],
        ['SNX29','TG'],
        ['SPATA3','ZNF483'],
        ['ST3GAL1','UBB'],
        ['STAT3','TRB@'],
        ['STK10','UBE2D2'],
        ['STK4-AS1','TOMM34'],
        ['TAMM41','VGLL4'],
        ['TBCEL','TECTA'],
        ['TDRD10','UBE2Q1-AS1'],
        ['TFDP1','TMCO3'],
        ['TFDP1','ZEB2'],
        ['TFDP2','XRN1'],
        ['TG','TIE1'],
        ['TG','ZNF12'],
        ['TMEM225B','ZNF655'],
        ['TMEM225B','ZSCAN25'],
        ['TOP1','TPRXL'],
        ['TPM1','UQCRB'],
        ['TPM1','XIRP2'],
        ['TRPM7','USP50'],
        ['TTC32','WDR35'],
        ['UBA2','WTIP'],
        ['USP10','ZDHHC7'],
        ['USP34','XPO1'],
        ['VRK2','WDPCP'],
        ['YPEL5','ZEB2'],
        ['Z93930.2','ZNRF3'],
        ['ZC2HC1A','ZFX'],
        ['ZDHHC12','ZER1'],
        ['ZFP30','ZNF571'],
        ['ZNF32-AS3','ZNF485'],
        ['ZNF616','ZNF836'],
        ['ZNF746','ZNF767P'],
        ['ZNF780A','ZNF780B'],
        ['ZNF815P','ZNF890P'],
        ['AC011603.2','MBP'],
        ['AFM','LINC02499'],
        ['ANAPC1','RMND5A'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['AZGP1','GJC3'],
        ['BCAP29','DUS4L'],
        ['BOLA2-SMG1P6','SMG1'],
        ['BRS3','HTATSF1'],
        ['C19MC','RPLP1'],
        ['CATSPER2','PPIP5K1'],
        ['COQ8B','NUMBL'],
        ['CTSB','HOOK2'],
        ['CTSB','RAP1GAP'],
        ['EEF1A1','EEF1A2'],
        ['FAM157B','FBXO25'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['MECOM','RPL22'],
        ['NAIP','OCLN'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['ABL1','EXOSC2'],
        ['ABR','CLUH'],
        ['AC005258.1','BSG'],
        ['AC005258.1','CCNG2'],
        ['AC005258.1','DDX5'],
        ['AC005258.1','SEPT5'],
        ['AC005258.1','UBTF'],
        ['AC009093.10','RRN3P2'],
        ['AC009093.9','CDR2'],
        ['AC010186.4','DCK'],
        ['AC013394.1','ELANE'],
        ['AC013394.1','ZBTB7A'],
        ['AC016876.2','DDX5'],
        ['AC023790.2','GPRC5D-AS1'],
        ['AC067930.2','HBB'],
        ['AC068580.4','LSP1'],
        ['AC079781.5','LINC00937'],
        ['AC092821.3','CLEC2D'],
        ['AC118758.2','AC138035.3'],
        ['AC244517.10','AC244517.6'],
        ['ACCS','EXT2'],
        ['ACSS1','APMAP'],
        ['ACTN4','EIF3K'],
        ['ADGRG7','TFG'],
        ['AGAP5','BMS1P4'],
        ['AKT2','C19ORF47'],
        ['AL022322.2','GALNT4'],
        ['ALG14','CNN3'],
        ['ANAPC1','RMND5A'],
        ['AP000345.2','DRICH1'],
        ['AP006222.1','DUX4'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARPC4-TTLL3','OGG1'],
        ['ATG16L1','INPP5D'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['B3GNTL1','METRNL'],
        ['BANK1','IGH@'],
        ['BCAP29','DUS4L'],
        ['BLOC1S6','SQOR'],
        ['BOLA2-SMG1P6','SMG1'],
        ['BPTF','LRRC37A2'],
        ['C12ORF73','HSP90B1'],
        ['C15ORF38-AP3S2','YPEL5'],
        ['C19MC','RPLP1'],
        ['C19ORF33','SPINT2'],
        ['C6ORF106','SPDEF'],
        ['CABP7','NF2'],
        ['CATSPER2','PPIP5K1'],
        ['CBWD2','DOCK8'],
        ['CBX3','CCDC32'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CDC42EP2','POLA2'],
        ['CDK2','RAB5B'],
        ['CDR2','RRN3P2'],
        ['CFD','ELANE'],
        ['CITED1','HDAC8'],
        ['COLQ','HACL1'],
        ['COMMD3-BMI1','DDX5'],
        ['COQ8B','NUMBL'],
        ['COX19','MAFK'],
        ['CPM','LYZ'],
        ['CRCP','TPST1'],
        ['CTDP1','PQLC1'],
        ['CTSC','RAB38'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZEB2'],
        ['CYTIP','ERMN'],
        ['DCUN1D1','MCCC1'],
        ['DDX42','MAP3K3'],
        ['DHX35','FAM83D'],
        ['DHX9','NPL'],
        ['DOCK9','STK24'],
        ['EIF4E3','FOXP1'],
        ['ENTPD5','FAM161B'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FOXP1','RYBP'],
        ['GAS7','RCVRN'],
        ['GGT1','IGSF3'],
        ['GNAQ','LYZ'],
        ['GNAS','SENP3-EIF4A1'],
        ['GOLT1A','KISS1'],
        ['HAPLN3','MFGE8'],
        ['HBB','RNF216'],
        ['HBB','TTC7A'],
        ['HBB','TTN'],
        ['HBB','USP15'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['IGH@','LAPTM5'],
        ['IGH@','MIR100HG'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','ZFP36L1'],
        ['IQGAP1','ZNF774'],
        ['KANSL1','LRRC37A3'],
        ['KLF2','OAZ1'],
        ['KLK4','KLKP1'],
        ['KMT2C','TPTE'],
        ['KMT5A','ZNF664'],
        ['LPGAT1','RPS6KC1'],
        ['LRRC37A3','NSF'],
        ['LYZ','NFATC2IP'],
        ['MANBAL','SRC'],
        ['MAPKBP1','MGA'],
        ['MICAL2','TEAD1'],
        ['MTSS1','TATDN1'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NCOA6','PIGU'],
        ['NCOR2','UBC'],
        ['NDRG1','ST3GAL1'],
        ['NFATC3','PLA2G15'],
        ['NFKBIZ','SRGN'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PMS2','POLR2J3'],
        ['PPP1CB','YPEL5'],
        ['PRDX1','TESK2'],
        ['PRKAG2','RHEB'],
        ['PXK','RPP14'],
        ['RAP1B','RBPJ'],
        ['RBMS2','SPRYD4'],
        ['RPL14','SRP14'],
        ['RPL38','TTYH2'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['STK10','UBE2D2'],
        ['TBCEL','TECTA'],
        ['TFDP2','XRN1'],
        ['UBA2','WTIP'],
        ['USP34','XPO1'],
        ['AATF','AC010542.3'],
        ['ABL1','EXOSC2'],
        ['ABR','CLUH'],
        ['ABR','PRB2'],
        ['AC002398.2','TNNT2'],
        ['AC005041.1','RTKN'],
        ['AC005258.1','BSG'],
        ['AC005258.1','CCNG2'],
        ['AC005258.1','DDX5'],
        ['AC005258.1','SEPT5'],
        ['AC005258.1','UBTF'],
        ['AC007000.3','AC211486.3'],
        ['AC008770.1','ZNF625'],
        ['AC008906.1','CAST'],
        ['AC009093.10','RRN3P2'],
        ['AC009093.9','CDR2'],
        ['AC010186.4','DCK'],
        ['AC013394.1','ELANE'],
        ['AC013394.1','ZBTB7A'],
        ['AC015468.1','LINC02153'],
        ['AC015688.4','AC098850.5'],
        ['AC015688.4','NOS2'],
        ['AC016876.2','DDX5'],
        ['AC016912.1','RHOQ'],
        ['AC018630.6','CST2'],
        ['AC018630.6','DGCR2'],
        ['AC018630.6','KIAA1324'],
        ['AC018630.6','TRPV6'],
        ['AC018638.5','AL606534.5'],
        ['AC019133.1','LINC02494'],
        ['AC019349.1','KRT6B'],
        ['AC020656.1','ACTG1'],
        ['AC020656.1','IVD'],
        ['AC020656.1','NEBL'],
        ['AC020656.1','NT5C2'],
        ['AC020656.1','PIK3R2'],
        ['AC020656.1','PRH1'],
        ['AC020915.6','ZSCAN22'],
        ['AC023790.2','GPRC5D-AS1'],
        ['AC025279.2','SNX29'],
        ['AC026412.1','AC124944.3'],
        ['AC027612.2','IGK@'],
        ['AC067930.2','ANKRD19P'],
        ['AC067930.2','HBB'],
        ['AC068580.4','LSP1'],
        ['AC068587.6','AC079781.5'],
        ['AC073063.1','SARNP'],
        ['AC073610.3','ARF1'],
        ['AC079781.5','LINC00937'],
        ['AC079781.5','RBFOX1'],
        ['AC087203.3','AF228730.7'],
        ['AC091390.4','RBAK'],
        ['AC092128.1','AC138627.1'],
        ['AC092821.3','CLEC2D'],
        ['AC092821.3','GOT2'],
        ['AC092821.3','TPT1-AS1'],
        ['AC104836.1','C1ORF146'],
        ['AC107918.4','DEFB131E'],
        ['AC109583.3','PRSS46'],
        ['AC118758.2','AC138035.3'],
        ['AC119674.2','TG'],
        ['AC124312.1','WDR4'],
        ['AC124944.3','LINC00969'],
        ['AC124944.3','PDCD6'],
        ['AC133919.3','AL606534.6'],
        ['AC244517.10','AC244517.6'],
        ['ACCS','EXT2'],
        ['ACE2','PIR'],
        ['ACSS1','APMAP'],
        ['ACTA1','ACTA2'],
        ['ACTB','XPO1'],
        ['ACTC1','ACTG2'],
        ['ACTN4','EIF3K'],
        ['ADAP1','SUN1'],
        ['ADGRE2','ADGRE5'],
        ['ADGRG7','TFG'],
        ['AFF1','PTPN13'],
        ['AFP','ALB'],
        ['AGAP5','BMS1P4'],
        ['AKT2','C19ORF47'],
        ['AL022322.2','GALNT4'],
        ['AL031432.4','TG'],
        ['AL132639.3','TRAPPC6B'],
        ['AL133481.3','AL359195.2'],
        ['AL136116.3','PARL'],
        ['AL136295.5','THADA'],
        ['AL136295.7','CHMP4A'],
        ['AL136309.2','C6ORF201'],
        ['AL138963.3','PRB2'],
        ['AL139142.1','TEX50'],
        ['AL162400.2','LINC01739'],
        ['AL627309.5','AL732372.2'],
        ['AL645937.4','AL672167.1'],
        ['AL662791.5','OR2J3'],
        ['ALDH2','ANKHD1-EIF4EBP3'],
        ['ALDH6A1','SPAST'],
        ['ALG14','CNN3'],
        ['ALKBH5','LLGL1'],
        ['AMACR','SLC45A2'],
        ['AMY2A','CPA1'],
        ['AMY2B','PRH1'],
        ['ANAPC1','RMND5A'],
        ['ANK2','CAMK2D'],
        ['ANKRD18B','FAM95C'],
        ['ANP32B','IGH@'],
        ['AP000345.2','DRICH1'],
        ['AP000553.4','PPIL2'],
        ['AP001350.2','CNTF'],
        ['AP001372.3','TCF3'],
        ['AP006222.1','DUX4'],
        ['AP3S1','COMMD10'],
        ['APBB3','SRA1'],
        ['ARHGAP18','TMEM244'],
        ['ARHGEF12','SMR3B'],
        ['ARHGEF17','P2RY6'],
        ['ARL17A','KANSL1'],
        ['ARL17B','KANSL1'],
        ['ARL6IP1','RPS15A'],
        ['ARPC4-TTLL3','OGG1'],
        ['ARSD','EPN1'],
        ['ART4','SMCO3'],
        ['ASL','CRCP'],
        ['ASNS','RBFOX1'],
        ['ATAD1','PTEN'],
        ['ATG16L1','INPP5D'],
        ['ATP1B1','CEL'],
        ['ATP5I','MFSD7'],
        ['AZGP1','GJC3'],
        ['B3GNTL1','METRNL'],
        ['BABAM2','RBKS'],
        ['BANK1','IGH@'],
        ['BCAP29','DUS4L'],
        ['BCAS3','TG'],
        ['BCKDHA','TMEM91'],
        ['BEST1','IGF2'],
        ['BGLAP','PMF1'],
        ['BLOC1S6','SQOR'],
        ['BMPR1B','PDLIM5'],
        ['BOLA2-SMG1P6','SMG1'],
        ['BPIFB1','IGH@'],
        ['BPTF','LRRC37A2'],
        ['BRK1','VHL'],
        ['BSDC1','FAM229A'],
        ['C12ORF73','HSP90B1'],
        ['C15ORF38-AP3S2','YPEL5'],
        ['C19MC','RPLP1'],
        ['C19ORF33','SPINT2'],
        ['C1ORF194','UQCR10'],
        ['C2ORF27B','LINC02564'],
        ['C6ORF106','SPDEF'],
        ['CABP7','NF2'],
        ['CATSPER2','PPIP5K1'],
        ['CAVIN1','STAT3'],
        ['CBWD2','DOCK8'],
        ['CBX3','CCDC32'],
        ['CCL28','PRH1'],
        ['CD74','PRH1'],
        ['CD81','TSSC4'],
        ['CD83','JARID2'],
        ['CDC42EP2','POLA2'],
        ['CDK11B','SLC35E2'],
        ['CDK19','REV3L'],
        ['CDK2','RAB5B'],
        ['CDKN1A','RAB44'],
        ['CDR2','RRN3P2'],
        ['CEL','CELP'],
        ['CELA3A','NEAT1'],
        ['CFD','ELANE'],
        ['CHST11','IGH@'],
        ['CITED1','HDAC8'],
        ['CLCF1','POLD4'],
        ['CLTC','VMP1'],
        ['COL14A1','DEPTOR'],
        ['COL3A1','COL4A2'],
        ['COLQ','HACL1'],
        ['COMMD3-BMI1','DDX5'],
        ['COQ8B','NUMBL'],
        ['COX19','MAFK'],
        ['CPA1','DPEP1'],
        ['CPD','STATH'],
        ['CPM','LYZ'],
        ['CRCP','TPST1'],
        ['CTDP1','PQLC1'],
        ['CTSC','RAB38'],
        ['CTSD','PRB2'],
        ['CX3CL1','PRH1'],
        ['CXCR4','UBXN4'],
        ['CXCR4','ZEB2'],
        ['CYTIP','ERMN'],
        ['DCUN1D1','MCCC1'],
        ['DDX42','MAP3K3'],
        ['DEPDC1B','PDE4D'],
        ['DES','PIK3IP1'],
        ['DHX35','FAM83D'],
        ['DHX9','NPL'],
        ['DIP2B','TMBIM6'],
        ['DLGAP4','MUC7'],
        ['DOCK9','STK24'],
        ['DPH7','PNPLA7'],
        ['EEF1A1','TG'],
        ['EEF1AKMT3','TSFM'],
        ['EEF2','PRH1'],
        ['EIF2S1','RBM25'],
        ['EIF4E3','FOXP1'],
        ['ELK4','SLC45A3'],
        ['ENTPD5','FAM161B'],
        ['ERBB2','GRB7'],
        ['ETFRF1','KRAS'],
        ['EZH2','ZNF425'],
        ['FAM129C','PGLS'],
        ['FAM13A-AS1','HERC3'],
        ['FAM157B','FBXO25'],
        ['FAM53B','ST20-MTHFS'],
        ['FBXO25','TDRP'],
        ['FGFR4','ZNF346'],
        ['FLJ22447','PRKCH'],
        ['FOXP1','RYBP'],
        ['FRMD4A','PRB3'],
        ['GAPDH','PRH1'],
        ['GAS7','RCVRN'],
        ['GGT1','IGSF3'],
        ['GNAQ','LYZ'],
        ['GNAS','SENP3-EIF4A1'],
        ['GOLT1A','KISS1'],
        ['GPT','PPP1R16A'],
        ['GYPB','SAV1'],
        ['H1F0','PRH1'],
        ['H6PD','SPSB1'],
        ['HAPLN3','MFGE8'],
        ['HBB','RNF216'],
        ['HBB','TTC7A'],
        ['HBB','TTN'],
        ['HBB','USP15'],
        ['HORMAD1','SLC22A20'],
        ['HSD17B14','PLEKHA4'],
        ['HSP90AB1','SLC29A1'],
        ['HSPE1','MOB4'],
        ['HTN3','MBNL1'],
        ['IGF2','RBM3'],
        ['IGH@','LAPTM5'],
        ['IGH@','MIR100HG'],
        ['IGH@','MUC1'],
        ['IGH@','NFKB2'],
        ['IGH@','PCSK7'],
        ['IGH@','TRA@'],
        ['IGH@','ZFP36L1'],
        ['IGK@','QSOX1'],
        ['IGSF10','MED12L'],
        ['IQGAP1','ZNF774'],
        ['ITM2B','RB1'],
        ['KANSL1','LRRC37A3'],
        ['KLF2','OAZ1'],
        ['KLK1','PRSS1'],
        ['KLK4','KLKP1'],
        ['KLK7','KLK8'],
        ['KMT2C','TPTE'],
        ['KMT5A','ZNF664'],
        ['LAMA3','RBBP8'],
        ['LARP1','SMR3B'],
        ['LATS1','NUP43'],
        ['LDAH','PUM2'],
        ['LPGAT1','RPS6KC1'],
        ['LRPPRC','PNLIP'],
        ['LRRC37A3','NSF'],
        ['LTBP2','PRH2'],
        ['LYNX1','SLURP2'],
        ['LYZ','NFATC2IP'],
        ['LYZ','SFXN1'],
        ['MAML2','MTMR2'],
        ['MANBAL','SRC'],
        ['MAPKBP1','MGA'],
        ['MECOM','RPL22'],
        ['MICAL2','TEAD1'],
        ['MINPP1','NUTM2A-AS1'],
        ['MLEC','UNC119B'],
        ['MRC1','TMEM236'],
        ['MSMB','NCOA4'],
        ['MSRB3','TMBIM4'],
        ['MTA3','THADA'],
        ['MTSS1','TATDN1'],
        ['MUC7','NEBL'],
        ['MYH11','PPT1'],
        ['MYH9','TRB@'],
        ['MYL2','MYL9'],
        ['NACA','PRIM1'],
        ['NAIP','OCLN'],
        ['NCOA6','PIGU'],
        ['NCOR2','UBC'],
        ['NDRG1','ST3GAL1'],
        ['NFATC3','PLA2G15'],
        ['NFKBIZ','SRGN'],
        ['NIN','SAV1'],
        ['NPEPL1','STX16'],
        ['NPEPPS','TBC1D3'],
        ['NPIPB5','SMG1'],
        ['NUP188','PHYHD1'],
        ['ODAM','PRH1'],
        ['OR6D1P','SMG1'],
        ['OR7E161P','OR7E91P'],
        ['PBXIP1','PMVK'],
        ['PIK3R1','PRH1'],
        ['PIM3','SCO2'],
        ['PIM3','TRABD'],
        ['PITPNC1','PRKCA'],
        ['PMS2','POLR2J3'],
        ['PNLIP','RAPGEF1'],
        ['PPM1B','SLC3A1'],
        ['PPP1CB','YPEL5'],
        ['PRB3','SERF2'],
        ['PRDX1','TESK2'],
        ['PRH1','PRR27'],
        ['PRH1','RBM47'],
        ['PRH1','RPL3'],
        ['PRH1','SFRP1'],
        ['PRH1','SIX4'],
        ['PRH1','SLC6A8'],
        ['PRH1','SND1'],
        ['PRH1','SORL1'],
        ['PRH1','SQSTM1'],
        ['PRKAG2','RHEB'],
        ['PRKCZ','SKI'],
        ['PRMT7','SLC7A6'],
        ['PSMB8','TAP1'],
        ['PSPC1','ZMYM5'],
        ['PTBP3','SUSD1'],
        ['PXK','RPP14'],
        ['RAP1B','RBPJ'],
        ['RASSF8','SSPN'],
        ['RBAK-RBAKDN','RNF216'],
        ['RBMS2','SPRYD4'],
        ['RDH11','VTI1B'],
        ['RIPK1','SERPINB9'],
        ['RLN1','RLN2'],
        ['RNF125','RNF138'],
        ['RPL14','SRP14'],
        ['RPL28','TMEM190'],
        ['RPL38','TTYH2'],
        ['RWDD3','TMEM56'],
        ['SAMD5','SASH1'],
        ['SCARB1','UBC'],
        ['SCNN1A','TNFRSF1A'],
        ['SERF1A','SMN1'],
        ['SIDT2','TAGLN'],
        ['SLC39A10','TG'],
        ['SLC6A16','TEAD2'],
        ['SMR3B','STIM1'],
        ['SMR3B','UVSSA'],
        ['SNTB2','VPS4A'],
        ['SREBF2','STATH'],
        ['STATH','TMED3'],
        ['STK10','UBE2D2'],
        ['STXBP4','TOM1L1'],
        ['TBCEL','TECTA'],
        ['TFDP2','XRN1'],
        ['TTTY15','USP9Y'],
        ['UBA2','WTIP'],
        ['USP10','ZDHHC7'],
        ['USP34','XPO1'],
        ['ZNF229','ZNF285'],
        ['ZNF354C','ZNF879'],
        ['ZNF586','ZNF587'],
        ['AC022400.3','TIMM23B'],
        ['AC080128.2','AL512356.3'],
        ['AC087473.1','DA750114'],
        ['AC087473.1','IGH@'],
        ['AF228730.7','AP003498.1'],
        ['AL359715.2','AL359715.5'],
        ['AL451042.1','AL451042.2'],
        ['ALK','IGH@'],
        ['DA750114','JAK2'],
        ['DA750114','TBC1D32'],
        ['HELLPAR','IGH@'],
        ['IGH@','KCNQ1OT1'],
        ['IGH@','OLMALINC'],
        ['IGK@','JAK2'],
        ['IGK@','LRRK1'],
        ['JAK2','TRA@'],
        ['NUTM2A','NUTM2B'],
        ['IGH@','JAK2'],
        ['IGH@','MIR100HG'],
        ['AC006042.2','AC006042.4'],
        ['AC022400.3','TIMM23B'],
        ['AC024940.1','AC092821.3'],
        ['AC080128.2','AL512356.3'],
        ['AC087473.1','DA750114'],
        ['AC087473.1','IGH@'],
        ['AF228730.7','AP003498.1'],
        ['AF228730.7','CECR7'],
        ['AL109811.1','AL109811.2'],
        ['AL359715.2','AL359715.5'],
        ['AL451042.1','AL451042.2'],
        ['ALK','IGH@'],
        ['C12ORF45','DA750114'],
        ['DA750114','JAK2'],
        ['DA750114','PIGN'],
        ['DA750114','TBC1D32'],
        ['HELLPAR','IGH@'],
        ['HELLPAR','IGK@'],
        ['IGH@','JAK2'],
        ['IGH@','KCNQ1OT1'],
        ['IGH@','OLMALINC'],
        ['IGK@','JAK2'],
        ['IGK@','LRRK1'],
        ['JAK2','TRA@'],
        ['JAK2','TRB@'],
        ['NUTM2A','NUTM2B'],
        ['OLMALINC','TRA@'],
        ['OR14J1','TRA@'],
        ['SMAD2','TRB@']
        

        
        ]
    # up is human



    data = fusions.get(options.organism.lower(),[])
    if data:

        file_symbols = os.path.join(options.output_directory,'synonyms.txt')
        loci = symbols.generate_loci(file_symbols)

        genes = symbols.read_genes_symbols(file_symbols)
        d = []
        for gg in data:
            if len(gg)>2 or len(gg)<2:
                print "ERROR:",gg
                sys.exit(1)
            g1 = gg[0]
            g2 = gg[1]
            if g1.upper() != g2.upper():
                #print g1,g2
                ens1 = symbols.ensembl(g1.upper(),genes,loci)
                ens2 = symbols.ensembl(g2.upper(),genes,loci)
                if ens1 and ens2:
                    for e1 in ens1:
                        for e2 in ens2:
                            if e1 != e2:
                                d.append([e1,e2])
                                #print '-->',e1,e2
        # create a banned list of gene based on loci
        for v in loci.values():
            if v:
                n = len(v)
                if n > 1:
                    for i in xrange(n-1):
                        for j in xrange(i+1,n):
                            if v[i].upper() != v[j].upper():
                                ens1 = symbols.ensembl(v[i].upper(),genes,loci)
                                ens2 = symbols.ensembl(v[j].upper(),genes,loci)
                                if ens1 and ens2:
                                    for e1 in ens1:
                                        for e2 in ens2:
                                            if e1 != e2:
                                                d.append([e1,e2])
        data = ['\t'.join(sorted(line)) + '\n' for line in d]
        data = sorted(set(data))

    print "List of banned fusions has ",len(data),'fusions.'
    file(os.path.join(options.output_directory,'banned.txt'),'w').writelines(data)
    #
